/**
 * @license
 * Copyright (C) Level Access 2015-2022 - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * By using this code you automatically agree to Section 2 Service Terms, Section 5.2 License and Use Restrictions; and Section 6 Representations, Warranties, Remedies and Disclaimers of Level Access??? Master Subscription Agreement (http://www.levelaccess.com/msa/)
 */

"use strict";var LevelAccess_AccessEngine=function(exports){'use strict';/**
	 * Access Engine Logger
	 */class Logger{/**
		 * @singleton
		 * @param {boolean} logging
		 * @returns {Logger}
		 */constructor(logging){if(!Logger.instance){if(logging===undefined){logging=false;}this._logging=logging;Logger.instance=this;}return Logger.instance;}/**
		 * @returns {boolean}
		 */get logging(){return this._logging;}/**
		 * @param {boolean} logging
		 */set logging(logging){this._logging=logging;}/**
		 * Wrapper for console.log so we can turn on logging only when needed
		 */log(){if(this._logging){console.log.apply(window,arguments);}}/**
		 * Wrapper for indicating there was a runtime error
		 */error(){if(this._logging){if(console.error){console.error.apply(window,arguments);}else{console.log.apply(window,arguments);}}}}/**
	 * @const
	 * @type {AEKernel}
	 */const logger=new Logger();let markdownStageExecutionTimeout=null;// this timeout is disabled by default
let didMarkdownStageExecutionTimeOut=false;/**
	 * Public interface to get the markdown stage execution timeout.
	 * @returns {number}
	 */function getMarkdownStageExecutionTimeout(){return markdownStageExecutionTimeout;}/**
	 * Public interface to set the markdown stage execution timeout.
	 * Set to null to disable timeout functionality.
	 * @param {number} newMarkdownStageExecutionTimeout
	 */function setMarkdownStageExecutionTimeout(newMarkdownStageExecutionTimeout){markdownStageExecutionTimeout=newMarkdownStageExecutionTimeout;}/**
	 * Public interface to get whether the markdown stage timeout occurred.
	 * @returns {boolean}
	 */function getDidMarkdownStageExecutionTimeOut(){return didMarkdownStageExecutionTimeOut;}/**
	 * Internal interface to set whether the markdown stage timeout occurred.
	 * @param {boolean} newDidMarkdownStageExecutionTimeOut
	 */function setDidMarkdownStageExecutionTimeOut(newDidMarkdownStageExecutionTimeOut){didMarkdownStageExecutionTimeOut=newDidMarkdownStageExecutionTimeOut;}// CSS Block Styles indexed from:
// https://developer.mozilla.org/en-US/docs/Web/Guide/CSS/Block_formatting_context
// (copied from CalcNames.js)
const blockStyles={display:["block","grid","table","flow-root","flex"],position:["absolute","fixed"],float:["left","right","inline"],clear:["left","right","both","inline"],overflow:["hidden","scroll","auto"],"column-count":["!auto"],"column-width":["!auto"],"column-span":["all"],contain:["layout","content","strict"]};/**
	 * Stage 5 markdown function, which sets data-ae_ar
	 * @param {Element} node
	 */function aeArFunc(node){var ae_ar=node.getAttribute("data-ae_ar");if(ae_ar===null){return"null";}return ae_ar;}/**
	 * Stage 1 + 3b core markdown function, which detects css visibility rule
	 * @param {{}} styleFromOriginal
	 */function visibilityFunc(styleFromOriginal){let visibility=styleFromOriginal["visibility"];if(visibility==="hidden"){return"visibility;hidden";}return null;}// at the start of markdown stage execution, invoke resetStartTime() to start our stopwatch
// periodically during markdown stage execution, invoke isRunningTooLong() to check to see if cumulative execution is taking too long
// if it is, abort execution, otherwise continue until execution has finished or we hit the time limit--whichever comes first
// either way, at the end of the markdown stage execution, invoke clearStartTime() to stop our stopwatch
let startTime;function resetStartTime(){startTime=performance.now();setDidMarkdownStageExecutionTimeOut(false);}function isRunningTooLong(){if(getMarkdownStageExecutionTimeout()&&startTime){const didMarkdownStageExecutionTimeOut=performance.now()-startTime>getMarkdownStageExecutionTimeout();if(didMarkdownStageExecutionTimeOut){setDidMarkdownStageExecutionTimeOut(didMarkdownStageExecutionTimeOut);}return didMarkdownStageExecutionTimeOut;}else{return false;}}function clearStartTime(){startTime=null;setDidMarkdownStageExecutionTimeOut(false);}function getRuntimeExceededMessage(){return`markdown runtime execution limit of ${getMarkdownStageExecutionTimeout()}ms exceeded`;}let detectShadowDOM=null;let stage1_markDownFunc_errors={};/**
	 * Pre-testing: Marks down DOM with computed styles info - DOM in live DOM for massive speed increase 1000 times faster
	 * e.g. ComputedStyles and events from node code
	 * ; is used in place of : as we create an object from the attribute in stage 2
	 * done in separate stage as it needs to be fast - this 1000 times faster than the previous stage1 solutions.  Fastest as to mark down original dom with attributes that hold computed styles you don't need uels - as info is transferred in clone
	 * @returns {Number}
	 */function stage1_markDown_liveDOM_copyInfo(testWind,testDoc,stage1MarkdownFuncs){try{detectShadowDOM=false;stage1_markDownFunc_errors={};// run test dependent functions
// collect all functions added to 
let addedStage1Funcs=Object.keys(stage1MarkdownFuncs);let markEach=function(n){do{if(isRunningTooLong()){throw getRuntimeExceededMessage();}if(n.nodeType===1){let styleFromOriginal=testWind.getComputedStyle(n,null);let oI=[];let visibility=visibilityFunc(styleFromOriginal);if(visibility!==null){oI.push(visibility);}for(let prop in blockStyles){const propValue=styleFromOriginal.getPropertyValue(prop);if(propValue){oI.push(`${prop};${propValue}`);}}// each added stage1 func must be standalone, and not rely on any other
// so, they can be run in any order
for(let i=addedStage1Funcs.length;i--;){if(isRunningTooLong()){throw getRuntimeExceededMessage();}try{// each added function has same interface
let extractedData=stage1MarkdownFuncs[addedStage1Funcs[i]](styleFromOriginal,testWind,n);if(extractedData!==null){oI.push(extractedData);}}catch(err){logger.error('Error',err);stage1_markDownFunc_errors.push("name of function");}}n.setAttribute("data-ae_styles",oI.join(";"));// runs recursively through all shadowRoots and nodes
if(n.shadowRoot){if(detectShadowDOM===false){detectShadowDOM=true;}var firstChild=n.shadowRoot.firstElementChild;if(firstChild!==null){markEach(n.shadowRoot.firstElementChild);}}if(n.hasChildNodes()){if(n.firstChild!==null){markEach(n.firstChild);}}// slot - even if it has children returns false
else{if(n.childNodes){if(n.childNodes.length>0){markEach(n.childNodes[0]);}}}}}while(n=n.nextSibling);};markEach(testDoc.querySelector("body, frameset"));return 1;}catch(err){logger.error('Error','stage1_markDown_liveDOM_copyInfo',err);stage1_markDownFunc_errors["markViewable_stage1_MarkDownDOM"]=err;return 0;}}// the below is required for CalcNames
// this is here as opposed to somewhere later in the import chain because we need to ensure modifying the window object is done prior to importing CalcNames
const AccNamePrototypeNameSpace='LevelAccess_CalcNames';window.AccNamePrototypeNameSpace=AccNamePrototypeNameSpace;/**
	 * Enum for AMP test types
	 * @type {{GLOBAL: number, IMPLIED_GLOBAL: number, AUTOMATIC: number, GUIDED_AUTOMATIC: number, MANUAL: number, GUIDED_MANUAL: number, ADVISORY: number, PREVIEW_MODE: number}}
	 */const testTypes={GLOBAL:2,IMPLIED_GLOBAL:3,AUTOMATIC:4,GUIDED_AUTOMATIC:5,MANUAL:6,GUIDED_MANUAL:30,ADVISORY:31,PREVIEW_MODE:-2// There is no corresponding AMP value for these
};/**
	 * Enum for MediaTypes to filter tests
	 * @type {{WEB:number, IOS:number, ANDROID:number}}
	 */const mediaTypes={WEB:1,IOS:2,ANDROID:3};/**
	 * Enum for Alchemy fix types
	 * @type {{ATTRIBUTE: number, TAG: number, INNERHTML: number}}
	 */const fixTypes={ATTRIBUTE:1,TAG:2,INNERHTML:3};/**
	 * 1x1 gif
	 * @type {string}
	 */const dummyImage="data:image/gif;base64,R0lGODlhAQABAIAAAAAAAP///yH5BAEAAAAALAAAAAABAAEAAAIBRAA7";/**
	 * An object of accepted IANA language subtags with regions codes whose keys are are the subtags and values are always 'true'.
	 * 
	 * @type {Object}
	 * 
	 */const ACCEPTED_LANGUAGE_FULLTAGS={"en-ca":true,"en-gb":true,"en-us":true,"es-es":true,"es-mx":true,"fr-ca":true,"fr-fr":true};/**
	 * An object of accepted IANA language subtags whose keys are the subtags and values are always 'true'.
	 * (An object is used here rather than an array or set or something for both computational complexity and browser support purposes.)
	 *
	 * @type {Object}
	 */const ACCEPTED_LANGUAGE_SUBTAGS={// two-character tags
aa:true,ab:true,ae:true,af:true,ak:true,am:true,an:true,ar:true,as:true,av:true,ay:true,az:true,ba:true,be:true,bg:true,bh:true,bi:true,bm:true,bn:true,bo:true,br:true,bs:true,ca:true,ce:true,ch:true,co:true,cr:true,cs:true,cu:true,cv:true,cy:true,da:true,de:true,dv:true,dz:true,ee:true,el:true,en:true,eo:true,es:true,et:true,eu:true,fa:true,ff:true,fi:true,fj:true,fo:true,fr:true,fy:true,ga:true,gd:true,gl:true,gn:true,gu:true,gv:true,ha:true,he:true,hi:true,ho:true,hr:true,ht:true,hu:true,hy:true,hz:true,ia:true,id:true,ie:true,ig:true,ii:true,ik:true,in:true,io:true,is:true,it:true,iu:true,iw:true,ja:true,ji:true,jv:true,jw:true,ka:true,kg:true,ki:true,kj:true,kk:true,kl:true,km:true,kn:true,ko:true,kr:true,ks:true,ku:true,kv:true,kw:true,ky:true,la:true,lb:true,lg:true,li:true,ln:true,lo:true,lt:true,lu:true,lv:true,mg:true,mh:true,mi:true,mk:true,ml:true,mn:true,mo:true,mr:true,ms:true,mt:true,my:true,na:true,nb:true,nd:true,ne:true,ng:true,nl:true,nn:true,no:true,nr:true,nv:true,ny:true,oc:true,oj:true,om:true,or:true,os:true,pa:true,pi:true,pl:true,ps:true,pt:true,qu:true,rm:true,rn:true,ro:true,ru:true,rw:true,sa:true,sc:true,sd:true,se:true,sg:true,sh:true,si:true,sk:true,sl:true,sm:true,sn:true,so:true,sq:true,sr:true,ss:true,st:true,su:true,sv:true,sw:true,ta:true,te:true,tg:true,th:true,ti:true,tk:true,tl:true,tn:true,to:true,tr:true,ts:true,tt:true,tw:true,ty:true,ug:true,uk:true,ur:true,uz:true,ve:true,vi:true,vo:true,wa:true,wo:true,xh:true,yi:true,yo:true,za:true,zh:true,zu:true,// three-character tags
bsq:true,cdo:true,chk:true,cho:true,chr:true,cjy:true,cmn:true,cpx:true,czh:true,czo:true,din:true,fil:true,gan:true,hak:true,haw:true,hmn:true,hsn:true,kar:true,lzh:true,mnp:true,nan:true,pon:true,wuu:true,yue:true,zhx:true};const IMG_SUSPICIOUS_TEXT_CSS_SELECTOR_POSTFIXES=['[data-ae_an*=".gif"]','[data-ae_an*=".jpg"]','[data-ae_an*=".png"]','[data-ae_an="img"]','[data-ae_an="spacer"]','[data-ae_an="alt"]','[data-ae_an="blank"]','[data-ae_an="_"]','[data-ae_an="null"]','[data-ae_an="photo"]','[data-ae_an="image"]','[data-ae_an="corner"]','[data-ae_an="tag"]','[data-ae_an="picture"]','[data-ae_an="header"]','[data-ae_an="histogram"]','[data-ae_an="chart"]','[data-ae_an="undefined"]'];const specialTagNames=["HTML","HEAD","BODY","FRAMESET"];/**
	 * Replaces the native cloneNode function, as cloneNode causes isses in pages with elements which contain shadowRoots
	 * @param {*} node
	 * @param {*} deep
	 */function cloneThisNode(node,deep){try{if(node.nodeType===3){var textNode=document.createTextNode(node.textContent);return textNode;}// if rootNode is a web component - we create a div in its place
var tree=null;var treeTagName=node.tagName.toLowerCase();const createDiv=()=>{// likely tried to create a non-standard HTML element
// fall back to creating a div instead
tree=document.createElement("div");tree.setAttribute("data-ae_origTagName",treeTagName);};if(treeTagName.indexOf("-")!==-1){// might be a web component
createDiv();}else{try{tree=document.createElement(treeTagName);}catch(err){if(err instanceof DOMException){createDiv();}else{throw err;}}}// copy across all the attributes
var attrs=node.attributes;for(var j=attrs.length;j--;){var attr=attrs[j];if(attr.name!=="is"){// if this is href or src - we need to change that
try{// catches invalid attributes, and prevents copying them
tree.setAttribute(attr.name,attr.value);}catch(err){// do nothing
}}}// deal with shallow clone
if(deep===undefined||deep===false){return tree;}// deal with deep clone
// initially
var parentEl=tree;var x=0;var origTagName=false;let markEach=function(n){do{if(n.nodeType===1){let tagName=n.tagName;const createChildDiv=()=>{// likely tried to create a non-standard HTML element
// fall back to creating a div instead
origTagName=tagName.toLowerCase();tagName="div";childElement=document.createElement(tagName);};let childElement;if(tagName.indexOf("-")!==-1){// might be a web component
createChildDiv();}else{try{childElement=document.createElement(tagName);}catch(err){if(err instanceof DOMException){createChildDiv();}else{throw err;}}}var attrs=n.attributes;for(var j=attrs.length;j--;){var attr=attrs[j];var attrName=attr.name;var attrValue=attr.value;// if this is href or src - we need to change that
if(attrName!=="is"){try{// catches invalid attributes, and prevents copying them
childElement.setAttribute(attrName,attrValue);}catch(err){// do nothing
}}}if(n.childNodes&&n.childNodes.length>0){childElement.setAttribute("data-ae_uel",n.getAttribute("data-ae_uel"));if(origTagName!==false){childElement.setAttribute("data-ae_origTagName",origTagName);origTagName=false;}childElement.setAttribute("data-ae_justadded","true");parentEl.appendChild(childElement);parentEl=parentEl.querySelector("[data-ae_justadded]");parentEl.removeAttribute("data-ae_justadded");x=x+1;var firstChild=n.childNodes[0];if(firstChild){markEach(firstChild);}}else{// no child elements, but could contain text
if(origTagName!==false){childElement.setAttribute("data-ae_origTagName",origTagName);origTagName=false;}childElement.innerHTML=n.innerHTML;parentEl.appendChild(childElement);}}if(n.nodeType===3){// just append that value to parentEl
var textNode=document.createTextNode(n.textContent);parentEl.appendChild(textNode);}if(n.nextSibling===null){x=x-1;parentEl=getParent(parentEl);}}while(n=n.nextSibling);};var firstChild=node.firstChild;if(firstChild){markEach(firstChild);}else{tree.innerHTML=node.innerHTML;return tree;}return tree;}catch(err){logger.error(err);return null;}}/**
	 * Specific version used in markdown stages - Replaces the native cloneNode function, as cloneNode causes isses in pages with elements which contain shadowRoots
	 * @param {*} node
	 * @param {*} deep
	 */function cloneThisNodeForMarkDown(node,deep){if(detectShadowDOM===false||node.nodeType===Node.COMMENT_NODE){return node.cloneNode(deep);}if(node.nodeType===3){var textNode=document.createTextNode(node.textContent);return textNode;}// issue - certain pure HTML elements are not able to have a shadowRoot - but if they are created using the "is" attribute they can have
// we create a div in its place, whatever the element - then record it's original tagName
var tree=null;var treeTagName=node.tagName.toLowerCase();tree=document.createElement("div");tree.setAttribute("data-ae_origTagName",treeTagName);// we need to record all element tagNames now, set everything to a div which can have a shadowRoot, then convert everything back in 3d - to its original tag name, so the tests work
// if "is" attribute is used, we use the original tagName instead of the "is" as the is is only extending base functionality.
// copy across all the attributes
var attrs=node.attributes;for(var j=attrs.length;j--;){if(isRunningTooLong()){throw getRuntimeExceededMessage();}var attr=attrs[j];if(attr.name!=="is"){// if this is href or src - we need to change that
try{// catches invalid attributes, and prevents copying them
tree.setAttribute(attr.name,attr.value);}catch(err){// do nothing
}}}// deal with shallow clone
if(deep===undefined||deep===false){return tree;}// deal with deep clone
// initially
var parentEl=tree;var x=0;var origTagName=false;try{let markEach=function(n){do{if(isRunningTooLong()){throw getRuntimeExceededMessage();}if(n.nodeType===1){var tagName=n.tagName;origTagName=tagName.toLowerCase();var childElement=document.createElement("div");childElement.setAttribute("data-ae_origTagName",origTagName);var attrs=n.attributes;for(var j=attrs.length;j--;){var attr=attrs[j];var attrName=attr.name;var attrValue=attr.value;// if this is href or src - we need to change that
if(attrName!=="is"){// if this is href or src - we need to change that
try{// catches invalid attributes, and prevents copying them
childElement.setAttribute(attrName,attrValue);}catch(err){// do nothing
}}}if(n.childNodes&&n.childNodes.length>0){childElement.setAttribute("data-ae_uel",n.getAttribute("data-ae_uel"));childElement.setAttribute("data-ae_justadded","true");parentEl.appendChild(childElement);parentEl=parentEl.querySelector("[data-ae_justadded]");parentEl.removeAttribute("data-ae_justadded");x=x+1;var firstChild=n.childNodes[0];if(firstChild){markEach(firstChild);}}else{// strip contents of script and style elements
if(origTagName!=="script"&&origTagName!=="style"&&origTagName!=="template"&&origTagName!=="noscript"){childElement.innerHTML=n.innerHTML;}parentEl.appendChild(childElement);}}if(n.nodeType===3){// just append that value to parentEl
var textNode=document.createTextNode(n.textContent);parentEl.appendChild(textNode);}if(n.nextSibling===null){x=x-1;parentEl=getParent(parentEl);}}while(n=n.nextSibling);};//IE11 lacks .children method so we fake one if needed
fixChildren(node);if(node.children.length>0){markEach(node.children[0]);}else{// copy across innerHTML
if(origTagName!=="script"&&origTagName!=="style"&&origTagName!=="template"&&origTagName!=="noscript"){tree.innerHTML=node.innerHTML;}}return tree;}catch(err){logger.error(err);return null;}}/*
	 * Pre-Testing: retrieves computed styles copied over from css in original into an object
	 * @returns {Obj}
	 */function retrieveComputedCssStyles(node,pseudoElt){const oIAttributes=node.getAttribute("data-ae_styles");const oI=oIAttributes!==null?oIAttributes.split(";"):[];const originalInformation={};for(let j=oI.length;j--;){if(j%2!==0){continue;}let propName=null;if(pseudoElt){if(oI[j].indexOf(pseudoElt)!==-1){propName=oI[j].replace(pseudoElt,"");}else{continue;}}else if(oI[j].indexOf(':before')===-1&&oI[j].indexOf(':after')===-1){propName=oI[j];}else{continue;}if(propName){let propValue=oI[j+1];if(propName==='content'){propValue=propValue.replace(/^"|\\|"$/g,"");}originalInformation[propName]=propValue;}}return originalInformation;}/**
	 * Stage 3b markdown function + stage 5
	 * @param {Element} node
	 */function aeSibFunc(node){// previous sibling - due to branch search generating previous id is tricky without attribute
var sib="1";var previousSibling=node.previousElementSibling;if(previousSibling){var prevSib=previousSibling.getAttribute("data-ae_sib");if(prevSib){sib=parseInt(prevSib)+1;}}return sib;}/**
	 * Stage 3b markdown function + stage 5
	 * This is the sibling position of the element in the actual DOM, rather than the Virtual DOM created for testing
	 * @param {Element} node
	 */function aeDOMSibFunc(node){// previous sibling - due to branch search generating previous id is tricky without attribute
var sib="1";var previousSibling=node.previousElementSibling;if(previousSibling){var prevSib=previousSibling.getAttribute("data-ae_domsib");if(prevSib){sib=parseInt(prevSib)+1;}}return sib;}/**
	 * parentElement is not natively supported on a number of elements
	 * @param {Element} element
	 */function getParent(element){let parentElement=element.parentElement;if(parentElement===undefined){parentElement=element.parentNode;}return parentElement;}/**
	 * Used in uelAccurate_FromRoot
	 * @param {Element} element
	 * @param {boolean} tagSpecific
	 * @returns {Number}
	 */function getPositionInParent(element){let tagSpecific=arguments.length>1&&arguments[1]!==undefined?arguments[1]:false;let position=1;let selectedElement=element;while((selectedElement=selectedElement.previousElementSibling)!==null){if(!tagSpecific||selectedElement.tagName===element.tagName){position++;}}return position;}/**
	 * Calculates a long CSS selector (UEL = Unique Element Locator) for finding that unique element from root node
	 * Use *:nth-child chain until we hit the body
	 * @param {Element} element
	 * @returns {String}
	 */function uelAccurate_FromRoot(element){// use lowercase tag names in locators;
// the use of uppercase tag names in selectors means elements (svg, math) get missed in IE
if(specialTagNames.includes(element.tagName)){return element.tagName.toLowerCase();}let locator=`*:nth-child(${getPositionInParent(element)})`;let parent=getParent(element);let previousParent=element;while(parent.tagName!=="HTML"){const parentTagName=parent.tagName;if(specialTagNames.includes(parentTagName)){const posSecondBracket=locator.indexOf(")")+1;const locatorSecondPart=locator.slice(posSecondBracket,locator.length);const locatorFirstPart=`${previousParent.tagName.toLowerCase()}:nth-of-type(${getPositionInParent(previousParent,true)})`;locator=`${parentTagName.toLowerCase()}>${locatorFirstPart}${locatorSecondPart}`;}else{locator=`*:nth-child(${getPositionInParent(parent)})>${locator}`;}previousParent=parent;parent=getParent(parent);}return locator;}/**
	 * Stage 5, 6 markdown function, which sets data-ae_uel
	 * @param {Element} node
	 */function aeUELFunc(node){return aeBaseUELFunc(node,"data-ae_uel","data-ae_sib");}/**
	 * Stage 5, 6 markdown function, which sets data-ae_domuel
	 * @param {Element} node
	 */function aeDOMUELFunc(node){return aeBaseUELFunc(node,"data-ae_domuel","data-ae_domsib");}/**
	 * Stage 5, 6 markdown function, which sets data-ae_domuel
	 * @param {Element} node
	 * @param attUel
	 * @param attSib
	 * @returns {*}
	 */function aeBaseUELFunc(node,attUel,attSib){let uel=null;const parent=getParent(node);let host;try{host=node.getRootNode().host;}catch(err){host=undefined;}if(parent===null&&typeof host!=="undefined"){// top level child in a web component
uel=host.getAttribute(attUel)+"|";// indicates that we are now in a web component
}const nameTagName=node.tagName;if(specialTagNames.includes(nameTagName)){uel=nameTagName.toLowerCase();}else{if(parent!==null){uel=parent.getAttribute(attUel);}if(uel===null){uel=uelAccurate_FromRoot(node);}else{// we also have to place the web component locator in the uel
// e.g. if uel element has data-ae_shdwrt then break uel and start it with [data-ae_shdwrt='']
if(uel.charAt(uel.length-1)==="|"){uel=`${uel}:host>*:nth-child(${node.getAttribute(attSib)})`;}else if(specialTagNames.includes(uel.toUpperCase())){uel=`${uel}>${node.tagName.toLowerCase()}:nth-of-type(${getPositionInParent(node,true)})`;}else{uel=`${uel}>*:nth-child(${node.getAttribute(attSib)})`;}}}return uel;}/**
	     * CSS breaks down as a locator within ShadowDOM in some older browsers
		 * @param {Element} startNode
		 * @param locator
	     */function findElementThroughTreeBranch(startNode,locator){var locatedNode=null;var markEach=function markEach(n){do{if(n.nodeType===1){if(n.getAttribute("data-ae_domuel")===locator){locatedNode=n;break;}if(n.shadowRoot){// need to paint all nodes in all shadowRoots.
// this makes it recursive
var firstChild=n.shadowRoot.firstElementChild;if(firstChild!==null){markEach(n.shadowRoot.firstElementChild);}}if(n.hasChildNodes()){if(n.firstChild!==null){markEach(n.firstChild);}}// slot - even if it has children returns false
else{if(n.childNodes){if(n.childNodes.length>0){markEach(n.childNodes[0]);}}}}}while(n=n.nextSibling);};markEach(startNode);return locatedNode;}/**
	 * Get element information from original shadowRoot
	 * @param {*} elementShdwrtLocation
	 * @param {HTMLDocument} testDoc
	 */function getElementFromOrigShadowRoot(elementShdwrtLocation,testDoc){// [id='app']|[id='footer']|>*:nth-child(2)>*:nth-child(3)>*:nth-child(6)>*:nth-child(1)
// we split the locator on "|". The piece before this is the web component which indicates 
// recursively in web components
var element=null;try{var levels=elementShdwrtLocation.split("|");// IMPORTANT: we need to reduce the length by 1 
// as the last piece of the locator is the css selector within the lowest level of web components
var currentShadowDOMHost=null;for(var i=0,len=levels.length;i<len;i++){// recursively find, and move to each shadowDOMHost
var locator=levels[i];if(i!==len-1){// this is handling locating subsequent shadowDOMs
// if the locator looks like *:nth-child(5) it is easy for other content to be located instead of the direct child needed
// as the direct child could be located after other matching content has been located
// so we recognise *:nth-child(x) 
// and if we match it - we move other to using direct children instead.
// noting that there is no :root css locator in a shadowDOM seemingly.
if(currentShadowDOMHost===null){currentShadowDOMHost=testDoc.querySelector(locator);}else{// here is the main issue for the *:nth-child(x) locator
// if the locator starts with *:nth-child( and there is no > then we'll have recognised the issue
if(locator.indexOf("*:nth-child(")===0&&locator.indexOf(">")===-1){var number=parseInt(locator.substring(12,locator.indexOf(")")));//IE11 lacks .children method so we fake one if needed
fixChildren(currentShadowDOMHost);currentShadowDOMHost=currentShadowDOMHost.shadowRoot.children[number-1];}else{currentShadowDOMHost=currentShadowDOMHost.shadowRoot.querySelector(locator);}}}else{if(currentShadowDOMHost===null){element=testDoc.querySelector(locator);}else{// here is the main issue for the *:nth-child(x) locator
// if the locator starts with *:nth-child( and there is no > then we'll have recognised the issue
if(locator.indexOf("*:nth-child(")===0&&locator.indexOf(">")===-1){var number=parseInt(locator.substring(12,locator.indexOf(")")));//IE11 lacks .children method so we fake one if needed
fixChildren(currentShadowDOMHost);element=currentShadowDOMHost.shadowRoot.children[number-1];}else{element=currentShadowDOMHost.shadowRoot.querySelector(locator);}}}}if(element===null){// needs to be made more efficient - this is a brute force way to find the elements when css does not work e.g. FF59
element=findElementThroughTreeBranch(testDoc.querySelector("body, frameset"),elementShdwrtLocation);}}catch(err){logger.error('Error',"aeUtils: getElementFromOrigShadowRoot",err);}return element;}/**
	 * To be moved to aeUtils as it is used in 3c also
	 * @param {Element} n
	 * @param {HTMLDocument} testDoc
	 */function rebuildShadowDOMFromOrig(n,testDoc){// domuel should be available from the clone
try{const domuel=n.getAttribute("data-ae_domuel");// use domuel to get the shadowDOM element 
const origElement=getElementFromOrigShadowRoot(domuel,testDoc);if(origElement){const origShadRoot=origElement.shadowRoot;if(origShadRoot){if(n.shadowRoot){// remove all content from existing shadowRoot as it will be partial
n.shadowRoot.innerHTML="";// copy across and append children from original shadowRoot
//IE11 lacks .children method so we fake one if needed
fixChildren(origShadRoot);const childrenOfOrigElementShadowRoot=origShadRoot.childNodes;for(let c=0,len=childrenOfOrigElementShadowRoot.length;c<len;c++){n.shadowRoot.appendChild(cloneThisNodeForMarkDown(childrenOfOrigElementShadowRoot[c],true));}}else{// we create a shadow root on the element which was meant to have one
// then we copy across the shadowRoot from the original element
const addedShadowRoot=n.attachShadow({mode:'open'});//IE11 lacks .children method so we fake one if needed
fixChildren(origShadRoot);const childrenOfOrigElementShadowRoot=origShadRoot.childNodes;for(let c=0,len=childrenOfOrigElementShadowRoot.length;c<len;c++){addedShadowRoot.appendChild(cloneThisNodeForMarkDown(childrenOfOrigElementShadowRoot[c],true));}}}}}catch(err){logger.error('Error',"aeUtils: rebuildShadowDOMFromOrig",err);}}/**
	 * Determines if a name or id value is valid
	 * @param {String} attValue
	 * @returns {Boolean}
	 */function isValidIdOrName(attValue){// Must contain at least one character
// Must not contain any space characters
var id=attValue.trim();if(id!==""){if(id.indexOf(" ")===-1){return true;}}return false;}/**
	 * Doctors IE11 lack of .children method on nodes by adding a fake one
	 * @param {Element} node
	 * @returns {Element} node
	 */function fixChildren(node){if(typeof node.children==="undefined"){var children=[];var theChildren=node.childNodes;for(var i=0,len_theChildren=theChildren.length;i<len_theChildren;i++){var nodeToCheck=theChildren[i];if(nodeToCheck.nodeType===1){children[children.length]=nodeToCheck;}}node.children=children;}return node;}/**
	 * Returns original width, height, x, y from css styles
	 * Used in Mobile
	 * @returns {String}
	 */function origPositionAttsFromStyle(styleString){var stylePieces=styleString.split(";");var result=["width","height","x","y"];for(var i=0,len_style=stylePieces.length;i<len_style;i++){var piece=stylePieces[i].trim();if(piece.indexOf(":")!==-1){var bits=piece.split(":");switch(bits[0]){case"width":result[0]=bits[1].replace("px","").trim();break;case"height":result[1]=bits[1].replace("px","").trim();break;case"left":result[2]=bits[1].replace("px","").trim();break;case"top":result[3]=bits[1].replace("px","").trim();break;}}}return result;}function decToHex(dec){var hex=parseInt(dec).toString(16);return hex.length===1?'0'+hex:hex;}function rgbToHex(rgb){var color=rgb.replace('rgb(','').replace(')','').split(',');for(var i=0;i<color.length;i++){color[i]=decToHex(color[i]);}return'#'+color.join('');}// source: https://stackoverflow.com/a/14794066
function isInt(value){let x;if(isNaN(value)){return false;}x=parseFloat(value);return(x|0)===x;}/**
	 * Returns html of instance with ae attributes removed
	 * @returns {Node}
	 */function removeAEAttributes(node){let clone=cloneThisNode(node,true);// remove ae data-attributes from clone
const attributes=clone.attributes;for(let i=attributes.length;i--;){const name=attributes[i].name;if(name.substring(0,8)==="data-ae_"||name.substring(0,8)==="data-la-"){clone.removeAttribute(name);}}// remove ae data-attributes from any children of clone
const descends=clone.querySelectorAll("*");for(let j=descends.length;j--;){const desc=descends[j];const descAttributes=desc.attributes;for(let k=descAttributes.length;k--;){const dname=descAttributes[k].name;if(dname.substring(0,8)==="data-ae_"||dname.substring(0,8)==="data-la-"){desc.removeAttribute(dname);}}}return clone;}let stage0_markDownFunc_errors={};/**
	 * Pre-testing: Marks down DOM with computed styles info - DOM in live DOM for massive speed increase 1000 times faster
	 * e.g. ComputedStyles and events from node code
	 * ; is used in place of : as we create an object from the attribute in stage 2
	 * done in separate stage as it needs to be fast - this 1000 times faster than the previous stage1 solutions.  Fastest as to mark down original dom with attributes that hold computed styles you don't need uels - as info is transferred in clone
	 * @returns {Number}
	 */function stage0_markDown_liveDOM_domuels(testDoc){try{resetStartTime();stage0_markDownFunc_errors={};const headNode=testDoc.querySelector('head');if(headNode){const firstHeadChild=headNode.firstChild;if(firstHeadChild){const markEachInHead=function(n){do{if(isRunningTooLong()){throw getRuntimeExceededMessage();}if(n.nodeType!==1){continue;}// any element that's not allowed to be in a head element according to the html spec should be flagged for removal from our eventual clone of this DOM
if(!['base','link','meta','noscript','script','style','template','title'].includes(n.tagName.toLowerCase())){n.setAttribute('data-ae_removefromclone','true');}// check children
if(n.shadowRoot){const firstChild=n.shadowRoot.firstElementChild;if(firstChild){markEachInHead(firstChild);}}if(n.hasChildNodes()){const firstChild=n.firstChild;if(firstChild){markEachInHead(firstChild);}}else if(n.childNodes&&n.childNodes.length>0){const firstChild=n.childNodes[0];if(firstChild){markEachInHead(firstChild);}}}while(n=n.nextSibling);};markEachInHead(firstHeadChild);}}let markEach=function(n){do{if(isRunningTooLong()){throw getRuntimeExceededMessage();}if(n.nodeType===1){if(n.hasAttribute("data-ae_ignore")&&n.hasAttribute("src")){// The data-ae_ignore attribute tags an img element for special beacon/tracking handling so the src is not fetched more than once.
// These elements will have their normal src replaced with a data URL that prevents any future hits to the server when the DOM is cloned.
// The URL is left in this inert state because restoring it would cause another fetch and a false tally on the tracking system.
n.setAttribute("src",dummyImage);}if(n.tagName&&["audio","video"].includes(n.tagName.toLowerCase())){const duration=n.duration;if(duration&&duration>3){n.setAttribute("data-ae_durationtoolong",duration);}}// IMPORTANT: For Web Components
// sib needs calculating before aeDOMUELFunc, as aeDOMUELFunc needs sib number
n.setAttribute("data-ae_domsib",aeDOMSibFunc(n));var calcUel=aeDOMUELFunc(n);n.setAttribute("data-ae_domuel",calcUel);if(n.shadowRoot){n.setAttribute("data-ae_shdwrt",calcUel);// need to paint all nodes in all shadowRoots.
// this makes it recursive
var firstChild=n.shadowRoot.firstElementChild;if(firstChild!==null){markEach(n.shadowRoot.firstElementChild);}}if(n.hasChildNodes()){if(n.firstChild!==null){markEach(n.firstChild);}}// slot - even if it has children returns false
else{if(n.childNodes){if(n.childNodes.length>0){markEach(n.childNodes[0]);}}}}}while(n=n.nextSibling);};markEach(testDoc.querySelector("body, frameset"));return 1;}catch(err){logger.error('Error','stage0_markDown_liveDOM_domuels',err);stage0_markDownFunc_errors["markViewable_stage0_MarkDownDOM"]=err;return 0;}}let stage2_markDownFunc_errors={};/**
	 * Pre-testing: Clones HTML Node
	 * @param {HTMLDocument} testDoc
	 * @returns {Node}
	 */function stage2_cloneDOM(testDoc){try{stage2_markDownFunc_errors={};let clone;// if shadow DOM isn't present, we can now safely clone the page DOM using its string representation to produce a new HTML document,
// otherwise we first need to use Engine's custom cloning mechanism before cloning the page DOM using its string representation in stage3d;
// shadow DOM isn't natively included in the page DOM's string representation
if(detectShadowDOM===false){const innerHTMLByUEL={};// temporarily strip contents of noscript elements from live DOM,
// persisting them to memory for restoration after the live DOM has been cloned
const noscriptElements=testDoc.querySelectorAll('noscript');for(let i=0;i<noscriptElements.length;i++){const element=noscriptElements[i];const uel=element.getAttribute('data-ae_domuel');innerHTMLByUEL[uel]=element.innerHTML;element.innerHTML='';}clone=new DOMParser().parseFromString(testDoc.documentElement.outerHTML,"text/html").documentElement;// restore saved contents of noscript elements from live DOM
const noscriptUELs=Object.keys(innerHTMLByUEL);for(let i=0;i<noscriptUELs.length;i++){const uel=noscriptUELs[i];const innerHTML=innerHTMLByUEL[uel];const element=testDoc.querySelector(uel);if(element){element.innerHTML=innerHTML;}}// delete any elements marked for removal from cloned DOM
const nodesToRemoveFromClone=clone.querySelectorAll('*[data-ae_removefromclone]');for(let i=0;i<nodesToRemoveFromClone.length;i++){const nodeToRemoveFromClone=nodesToRemoveFromClone[i];nodeToRemoveFromClone.parentNode.removeChild(nodeToRemoveFromClone);}// strip contents of script and style elements
const nodesToStrip=clone.querySelectorAll('script, style, template');for(let i=0;i<nodesToStrip.length;i++){const nodeToStrip=nodesToStrip[i];nodeToStrip.innerHTML='';}}else{clone=cloneThisNodeForMarkDown(testDoc.documentElement,true);}return clone;}catch(err){logger.error('Error','stage2_cloneDOM',err);stage2_markDownFunc_errors["stage2_cloneDOM"]=err;return null;}}let stage3_markDownFunc_errors={};/**
	 * Pre-testing: Removes data-ae_ attributes from live DOM
	 */function stage3_cleanUpLiveDOM(testDoc){try{stage3_markDownFunc_errors={};const markEachInHead=function(n){do{if(n.nodeType===1){n.removeAttribute("data-ae_removefromclone");// check children
if(n.shadowRoot){const firstChild=n.shadowRoot.firstElementChild;if(firstChild){markEachInHead(firstChild);}}if(n.hasChildNodes()){const firstChild=n.firstChild;if(firstChild){markEachInHead(firstChild);}}else if(n.childNodes&&n.childNodes.length>0){const firstChild=n.childNodes[0];if(firstChild){markEachInHead(firstChild);}}}}while(n=n.nextSibling);};markEachInHead(testDoc.querySelector("head"));const markEach=function(n){do{if(n.nodeType===1){n.removeAttribute("data-ae_durationtoolong");n.removeAttribute("data-ae_styles");n.removeAttribute("data-ae_shdwrt");n.removeAttribute("data-ae_domuel");n.removeAttribute("data-ae_domsib");// check children
if(n.shadowRoot){const firstChild=n.shadowRoot.firstElementChild;if(firstChild){markEach(firstChild);}}if(n.hasChildNodes()){const firstChild=n.firstChild;if(firstChild){markEach(firstChild);}}else if(n.childNodes&&n.childNodes.length>0){const firstChild=n.childNodes[0];if(firstChild){markEach(firstChild);}}}}while(n=n.nextSibling);};markEach(testDoc.querySelector("body, frameset"));return 1;}catch(err){stage3_markDownFunc_errors["markViewable_stage2_CleanUpLiveDOM"]=err;return 0;}}let stage3a_markDownFunc_errors={};/**
	 * Function to transfer in only the top level shadowRoots from elements which have shadowRoots in the live DOM.
	 * Adding the shadowRoots for top-level elements automatically pull in all nested shadowDOMs. [This is not correct]
	 * ShadowRoots from heavily nested web components are not pulled in - really just ones in the first level
	 */function stage3a_CopyOverShadowHosts(rootTestNode,testDoc){try{stage3a_markDownFunc_errors={};let markEach=function(n){do{if(isRunningTooLong()){throw getRuntimeExceededMessage();}if(n.nodeType===1){rebuildShadowDOMFromOrig(n,testDoc);if(n.shadowRoot){// IMPORTANT: this makes it recursive
var firstChild=n.shadowRoot.firstElementChild;if(firstChild!==null){markEach(n.shadowRoot.firstElementChild);}}if(n.hasChildNodes()){if(n.firstChild!==null){markEach(n.firstChild);}}// slot - even if it has children returns false
else{if(n.childNodes){if(n.childNodes.length>0){markEach(n.childNodes[0]);}}}}}while(n=n.nextSibling);};// we have not reverted back to original element tagNames yet
markEach(rootTestNode.querySelector('[data-ae_origTagName="body"], [data-ae_origTagName="frameset"]'));return rootTestNode;}catch(err){logger.error('Error','stage3a_markDownFunc_errors',err);stage3a_markDownFunc_errors["markViewable_stage3a_MarkDownDOM"]=err;return null;}}let stage3c_markDownFunc_errors={};/**
	 * Replace the slot content - ShadowDOM v1
	 * @param {*} node
	 * @param {*} shadDOM
	 */function replaceSlotsInShadowDOM(node,shadDOM,testDoc){while(shadDOM.querySelector('[data-ae_origTagName="slot"]')){var slotToReplace=shadDOM.querySelector('[data-ae_origTagName="slot"]');var parent=getParent(slotToReplace);if(parent===null){parent=shadDOM;}var slotFromLiveDOM=getElementFromOrigShadowRoot(slotToReplace.getAttribute("data-ae_domuel"),testDoc);// this is now the live one
if(slotFromLiveDOM&&'assignedNodes'in slotFromLiveDOM){var contentInSlot=slotFromLiveDOM.assignedNodes({flatten:true});if(typeof contentInSlot!=="undefined"){// we append all content from contentInSlot to the parent
for(var i=0,len=contentInSlot.length;i<len;i++){// IMPORTANT: clone the content in the slot
var contentInSlotClone=cloneThisNodeForMarkDown(contentInSlot[i],true);// the following does and insertAfter - and preserves order
parent.insertBefore(contentInSlotClone,slotToReplace.nextSibling);}}}// then we remove the slot
parent.removeChild(slotToReplace);}// we want to remove the content from the body of the node
node.innerHTML="";}/**
	 * Replaces the contents of those slot elements located in the shadowDOM with the relevant content from the web components body e.g. host root
	 * @param {Element} node
	 * @param {*} shadDOM
	 * @param {HTMLDocument} testDoc
	 * At this stage - we need to check if all the shadowRoots are transferred through a clone + append
	 */function buildOutVirtualDOMNode(node,shadDOM,testDoc){// cannot clone shadowRoot - currently
var childNs=shadDOM.childNodes;// appending is better than swapping outer for inner html, as it preserves properties
// we also append text nodes
for(var c=0,len=childNs.length;c<len;c++){var childN=childNs[c];if(typeof childN!=="undefined"){var nodeType=childN.nodeType;if(typeof nodeType!=="undefined"){if(nodeType===1){// element clone
var childNClone=null;childNClone=cloneThisNodeForMarkDown(childN,true);// set a temp id to find the appended child by in the node
// it will be removed later
childNClone.setAttribute("data-ae_tempcloneid","");node.appendChild(childNClone);var elInClone=node.querySelector("[data-ae_tempcloneid]");if(elInClone){elInClone.removeAttribute("data-ae_tempcloneid");// you need to reattach the shadowRoot of the cloned element
if(elInClone.shadowRoot){// you have to re-build from the original element in the live DOM
rebuildShadowDOMFromOrig(elInClone,testDoc);}// then if the clone has children 
// then you need to reattach the shadowRoot of any children of the cloned element
if(elInClone.children.length>0){let markEachChild=function(n){do{if(n.nodeType===1){fixChildren(n);rebuildShadowDOMFromOrig(n,testDoc);if(n.children.length>0){markEachChild(n.children[0]);}}}while(n=n.nextSibling);};markEachChild(elInClone.children[0]);}}}if(nodeType===3){// text node
var childTextNClone=cloneThisNodeForMarkDown(childN,true);node.appendChild(childTextNClone);}}}}return node;// now contains slot replacements
}/**
	 * Builds out the cloned HTML DOM into a true Virtualised DOM that Access Engine can test
	 * @param {*} rootTestNode
	 * @param {*} testDoc
	 */function stage3c_BuildOutShadowDOM(rootTestNode,testDoc){try{stage3c_markDownFunc_errors={};let markEach=function(n){do{if(isRunningTooLong()){throw getRuntimeExceededMessage();}if(n.nodeType===1){rebuildShadowDOMFromOrig(n,testDoc);if(n.shadowRoot){// replace slots here as the slot contents has been given the correct position from 3b
replaceSlotsInShadowDOM(n,n.shadowRoot,testDoc);buildOutVirtualDOMNode(n,n.shadowRoot,testDoc);n.shadowRoot.innerHTML="";}fixChildren(n);if(n.children.length>0){var firstChild=n.children[0];if(firstChild!==null){markEach(firstChild);}}}}while(n=n.nextSibling);};markEach(rootTestNode.querySelector('[data-ae_origTagName="body"], [data-ae_origTagName="frameset"]'));return rootTestNode;}catch(err){logger.error('Error','stage3c_markDownFunc_errors',err);stage3c_markDownFunc_errors["stage3c_BuildOutShadowDOM"]=err;return null;}}let stage3d_markDownFunc_errors={};/**
	 * Alter back elements to their original tagName - so that tests continue to work
	 * @param {*} node 
	 */function stage3d_RevertElementsToOrigTagName(node){stage3d_markDownFunc_errors={};var tree;var treeTagName=node.getAttribute("data-ae_origTagName");tree=document.createElement(treeTagName);// copy across all the attributes
// just copy attributes, as all manipulation is already done in earlier stages
var attrs=node.attributes;for(var j=attrs.length;j--;){var attr=attrs[j];tree.setAttribute(attr.name,attr.value);}var parentEl=tree;var x=0;var origTagName=false;try{var markEach=function markEach(n){do{if(isRunningTooLong()){throw getRuntimeExceededMessage();}if(n.nodeType===1){fixChildren(n);let tagName=n.getAttribute("data-ae_origTagName");if(tagName!=="html"){const createChildDiv=()=>{// likely tried to create a non-standard HTML element
// fall back to creating a div instead
origTagName=tagName;tagName="div";childElement=document.createElement(tagName);};let childElement;if(tagName.indexOf("-")!==-1){// might be a web component
createChildDiv();}else{try{childElement=document.createElement(tagName);}catch(err){if(err instanceof DOMException){createChildDiv();}else{throw err;}}}// just copy attributes, as all manipulation is already done in earlier stages
var attrs=n.attributes;for(var j=attrs.length;j--;){var attr=attrs[j];childElement.setAttribute(attr.name,attr.value);}// > instead of !==
if(n.childNodes.length>0){childElement.setAttribute("data-ae_justadded","true");parentEl.appendChild(childElement);//added
parentEl=parentEl.querySelector("[data-ae_justadded]");parentEl.removeAttribute("data-ae_justadded");//added ended
x=x+1;markEach(n.childNodes[0]);}else{childElement.innerHTML=n.innerHTML;// IE will set "Submit Query" or "Reset" on INPUT elements of type submit | reset which lack a value
if(childElement.tagName==="INPUT"){if(childElement.hasAttribute("value")===true){var nVal=n.getAttribute("value");var childVal=childElement.getAttribute("value");if(nVal!==childVal){childElement.setAttribute("value",nVal);}}}parentEl.appendChild(childElement);}}}if(n.nodeType===3){// just append that value to parentEl
var orig=n.parentElement.getAttribute("data-ae_origTagName");if(orig!=="style"&&orig!=="script"&&orig!=="noscript"&&orig!=="template"){var textNode=document.createTextNode(n.textContent);parentEl.appendChild(textNode);}}if(n.nextSibling===null){x=x-1;parentEl=getParent(parentEl);}}while(n=n.nextSibling);};fixChildren(node);if(node.children.length>0){markEach(node.children[0]);}// produce a new HTML document by cloning the page DOM using its string representation,
// which mimics what's done in stage2 when there's no shadow DOM,
// but we couldn't do this before because we had yet to resolve all the shadow DOM stuff
const clonedDocument=new DOMParser().parseFromString(tree.outerHTML,"text/html");tree=clonedDocument.documentElement;return tree;}catch(err){logger.error("Error: stage3d_markDownFunc_errors",err);stage3d_markDownFunc_errors["stage3d_RevertElementsToOrigTagName"]=err;return null;}}let stage4_markDownFunc_errors={};/*
	 * Pre-Testing: Finds elements that have role attribute and adds an "data-ae_ar" attribute.
	 * Note" this does not add null value to all elements without a role, or a role='' - which is needed.  That happens in stage2_runoff.
	 * @returns {Number}
	 */function addRoleAtt(rootTestNode){try{var els=rootTestNode.querySelectorAll("*[role]:not([role=''])");for(var i=els.length;i--;){if(isRunningTooLong()){throw getRuntimeExceededMessage();}var el=els[i];var roleAttribute=el.getAttribute("role").trim();var firstRole=roleAttribute.split(" ")[0];el.setAttribute("data-ae_ar",firstRole.toLowerCase());}return rootTestNode;}catch(err){logger.error('Error','stage4_addRoleAtt',err);stage4_markDownFunc_errors["stage4_addRoleAtt"]=err;return null;}}/*
	 * Pre-Testing: Finds elements that are display:none / visibility:hidden data-ae_invis=true attribute.
	 * exclude "datalist, param, script, template" as they are display:none by default.
	 * @returns {Number}
	 */function invisEls(rootTestNode){try{var els=rootTestNode.querySelectorAll("*[data-ae_styles*='visibility;hidden'], *[data-ae_styles*='display;none']:not(datalist):not(param):not(script):not(template), noscript");for(var i=els.length;i--;){if(isRunningTooLong()){throw getRuntimeExceededMessage();}els[i].setAttribute("data-ae_invis","true");}return rootTestNode;}catch(err){logger.error('Error','stage4_invisEls',err);stage4_markDownFunc_errors["stage4_invisEls"]=err;return null;}}/*
	 * Pre-Testing: Finds elements that are descendants of elements marked as data-ae_invis.
	 * @returns {Number}
	 */function invisElsDescendants(rootTestNode){try{var els=rootTestNode.querySelectorAll("*[data-ae_invis] *");for(var i=els.length;i--;){if(isRunningTooLong()){throw getRuntimeExceededMessage();}els[i].setAttribute("data-ae_invis","true");}return rootTestNode;}catch(err){logger.error('Error','stage4_invisElsDescendants',err);stage4_markDownFunc_errors["stage4_invisElsDescendants"]=err;return null;}}/*
	 * Pre-Testing: Finds elements that are aria-hidden and sets data-ae_at_hidden=true attribute.
	 * @returns {Number}
	 */function ariaHiddenEls(rootTestNode){try{var els=rootTestNode.querySelectorAll("*[aria-hidden='true']");for(var i=els.length;i--;){if(isRunningTooLong()){throw getRuntimeExceededMessage();}els[i].setAttribute("data-ae_at_hidden","true");}return rootTestNode;}catch(err){logger.error('Error','stage4_ariaHiddenEls',err);stage4_markDownFunc_errors["stage4_ariaHiddenEls"]=err;return null;}}/*
	 * Pre-Testing: Finds elements that are descendants of elements marked as data-ae_at_hidden.
	 * @returns {Number}
	 */function ariaHiddenElsDescendants(rootTestNode){try{var els=rootTestNode.querySelectorAll("*[data-ae_at_hidden] *");for(var i=els.length;i--;){if(isRunningTooLong()){throw getRuntimeExceededMessage();}els[i].setAttribute("data-ae_at_hidden","true");}return rootTestNode;}catch(err){logger.error('Error','stage4_ariaHiddenElsDescendants',err);stage4_markDownFunc_errors["stage4_ariaHiddenElsDescendants"]=err;return null;}}/*
	 * Pre-Testing: Finds elements that marked data-ae_ar=none; or data-ae_ar=presentation.
	 * @returns {Number}
	 */function ariaRolePreNone(rootTestNode){try{var els=rootTestNode.querySelectorAll("*[data-ae_ar='none'], *[data-ae_ar='presentation']");for(var i=els.length;i--;){if(isRunningTooLong()){throw getRuntimeExceededMessage();}els[i].setAttribute("data-ae_at_none","true");}return rootTestNode;}catch(err){logger.error('Error','stage4_ariaRolePreNone',err);stage4_markDownFunc_errors["stage4_ariaRolePreNone"]=err;return null;}}/*
	 * Pre-Testing: Finds elements impacted from having a parent marked data-ae_at_none=true.
	 * @returns {Number}
	 */function affectedByParentsAria(rootTestNode){try{var els=rootTestNode.querySelectorAll("*[data-ae_at_none] *[data-ae_ar='columnheader'], *[data-ae_at_none] *[data-ae_ar='gridcell'], *[data-ae_at_none] *[data-ae_ar='rowheader'], *[data-ae_at_none] *[data-ae_ar='row'], *[data-ae_at_none] *[data-ae_ar='listitem'], *[data-ae_at_none] *[data-ae_ar='menuitem'], *[data-ae_at_none] *[data-ae_ar='menuitemcheckbox'], *[data-ae_at_none] *[data-ae_ar='menuitemradio'], *[data-ae_at_none] *[data-ae_ar='option'], *[data-ae_at_none] *[data-ae_ar='rowgroup'], *[data-ae_at_none] *[data-ae_ar='tab'], *[data-ae_at_none] *[data-ae_ar='treeitem'], *[data-ae_at_none] *[data-ae_ar='menubar'], *[data-ae_at_none] *[data-ae_ar='grid'], *[data-ae_at_none] thead:not([data-ae_ar]), *[data-ae_at_none] tbody:not([data-ae_ar]), *[data-ae_at_none] tfoot:not([data-ae_ar]), *[data-ae_at_none] tr:not([data-ae_ar]), *[data-ae_at_none] th:not([data-ae_ar]), *[data-ae_at_none] td:not([data-ae_ar]), *[data-ae_at_none] caption:not([data-ae_ar]), *[data-ae_at_none] li:not([data-ae_ar]), *[data-ae_at_none] dt:not([data-ae_ar]), *[data-ae_at_none] dd:not([data-ae_ar]), *[data-ae_at_none] option:not([data-ae_ar]), *[data-ae_at_none] menuitem:not([data-ae_ar])");for(var i=els.length;i--;){if(isRunningTooLong()){throw getRuntimeExceededMessage();}els[i].setAttribute("data-ae_at_none","true");}return rootTestNode;}catch(err){logger.error('Error','stage4_affectedByParentsAria',err);stage4_markDownFunc_errors["stage4_affectedByParentsAria"]=err;return null;}}/*
	 * Pre-Testing: Finds elements that do not have a data-ae_at_hidden and data-ae_at_none attribute.
	 * @returns {Number}
	 */function availableToAT(rootTestNode){try{var els=rootTestNode.querySelectorAll("*:not([data-ae_at_hidden]):not([data-ae_at_none]):not(template):not(script):not(style)");for(var i=els.length;i--;){if(isRunningTooLong()){throw getRuntimeExceededMessage();}els[i].setAttribute("data-ae_avat","true");}return rootTestNode;}catch(err){logger.error('Error','stage4_availableToAT',err);stage4_markDownFunc_errors["stage4_availableToAT"]=err;return null;}}/*
	 * Pre-Testing: Finds elements that are descendants of elements marked as data-ae_invis.
	 * @returns {Number}
	 */function shadowElsDescendants(rootTestNode){try{const els=rootTestNode.querySelectorAll("*[data-ae_shdwrt] *");for(let i=els.length;i--;){if(isRunningTooLong()){throw getRuntimeExceededMessage();}els[i].setAttribute("data-ae_inshdw","true");}return rootTestNode;}catch(err){logger.error('Error','stage4_shadowElsDescendants',err);stage4_markDownFunc_errors["stage4_shadowElsDescendants"]=err;return null;}}function emptyOpenDialogs(rootTestNode){try{const els=rootTestNode.querySelectorAll('dialog[open]:not([hidden])[data-ae_avat], *[data-ae_ar="dialog"]:not([hidden]):not([data-ae_invis])[data-ae_avat]');for(let i=els.length;i--;){if(isRunningTooLong()){throw getRuntimeExceededMessage();}const el=els[i];if(el.innerHTML.trim().length<=0){el.setAttribute("data-ae_emptyopendialog","true");}}return rootTestNode;}catch(err){logger.error('Error','stage4_emptyOpenDialogs',err);stage4_markDownFunc_errors["stage4_emptyOpenDialogs"]=err;return null;}}/**
	 *
	 */function stage4_markDown_clonedDOM(rootTestNode,stage4MarkdownFuncs){stage4_markDownFunc_errors={};// standard functions used in a large number of tests
// rootTestNode = removeShadowRootsInVirtualDOM(rootTestNode);
// The following is removed, as in a DOM where the ShadowDOM models have already been registered
// changing the innerHTML will recreate the elements - with their DOM model
// rootTestNode = replaceHTMLEntitiesInBody(rootTestNode);
rootTestNode=addRoleAtt(rootTestNode);rootTestNode=invisEls(rootTestNode);rootTestNode=invisElsDescendants(rootTestNode);rootTestNode=ariaHiddenEls(rootTestNode);rootTestNode=ariaHiddenElsDescendants(rootTestNode);rootTestNode=ariaRolePreNone(rootTestNode);rootTestNode=affectedByParentsAria(rootTestNode);rootTestNode=availableToAT(rootTestNode);rootTestNode=shadowElsDescendants(rootTestNode);if("professional"==="professional"){rootTestNode=emptyOpenDialogs(rootTestNode);}// collect all functions added from tests
let addedStage4Funcs=Object.keys(stage4MarkdownFuncs);// each added stage2 func must be standalone, and not rely on any other
// so, they can be run in any order
for(let i=addedStage4Funcs.length;i--;){rootTestNode=stage4MarkdownFuncs[addedStage4Funcs[i]](rootTestNode);}return rootTestNode;}let stage5_markDownFunc_errors={};/**
	 * Pre-testing: Marks-up code with data-ae_attributes to enable more tests to use CSS selectors
	 * // Clone html node is marked-down, and all tests run on the clone
	 * @returns {Node}
	 */function stage5_markDown_clonedDOM(rootTestNode,stage5MarkdownFuncs){try{stage5_markDownFunc_errors={};let addedStage5Funcs=Object.keys(stage5MarkdownFuncs);function runOffFunc(n){do{if(isRunningTooLong()){throw getRuntimeExceededMessage();}if(n.nodeType===1){n.setAttribute("data-ae_ar",aeArFunc(n));var ae_invis=n.getAttribute("data-ae_invis");if(ae_invis===null){n.setAttribute("data-ae_vis","true");}n.setAttribute("data-ae_sib",aeSibFunc(n));n.setAttribute("data-ae_uel",aeUELFunc(n));var originalInformation=retrieveComputedCssStyles(n);// each added stage2 func must be standalone, and not rely on any other
// so, they can be run in any order
for(let i=addedStage5Funcs.length;i--;){if(isRunningTooLong()){throw getRuntimeExceededMessage();}var attrToAdd=stage5MarkdownFuncs[addedStage5Funcs[i]](originalInformation,n);if(attrToAdd!==null){n.setAttribute(attrToAdd[0],attrToAdd[1]);}}var firstChild=n.firstChild;if(firstChild){runOffFunc(firstChild);}}else if(n.nodeType===3){// to remove <th>&nbsp;</th> issues, and ensure :empty works
n.textContent=n.textContent.trim();}}while(n=n.nextSibling);}runOffFunc(rootTestNode.querySelector("body, frameset"));return rootTestNode;}catch(err){stage5_markDownFunc_errors["stage5_markDown_clonedDOM"]=err;return null;}}let stage6_markDownFunc_errors={};/**
	 * Pre-testing: Marks-up code within head with data-ae_attributes to enable more tests to use CSS selectors
	 * @returns {Node}
	 */function stage6_markDown_clonedDOM(rootTestNode,stage6MarkdownFuncs){try{stage6_markDownFunc_errors={};//Decorate HTML element as it is not covered by recursive tree walker
if(rootTestNode.tagName==="HTML"){rootTestNode.setAttribute("data-ae_uel","html");}let addedStage6Funcs=Object.keys(stage6MarkdownFuncs);function runOffFunc(n){do{if(isRunningTooLong()){throw getRuntimeExceededMessage();}if(n.nodeType===1){if(n.tagName==="BODY"){// otherwise it roles into body
break;}n.setAttribute("data-ae_sib",aeSibFunc(n));n.setAttribute("data-ae_uel",aeUELFunc(n));n.setAttribute("data-ae_ar",aeArFunc(n));// each added stage2 func must be standalone, and not rely on any other
// so, they can be run in any order
for(let i=addedStage6Funcs.length;i--;){if(isRunningTooLong()){throw getRuntimeExceededMessage();}var attrToAdd=stage6MarkdownFuncs[addedStage6Funcs[i]](n);if(attrToAdd!==null){n.setAttribute(attrToAdd[0],attrToAdd[1]);}}if(n.hasChildNodes()){runOffFunc(n.firstChild);}}}while(n=n.nextSibling);}runOffFunc(rootTestNode.querySelector("head"));clearStartTime();return rootTestNode;}catch(err){clearStartTime();stage6_markDownFunc_errors["stage6_markDown_clonedDOM"]=err;return null;}}let success=null;/**
	 * Public interface to get whether or not Engine successfully finished its last DOM capturing (and, if analytics_runAllTests_returnInstances_JSON_timer was invoked, test execution) run without any errors.
	 * Returns null if a run hasn't been started yet or if one is still in progress.
	 * @returns {boolean}
	 */function getSuccess(){return success;}/**
	 * @param {boolean} newSuccess
	 */function setSuccess(newSuccess){success=newSuccess;}/**
	 * Access Engine Kernel
	 */class AEKernel{/**
		 * @singleton
		 * @returns {AEKernel}
		 */constructor(){if(!AEKernel.instance){this._version="2.21.0";this._stage1MarkdownFuncs={};this._stage2MarkdownFuncs={};this._stage4MarkdownFuncs={};this._stage5MarkdownFuncs={};this._stage6MarkdownFuncs={};this._rootTestNode=null;this._automaticTests={};this._guidedAutomaticTests={};// might only be needed in extended object
this._advisoryAutomaticTests={};// might only be needed in extended object
this._previewModes={};// might only be needed in extended object
this._window=window;// default is current Window
this._document=document;// default is current Document
this._markdownFuncs_failedToRun={};this._tests_failedToRun={};this._test_outcomes={};// used for collecting results from timer apis
this._test_analytics_outcomes=[];// used for collecting results from timer apis in Analytics V3
this._getTestInfo={};// get data about all the tests
this._isLiveDOMMarkedUp=false;AEKernel.instance=this;}return AEKernel.instance;}/* ******************************************************************************** */ /* Getters / Setters */ /* ******************************************************************************** */ /**
		 *
		 * @returns {string}
		 */get version(){return this._version;}/**
		 *
		 * @param {string} version
		 */set version(version){this._version=version;}/**
		 * @returns {function[]}
		 */get stage1MarkdownFuncs(){return this._stage1MarkdownFuncs;}/**
		 * @returns {function[]}
		 */get stage2MarkdownFuncs(){return this._stage2MarkdownFuncs;}/**
		 *
		 * @returns {Window|*}
		 */get window(){return this._window;}/**
		 *
		 * @returns {HTMLDocument}
		 */get document(){return this._document;}/**
		 *
		 * @returns {Array}
		 */get markdownFuncs_failedToRun(){return this._markdownFuncs_failedToRun;}/**
		 *
		 * @param {Array} markdownFuncs_failedToRun
		 */set markdownFuncs_failedToRun(markdownFuncs_failedToRun){this._markdownFuncs_failedToRun=markdownFuncs_failedToRun;}/**
		 *
		 * @returns {Array}
		 */get tests_failedToRun(){return this._tests_failedToRun;}/**
		 *
		 * @param {Array} tests_failedToRun
		 */set tests_failedToRun(tests_failedToRun){this._tests_failedToRun=tests_failedToRun;}/**
		 * @returns {Node}
		 */get rootTestNode(){return this._rootTestNode;}/**
		 * @param {Node} rootTestNode
		 */set rootTestNode(rootTestNode){this._rootTestNode=rootTestNode;}/**
		 * @returns {function[]}
		 */get stage4MarkdownFuncs(){return this._stage4MarkdownFuncs;}/**
		 * @param {function[]} stage4MarkdownFuncs
		 */set stage4MarkdownFuncs(stage4MarkdownFuncs){this._stage4MarkdownFuncs=stage4MarkdownFuncs;}/**
		 * @returns {function[]}
		 */get stage5MarkdownFuncs(){return this._stage5MarkdownFuncs;}/**
		 * @param {function[]} stage5MarkdownFuncs
		 */set stage5MarkdownFuncs(stage5MarkdownFuncs){this._stage5MarkdownFuncs=stage5MarkdownFuncs;}/**
		 * @returns {function[]}
		 */get stage6MarkdownFuncs(){return this._stage6MarkdownFuncs;}/**
		 * @param {function[]} stage6MarkdownFuncs
		 */set stage6MarkdownFuncs(stage6MarkdownFuncs){this._stage6MarkdownFuncs=stage6MarkdownFuncs;}/* needed for timer API */ /**
		 * @returns {{}}
		 */get test_outcomes(){return this._test_outcomes;}/**
		 * @param {{}} test_outcomes
		 */set test_outcomes(test_outcomes){this._test_outcomes=test_outcomes;}/* needed for analytics timer API */ /**
		 * @returns {{}}
		 */get test_analytics_outcomes(){return this._test_analytics_outcomes;}/**
		 * @param {{}} test_analytics_outcomes
		 */set test_analytics_outcomes(test_analytics_outcomes){this._test_analytics_outcomes=test_analytics_outcomes;}get isLiveDOMMarkedUp(){return this._isLiveDOMMarkedUp;}set isLiveDOMMarkedUp(isLiveDOMMarkedUp){this._isLiveDOMMarkedUp=isLiveDOMMarkedUp;}/* ******************************************************************************** */ /**
		 * Change the window and document under test
		 * If you are using window.opener model from Access Analyst you will want to run Access Engine from the new window.
		 * Doing this allows you to set these properties, and hold the code for AccessEngine away from the test page - does not have to be included in the page being tested.
		 * Inheriting a constructor can be tricky so this is here for objects that extend AccessEngine
		 *
		 * @param {Window} refWindow
		 */setWindowUnderTest(refWindow){this._window=refWindow;this._document=refWindow.document;}/**
		 * Change rootTestNode is needed for nodeCapture APIs
		 *
		 * @param {Node} testNode
		 */setRootTestNode(testNode){this._rootTestNode=testNode;}/**
		 * Provides a clone of the current root test node
		 * We may want to enable it to be stored in the toolbar, for record / playback analysis
		 *
		 * @returns {Node}
		 */getCloneRootTestNode(){return cloneThisNode(this._rootTestNode,true);}/**
		 * Add failed to run functions to relevant holder
		 * @param {Obj} targetCollectionToAddTo
		 * @param {Obj} collectionOfFuncsWhichFailedToRun
		 */addFailedToRunFuncs(targetCollectionToAddTo,collectionOfFuncsWhichFailedToRun){var funcNames=Object.keys(collectionOfFuncsWhichFailedToRun);for(var i=funcNames.length;i--;){var funcName=funcNames[i];targetCollectionToAddTo[funcName]=collectionOfFuncsWhichFailedToRun[funcName];}}/**
		 * Add Stage1 test dependent Markdown Func
		 * Runs on live DOM
		 * @param {String} name
		 * @param {function} func
		 */addStage1MarkdownFunc(name,func){this._stage1MarkdownFuncs[name]=func;}/**
		 * Add Stage 4 test dependent Markdown Func
		 * Runs on cloned DOM
		 * @param {String} name
		 * @param {function} func
		 */addStage4MarkdownFunc(name,func){this._stage4MarkdownFuncs[name]=func;}/**
		 * Add Stage 5 test dependent Markdown Func
		 * Runs on cloned DOM
		 * @param {String} name
		 * @param {function} func
		 */addStage5MarkdownFunc(name,func){this._stage5MarkdownFuncs[name]=func;}/**
		 * Add Stage 6 test dependent Markdown Func
		 * Runs on cloned DOM
		 * @param {String} name
		 * @param {function} func
		 */addStage6MarkdownFunc(name,func){this._stage6MarkdownFuncs[name]=func;}/* split stages for DOM capture to work with async api calls */ /**
		 *
		 */stage0_capture(){// markdown live DOM + all nested ShadowDOMs
// we do full tree search of DOM + shadowRoots 
// defining which elements should have a shadowRoot
const result=stage0_markDown_liveDOM_domuels(this.document);this.isLiveDOMMarkedUp=true;this.addFailedToRunFuncs(this._markdownFuncs_failedToRun,stage0_markDownFunc_errors);return result;}/**
		 *
		 */stage1_capture(){// markdown live DOM
// add style information on only nodes in the live DOM
const result=stage1_markDown_liveDOM_copyInfo(this.window,this.document,this.stage1MarkdownFuncs);this.isLiveDOMMarkedUp=true;this.addFailedToRunFuncs(this._markdownFuncs_failedToRun,stage1_markDownFunc_errors);return result;}/**
		 *
		 */stage2_capture(){// clone the DOM node
this.rootTestNode=stage2_cloneDOM(this.document);this.addFailedToRunFuncs(this._markdownFuncs_failedToRun,stage2_markDownFunc_errors);return this.rootTestNode;}/**
		 *
		 */stage3a_capture(){if(detectShadowDOM===true){// Build out ShadowDOM
this.rootTestNode=stage3a_CopyOverShadowHosts(this.rootTestNode,this.document);this.addFailedToRunFuncs(this._markdownFuncs_failedToRun,stage3a_markDownFunc_errors);}return this.rootTestNode;}/**
		 *
		 */stage3c_capture(){if(detectShadowDOM===true){// Build out ShadowDOM
this.rootTestNode=stage3c_BuildOutShadowDOM(this.rootTestNode,this.document);this.addFailedToRunFuncs(this._markdownFuncs_failedToRun,stage3c_markDownFunc_errors);}return this.rootTestNode;}/**
		 *
		 */stage3d_capture(){if(detectShadowDOM===true){// Copy across DOM information from shadowDOM web components, just like we do in stage 1
this.rootTestNode=stage3d_RevertElementsToOrigTagName(this.rootTestNode);this.addFailedToRunFuncs(this._markdownFuncs_failedToRun,stage3d_markDownFunc_errors);}return this.rootTestNode;}/**
		 *
		 */stage3_capture(){// clean-up live DOM
const result=stage3_cleanUpLiveDOM(this.document);if(result){this.isLiveDOMMarkedUp=false;}this.addFailedToRunFuncs(this._markdownFuncs_failedToRun,stage3_markDownFunc_errors);return result;}/**
		 *
		 */stage4_capture(){// markdown built-out cloned node - bulk 
this.rootTestNode=stage4_markDown_clonedDOM(this.rootTestNode,this.stage4MarkdownFuncs);this.addFailedToRunFuncs(this._markdownFuncs_failedToRun,stage4_markDownFunc_errors);return this.rootTestNode;}/**
		 *
		 */stage5_capture(){// markdown cloned node - tree search in body
this.rootTestNode=stage5_markDown_clonedDOM(this.rootTestNode,this.stage5MarkdownFuncs);this.addFailedToRunFuncs(this._markdownFuncs_failedToRun,stage5_markDownFunc_errors);return this.rootTestNode;}/**
		 *
		 */stage6_capture(){// markdown cloned node - tree search in head
this.rootTestNode=stage6_markDown_clonedDOM(this.rootTestNode,this.stage6MarkdownFuncs);this.addFailedToRunFuncs(this._markdownFuncs_failedToRun,stage6_markDownFunc_errors);return this.rootTestNode;}/**
		 * Captures DOM for testing - allowing multi-DOM testing
		 * for sync apis
		 */captureDOM(){const tasks=[this.stage0_capture,this.stage1_capture,this.stage2_capture,this.stage3a_capture,this.stage3c_capture,this.stage3d_capture,this.stage3_capture,// clean-up happens after clone has been created, as shadowRoots need to be copied into the clone
this.stage4_capture,this.stage5_capture,this.stage6_capture];try{for(let i=0;i<tasks.length;i++){const task=tasks[i];if(!task.bind(this)()){return false;}}}finally{if(this.isLiveDOMMarkedUp){this.stage3_capture.bind(this)();}}return true;}/**
		 * @param {String} testId
		 * @param {{}} testObj
		 */addAutomaticTest(testId,testObj){this._automaticTests[testId]=testObj;}/**
		 * @param {String} testId
		 * @param {{}} testObj
		 */addGuidedAutomaticTest(testId,testObj){this._guidedAutomaticTests[testId]=testObj;}/**
		 * @param {String} testId
		 * @param {{}} testObj
		 */addAdvisoryAutomaticTest(testId,testObj){this._advisoryAutomaticTests[testId]=testObj;}/**
		 * @param {String} previewModeId
		 * @param {{}} previewModeObj
		 */addPreviewMode(previewModeId,previewModeObj){this._previewModes[previewModeId]=previewModeObj;}/**
		 * Gets a single test from test id
		 *
		 * @param {String} testId
		 */getTestToRun(testId){let test=this._automaticTests[testId];if(test===undefined){test=this._guidedAutomaticTests[testId];}else{return test;}if(test===undefined){test=this._previewModes[testId];}else{return test;}if(test===undefined){test=this._advisoryAutomaticTests[testId];}else{return test;}if(test===undefined){return null;}return test;}/**
		 * Gets a list of tests according to enum test type
		 * @param {Number} testType
		 */getTestsToRun(testType){let tests=null;switch(testType){case testTypes.AUTOMATIC:tests=this._automaticTests;break;case testTypes.GUIDED_AUTOMATIC:tests=this._guidedAutomaticTests;break;case testTypes.PREVIEW_MODE:tests=this._previewModes;break;case testTypes.ADVISORY:tests=this._advisoryAutomaticTests;break;}return tests;}/**
		 * Provide catalog of test data
		 * @returns {object}
		 * @param {object} columns
		 */getTestInfo(columns){return this._getTestInfo;}/**
		 * Updates the public flag that indicates whether or not test execution (including DOM capturing) was successful.
		 * If no boolean is passed in as a parameter to this function, then the value to update with is derived from the 'failedToRun' objects that are populated during execution if any errors occur.
		 * @param {Boolean?} success
		 */updateSuccess(){let success=arguments.length>0&&arguments[0]!==undefined?arguments[0]:undefined;setSuccess(success===undefined?Object.keys(this._markdownFuncs_failedToRun).length===0&&Object.keys(this._tests_failedToRun).length===0:success);}}/**
	 * @const
	 * @type {AEKernel}
	 */const aeKernel=new AEKernel();/*
	 * Creates an instance of AutoTestShell which is used as the basis for all AccessEngine tests
	 * @constructor
	 * @this {AutoTestShell}
	 */ // TODO - we need to set rootTestNode prior to the test being run
class AutoTestShell{constructor(isRelevantCssSelector){this._isRelevantCssSelector=isRelevantCssSelector;this._rootTestNode=null;this._result={};this._candidateSetNodesFuncArgs=null;this._relevantNodes=0;this._numberOfRelevantNodes=0;this._cssSelector=arguments[1];this._applicabilityTestArgs=null;this._applicableNodesFuncArgs=null;}// Getters / Setters
get rootTestNode(){return this._rootTestNode;}set rootTestNode(rootTestNode){this._rootTestNode=rootTestNode;}get numberOfRelevantNodes(){return this._numberOfRelevantNodes;}set numberOfRelevantNodes(numberOfRelevantNodes){this._numberOfRelevantNodes=numberOfRelevantNodes;}get relevantNodes(){return this._relevantNodes;}set relevantNodes(relevantNodes){this._relevantNodes=relevantNodes;}/**
		 * Default function that allows a candidate set to be collected from a cssSelector.
		 * @return {Element[]} The candidate set.
		 */candidateSetNodesFunc(){return this.querySelectorAll(this._cssSelector);}/**
		 * Function allows the default candidateSetNodes function to be overwritten, with a custom Candidate Set collector function.
		 * @param {Function} func The custom Candidate Set collector function.
		 * @param {Array} funcArgs The arguments that are needed for the custom Candidate Set collector function.
		 * @return {NodeList} The candidate set.
		 */set_candidateSetNodesFunc(func,funcArgs){this.candidateSetNodesFunc=func;if(funcArgs){this._candidateSetNodesFuncArgs=funcArgs;}}/**
		 * Default applicability test function that determines if a check function is applicable based on one or more nodes being returned from a css selector.
		 * @return {Boolean} A true | false outcome.
		 */applicabilityTest(){return this.querySelector(this._isRelevantCssSelector)!=null;}/**
		 * Function allows the default applicabilityTest function to be overwritten, with a custom function.
		 * @param {Function} func The custom function.
		 * @param {Array} funcArgs The arguments that are needed for the custom function.
		 */set_applicabilityTest(func,funcArgs){this.applicabilityTest=func;if(funcArgs){this._applicabilityTestArgs=funcArgs;}}/**
		 * Default function that allows an applicability set to be collected from a cssSelector.
		 * @return {Element[]} The applicability set.
		 */applicableNodesFunc(){return this.querySelectorAll(this._isRelevantCssSelector);}/**
		 * Function allows the default applicableNodesFunc function to be overwritten, with a custom Applicability Set collector function.
		 * @param {Function} func The custom Applicability Set collector function.
		 * @param {Array} funcArgs The arguments that are needed for the custom Applicability Set collector function.
		 * @return {NodeList} The applicability set.
		 */set_applicableNodesFunc(func,funcArgs){this.applicableNodesFunc=func;if(funcArgs){this._applicableNodesFuncArgs=funcArgs;}}/**
		 * Default function that determines if a check is pass or fail, based on one or more nodes being returned the Candidate Set collector function.
		 * @return {string} A "pass | fail" outcome.
		 */passFailFunc(){let outcome="pass";if(this._numberOfRelevantNodes>0){outcome="fail";}return outcome;}/**
		 * Default function that determines if a check is pass or fail, and if failed the number of fail instances, based on one or more nodes being returned the Candidate Set collector function.
		 * @return {string} A "pass | fail, no. issues" outcome.
		 */numberIssuesFunc(){let outcome="pass";if(this._numberOfRelevantNodes>0){outcome=["fail",this._numberOfRelevantNodes];}return outcome;}/**
		 * Default function that determines if a check is pass or fail, and if failed the fail instances, based on one or more nodes being returned the Candidate Set collector function.
		 * @return {String|Array} A [pass | fail, instances]" outcome.
		 */issueInstancesFunc(){// default function
let outcome="pass";if(this._numberOfRelevantNodes>0){outcome=["fail",this._relevantNodes];}return outcome;}/**
		 * Function determines the outcome of the test function, with 3 possible outcomes.
		 * Variant 1 - na | pass | fail
		 * Variant 2 - na | pass | fail, number of issues
		 * Variant 3 - na | pass | fail, instances
		 * @param {result}
		 */getResult(){if(this.applicabilityTest()===false){this._result.outcome="na";return this._result;}this._relevantNodes=this.candidateSetNodesFunc(this._candidateSetNodesFuncArgs);// can remove when complete
this._numberOfRelevantNodes=this._relevantNodes.length;// arguments[0] is collectType
// Built with comments like this to enable automated build into 3 variants - underscore.js template mark-down
// <% if (dev) { %>
if(arguments[0]===undefined){// <% } %>
// <% if (var1) { %>
this._result.outcome=this.passFailFunc();// <% } %>
// <% if (dev) { %>
}// <% } %>
// <% if (dev) { %>
if(arguments[0]==="numbers"){// <% } %>
// <% if (var2) { %>
const outcome=this.numberIssuesFunc();if(typeof outcome==="string"){this._result.outcome=outcome;}else{this._result.outcome=outcome[0];this._result.numberOfIssues=outcome[1];}// <% } %>
// <% if (dev) { %>
}// <% } %>
// <% if (dev) { %>
if(arguments[0]==="instances"){// <% } %>
// <% if (var3) { %>
const outcome=this.issueInstancesFunc(this._relevantNodes);if(typeof outcome==="string"){this._result.outcome=outcome;}else{this._result.outcome=outcome[0];this._result.instances=outcome[1];}// <% } %>
// <% if (dev) { %>
}// <% } %>
return this._result;}getNextGenResultsMetadata(){const isApplicable=this.applicabilityTest(this._applicabilityTestArgs);let outcome;if(isApplicable){this._relevantNodes=Array.from(this.candidateSetNodesFunc(this._candidateSetNodesFuncArgs));this._numberOfRelevantNodes=this._relevantNodes.length;outcome=this._numberOfRelevantNodes<=0?"pass":"fail";}else{this._relevantNodes=[];this._numberOfRelevantNodes=0;outcome="na";}return{outcome:outcome,failingNodes:this._relevantNodes};}/**
		 * An extension of this._rootTestNode.querySelector that also operates over the root test node, not just its children.
		 * @return {Element}
		 */querySelector(cssSelector){// source: https://stackoverflow.com/a/60831454
return this._rootTestNode.matches(cssSelector)&&this._rootTestNode||this._rootTestNode.querySelector(cssSelector);}/**
		 * An extension of this._rootTestNode.querySelectorAll that also operates over the root test node, not just its children.
		 * @return {Element[]}
		 */querySelectorAll(cssSelector){// source: https://stackoverflow.com/a/59838990
return[this._rootTestNode,...this._rootTestNode.querySelectorAll(cssSelector)].filter(el=>el.matches(cssSelector));}}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-8
	 * @returns {AutoTestShell} testShell
	 */function test8(){aeKernel.addAutomaticTest("8",{bestPractice:1626,testId:8,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM and available to assistive technologies, has an aria-details attribute that is not set to a valid id.",metaText:"This element has an aria-details attribute that is set to an invalid or duplicate id",metaTextDetail:"This {{tag-name}} has an aria-details attribute that is set to an invalid or duplicate id of {{aria-details}}",fixType:"",testFunc:function(){var testShell=new AutoTestShell('*[data-ae_vis][data-ae_avat][aria-details]:not([aria-details=""])');testShell.set_candidateSetNodesFunc(function(){var nodesWithBadIds=[];var res=aeKernel.rootTestNode.querySelectorAll('*[data-ae_vis][data-ae_avat][aria-details]:not([aria-details=""])');for(var i=0,len_res=res.length;i<len_res;i++){var n=res[i];var detailsId=n.getAttribute("aria-details");if(isValidIdOrName(detailsId)===true){// "[id=" is used in place of "#" + id as ids can included colons which breaks css selector, although is valid html
var refNode=aeKernel.rootTestNode.querySelectorAll("[id='"+detailsId.replace(/'/g,"\\'")+"']");var refNodeLength=refNode.length;if(refNodeLength===0||refNodeLength>1){nodesWithBadIds.push(n);}}else{nodesWithBadIds.push(n);}}return nodesWithBadIds;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-14
	 * @returns {AutoTestShell} testShell
	 */function test14(){aeKernel.addAutomaticTest("14",{bestPractice:1626,testId:14,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements, excluding th elements (without an ARIA-assigned role), and elements with a role='rowheader... | columnheader...' attribute, not intentionally hidden in the DOM and available to assistive technologies, has an aria-sort attribute.",metaText:"The aria-sort attribute is not allowed on this element",metaTextDetail:"The aria-sort attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){const relevantCssSelector='*:not(th[data-ae_ar="null"]):not([data-ae_ar="rowheader"]):not([data-ae_ar="columnheader"])[data-ae_vis][data-ae_avat]';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[aria-sort]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-15
	 * @returns {AutoTestShell} testShell
	 */function test15(){aeKernel.addAutomaticTest("15",{bestPractice:338,testId:15,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more input elements, excluding input type='image' elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has an alt attribute.",metaText:"The alt attribute is not allowed on this element",metaTextDetail:"The alt attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_vis][data-ae_avat][data-ae_ar="null"]:not([type="image"])','input[data-ae_vis][data-ae_avat][data-ae_ar="null"]:not([type="image"])[alt]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-16
	 * @returns {AutoTestShell} testShell
	 */function test16(){aeKernel.addAutomaticTest("16",{bestPractice:1626,testId:16,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements, excluding textarea elements (without an ARIA-assigned role) and elements with a role='textbox...' attribute, not intentionally hidden in the DOM and available to assistive technologies, has an aria-multiline attribute.",metaText:"The aria-multiline attribute is not allowed on this element",metaTextDetail:"The aria-multiline attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){const relevantCssSelector='*:not(textarea[data-ae_ar="null"]):not([data-ae_ar="textbox"])[data-ae_vis][data-ae_avat]';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[aria-multiline]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-21
	 * @returns {AutoTestShell} testShell
	 */function test21(){aeKernel.addAutomaticTest("21",{bestPractice:1626,testId:21,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM and available to assistive technologies, has an aria-live attribute that is not set to one of the following text values: 'off | polite | assertive'.",metaText:"This element's aria-live attribute should be set to one of the following text values: off | polite | assertive",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][aria-live]','*[data-ae_vis][data-ae_avat][aria-live]:not([aria-live="off"]):not([aria-live="polite"]):not([aria-live="assertive"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-22
	 * @returns {AutoTestShell} testShell
	 */function test22(){aeKernel.addAutomaticTest("22",{bestPractice:967,testId:22,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more A elements, without an ARIA-assigned role, not intentionally hidden in the DOM, contained one or more input elements (without an assigned ARIA role and not type=hidden), keygen elements (without an assigned ARIA role), select elements (without an assigned ARIA role), textarea elements (without an assigned ARIA role) or button elements (without an assigned ARIA role).",metaText:"This element should not contain input elements (without an assigned ARIA role and not type=hidden), keygen elements (without an assigned ARIA role), select elements (without an assigned ARIA role), textarea elements (without an assigned ARIA role) or button elements (without an assigned ARIA role)",metaTextDetail:"This {{tag-name}} should not contain input elements (without an assigned ARIA role and not type=hidden), keygen elements (without an assigned ARIA role), select elements (without an assigned ARIA role), textarea elements (without an assigned ARIA role) or button elements (without an assigned ARIA role)",fixType:"",testFunc:function(){const relevantCssSelector='a[data-ae_vis][data-ae_ar="null"]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const notProper=[];const res=aeKernel.rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0;i<res.length;i++){const result=res[i];const res1=result.querySelector('input[data-ae_ar="null"]:not([type="hidden"]), keygen[data-ae_ar="null"], select[data-ae_ar="null"], textarea[data-ae_ar="null"], button[data-ae_ar="null"]');if(res1){notProper.push(result);}}return notProper;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-26
	 * @returns {AutoTestShell} testShell
	 */function test26(){aeKernel.addAutomaticTest("26",{bestPractice:1626,testId:26,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='radio' attribute, not intentionally hidden in the DOM and available to assistive technologies, does not have an aria-checked attribute set to one of the text values: 'true | false'.",metaText:"This element (role=radio) does not have an aria-checked attribute set to one of the text values: true | false",metaTextDetail:"This {{tag-name}} (role=radio) does not have an aria-checked attribute set to one of the text values: true | false",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="radio"]','*[data-ae_vis][data-ae_avat][data-ae_ar="radio"][aria-checked]:not([aria-checked="true"]):not([aria-checked="false"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-27
	 * @returns {AutoTestShell} testShell
	 */function test27(){aeKernel.addAutomaticTest("27",{bestPractice:1626,testId:27,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='option' attribute and is a child of an element with a aria-multiselectable attribute, not intentionally hidden in the DOM and available to assistive technologies, does not have an aria-selected attribute set one of the text values: 'true | false'.",metaText:"This element (role=option) does not have an aria-selected attribute set to one of the text values: true | false",metaTextDetail:"This {{tag-name}} (role=option) does not have an aria-selected attribute set to one of the text values: true | false",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][aria-multiselectable] *[data-ae_vis][data-ae_avat][data-ae_ar="option"]','*[data-ae_vis][data-ae_avat][aria-multiselectable] *[data-ae_vis][data-ae_avat][data-ae_ar="option"][aria-selected]:not([aria-selected="true"]):not([aria-selected="false"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-28
	 * @returns {AutoTestShell} testShell
	 */function test28(){aeKernel.addAutomaticTest("28",{bestPractice:1626,testId:28,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='scrollbar' attribute, not intentionally hidden in the DOM and available to assistive technologies, does not have an aria-controls attribute, an aria-orientation attribute, an aria-valuemax attribute, an aria-valuemin attribute and an aria-valuenow attribute.",metaText:"This element (role=scrollbar) must have all of the following attributes: aria-controls | aria-orientation | aria-valuemax | aria-valuemin | aria-valuenow",metaTextDetail:"This {{tag-name}} (role=scrollbar) must have all of the following attributes: aria-controls | aria-orientation | aria-valuemax | aria-valuemin | aria-valuenow",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="scrollbar"]',`*[data-ae_vis][data-ae_avat][data-ae_ar="scrollbar"]:not([aria-controls]), 
				*[data-ae_vis][data-ae_avat][data-ae_ar="scrollbar"]:not([aria-orientation]), 
				*[data-ae_vis][data-ae_avat][data-ae_ar="scrollbar"]:not([aria-valuemax]), 
				*[data-ae_vis][data-ae_avat][data-ae_ar="scrollbar"]:not([aria-valuemin]), 
				*[data-ae_vis][data-ae_avat][data-ae_ar="scrollbar"]:not([aria-valuenow])`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-31
	 * @returns {AutoTestShell} testShell
	 */function test31(){aeKernel.addAutomaticTest("31",{bestPractice:389,testId:31,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more track elements with a kind='captions' attribute, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a srclang attribute set to a text value.",metaText:"This element does not have a srclang attribute set to a text value",metaTextDetail:"This {{tag-name}} does not have a srclang attribute set to a text value",fixType:"",testFunc:function(){return new AutoTestShell('track[data-ae_ar="null"][data-ae_vis][data-ae_avat][kind="captions"]','track[data-ae_ar="null"][data-ae_vis][data-ae_avat][kind="captions"]:not([srclang]),track[data-ae_ar="null"][data-ae_vis][data-ae_avat][kind="captions"][srclang=""]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-33
	 * @returns {AutoTestShell} testShell
	 */function test33(){aeKernel.addAutomaticTest("33",{bestPractice:1626,testId:33,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='spinbutton' attribute, not intentionally hidden in the DOM and available to assistive technologies, does not have an aria-valuemax attribute, an aria-valuemin attribute and an aria-valuenow attribute.",metaText:"This element (role=spinbutton) does not have an aria-valuemax attribute, an aria-valuemin attribute and an aria-valuenow attribute",metaTextDetail:"This {{tag-name}} (role=spinbutton) does not have an aria-valuemax attribute, an aria-valuemin attribute and an aria-valuenow attribute",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="spinbutton"]','*[data-ae_vis][data-ae_avat][data-ae_ar="spinbutton"]:not([aria-valuemax]), *[data-ae_vis][data-ae_avat][data-ae_ar="spinbutton"]:not([aria-valuemin]), *[data-ae_vis][data-ae_avat][data-ae_ar="spinbutton"]:not([aria-valuenow])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-34
	 * @returns {AutoTestShell} testShell
	 */function test34(){aeKernel.addAutomaticTest("34",{bestPractice:1342,testId:34,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more blink elements, without an ARIA-assigned role, not intentionally hidden in the DOM, are present in the DOM.",metaText:"The blink element is obsolete and should not be used",fixType:"",testFunc:function(){const relevantCssSelector='blink';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[data-ae_ar="null"][data-ae_vis]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-35
	 * @returns {AutoTestShell} testShell
	 */function test35(){aeKernel.addAutomaticTest("35",{bestPractice:941,testId:35,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more th elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, were found to be empty.",metaText:"This th element should not be empty",fixType:"",testFunc:function(){return new AutoTestShell('th[data-ae_vis][data-ae_avat][data-ae_ar="null"]','th[data-ae_vis][data-ae_avat][data-ae_ar="null"]:empty');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-37
	 * @returns {AutoTestShell} testShell
	 */function test37(){aeKernel.addAutomaticTest("37",{bestPractice:1626,testId:37,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more A elements with an href attribute, excluding those with a role='link...' attribute, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to a value that is not: 'button... | checkbox... | menuitem... | menuitemcheckbox... | menuitemradio... | option... | radio... | tab... | switch... | treeitem...'",metaText:"The role attribute value given to this element is not allowed.  The element's role attribute should be set to one of the following text values: button | checkbox | menuitem | menuitemcheckbox | menuitemradio | radio | tab | switch | treeitem; or the role attribute can be removed",metaTextDetail:"The role attribute value of '{{data-ae_ar}}' given to this {{tag-name}} is not allowed.  The element's role attribute should be set to one of the following text values: button | checkbox | menuitem | menuitemcheckbox | menuitemradio | radio | tab | switch | treeitem; or the role attribute can be removed",fixType:"",testFunc:function(){return new AutoTestShell('a[data-ae_vis][data-ae_avat][href]:not([data-ae_ar="null"]):not([data-ae_ar="link"])','a[data-ae_vis][data-ae_avat][href]:not([data-ae_ar="null"]):not([data-ae_ar="link"]):not([data-ae_ar="button"]):not([data-ae_ar="checkbox"]):not([data-ae_ar="menuitem"]):not([data-ae_ar="menuitemcheckbox"]):not([data-ae_ar="menuitemradio"]):not([data-ae_ar="option"]):not([data-ae_ar="radio"]):not([data-ae_ar="tab"]):not([data-ae_ar="switch"]):not([data-ae_ar="treeitem"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-38
	 * @returns {AutoTestShell} testShell
	 */function test38(){aeKernel.addAutomaticTest("38",{bestPractice:1626,testId:38,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements, excluding button elements (without an ARIA-assigned role), input type='button | submit | reset | image' elements (without an ARIA-assigned role), and elements with a role='button...' attribute, not intentionally hidden in the DOM and available to assistive technologies, has an aria-pressed attribute.",metaText:"This element should not have an aria-pressed attribute",metaTextDetail:"This {{tag-name}} with a role of '{{data-ae_ar}}' should not have an aria-pressed attribute",fixType:"",testFunc:function(){const relevantCssSelector='*:not(button[data-ae_ar="null"]):not(input[type="button"][data-ae_ar="null"]):not(input[type="submit"][data-ae_ar="null"]):not(input[type="reset"][data-ae_ar="null"]):not(input[type="image"][data-ae_ar="null"]):not([data-ae_ar="button"])';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[aria-pressed]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-40
	 * @returns {AutoTestShell} testShell
	 */function test40(){aeKernel.addAutomaticTest("40",{bestPractice:2049,testId:40,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more track elements with a kind='captions' attribute, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a label attribute set to a text value.",metaText:"This element does not have a label attribute set to a text value",metaTextDetail:"This {{tag-name}} does not have a label attribute set to a text value",fixType:"",testFunc:function(){return new AutoTestShell('track[data-ae_ar="null"][data-ae_vis][data-ae_avat][kind="captions"]','track[data-ae_ar="null"][data-ae_vis][data-ae_avat][kind="captions"]:not([label]),track[data-ae_ar="null"][data-ae_vis][data-ae_avat][kind="captions"][label=""]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-41
	 * @returns {AutoTestShell} testShell
	 */function test41(){aeKernel.addAutomaticTest("41",{bestPractice:1626,testId:41,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='switch' attribute, not intentionally hidden in the DOM and available to assistive technologies, does not have an aria-checked attribute set to one of the text values: 'true | false'.",metaText:"This element (role=switch) does not have an aria-checked attribute set to one of the text values: true | false",metaTextDetail:"This {{tag-name}} (role=switch) does not have an aria-checked attribute set to one of the text values: true | false",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="switch"]','*[data-ae_vis][data-ae_avat][data-ae_ar="switch"][aria-checked]:not([aria-checked="true"]):not([aria-checked="false"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-43
	 * @returns {AutoTestShell} testShell
	 */function test43(){aeKernel.addAutomaticTest("43",{bestPractice:441,testId:43,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more marquee elements, without an ARIA-assigned role, not intentionally hidden in the DOM, are present in the DOM.",metaText:"The marquee element is obsolete and should not be used",fixType:"",testFunc:function(){const relevantCssSelector='marquee';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[data-ae_ar="null"][data-ae_vis]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-44
	 * datalist is display:none by default
	 * @returns {AutoTestShell} testShell
	 */function test44(){aeKernel.addAutomaticTest("44",{bestPractice:1626,testId:44,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more hr elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute.",metaText:"The role attribute is not allowed on this element",metaTextDetail:"The role attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){return new AutoTestShell('hr[data-ae_vis][data-ae_avat]','hr[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-48
	 * @returns {AutoTestShell} testShell
	 */function test48(){aeKernel.addAutomaticTest("48",{bestPractice:414,testId:48,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more tr elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, contains two or more descendant elements (without an ARIA-assigned role) that have been given a scope attribute set to the text value 'row'.",metaText:"This element should not contain two or more descendant elements (without an ARIA-assigned role) that have been given a scope attribute set to the text value: row",metaTextDetail:"This {{tag-name}} should not contain two or more descendant elements (without an ARIA-assigned role) that have been given a scope attribute set to the text value: row",fixType:"",testFunc:function(){const relevantCssSelector='tr[data-ae_vis][data-ae_avat][data-ae_ar="null"]:not(:empty)';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const notAllowed=[];const res=aeKernel.rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0;i<res.length;i++){const result=res[i];let numBadChildren=0;const tableAncestor=result.closest('table');const childrenWithScope=result.querySelectorAll('*[data-ae_ar="null"][scope="row"]');for(let j=0;j<childrenWithScope.length;j++){const childWithScope=childrenWithScope[j];// ignore nested tables; the elements in nested tables have either already been checked or will be checked by a different iteration of the outer 'for' loop
const childWithScopeTableAncestor=childWithScope.closest('table');if(childWithScopeTableAncestor!==tableAncestor){continue;}numBadChildren++;}if(numBadChildren>=2){notAllowed.push(result);}}return notAllowed;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-50
	 * @returns {AutoTestShell} testShell
	 */function test50(){aeKernel.addAutomaticTest("50",{bestPractice:1626,testId:50,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM and available to assistive technologies, has an aria-busy attribute that is not set to one of the text values: 'true | false'.",metaText:"This element's aria-busy attribute should be set to one of the following text values: true | false",metaTextDetail:"This {{tag-name}}'s aria-busy attribute should be set to one of the following text values: true | false",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][aria-busy]','*[data-ae_vis][data-ae_avat][aria-busy]:not([aria-busy="true"]):not([aria-busy="false"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-51
	 * @returns {AutoTestShell} testShell
	 */function test51(){aeKernel.addAutomaticTest("51",{bestPractice:1626,testId:51,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM and available to assistive technologies, has an aria-errormessage attribute that is not set to a valid id.",metaText:"This element has an aria-errormessage attribute that is set to an invalid or duplicate id",metaTextDetail:"This {{tag-name}} has an aria-errormessage attribute that is set to an invalid or duplicate id of {{aria-errormessage}}",fixType:"",testFunc:function(){var testShell=new AutoTestShell('*[data-ae_vis][data-ae_avat][aria-errormessage]:not([aria-errormessage=""])');testShell.set_candidateSetNodesFunc(function(){var nodesWithBadIds=[];var res=aeKernel.rootTestNode.querySelectorAll('*[data-ae_vis][data-ae_avat][aria-errormessage]:not([aria-errormessage=""])');for(var i=0,len_res=res.length;i<len_res;i++){var n=res[i];var detailsId=n.getAttribute("aria-errormessage");if(isValidIdOrName(detailsId)===true){// "[id=" is used in place of "#" + id as ids can included colons which breaks css selector, although is valid html
var refNode=aeKernel.rootTestNode.querySelectorAll("[id='"+detailsId.replace(/'/g,"\\'")+"']");var refNodeLength=refNode.length;if(refNodeLength===0||refNodeLength>1){nodesWithBadIds.push(n);}}else{nodesWithBadIds.push(n);}}return nodesWithBadIds;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-53
	 * @returns {AutoTestShell} testShell
	 */function test53(){aeKernel.addAutomaticTest("53",{bestPractice:1626,testId:53,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM and available to assistive technologies, has an aria-relevant attribute that is set to a value that contains values other than: 'additions | additions text | additions removals | all | removals | text'.",metaText:"This element's aria-relevant attribute should only include one or more of the following text values: additions | additions text | additions removals | all | removals | text",metaTextDetail:"This {{tag-name}}'s aria-relevant attribute should only include one or more of the following text values: additions | additions text | additions removals | all | removals | text",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][aria-relevant]','*[data-ae_vis][data-ae_avat][aria-relevant]:not([aria-relevant="additions"]):not([aria-relevant="additions text"]):not([aria-relevant="additions removals"]):not([aria-relevant="all"]):not([aria-relevant="removals"]):not([aria-relevant="text"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-54
	 * @returns {AutoTestShell} testShell
	 */function test54(){aeKernel.addAutomaticTest("54",{bestPractice:1626,testId:54,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM and available to assistive technologies, has an aria-atomic attribute that is not set to one of the text values: 'true | false'.",metaText:"This element's aria-atomic attribute should be set to one of the following text values: true | false",metaTextDetail:"This {{tag-name}}'s aria-atomic attribute should be set to one of the following text values: true | false",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][aria-atomic]','*[data-ae_vis][data-ae_avat][aria-atomic]:not([aria-atomic="true"]):not([aria-atomic="false"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-55
	 * @returns {AutoTestShell} testShell
	 */function test55(){aeKernel.addAutomaticTest("55",{bestPractice:410,testId:55,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more meta elements, in the head element, has a content attribute set to a value that includes an instruction that stops a user scaling the viewport size i.e. 'user-scalable=no'.",metaText:"This element has a content attribute set to a value that includes an instruction that stops a user scaling the viewport size",metaTextDetail:"This {{tag-name}} has a content attribute set to a value that includes an instruction that stops a user scaling the viewport size",fixType:{"fixType":1,"domSpec":false,"helperText":'Select "Save" to change the viewport content attribute to "width=device-width, initial-scale=1.0", or "Cancel" if the fix should not be applied.',"fix":{"content":"width=device-width, initial-scale=1.0"}},testFunc:function(){const relevantCssSelector='head meta[content]';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[content~="user-scalable=no"]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-57
	 * @returns {AutoTestShell} testShell
	 */function test57(){aeKernel.addAutomaticTest("57",{bestPractice:1626,testId:57,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM and available to assistive technologies, has an aria-orientation attribute that is not set to one of the following text values: 'horizontal | vertical'.",metaText:"This element's aria-orientation attribute should be set to one of the following text values: horizontal | vertical",metaTextDetail:"This {{tag-name}}'s aria-orientation attribute should be set to one of the following text values: horizontal | vertical",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][aria-orientation]','*[data-ae_vis][data-ae_avat][aria-orientation]:not([aria-orientation="horizontal"]):not([aria-orientation="vertical"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-59
	 * @returns {AutoTestShell} testShell
	 */function test59(){aeKernel.addAutomaticTest("59",{bestPractice:410,testId:59,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more meta elements, in the head element, has a content attribute set to a value that includes an instruction that sets the viewport's maximum-scale to less than 2 e.g. 'maximum-scale=1.0'.",metaText:"This element has a content attribute set to a value that includes an instruction that sets the viewport's maximum-scale to less than 2",metaTextDetail:"This {{tag-name}} has a content attribute set to a value that includes an instruction that sets the viewport's maximum-scale to less than 2",fixType:{"fixType":1,"domSpec":false,"helperText":'Select "Save" to change the viewport content attribute to "width=device-width, initial-scale=1.0", or "Cancel" if the fix should not be applied.',"fix":{"content":"width=device-width, initial-scale=1.0"}},testFunc:function(){const relevantCssSelector='head meta[content]';const failConditionSubSelectors=['[content*="maximum-scale=0"]','[content*="maximum-scale=1,"]','[content*="maximum-scale=1 "]','[content$="maximum-scale=1"]','[content*="maximum-scale=1."]','[content*="maximum-scale=yes"]','[content*="maximum-scale=-"]'];return new AutoTestShell(relevantCssSelector,failConditionSubSelectors.map(subSelector=>`${relevantCssSelector}${subSelector}`).join(', '));}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-61
	 * @returns {AutoTestShell} testShell
	 */function test61(){aeKernel.addAutomaticTest("61",{bestPractice:347,testId:61,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more meta elements, in the head element, that had an http-equiv attribute set to the text value 'refresh' did not have a zero second time limit set in their content attribute.",metaText:"This meta element has an http-equiv attribute set to the value: refresh; with a content attribute set greater than zero seconds",fixType:"",testFunc:function(){const relevantCssSelector='head meta[http-equiv="refresh"][content]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){var notAllowed=[];var res=aeKernel.rootTestNode.querySelectorAll(`${relevantCssSelector}:not([content=""])`);for(var i=0,len_res=res.length;i<len_res;i++){var result=res[i];var content=result.getAttribute("content");var semiColonPos=content.indexOf(";");if(semiColonPos!==-1){var charAfterSemiColon=content.substring(0,semiColonPos);if(charAfterSemiColon!=="0"){notAllowed.push(result);}}}return notAllowed;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-64
	 * @returns {AutoTestShell} testShell
	 */function test64(){aeKernel.addAutomaticTest("64",{bestPractice:1626,testId:64,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM and available to assistive technologies, has an aria-valuemax attribute that is not set to 0, or a + / - integer.",metaText:"This element has an aria-valuemax attribute that is not set to a + / - integer",metaTextDetail:"This {{tag-name}} has an aria-valuemax attribute that is not set to a + / - integer",fixType:"",testFunc:function(){var testShell=new AutoTestShell('*[data-ae_vis][data-ae_avat][aria-valuemax]:not([aria-valuemax=""])');testShell.set_candidateSetNodesFunc(function(){var nodesWithBadValues=[];var res=aeKernel.rootTestNode.querySelectorAll('*[data-ae_vis][data-ae_avat][aria-valuemax]:not([aria-valuemax=""])');for(var i=0,len_res=res.length;i<len_res;i++){var n=res[i];var val=n.getAttribute("aria-valuemax");// check if this is a + / - integer / float
if(isNaN(val)===true){nodesWithBadValues.push(n);}}return nodesWithBadValues;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-65
	 * @returns {AutoTestShell} testShell
	 */function test65(){aeKernel.addAutomaticTest("65",{bestPractice:389,testId:65,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements, excluding the root html node, not intentionally hidden in the DOM and available to assistive technologies, has a lang attribute set to a text value that is not empty and does not start with an IANA primary language value.",metaText:"This element has a lang attribute set to a text value that does not start with an IANA Language Subtag Registry language",metaTextDetail:'This {{tag-name}} has a lang attribute set to a text value of "{{lang}}", which does not start with an IANA Language Subtag Registry language',fixType:"",testFunc:function(){var testShell=new AutoTestShell('*:not(:root)[data-ae_vis][data-ae_avat][lang]:not([lang=""])');testShell.set_candidateSetNodesFunc(function(){const notProper=[];const res=aeKernel.rootTestNode.querySelectorAll('*[data-ae_vis][data-ae_avat][lang]:not(:root)');for(let i=0,len_res=res.length;i<len_res;i++){const result=res[i];let lang=result.getAttribute("lang");// We only care about the main language attribute before the first dash, if there is one
lang=lang.split("-")[0].toLowerCase();if(lang.length<2||lang.length>3||!ACCEPTED_LANGUAGE_SUBTAGS[lang]){notProper.push(result);}}return notProper;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-66
	 * @returns {AutoTestShell} testShell
	 */function test66(){aeKernel.addAutomaticTest("66",{bestPractice:1626,testId:66,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more head elements, has a role attribute and/or an aria attribute.",metaText:"This element has a role attribute and/or an aria attribute",metaTextDetail:"This {{tag-name}} has a role attribute and/or an aria attribute",fixType:"",testFunc:function(){const testShell=new AutoTestShell('head');testShell.set_candidateSetNodesFunc(function(){var notProper=[];var head=this.querySelector('head');if(head.hasAttribute("role")){notProper.push(head);return notProper;}var head_attributes=head.attributes;for(var i=head_attributes.length;i--;){if(head_attributes[i].name.substring(0,4).toLowerCase()==="aria"){notProper.push(head);return notProper;}}return notProper;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-67
	 * @returns {AutoTestShell} testShell
	 */function test67(){aeKernel.addAutomaticTest("67",{bestPractice:1626,testId:67,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM and available to assistive technologies, has an aria-valuenow attribute that is not set to 0, or a + / - integer.",metaText:"This element has an aria-valuenow attribute that is not set to a + / - integer",metaTextDetail:"This {{tag-name}} has an aria-valuenow attribute that is not set to a + / - integer",fixType:"",testFunc:function(){var testShell=new AutoTestShell('*[data-ae_vis][data-ae_avat][aria-valuenow]:not([aria-valuenow=""])');testShell.set_candidateSetNodesFunc(function(){var nodesWithBadValues=[];var res=aeKernel.rootTestNode.querySelectorAll('*[data-ae_vis][data-ae_avat][aria-valuenow]:not([aria-valuenow=""])');for(var i=0,len_res=res.length;i<len_res;i++){var n=res[i];var val=n.getAttribute("aria-valuenow");// check if this is a + / - integer / float
if(isNaN(val)===true){nodesWithBadValues.push(n);}}return nodesWithBadValues;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-69
	 * @returns {AutoTestShell} testShell
	 */function test69(){aeKernel.addAutomaticTest("69",{bestPractice:1626,testId:69,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM and available to assistive technologies, has an aria-valuemin attribute that is not set to 0, or a + / - integer.",metaText:"This element has an aria-valuemin attribute that is not set to a + / - integer",metaTextDetail:"This {{tag-name}} has an aria-valuemin attribute that is not set to a + / - integer",fixType:"",testFunc:function(){var testShell=new AutoTestShell('*[data-ae_vis][data-ae_avat][aria-valuemin]:not([aria-valuemin=""])');testShell.set_candidateSetNodesFunc(function(){var nodesWithBadValues=[];var res=aeKernel.rootTestNode.querySelectorAll('*[data-ae_vis][data-ae_avat][aria-valuemin]:not([aria-valuemin=""])');for(var i=0,len_res=res.length;i<len_res;i++){var n=res[i];var val=n.getAttribute("aria-valuemin");// check if this is a + / - integer / float
if(isNaN(val)===true){nodesWithBadValues.push(n);}}return nodesWithBadValues;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-71
	 * @returns {AutoTestShell} testShell
	 */function test71(){aeKernel.addAutomaticTest("71",{bestPractice:1626,testId:71,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='heading' attribute, not intentionally hidden in the DOM and available to assistive technologies, does not have an aria-level attribute.",metaText:"This element (role=heading) does not have an aria-level attribute",metaTextDetail:"This {{tag-name}} (role=heading) does not have an aria-level attribute",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="heading"]','*[data-ae_vis][data-ae_avat][data-ae_ar="heading"]:not([aria-level])');}});}/**
	 * NOTE: aria-level='' is applicable as it defaults to aria-level='0' in IE
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-74
	 * @returns {AutoTestShell} testShell
	 */function test74(){aeKernel.addAutomaticTest("74",{bestPractice:1626,testId:74,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM and available to assistive technologies, has an aria-level attribute that is not set to an integer value that is greater than, or equal to, 1.",metaText:"This element has an aria-level attribute that is not set to an integer value that is greater than, or equal to, 1",metaTextDetail:"This {{tag-name}} has an aria-level attribute of {{aria-level}}, which is not an integer value greater than, or equal to, 1",fixType:"",testFunc:function(){var testShell=new AutoTestShell('*[data-ae_vis][data-ae_avat][aria-level]');testShell.set_candidateSetNodesFunc(function(){var nodesWithBadValues=[];var res=aeKernel.rootTestNode.querySelectorAll('*[data-ae_vis][data-ae_avat][aria-level]');for(var i=0,len_res=res.length;i<len_res;i++){var n=res[i];var val=n.getAttribute("aria-level");// check if this is a + / - integer / float
if(isNaN(val)===false){var num=parseFloat(val);// isInteger is not supported in IE so we use ((num ^ 0) === num) === false)
if(num<=0||(num^0)===num===false){nodesWithBadValues.push(n);}}else{nodesWithBadValues.push(n);}}return nodesWithBadValues;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-78
	 * Template Test: 3853
	 * @returns {AutoTestShell} testShell
	 */function test78(){aeKernel.addAutomaticTest("78",{bestPractice:1626,testId:78,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM and available to assistive technologies, has an aria-flowto attribute value that includes one or more invalid ids.",metaText:"This element has an aria-flowto attribute value that includes one or more invalid or duplicate ids",metaTextDetail:"This {{tag-name}} has an aria-flowto attribute value of '{{aria-flowto}}', which includes one or more invalid or duplicate ids",fixType:"",testFunc:function(){var testShell=new AutoTestShell('*[data-ae_vis][data-ae_avat][aria-flowto]:not([aria-flowto=""])');testShell.set_candidateSetNodesFunc(function(){var nodesWithBadIds=[];var res=aeKernel.rootTestNode.querySelectorAll('*[data-ae_vis][data-ae_avat][aria-flowto]:not([aria-flowto=""])');for(var i=0,len_res=res.length;i<len_res;i++){var n=res[i];var labelledByIds=n.getAttribute("aria-flowto").trim().split(" ");for(var j=0,len_labelledByIds=labelledByIds.length;j<len_labelledByIds;j++){var labelledById=labelledByIds[j];if(isValidIdOrName(labelledById)===true){// "[id=" is used in place of "#" + id as ids can included colons which breaks css selector, although is valid html
var refNode=aeKernel.rootTestNode.querySelectorAll("[id='"+labelledById.replace(/'/g,"\\'")+"']");var refNodeLength=refNode.length;if(refNodeLength===0||refNodeLength>1){nodesWithBadIds.push(n);break;}}else{nodesWithBadIds.push(n);break;}}}return nodesWithBadIds;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-80
	 * @returns {AutoTestShell} testShell
	 */function test80(){aeKernel.addAutomaticTest("80",{bestPractice:1626,testId:80,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more dialog elements with an open attribute, excluding those with a role='dialog...' attribute and available to assistive technologies, has a role attribute set to a value that is not: 'alertdialog...'.",metaText:"The role attribute value given to this element is not allowed.  This element's role attribute can be set to the text value: alertdialog; or the role attribute can be removed",metaTextDetail:"The role attribute value of '{{data-ae_ar}}' given to this {{tag-name}} is not allowed.  This element's role attribute can be set to the text value: alertdialog; or the role attribute can be removed",fixType:"",testFunc:function(){return new AutoTestShell('dialog[open]:not([data-ae_ar="null"]):not([data-ae_ar="dialog"])[data-ae_avat]','dialog[open]:not([data-ae_ar="null"]):not([data-ae_ar="dialog"])[data-ae_avat]:not([data-ae_ar="alertdialog"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-81
	 * @returns {AutoTestShell} testShell
	 */function test81(){aeKernel.addAutomaticTest("81",{bestPractice:358,testId:81,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more img elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has an ismap attribute.",metaText:"This img element has an ismap attribute. Server side image maps are not keyboard accessible and the regions cannot be given accessible names",fixType:"",testFunc:function(){return new AutoTestShell('img[data-ae_vis][data-ae_avat][data-ae_ar="null"]','img[data-ae_vis][data-ae_avat][data-ae_ar="null"][ismap]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-83
	 * @returns {AutoTestShell} testShell
	 */function test83(){aeKernel.addAutomaticTest("83",{bestPractice:1626,testId:83,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM and available to assistive technologies, has an aria-describedby attribute value that includes one or more invalid ids.",metaText:"This element has an aria-describedby attribute value that includes one or more invalid or duplicate ids",metaTextDetail:"This {{tag-name}} has an aria-describedby attribute value of '{{aria-describedby}}', which includes one or more invalid or duplicate ids",fixType:"",testFunc:function(){var testShell=new AutoTestShell('*[data-ae_vis][data-ae_avat][aria-describedby]:not([aria-describedby=""])');testShell.set_candidateSetNodesFunc(function(){var nodesWithBadIds=[];var res=aeKernel.rootTestNode.querySelectorAll('*[data-ae_vis][data-ae_avat][aria-describedby]:not([aria-describedby=""])');for(var i=0,len_res=res.length;i<len_res;i++){var n=res[i];var labelledByIds=n.getAttribute("aria-describedby").trim().split(" ");for(var j=0,len_labelledByIds=labelledByIds.length;j<len_labelledByIds;j++){var labelledById=labelledByIds[j];if(isValidIdOrName(labelledById)===true){// "[id=" is used in place of "#" + id as ids can included colons which breaks css selector, although is valid html
var refNode=aeKernel.rootTestNode.querySelectorAll("[id='"+labelledById.replace(/'/g,"\\'")+"']");var refNodeLength=refNode.length;if(refNodeLength===0||refNodeLength>1){nodesWithBadIds.push(n);break;}}else{nodesWithBadIds.push(n);break;}}}return nodesWithBadIds;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-85
	 * Template Test: 95
	 * @returns {AutoTestShell} testShell
	 */function test85(){aeKernel.addAutomaticTest("85",{bestPractice:1626,testId:85,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM and available to assistive technologies, has an aria-controls attribute value that includes one or more invalid ids.",metaText:"This element has an aria-controls attribute value that includes one or more invalid or duplicate ids",metaTextDetail:"This {{tag-name}} has an aria-controls attribute value of '{{aria-controls}}', which includes one or more invalid or duplicate ids",fixType:"",testFunc:function(){var testShell=new AutoTestShell('*[data-ae_vis][data-ae_avat][aria-controls]:not([aria-controls=""])');testShell.set_candidateSetNodesFunc(function(){var nodesWithBadIds=[];var res=aeKernel.rootTestNode.querySelectorAll('*[data-ae_vis][data-ae_avat][aria-controls]:not([aria-controls=""])');for(var i=0,len_res=res.length;i<len_res;i++){var n=res[i];var labelledByIds=n.getAttribute("aria-controls").trim().split(" ");for(var j=0,len_labelledByIds=labelledByIds.length;j<len_labelledByIds;j++){var labelledById=labelledByIds[j];if(isValidIdOrName(labelledById)===true){// "[id=" is used in place of "#" + id as ids can included colons which breaks css selector, although is valid html
var refNode=aeKernel.rootTestNode.querySelectorAll("[id='"+labelledById.replace(/'/g,"\\'")+"']");var refNodeLength=refNode.length;if(refNodeLength===0||refNodeLength>1){nodesWithBadIds.push(n);break;}}else{nodesWithBadIds.push(n);break;}}}return nodesWithBadIds;});return testShell;}});}function fromStyles(computedStyles,testWind,node,pseudoElt){const computedStyle=testWind.getComputedStyle(node,pseudoElt);let pseudoEltFormattedContent=computedStyle.getPropertyValue('content').replace(/^"|\\|"$/g,"");if(pseudoEltFormattedContent===""||pseudoEltFormattedContent==="none"){return null;}const props=[`content${pseudoElt};${pseudoEltFormattedContent}`];for(let prop in blockStyles){props.push(`${prop}${pseudoElt};${computedStyle.getPropertyValue(prop)}`);}return props.join(';');}/**
	 * Extract the information we need for color contrast testing from computed styles
	 */function getBefore_FromStyles(computedStyles,testWind,node){return fromStyles(computedStyles,testWind,node,':before');}/**
	 * Extract the information we need for color contrast testing from computed styles
	 */function getAfter_FromStyles(computedStyles,testWind,node){return fromStyles(computedStyles,testWind,node,':after');}var commonjsGlobal=typeof globalThis!=='undefined'?globalThis:typeof window!=='undefined'?window:typeof global!=='undefined'?global:typeof self!=='undefined'?self:{};function commonjsRequire(path){throw new Error('Could not dynamically require "'+path+'". Please configure the dynamicRequireTargets or/and ignoreDynamicRequires option of @rollup/plugin-commonjs appropriately for this require call to work.');}var recursion={exports:{}};/*@license
	CalcNames: The AccName Computation Prototype, compute the Name and Description property values for a DOM node
	Returns an object with 'name' and 'desc' properties.
	Functionality mirrors the steps within the W3C Accessible Name and Description computation algorithm.
	https://w3c.github.io/accname/
	Author: Bryan Garaventa
	https://github.com/whatsock/w3c-alternative-text-computation
	Distributed under the terms of the Open Source Initiative OSI - MIT License
	*/(function(module){(function(){var nameSpace=window.AccNamePrototypeNameSpace||window;if(nameSpace&&typeof nameSpace==="string"&&nameSpace.length){window[nameSpace]={};nameSpace=window[nameSpace];}nameSpace.getAccNameVersion="2.57";// AccName Computation Prototype
nameSpace.getAccName=nameSpace.calcNames=function(node,fnc,preventVisualARIASelfCSSRef,overrides){overrides=overrides||{};var docO=overrides.document||document;var props={name:"",desc:"",error:""};var nameFromPlaceholder=false;var nameFromUserAgent=false;try{if(!node||node.nodeType!==1){return props;}var rootNode=node;var rootRole=trim(node.getAttribute("role")||"");// Track nodes to prevent duplicate node reference parsing.
var nodes=[];// Track aria-owns references to prevent duplicate parsing.
var owns=[];// Recursively process a DOM node to compute an accessible name in accordance with the spec
var walk=function(refNode,stop,skip,nodesToIgnoreValues,skipAbort,ownedBy,skipTo){skipTo=skipTo||{};skipTo.tag=skipTo.tag||false;skipTo.role=skipTo.role||false;skipTo.go=skipTo.go||false;var fullResult={name:"",title:""};var hasLabel=false;/*
	  ARIA Role Exception Rule Set 1.1
	  The following Role Exception Rule Set is based on the following ARIA Working Group discussion involving all relevant browser venders.
	  https://lists.w3.org/Archives/Public/public-aria/2017Jun/0057.html
	Plus roles extended for the Role Parity project.
	  */var isException=function(node,refNode){if(!refNode||!node||refNode.nodeType!==1||node.nodeType!==1){return false;}var role=getRole(node);var tag=node.nodeName.toLowerCase();var inList=function(node,list){return role&&list.roles.indexOf(role)>=0||!role&&list.tags.indexOf(tag)>=0;};// The list3 overrides must be checked first.
if(inList(node,list3)){if(node===refNode&&!(node.id&&ownedBy[node.id]&&ownedBy[node.id].node)){return!isFocusable(node);}else{// Note: the inParent checker needs to be present to allow for embedded roles matching list3 when the referenced parent is referenced using aria-labelledby, aria-describedby, or aria-owns.
return!(inParent(node,ownedBy.top)&&node.nodeName.toLowerCase()!=="select"||inList(refNode,list1));}}// Otherwise process list2 to identify roles to ignore processing name from content.
else{return!!((inList(node,list2)||node===rootNode&&!inList(node,list1))&&!(!role&&["section"].indexOf(tag)!==-1&&!(node.getAttribute("aria-labelledby")||node.getAttribute("aria-label")))&&!skipTo.go);}};var inParent=function(node,parent){var trackNodes=[];while(node){if(node.id&&ownedBy[node.id]&&ownedBy[node.id].node&&trackNodes.indexOf(node)===-1){trackNodes.push(node);node=ownedBy[node.id].node;}else{node=node.parentNode;}if(node&&node===parent){return true;}else if(!node||node===ownedBy.top||node===docO.body){return false;}}return false;};// Placeholder for storing CSS before and after pseudo element text values for the top level node
var cssOP={before:"",after:""};if(!skipTo.tag&&!skipTo.role&&nodes.indexOf(refNode)===-1){// Store the before and after pseudo element 'content' values for the top level DOM node
// Note: If the pseudo element includes block level styling, a space will be added, otherwise inline is asumed and no spacing is added.
cssOP=getCSSText(refNode,null);// Enabled in Visual ARIA to prevent self referencing by Visual ARIA tooltips
if(preventVisualARIASelfCSSRef){if(cssOP.before.indexOf(" [ARIA] ")!==-1||cssOP.before.indexOf(" aria-")!==-1||cssOP.before.indexOf(" accName: ")!==-1)cssOP.before="";if(cssOP.after.indexOf(" [ARIA] ")!==-1||cssOP.after.indexOf(" aria-")!==-1||cssOP.after.indexOf(" accDescription: ")!==-1)cssOP.after="";}}// Recursively apply the same naming computation to all nodes within the referenced structure
var walkDOM=function(node,fn,refNode){var res={name:"",title:""};if(!node){return res;}var nodeIsBlock=!!(node&&node.nodeType===1&&isBlockLevelElement(node));var currentNode=node;var fResult=fn(node)||{};if(fResult.name&&fResult.name.length){res.name+=fResult.name;}if(!fResult.skip&&!isException(node,ownedBy.top)){if(skipTo.go)skipTo.go=false;node=node.firstChild;while(node){res.name+=walkDOM(node,fn,refNode).name;node=node.nextSibling;}}res.name+=fResult.owns||"";if(rootNode===currentNode&&refNode===currentNode&&!trim(res.name)&&trim(fResult.title)){res.name=addSpacing(fResult.title);}else if(rootNode===currentNode&&refNode===currentNode&&trim(fResult.title)){res.title=addSpacing(fResult.title);}if(rootNode===currentNode&&refNode===currentNode&&trim(fResult.desc)){res.title=addSpacing(fResult.desc);}if(rootNode===currentNode&&refNode===currentNode&&trim(fResult.placeholder)&&!trim(res.name)){res.name=addSpacing(fResult.placeholder);nameFromPlaceholder=true;}else if(rootNode===currentNode&&refNode===currentNode&&trim(fResult.placeholder)&&!trim(res.title)){res.title=addSpacing(fResult.placeholder);}if(nodeIsBlock||fResult.isWidget){res.name=addSpacing(res.name);}return res;};fullResult=walkDOM(refNode,function(node){var i=0;var element=null;var ids=[];var parts=[];var result={name:"",title:"",owns:"",skip:false};var isEmbeddedNode=!!(node&&node.nodeType===1&&nodesToIgnoreValues&&nodesToIgnoreValues.length&&nodesToIgnoreValues.indexOf(node)!==-1&&node===rootNode&&node!==refNode);var hLabel=false;if((skip||!node||isHidden(node,ownedBy.top))&&!skipAbort&&!isEmbeddedNode){// Abort if algorithm step is already completed, or if node is a hidden child of refNode, or skip abort if aria-labelledby self references same node.
return result;}if(!skipTo.tag&&!skipTo.role&&nodes.indexOf(node)===-1){nodes.push(node);}else{// Abort if this node has already been processed.
return result;}// Store name for the current node.
var name="";// Store name from aria-owns references if detected.
var ariaO="";// Placeholder for storing CSS before and after pseudo element text values for the current node container element
var cssO={before:"",after:""};var parent=refNode===node?node:node.parentNode;if(!skipTo.tag&&!skipTo.role&&nodes.indexOf(parent)===-1){nodes.push(parent);// Store the before and after pseudo element 'content' values for the current node container element
// Note: If the pseudo element includes block level styling, a space will be added, otherwise inline is asumed and no spacing is added.
cssO=getCSSText(parent,refNode);// Enabled in Visual ARIA to prevent self referencing by Visual ARIA tooltips
if(preventVisualARIASelfCSSRef){if(cssO.before.indexOf(" [ARIA] ")!==-1||cssO.before.indexOf(" aria-")!==-1||cssO.before.indexOf(" accName: ")!==-1)cssO.before="";if(cssO.after.indexOf(" [ARIA] ")!==-1||cssO.after.indexOf(" aria-")!==-1||cssO.after.indexOf(" accDescription: ")!==-1)cssO.after="";}}// Process standard DOM element node
if(node.nodeType===1){var nTag=node.nodeName.toLowerCase();var nRole=getRole(node);var aLabelledby=!skipTo.tag&&!skipTo.role&&node.getAttribute("aria-labelledby")||"";var aDescribedby=!skipTo.tag&&!skipTo.role&&node.getAttribute("aria-describedby")||"";var aDescription=!skipTo.tag&&!skipTo.role&&node.getAttribute("aria-description");var aLabel=!skipTo.tag&&!skipTo.role&&node.getAttribute("aria-label")||"";var nTitle=!skipTo.tag&&!skipTo.role&&node.getAttribute("title")||"";var isNativeFormField=nativeFormFields.indexOf(nTag)!==-1;var isNativeButton=["input"].indexOf(nTag)!==-1;var isRangeWidgetRole=rangeWidgetRoles.indexOf(nRole)!==-1;var isEditWidgetRole=editWidgetRoles.indexOf(nRole)!==-1;var isSelectWidgetRole=selectWidgetRoles.indexOf(nRole)!==-1;var isSimulatedFormField=isRangeWidgetRole||isEditWidgetRole||isSelectWidgetRole||nRole==="combobox";var isWidgetRole=(isSimulatedFormField||otherWidgetRoles.indexOf(nRole)!==-1)&&nRole!=="link";result.isWidget=isNativeFormField||isWidgetRole;var hasName=false;var hasDesc=false;var aOwns=node.getAttribute("aria-owns")||"";var isSeparatChildFormField=!!(!skipTo.tag&&!skipTo.role&&!isEmbeddedNode&&(node!==refNode&&(isNativeFormField||isSimulatedFormField)||node.id&&ownedBy[node.id]&&ownedBy[node.id].target&&ownedBy[node.id].target===node));// Check for non-empty value of aria-labelledby on current node, follow each ID ref, then stop and process no deeper.
if(!stop&&!skipTo.tag&&!skipTo.role&&aLabelledby){ids=aLabelledby.split(/\s+/);parts=[];for(i=0;i<ids.length;i++){element=docO.getElementById(ids[i]);// Also prevent the current form field from having its value included in the naming computation if nested as a child of label
parts.push(walk(element,true,skip,[node],element===refNode,{ref:ownedBy,top:element}).name);}// Check for blank value, since whitespace chars alone are not valid as a name
name=trim(parts.join(" "));if(trim(name)){hasName=true;hLabel=true;hasLabel=true;// Abort further recursion if name is valid.
result.skip=true;}}// Check for non-empty value of aria-describedby/description if current node equals reference node, follow each ID ref, then stop and process no deeper.
if(!stop&&node===refNode&&!skipTo.tag&&!skipTo.role&&(aDescribedby||aDescription)){if(aDescribedby){var desc;ids=aDescribedby.split(/\s+/);parts=[];for(i=0;i<ids.length;i++){element=docO.getElementById(ids[i]);// Also prevent the current form field from having its value included in the naming computation if nested as a child of label
parts.push(walk(element,true,false,[node],false,{ref:ownedBy,top:element}).name);}// Check for blank value, since whitespace chars alone are not valid as a name
desc=trim(parts.join(" "));}else{desc=trim(aDescription);}if(trim(desc)){result.desc=desc;hasDesc=true;}}// Otherwise, if current node has a non-empty aria-label then set as name and process no deeper within the branch.
if(!skipTo.tag&&!skipTo.role&&!hasName&&trim(aLabel)&&!isSeparatChildFormField){name=aLabel;// Check for blank value, since whitespace chars alone are not valid as a name
if(trim(name)){hasName=true;hLabel=true;if(node===refNode){// If name is non-empty and both the current and refObject nodes match, then don't process any deeper within the branch.
skip=true;hasLabel=true;}}}var rolePresentation=!skipTo.tag&&!skipTo.role&&!hasName&&nTag!=="iframe"&&nRole&&presentationRoles.indexOf(nRole)!==-1&&!isFocusable(node)&&!hasGlobalAttr(node);// Otherwise, if the current node is not a nested widget control within the parent ref obj, but is instead a native markup element that includes a host-defined labelling mechanism, then set the name and description accordingly if present.
if(!isSeparatChildFormField){// Otherwise, if name is still empty and the current node matches the ref node and is a standard form field with a non-empty associated label element, process label with same naming computation algorithm.
if(!skipTo.tag&&!skipTo.role&&!hasName&&node===refNode&&isNativeFormField){// Logic modified to match issue
// https://github.com/WhatSock/w3c-alternative-text-computation/issues/12 */
var labels=docO.querySelectorAll("label");var lblName="";var implicitLabel=getParent(node,"label")||false;for(i=0;i<labels.length;i++){if((labels[i]===implicitLabel&&typeof implicitLabel.getAttribute("for")!=="string"||labels[i].getAttribute("for")===node.id)&&!isParentHidden(labels[i],docO.body,true)){lblName+=addSpacing(walk(labels[i],true,skip,[node],false,{ref:ownedBy,top:labels[i]}).name);}}name=lblName;if(trim(name)){hasName=true;}}// Process native form field buttons in accordance with the HTML AAM
// https://w3c.github.io/html-aam/#accessible-name-and-description-computation
var btnType=!skipTo.tag&&!skipTo.role&&isNativeButton&&(node.getAttribute("type")||"").toLowerCase()||false;var btnValue=!skipTo.tag&&!skipTo.role&&btnType&&trim(node.getAttribute("value"))||false;var nAlt=rolePresentation&&nTag==="img"?"":trim(node.alt||node.getAttribute("alt"));// Otherwise, if name is still empty and current node is a standard non-presentational img or image button with a non-empty alt or title attribute, set alt or title attribute value as the accessible name.
if(!skipTo.tag&&!skipTo.role&&!hasName&&!rolePresentation&&(nRole==="img"||nTag==="img"||btnType==="image")&&(nAlt||trim(nTitle))){// Check for blank value, since whitespace chars alone are not valid as a name
name=trim(nAlt)||trim(nTitle);if(trim(name)){hasName=true;}}// Process native HTML area tags to use alt as name when not explicitly set using aria-labelledby or aria-label.
if(!skipTo.tag&&!skipTo.role&&!hasName&&!rolePresentation&&nTag==="area"&&nAlt){// Check for blank value, since whitespace chars alone are not valid as a name
name=trim(nAlt);if(trim(name)){hasName=true;}}// Process native HTML optgroup tags to use label as name when not explicitly set using aria-labelledby or aria-label.
if(nTag==="optgroup"){if(!skipTo.tag&&!skipTo.role&&!hasName&&!rolePresentation&&node.getAttribute("label")){// Check for blank value, since whitespace chars alone are not valid as a name
name=trim(node.getAttribute("label"));if(trim(name)){hasName=true;}}result.skip=true;}// Process the accessible names for native HTML buttons
if(!skipTo.tag&&!skipTo.role&&!hasName&&node===refNode&&btnType&&["button","submit","reset"].indexOf(btnType)!==-1){if(btnValue){name=btnValue;}else{switch(btnType){case"submit":name="submit";break;case"reset":name="reset";break;default:name="";}}if(trim(name)){hasName=true;}}if(!skipTo.tag&&!skipTo.role&&hasName&&node===refNode&&btnType&&["button","submit","reset"].indexOf(btnType)!==-1&&btnValue&&btnValue!==name&&!result.desc){result.desc=btnValue;hasDesc=true;}// Process the accessible names for native HTML image buttons
if(!skipTo.tag&&!skipTo.role&&!hasName&&node===refNode&&btnType&&btnType==="image"){name="Submit Query";hasName=true;nameFromUserAgent=true;}var isFieldset=!skipTo.tag&&!skipTo.role&&!hasName&&node===rootNode&&(nRole==="group"||nRole==="radiogroup"||!nRole&&nTag==="fieldset");// Otherwise, if name is still empty and the current node matches the root node and is a standard fieldset element with a non-empty associated legend element as the first child node, process legend with same naming computation algorithm.
// Plus do the same for role="group" and role="radiogroup" with embedded role="legend", or a combination of these.
if(isFieldset){var fChild=firstChild(node,["legend"],["legend"])||false;if(fChild){name=trim(walk(fChild,stop,false,[],false,{ref:ownedBy,top:fChild}).name);}if(trim(name)){hasName=true;}skip=true;}var isTable=!skipTo.tag&&!skipTo.role&&!hasName&&node===rootNode&&(nRole==="table"||!nRole&&nTag==="table");// Otherwise, if name is still empty and the current node matches the root node and is a standard table element with a non-empty associated caption element as the first child node, process caption with same naming computation algorithm.
// Plus do the same for role="table" with embedded role="caption", or a combination of these.
if(isTable){fChild=firstChild(node,["caption"],["caption"])||false;if(fChild){name=trim(walk(fChild,stop,false,[],false,{ref:ownedBy,top:fChild}).name);}if(trim(name)){hasName=true;}skip=true;}// Otherwise, if name is still empty and the root node and the current node are the same and node is an svg element, then parse the content of the title element to set the name and the desc element to set the description.
if(!skipTo.tag&&!skipTo.role&&nTag==="svg"){var svgT=node.querySelector("title")||false;var svgD=node===rootNode&&node.querySelector("desc")||false;if(!hasName&&svgT){name=trim(walk(svgT,true,false,[],false,{ref:ownedBy,top:svgT}).name);}if(!hasDesc&&svgD){var dE=trim(walk(svgD,true,false,[],false,{ref:ownedBy,top:svgD}).name);if(trim(dE)){result.desc=dE;}}result.skip=true;}}// Otherwise, if the current node is a nested widget control within the parent ref obj, then add only its value and process no deeper within the branch.
if(!skipTo.tag&&!skipTo.role&&isSeparatChildFormField){// Prevent the referencing node from having its value included in the case of form control labels that contain the element with focus.
if(!(nodesToIgnoreValues&&nodesToIgnoreValues.length&&nodesToIgnoreValues.indexOf(node)!==-1)){if(isRangeWidgetRole){// For range widgets, append aria-valuetext if non-empty, or aria-valuenow if non-empty, or node.value if applicable.
name=getObjectValue(nRole,node,true);}else if(isEditWidgetRole||nRole==="combobox"&&isNativeFormField){// For simulated edit widgets, append text from content if applicable, or node.value if applicable.
name=getObjectValue(nRole,node,false,true);}else if(isSelectWidgetRole){// For simulated select widgets, append same naming computation algorithm for all child nodes including aria-selected="true" separated by a space when multiple.
// Also filter nodes so that only valid child roles of relevant parent role that include aria-selected="true" are included.
name=getObjectValue(nRole,node,false,false,true);}else if(isNativeFormField&&["input","textarea"].indexOf(nTag)!==-1&&(!isWidgetRole||isEditWidgetRole)){// For native edit fields, append node.value when applicable.
name=getObjectValue(nRole,node,false,false,false,true);}else if(isNativeFormField&&nTag==="select"&&(!isWidgetRole||nRole==="combobox")){// For native select fields, get text from content for all options with selected attribute separated by a space when multiple, but don't process if another widget role is present unless it matches role="combobox".
// Reference: https://github.com/WhatSock/w3c-alternative-text-computation/issues/7
name=getObjectValue(nRole,node,false,false,true,true);}// Check for blank value, since whitespace chars alone are not valid as a name
name=trim(name);}}// Otherwise, if current node is the same as rootNode and is non-presentational and includes a non-empty title attribute, store title attribute value as the accessible name if name is still empty, or the description if not.
// Processing for this is handled within the walkDOM function.
if(!skipTo.tag&&!skipTo.role&&!rolePresentation&&trim(nTitle)){if(!(name&&aDescription===" ")){result.title=trim(nTitle);}}var nType=isNativeFormField&&trim(node.getAttribute("type")||"").toLowerCase();if(!nType)nType="text";var placeholder=!skipTo.tag&&!skipTo.role&&node===rootNode&&node===refNode&&(isEditWidgetRole||isNativeFormField&&(nTag==="textarea"||nTag==="input"&&["password","search","tel","text","url","email"].indexOf(nType)!==-1))&&trim(node.getAttribute("placeholder")||node.getAttribute("aria-placeholder"));if(placeholder){result.placeholder=placeholder;}var isSkipTo=skipTo.role&&skipTo.role===nRole||!nRole&&skipTo.tag&&skipTo.tag===nTag;// Process custom tag and role searches if needed.
if(isSkipTo){name=trim(walk(node,stop,false,[],false,{ref:ownedBy,top:node}).name);if(trim(name)){skip=true;}}// Check for non-empty value of aria-owns, follow each ID ref, then process with same naming computation.
// Also abort aria-owns processing if contained on an element that does not support child elements.
if(!isSkipTo&&aOwns&&["input","img","progress"].indexOf(nTag)===-1){ids=aOwns.split(/\s+/);parts=[];for(i=0;i<ids.length;i++){element=docO.getElementById(ids[i]);// Abort processing if the referenced node has already been traversed
if(element&&owns.indexOf(ids[i])===-1){owns.push(ids[i]);var oBy={ref:ownedBy,top:ownedBy.top};oBy[ids[i]]={refNode:refNode,node:node,target:element};if(!isParentHidden(element,docO.body,true)){parts.push(walk(element,true,skip,[],false,oBy).name);}}}// Join without adding whitespace since this is already handled by parsing individual nodes within the algorithm steps.
ariaO=parts.join("");}}// Otherwise, process text node
else if(!skipTo.tag&&!skipTo.role&&node.nodeType===3){name=node.data;}if(!hLabel){// Prepend and append the current CSS pseudo element text, plus normalize all whitespace such as newline characters and others into flat spaces.
name=cssO.before+name.replace(/\s+/g," ")+cssO.after;}if(name.length&&!hasParentLabelOrHidden(node,ownedBy.top,ownedBy)){result.name=name;}result.owns=ariaO;return result;},refNode);if(!hasLabel){// Prepend and append the refObj CSS pseudo element text, plus normalize whitespace chars into flat spaces.
fullResult.name=cssOP.before+fullResult.name.replace(/\s+/g," ")+cssOP.after;}return fullResult;};var firstChild=function(e,t,r,s){e=e?e.firstChild:null;while(e){var tr=getRole(e)||false;if(e.nodeType===1&&(!t&&!r||tr&&r&&r.indexOf(tr)!==-1||!tr&&t&&t.indexOf(e.nodeName.toLowerCase())!==-1)){return e;}else if(!s&&e.nodeType===1&&(t||r)){return null;}e=e.nextSibling;}return e;};var lastChild=function(e,t,r,s){e=e?e.lastChild:null;while(e){var tr=getRole(e)||false;if(e.nodeType===1&&(!t&&!r||tr&&r&&r.indexOf(tr)!==-1||!tr&&t&&t.indexOf(e.nodeName.toLowerCase())!==-1)){return e;}else if(!s&&e.nodeType===1&&(t||r)){return null;}e=e.previousSibling;}return e;};var getRole=function(node){var role=node&&node.getAttribute?(node.getAttribute("role")||"").toLowerCase():"";if(!trim(role)){return"";}var inList=function(list){return trim(role).length>0&&list.roles.indexOf(role)>=0;};var roles=role.split(/\s+/);for(var i=0;i<roles.length;i++){role=roles[i];if(inList(list1)||inList(list2)||inList(list3)||inList(list4)||presentationRoles.indexOf(role)!==-1){return role;}}return"";};var isFocusable=function(node){var nodeName=node.nodeName.toLowerCase();if(node.getAttribute("tabindex")){return true;}if(nodeName==="a"&&node.getAttribute("href")){return true;}return["button","input","select","textarea"].indexOf(nodeName)!==-1&&(node.getAttribute("type")||"").toLowerCase()!=="hidden";};// ARIA Role Exception Rule Set 1.2
// The following Role Exception Rule Set is based on the following ARIA Working Group discussion involving all relevant browser venders.
// https://lists.w3.org/Archives/Public/public-aria/2017Jun/0057.html
// Always include name from content when the referenced node matches list1, as well as when child nodes match those within list3
// Note: gridcell was added to list1 to account for focusable gridcells that match the ARIA 1.0 paradigm for interactive grids.
// So too was row to match 'name from author' and 'name from content' in accordance with the spec.
var list1={roles:["button","checkbox","link","option","radio","switch","tab","treeitem","menuitem","menuitemcheckbox","menuitemradio","row","cell","gridcell","columnheader","rowheader","tooltip","heading"],tags:["a","button","summary","input","h1","h2","h3","h4","h5","h6","menuitem","option","tr","td","th"]};// Never include name from content when current node matches list2
// The rowgroup role was added to prevent 'name from content' in accordance with relevant ARIA 1.1 spec changes.
// The fieldset element and group role was added to account for implicit mappings where name from content is not supported.
var list2={roles:["application","alert","log","marquee","timer","alertdialog","dialog","banner","complementary","form","main","navigation","region","search","article","document","feed","figure","img","math","toolbar","menu","menubar","grid","listbox","radiogroup","textbox","searchbox","spinbutton","scrollbar","slider","tablist","tabpanel","tree","treegrid","separator","rowgroup","group"],tags:["article","aside","body","select","datalist","optgroup","dialog","figure","footer","form","header","hr","iframe","img","textarea","input","main","math","menu","nav","section","thead","tbody","tfoot","fieldset"]};// As an override of list2, conditionally include name from content if current node is focusable, or if the current node matches list3 while the referenced parent node (root node) matches list1.
var list3={roles:["term","definition","directory","list","note","status","table","contentinfo"],tags:["dl","ul","ol","dd","details","output","table"]};// Subsequent roles added as part of the Role Parity project for ARIA 1.2.
// Tracks roles that don't specifically belong within the prior process lists.
var list4={roles:["legend","caption"],tags:["legend","caption","figcaption"]};var nativeFormFields=["button","input","select","textarea"];var rangeWidgetRoles=["scrollbar","slider","spinbutton"];var editWidgetRoles=["searchbox","textbox"];var selectWidgetRoles=["grid","listbox","tablist","tree","treegrid"];var otherWidgetRoles=["button","checkbox","link","switch","option","menu","menubar","menuitem","menuitemcheckbox","menuitemradio","radio","tab","treeitem","gridcell"];var presentationRoles=["presentation","none"];var hasGlobalAttr=function(node){var globalPropsAndStates=["labelledby","label","describedby","busy","controls","current","details","disabled","dropeffect","errormessage","flowto","grabbed","haspopup","invalid","keyshortcuts","live","owns","roledescription"];for(var i=0;i<globalPropsAndStates.length;i++){var a=trim(node.getAttribute("aria-"+globalPropsAndStates[i]));if(a){return true;}}return false;};var isHidden=overrides.isHidden||function(node,refNode){var hidden=function(node){if(!node||node.nodeType!==1||node===refNode){return false;}if(node.getAttribute("aria-hidden")==="true"){return true;}if(node.getAttribute("hidden")){return true;}var style=getStyleObject(node);return style["display"]==="none"||style["visibility"]==="hidden";};return hidden(node);};var isParentHidden=function(node,refNode,skipOwned,skipCurrent){while(node&&node!==refNode){if(!skipCurrent&&node.nodeType===1&&isHidden(node,refNode)){return true;}else skipCurrent=false;node=node.parentNode;}return false;};var getStyleObject=overrides.getStyleObject||function(node){var style={};if(docO.defaultView&&docO.defaultView.getComputedStyle){style=docO.defaultView.getComputedStyle(node,"");}else if(node.currentStyle){style=node.currentStyle;}return style;};var cleanCSSText=function(node,text){var s=text;if(s.indexOf("attr(")!==-1){var m=s.match(/attr\((.|\n|\r\n)*?\)/g);for(var i=0;i<m.length;i++){var b=m[i].slice(5,-1);b=node.getAttribute(b)||"";s=s.replace(m[i],b);}}s=s.replace(/url\((.*?)\)\s+\/|url\((.*?)\)/g,"").replace(/^\s+|\s+$/g,"").replace(/\"/g,"");return s;};var isBlockLevelElement=function(node,cssObj){var styleObject=cssObj||getStyleObject(node);for(var prop in blockStyles){var values=blockStyles[prop];for(var i=0;i<values.length;i++){if(styleObject[prop]&&(values[i].indexOf("!")===0&&[values[i].slice(1),"inherit","initial","unset"].indexOf(styleObject[prop])===-1||styleObject[prop].indexOf(values[i])===0)){return true;}}}return!cssObj&&node.nodeName&&blockElements.indexOf(node.nodeName.toLowerCase())!==-1&&!(styleObject["display"]&&styleObject["display"].indexOf("inline")===0&&node.nodeName.toLowerCase()!=="br");};// CSS Block Styles indexed from:
// https://developer.mozilla.org/en-US/docs/Web/Guide/CSS/Block_formatting_context
var blockStyles={display:["block","grid","table","flow-root","flex"],position:["absolute","fixed"],float:["left","right","inline"],clear:["left","right","both","inline"],overflow:["hidden","scroll","auto"],"column-count":["!auto"],"column-width":["!auto"],"column-span":["all"],contain:["layout","content","strict"]};// HTML5 Block Elements indexed from:
// https://github.com/webmodules/block-elements
// Note: 'br' was added to this array because it impacts visual display and should thus add a space .
// Reference issue: https://github.com/w3c/accname/issues/4
// Note: Added in 1.13, td, th, tr, and legend
var blockElements=["address","article","aside","blockquote","br","canvas","dd","div","dl","dt","fieldset","figcaption","figure","footer","form","h1","h2","h3","h4","h5","h6","header","hgroup","hr","legend","li","main","nav","noscript","ol","output","p","pre","section","table","td","tfoot","th","tr","ul","video"];var getObjectValue=function(role,node,isRange,isEdit,isSelect,isNative){var val="";var bypass=false;if(isRange&&!isNative){val=node.getAttribute("aria-valuetext")||node.getAttribute("aria-valuenow")||"";}else if(isEdit&&!isNative){val=getText(node)||"";}else if(isSelect&&!isNative){var childRoles=[];if(role==="grid"||role==="treegrid"){childRoles=["gridcell","rowheader","columnheader"];}else if(role==="listbox"){childRoles=["option"];}else if(role==="tablist"){childRoles=["tab"];}else if(role==="tree"){childRoles=["treeitem"];}val=joinSelectedParts(node,node.querySelectorAll('*[aria-selected="true"]'),false,childRoles);bypass=true;}val=trim(val);if(!val&&(isRange||isEdit)&&node.value){val=node.value;}if(!bypass&&!val&&isNative){if(isSelect){val=joinSelectedParts(node,node.querySelectorAll("option[selected]"),true);}else{val=node.value;}}return val;};var addSpacing=function(s){return trim(s).length?" "+s+" ":" ";};var joinSelectedParts=function(node,nOA,isNative,childRoles){if(!nOA||!nOA.length){return"";}var parts=[];for(var i=0;i<nOA.length;i++){var role=getRole(nOA[i]);var isValidChildRole=!childRoles||childRoles.indexOf(role)!==-1;if(isValidChildRole){parts.push(isNative?getText(nOA[i]):walk(nOA[i],true,false,[],false,{top:nOA[i]}).name);}}return parts.join(" ");};var getPseudoElStyleObj=overrides.getPseudoElStyleObj||function(node,position){var styleObj={};for(var prop in blockStyles){styleObj[prop]=docO.defaultView.getComputedStyle(node,position).getPropertyValue(prop);}styleObj["content"]=docO.defaultView.getComputedStyle(node,position).getPropertyValue("content").replace(/^"|\\|"$/g,"");return styleObj;};var getText=function(node,position){if(!position&&node.nodeType===1){return node.innerText||node.textContent||"";}var styles=getPseudoElStyleObj(node,position);var text=styles["content"];if(!text||text==="none"){return"";}if(isBlockLevelElement({},styles)){if(position===":before"){text+=" ";}else if(position===":after"){text=" "+text;}}return text;};var getCSSText=overrides.getCSSText||function(node,refNode){if(node&&node.nodeType!==1||node===refNode||["input","select","textarea","img","iframe","optgroup"].indexOf(node.nodeName.toLowerCase())!==-1){return{before:"",after:""};}return{before:cleanCSSText(node,getText(node,":before")),after:cleanCSSText(node,getText(node,":after"))};};var getParent=function(node,nTag,nRole,noRole){noRole=!!noRole;while(node){node=node.parentNode;if(node&&(nRole&&getRole(node)===nRole||nTag&&node.nodeName&&node.nodeName.toLowerCase()===nTag&&(!noRole||getRole(node).length<1))){return node;}}return{};};var hasParentLabelOrHidden=function(node,refNode,ownedBy,ignoreHidden){var trackNodes=[];while(node&&node!==refNode){if(node.id&&ownedBy&&ownedBy[node.id]&&ownedBy[node.id].node&&trackNodes.indexOf(node)===-1){trackNodes.push(node);node=ownedBy[node.id].node;}else{node=node.parentNode;}if(node&&node.getAttribute){if(trim(node.getAttribute("aria-label"))||!ignoreHidden&&isHidden(node,refNode)){return true;}}}return false;};if(isParentHidden(node,docO.body,true,!!(node&&node.nodeName&&node.nodeName.toLowerCase()==="area"))){return props;}// Compute accessible Name and Description properties value for node
var accProps=walk(node,false,false,[],false,{top:node});var accName=trim(accProps.name.replace(/\s+/g," "));var accDesc=trim(accProps.title.replace(/\s+/g," "));if(accName===accDesc){// If both Name and Description properties match, then clear the Description property value.
accDesc="";}props.hasUpperCase=rootRole&&rootRole!==rootRole.toLowerCase()?true:false;props.name=accName;props.desc=accDesc;// Clear track variables
nodes=[];owns=[];}catch(e){props.error=e;}props.placeholder=nameFromPlaceholder;props.userAgent=nameFromUserAgent;if(fnc&&typeof fnc==="function"){return fnc.apply(node,[props,node]);}else{return props;}};var trim=function(str){if(typeof str!=="string"){return"";}return str.replace(/^\s+|\s+$/g,"");};// Customize returned string for testable statements
nameSpace.getAccNameMsg=nameSpace.getNames=function(node,overrides){var props=nameSpace.getAccName(node,null,false,overrides);if(props.error){return props.error+"\n\n"+"An error has been thrown in AccName Prototype version "+nameSpace.getAccNameVersion+". Please copy this error message and the HTML markup that caused it, and submit both as a new GitHub issue at\n"+"https://github.com/whatsock/w3c-alternative-text-computation";}var r='accName: "'+props.name+'"\n\naccDesc: "'+props.desc+'"\n\n';if(props.placeholder)r+="Name from placeholder: true\n\n";if(props.userAgent)r+="Name from user agent: true\n\n";r+="(Running AccName Computation Prototype version: "+nameSpace.getAccNameVersion+")";return r;};if(module.exports){module.exports={getNames:nameSpace.getNames,calcNames:nameSpace.calcNames};}})();})(recursion);const calcNames=window[AccNamePrototypeNameSpace].calcNames;function invokeCalcNames(rootTestDocument,node){return calcNames(node,null,false,{document:rootTestDocument,isHidden:function(node,refNode){if(!node||node.nodeType!==1||node===refNode){return false;}return node.hasAttribute("data-ae_invis");},getStyleObject:function(node){return retrieveComputedCssStyles(node);},getPseudoElStyleObj:function(node,position){return retrieveComputedCssStyles(node,position);}});}/*
	 * Pre-Testing: Finds elements that need an accessible name calculating, then calculates the value and sets their date-ae_an attribute.
	 * @returns {Number}
	 */function calcAccNamesForCertainElements(rootTestNode){try{const els=rootTestNode.querySelectorAll("a:not([data-ae_ar]), area:not([data-ae_ar]), audio:not([data-ae_ar]), button:not([data-ae_ar]), canvas:not([data-ae_ar]), embed:not([data-ae_ar]), fieldset:not([data-ae_ar]), frame:not([data-ae_ar]), iframe, img:not([data-ae_ar]), input:not([data-ae_ar]), meter:not([data-ae_ar]), object:not([data-ae_ar]), optgroup:not([data-ae_ar]), progress:not([data-ae_ar]), select:not([data-ae_ar]), svg:not([data-ae_ar]), svg[data-ae_ar*='graphics-'], textarea:not([data-ae_ar]), video:not([data-ae_ar]), *[data-ae_ar^='img'], *[data-ae_ar='link'], *[data-ae_ar='radiogroup'], *[data-ae_ar='checkbox'], *[data-ae_ar='radio'], *[data-ae_ar='slider'], *[data-ae_ar='textbox'], *[data-ae_ar='combobox'], *[data-ae_ar='button'], *[data-ae_ar='spinbutton'], aside:not([data-ae_ar]), main:not([data-ae_ar]), nav:not([data-ae_ar]), *[data-ae_ar='dialog'], dialog:not([data-ae_ar]), picture:not([data-ae_ar]), figure:not([data-ae_ar]), summary:not([data-ae_ar]), *[data-ae_ar^='graphics'], *[data-ae_ar='region'], *[data-ae_ar='progressbar']");for(let i=els.length;i--;){if(isRunningTooLong()){throw getRuntimeExceededMessage();}const el=els[i];const payload=invokeCalcNames(rootTestNode.ownerDocument,el);if(payload.error){throw payload.error;}const accessibleName=payload.name||null;if(accessibleName!==null){el.setAttribute("data-ae_an",accessibleName.toLowerCase());if(payload.placeholder){el.setAttribute("data-ae_anp","true");}if(payload.userAgent){el.setAttribute("data-ae_anua","true");}}}return rootTestNode;}catch(err){logger.error("stage4_calcAccNamesForCertainElements",err);return null;}}/*
	 * Pre-Testing: Finds elements that need an accessible description calculating, then calculates the value and sets their date-ae_ad attribute.
	 * @returns {Number}
	 */function calcAccDescriptionsForCertainElements(rootTestNode){try{const els=rootTestNode.querySelectorAll("a[target='_blank']:not([data-ae_ar])");for(let i=els.length;i--;){if(isRunningTooLong()){throw getRuntimeExceededMessage();}const el=els[i];const payload=invokeCalcNames(rootTestNode.ownerDocument,el);if(payload.error){throw payload.error;}const accessibleDescription=payload.desc||null;if(accessibleDescription!==null){el.setAttribute("data-ae_ad",accessibleDescription);}}return rootTestNode;}catch(err){logger.error(err);return null;}}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-87
	 * @returns {AutoTestShell} testShell
	 */function test87(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("87",{bestPractice:362,testId:87,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements with a role attribute set to a value that starts with 'img' or 'graphics', excluding children of button elements and children of anchor elements, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This element (role=img, role=graphics-symbol or role=graphics-document) does not have a mechanism that allows an accessible name value to be calculated",metaTextDetail:"This {{tag-name}} (role={{data-ae_ar}}) does not have a mechanism that allows an accessible name value to be calculated",fixType:{"fixType":1,"domSpec":false,"helperText":"Specify label text that describes the content conveyed by the image.","fix":{"aria-label":null,"alt":false,"aria-labelledby":false}},testFunc:function(){const relevantCssSelectora='*[data-ae_ar^="img"]:not(button *):not(a *):not([role="button"] *):not([role="link"] *)[data-ae_vis][data-ae_avat]';const relevantCssSelectorb='*[data-ae_ar^="graphics"]:not(button *):not(a *):not([role="button"] *):not([role="link"] *)[data-ae_vis][data-ae_avat]';return new AutoTestShell(`${relevantCssSelectora},${relevantCssSelectorb}`,`${relevantCssSelectora}:not([data-ae_an]), ${relevantCssSelectorb}:not([data-ae_an])`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-89
	 * @returns {AutoTestShell} testShell
	 */function test89(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("89",{bestPractice:362,testId:89,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more img elements, excluding img elements with an alt attribute set to a null value, without an ARIA-assigned role other than img, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This img element does not have a mechanism that allows an accessible name value to be calculated",fixType:{"fixType":1,"domSpec":false,"helperText":"Provide alternative text that describes the content conveyed by the image.","fix":{"alt":null,"aria-label":false,"aria-labelledby":false}},testFunc:function(){return new AutoTestShell('img[data-ae_ar="null"][data-ae_vis][data-ae_avat]:not([alt=""]), img[data-ae_ar="img"][data-ae_vis][data-ae_avat]:not([alt=""])','img[data-ae_ar="null"][data-ae_vis][data-ae_avat]:not([alt=""]):not([data-ae_an]), img[data-ae_ar="img"][data-ae_vis][data-ae_avat]:not([alt=""]):not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-94
	 * @returns {AutoTestShell} testShell
	 */function test94(){aeKernel.addAutomaticTest("94",{bestPractice:1352,testId:94,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements contain an id attribute set to a value that is not unique in the same DOM.",metaText:"This element has an id attribute set to a value that is not unique in the same DOM",metaTextDetail:"This {{tag-name}} has an id attribute of '{{id}}', which is not unique in the same DOM",fixType:"",testFunc:function(){const testShell=new AutoTestShell();const relevantCssSelector='*[id]:not([id=""])';testShell.set_applicabilityTest(function(){return this.applicableNodesFunc(this._applicableNodesFuncArgs).length>0;});testShell.set_applicableNodesFunc(function(){// if there is more than 1 element with an ID then the test is applicable
const applicableNodes=this.querySelectorAll(relevantCssSelector);return applicableNodes.length>1?applicableNodes:[];});testShell.set_candidateSetNodesFunc(function(){var notUnique=[];var nodeStore={};var res=this.querySelectorAll(relevantCssSelector);for(var i=0;i<res.length;i++){var n=res[i];var id=n.getAttribute("id");try{nodeStore[id].push(n);}catch(err){nodeStore[id]=[];nodeStore[id].push(n);}}var keys=Object.keys(nodeStore);for(var j=0;j<keys.length;j++){var nArray=nodeStore[keys[j]];//If any of nArrays elements has more than one member then those members share a duplicate ID
if(nArray.length>1){//If we have shadowDOM on the page also check the UEL fingerprints for each collection of nodes that share an ID to see if they are in the same context
if(detectShadowDOM===true){var fingerprints={};for(var k=0;k<nArray.length;k++){var o=nArray[k];var f=o.getAttribute("data-ae_domuel");//Reduce UEL to parent of shadowDOM
f=f.substr(0,f.lastIndexOf("|"));try{fingerprints[f].push(o);}catch(err){fingerprints[f]=[];fingerprints[f].push(o);}}//fingerprints is now an array of arrays with nodes stored under their fingerprints
var fkeys=Object.keys(fingerprints);for(var l=0;l<fkeys.length;l++){var fArray=fingerprints[fkeys[l]];if(fArray.length>1){notUnique=notUnique.concat(fArray);}}}//No shadowDOM so shared IDs are always bad
else{notUnique=notUnique.concat(nArray);}}}return notUnique;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-95
	 * @returns {AutoTestShell} testShell
	 */function test95(){aeKernel.addAutomaticTest("95",{bestPractice:1626,testId:95,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM and available to assistive technologies, has an aria-labelledby attribute value that includes one or more invalid ids.",metaText:"This element has an aria-labelledby attribute value that includes one or more invalid or duplicate ids",metaTextDetail:"This {{tag-name}} has an aria-labelledby attribute value of '{{aria-labelledby}}', which includes one or more invalid or duplicate ids",fixType:{"fixType":1,"domSpec":false,"helperText":"Provide the ID of an element which contains the label content for this element.","fix":{"aria-labelledby":null}},testFunc:function(){var testShell=new AutoTestShell('*[data-ae_vis][data-ae_avat][aria-labelledby]:not([aria-labelledby=""])');testShell.set_candidateSetNodesFunc(function(){var nodesWithBadIds=[];var res=aeKernel.rootTestNode.querySelectorAll('*[data-ae_vis][data-ae_avat][aria-labelledby]:not([aria-labelledby=""])');for(var i=0,len_res=res.length;i<len_res;i++){var n=res[i];var labelledByIds=n.getAttribute("aria-labelledby").trim().split(" ");for(var j=0,len_labelledByIds=labelledByIds.length;j<len_labelledByIds;j++){var labelledById=labelledByIds[j];if(isValidIdOrName(labelledById)===true){// "[id=" is used in place of "#" + id as ids can included colons which breaks css selector, although is valid html
var refNode=aeKernel.rootTestNode.querySelectorAll("[id='"+labelledById.replace(/'/g,"\\'")+"']");var refNodeLength=refNode.length;if(refNodeLength===0||refNodeLength>1){nodesWithBadIds.push(n);break;}}else{nodesWithBadIds.push(n);break;}}}return nodesWithBadIds;});return testShell;}});}/**
	 * Finds the set of elements that is in a large group defined by a CSS selector, but not in a smaller group also defined by a CSS selector.
	 * @param {Array} funcArgs
	 * @returns {Array}
	 */function testUtil_ifDiff(rootTestNode,funcArgs){var diff=[];var res1=rootTestNode.querySelectorAll(funcArgs[0]);var res2=rootTestNode.querySelectorAll(funcArgs[1]);//need to optimise following - is there a way to cast to an array?
var res2AsArray=[];for(var x=0,len_res2=res2.length;x<len_res2;x++){res2AsArray[res2AsArray.length]=res2[x];}for(var i=0,len_res1=res1.length;i<len_res1;i++){var res=res1[i];if(res2AsArray.indexOf(res)===-1){diff[diff.length]=res;}}return diff;}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-99
	 * @returns {AutoTestShell} testShell
	 */function test99(){aeKernel.addAutomaticTest("99",{bestPractice:464,testId:99,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more li elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a ul element (without an ARIA-assigned role); ol element (without an ARIA-assigned role); or element with a role set to the value 'list' as a parent.",metaText:"This element does not have a ul element (without an ARIA-assigned role); ol element (without an ARIA-assigned role); or element with a role set to the value: list as a parent",metaTextDetail:"This {{tag-name}} does not have a ul element (without an ARIA-assigned role); ol element (without an ARIA-assigned role); or element with a role set to the value: list as a parent",fixType:"",testFunc:function(){var testShell=new AutoTestShell('li[data-ae_ar="null"][data-ae_vis][data-ae_avat]');testShell.set_candidateSetNodesFunc(function(){return testUtil_ifDiff(aeKernel.rootTestNode,['li[data-ae_ar="null"][data-ae_vis][data-ae_avat]','ol[data-ae_ar="null"][data-ae_vis][data-ae_avat] > li[data-ae_ar="null"][data-ae_vis][data-ae_avat], ul[data-ae_ar="null"][data-ae_vis][data-ae_avat] > li[data-ae_ar="null"][data-ae_vis][data-ae_avat], *[data-ae_vis][data-ae_avat][data-ae_ar="list"] > li[data-ae_ar="null"][data-ae_vis][data-ae_avat]']);});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-101
	 * @returns {AutoTestShell} testShell
	 */function test101(){aeKernel.addAutomaticTest("101",{bestPractice:429,testId:101,introduced:"0.9",mediaType:mediaTypes.WEB,description:"The root html element does not have a lang attribute",metaText:"This element does not have a lang attribute",metaTextDetail:"This {{tag-name}} does not have a lang attribute",fixType:{"fixType":4,"domSpec":false,"helperText":"Enter the appropriate language value for the page.","fix":{"lang":null}},testFunc:function(){// head is used as rootTestNode is html - so cannot be reached by querySelector
// head is a good indicator that an entire page has been selected - with regard to nodeCapture
var testShell=new AutoTestShell('head');testShell.set_candidateSetNodesFunc(function(){var notProper=[];if(aeKernel.rootTestNode.hasAttribute("lang")===false){notProper.push(aeKernel.rootTestNode);}return notProper;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-113
	 * @returns {AutoTestShell} testShell
	 */function test113(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("113",{bestPractice:338,testId:113,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more textarea elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This textarea element does not have a mechanism that allows an accessible name value to be calculated",fixType:{"fixType":1,"domSpec":false,"helperText":"Specify a descriptive label for the textarea.","fix":{"aria-label":null,"aria-labelledby":false}},testFunc:function(){return new AutoTestShell('textarea[data-ae_ar="null"][data-ae_vis][data-ae_avat]','textarea[data-ae_ar="null"][data-ae_vis][data-ae_avat]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-114
	 * @returns {AutoTestShell} testShell
	 */function test114(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("114",{bestPractice:338,testId:114,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more select elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This select element does not have a mechanism that allows an accessible name value to be calculated",fixType:{"fixType":1,"domSpec":false,"helperText":"Specify a descriptive label for the select.","fix":{"aria-label":null,"aria-labelledby":false}},testFunc:function(){return new AutoTestShell('select[data-ae_ar="null"][data-ae_vis][data-ae_avat]','select[data-ae_ar="null"][data-ae_vis][data-ae_avat]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-116
	 * @returns {AutoTestShell} testShell
	 */function test116(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("116",{bestPractice:338,testId:116,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more input type='button' elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This input (type=button) element does not have a mechanism that allows an accessible name value to be calculated",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="button"]','input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="button"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-118
	 * @returns {AutoTestShell} testShell
	 */function test118(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("118",{bestPractice:338,testId:118,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more input type='text' elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This input (type=text) element does not have a mechanism that allows an accessible name value to be calculated",fixType:{"fixType":1,"domSpec":false,"helperTex":"Specify a descriptive label for the text input.","fix":{"aria-label":null,"aria-labelledby":false}},testFunc:function(){return new AutoTestShell('input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="text"]','input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="text"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-123
	 * @returns {AutoTestShell} testShell
	 */function test123(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("123",{bestPractice:362,testId:123,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more svg elements, excluding children of button elements and children of anchor elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This svg element does not have a mechanism that allows an accessible name value to be calculated",fixType:{"fixType":1,"domSpec":false,"helperText":"Provide text that describes the content conveyed by the SVG.","fix":{"alt":false,"aria-label":null,"aria-labelledby":false}},testFunc:function(){const relevantCssSelector='svg:not(button svg):not(a svg):not([role="button"] svg):not([role="link"] svg)[data-ae_ar="null"][data-ae_vis][data-ae_avat]';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}:not([data-ae_an])`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-131
	 * @returns {AutoTestShell} testShell
	 */function test131(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("131",{bestPractice:373,testId:131,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more input type='image' elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This input (type=image) element does not have a mechanism that allows an accessible name value to be calculated",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="image"]','input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="image"][data-ae_anua], input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="image"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-132
	 * @returns {AutoTestShell} testShell
	 */function test132(){aeKernel.addAutomaticTest("132",{bestPractice:1626,testId:132,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='slider' attribute, not intentionally hidden in the DOM and available to assistive technologies, does not have an aria-valuemax attribute, an aria-valuemin attribute and an aria-valuenow attribute.",metaText:"This element (role=slider) must have all of the following attributes: aria-valuemax | aria-valuemin | aria-valuenow",metaTextDetail:"This {{tag-name}} (role=slider) must have all of the following attributes: aria-valuemax | aria-valuemin | aria-valuenow",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="slider"]','*[data-ae_vis][data-ae_avat][data-ae_ar="slider"]:not([aria-valuemax]), *[data-ae_vis][data-ae_avat][data-ae_ar="slider"]:not([aria-valuemin]), *[data-ae_vis][data-ae_avat][data-ae_ar="slider"]:not([aria-valuenow])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-134
	 * @returns {AutoTestShell} testShell
	 */function test134(){aeKernel.addAutomaticTest("134",{bestPractice:1626,testId:134,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='combobox' attribute, not intentionally hidden in the DOM and available to assistive technologies, does not have an aria-expanded attribute set to one of the text values: 'true | false'.",metaText:"This element's (role=combobox) aria-expanded attribute should be set to one of the text values: true | false",metaTextDetail:"This {{tag-name}}'s (role=combobox) aria-expanded attribute should be set to one of the text values: true | false",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="combobox"]','*[data-ae_vis][data-ae_avat][data-ae_ar="combobox"][aria-expanded]:not([aria-expanded="true"]):not([aria-expanded="false"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-140
	 * @returns {AutoTestShell} testShell
	 */function test140(){aeKernel.addAutomaticTest("140",{bestPractice:1626,testId:140,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='checkbox' attribute, not intentionally hidden in the DOM and available to assistive technologies, does not have an aria-checked attribute set to one of the text values: 'true | false | mixed'.",metaText:"This element's (role=checkbox) aria-checked attribute should be set to one of the text values: true | false | mixed",metaTextDetail:"This {{tag-name}}'s (role=checkbox) aria-checked attribute should be set to one of the text values: true | false | mixed",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="checkbox"]','*[data-ae_vis][data-ae_avat][data-ae_ar="checkbox"][aria-checked]:not([aria-checked="true"]):not([aria-checked="false"]):not([aria-checked="mixed"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-141
	 * @returns {AutoTestShell} testShell
	 */function test141(){aeKernel.addAutomaticTest("141",{bestPractice:401,testId:141,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more A elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has an href attribute set to an image file reference.",metaText:"This A element has an href attribute set to an image file reference",fixType:"",testFunc:function(){//Checks for various known image file endings.
//Will ignore href with url parameters so that href="viewer.html?foo=bob.png" will not be flagged
//Will not flag href="image.png?foo=bar" but we've not seen that in the wild
return new AutoTestShell("a[data-ae_ar='null'][data-ae_vis][data-ae_avat][href]:not([href=''])","a[data-ae_ar='null'][data-ae_vis][data-ae_avat][href$='.gif']:not([href*='?']):not([href*='.htm']), a[data-ae_ar='null'][data-ae_vis][data-ae_avat][href$='.GIF']:not([href*='?']):not([href*='.htm']), a[data-ae_ar='null'][data-ae_vis][data-ae_avat][href$='.jpg']:not([href*='?']):not([href*='.htm']), a[data-ae_ar='null'][data-ae_vis][data-ae_avat][href$='.JPG']:not([href*='?']):not([href*='.htm']), a[data-ae_ar='null'][data-ae_vis][data-ae_avat][href$='.jpeg']:not([href*='?']):not([href*='.htm']), a[data-ae_ar='null'][data-ae_vis][data-ae_avat][href$='.JPEG']:not([href*='?']):not([href*='.htm']), a[data-ae_ar='null'][data-ae_vis][data-ae_avat][href$='.bmp']:not([href*='?']):not([href*='.htm']), a[data-ae_ar='null'][data-ae_vis][data-ae_avat][href$='.BMP']:not([href*='?']):not([href*='.htm']), a[data-ae_ar='null'][data-ae_vis][data-ae_avat][href$='.png']:not([href*='?']):not([href*='.htm']), a[data-ae_ar='null'][data-ae_vis][data-ae_avat][href$='.PNG']:not([href*='?']):not([href*='.htm']), a[data-ae_ar='null'][data-ae_vis][data-ae_avat][href$='.tiff']:not([href*='?']):not([href*='.htm']), a[data-ae_ar='null'][data-ae_vis][data-ae_avat][href$='.TIFF']:not([href*='?']):not([href*='.htm']), a[data-ae_ar='null'][data-ae_vis][data-ae_avat][href$='.eps']:not([href*='?']):not([href*='.htm']), a[data-ae_ar='null'][data-ae_vis][data-ae_avat][href$='.EPS']:not([href*='?']):not([href*='.htm']), a[data-ae_ar='null'][data-ae_vis][data-ae_avat][href$='.pcx']:not([href*='?']):not([href*='.htm']), a[data-ae_ar='null'][data-ae_vis][data-ae_avat][href$='.PCX']:not([href*='?']):not([href*='.htm'])");}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-152
	 * @returns {AutoTestShell} testShell
	 */function test152(){aeKernel.addAutomaticTest("152",{bestPractice:364,testId:152,introduced:"0.9",mediaType:mediaTypes.WEB,description:"The first document title element located was found to be empty.",metaText:"This title element should not be empty",fixType:{"fixType":3,"domSpec":true,"helperText":"Specify a unique and descriptive title for the page.","fix":{"innerHTML":null}},testFunc:function(){const relevantCssSelector='html title:not(svg title):not([data-ae_inshdw]), head title:not(svg title):not([data-ae_inshdw]), body title:not(svg title):not([data-ae_inshdw])';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const firstTitle=this.querySelector(relevantCssSelector);if(firstTitle.childNodes.length<=0){badNodes.push(firstTitle);}return badNodes;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-165
	 * @returns {AutoTestShell} testShell
	 */function test165(){aeKernel.addAutomaticTest("165",{bestPractice:1626,testId:165,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='color' elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute.",metaText:"The role attribute is not allowed on this element",metaTextDetail:"The role attribute is not allowed on this {{tag-name}} with type=color",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_vis][data-ae_avat][type="color"]','input[data-ae_vis][data-ae_avat][type="color"]:not([data-ae_ar="null"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-166
	 * @returns {AutoTestShell} testShell
	 */function test166(){aeKernel.addAutomaticTest("166",{bestPractice:1626,testId:166,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='date' elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute.",metaText:"The role attribute is not allowed on this element",metaTextDetail:"The role attribute is not allowed on this {{tag-name}} with type=date",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_vis][data-ae_avat][type="date"]','input[data-ae_vis][data-ae_avat][type="date"]:not([data-ae_ar="null"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-167
	 * @returns {AutoTestShell} testShell
	 */function test167(){aeKernel.addAutomaticTest("167",{bestPractice:1626,testId:167,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='datetime-local' elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute.",metaText:"The role attribute is not allowed on this element",metaTextDetail:"The role attribute is not allowed on this {{tag-name}} with type=datetime-local",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_vis][data-ae_avat][type="datetime-local"]','input[data-ae_vis][data-ae_avat][type="datetime-local"]:not([data-ae_ar="null"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-168
	 * @returns {AutoTestShell} testShell
	 */function test168(){aeKernel.addAutomaticTest("168",{bestPractice:1626,testId:168,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='file' elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute.",metaText:"The role attribute is not allowed on this element",metaTextDetail:"The role attribute is not allowed on this {{tag-name}} with type=file",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_vis][data-ae_avat][type="file"]','input[data-ae_vis][data-ae_avat][type="file"]:not([data-ae_ar="null"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-169
	 * @returns {AutoTestShell} testShell
	 */function test169(){aeKernel.addAutomaticTest("169",{bestPractice:1626,testId:169,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='image' elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'button...'.",metaText:"This element has a redundant role attribute",metaTextDetail:"This {{tag-name}} with type=image has a redundant role attribute of {{data-ae_ar}}",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_vis][data-ae_avat][type="image"]','input[data-ae_vis][data-ae_avat][type="image"][data-ae_ar="button"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-170
	 * @returns {AutoTestShell} testShell
	 */function test170(){aeKernel.addAutomaticTest("170",{bestPractice:1626,testId:170,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='month' elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute.",metaText:"The role attribute is not allowed on this element",metaTextDetail:"The role attribute is not allowed on this {{tag-name}} with type=month",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_vis][data-ae_avat][type="month"]','input[data-ae_vis][data-ae_avat][type="month"]:not([data-ae_ar="null"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-171
	 * @returns {AutoTestShell} testShell
	 */function test171(){aeKernel.addAutomaticTest("171",{bestPractice:1626,testId:171,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='radio' elements, excluding those with a role='radio...' attribute, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to a value that is not: 'menuitemradio...'.",metaText:"This element has a role attribute that is not set to the following text value:  menuitemradio",metaTextDetail:"This {{tag-name}} with type=radio has a role attribute that is not set to the following text value:  menuitemradio",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_vis][data-ae_avat][type="radio"]:not([data-ae_ar="null"]):not([data-ae_ar="radio"])','input[data-ae_vis][data-ae_avat][type="radio"]:not([data-ae_ar="null"]):not([data-ae_ar="radio"])[role]:not([data-ae_ar="menuitemradio"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-173
	 * @returns {AutoTestShell} testShell
	 */function test173(){aeKernel.addAutomaticTest("173",{bestPractice:1626,testId:173,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='time' elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute.",metaText:"The role attribute is not allowed on this element",metaTextDetail:"The role attribute is not allowed on this {{tag-name}} with type=time",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_vis][data-ae_avat][type="time"]','input[data-ae_vis][data-ae_avat][type="time"]:not([data-ae_ar="null"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-174
	 * @returns {AutoTestShell} testShell
	 */function test174(){aeKernel.addAutomaticTest("174",{bestPractice:1626,testId:174,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='week' elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute.",metaText:"The role attribute is not allowed on this element",metaTextDetail:"The role attribute is not allowed on this {{tag-name}} with role=week",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_vis][data-ae_avat][type="week"]','input[data-ae_vis][data-ae_avat][type="week"]:not([data-ae_ar="null"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-175
	 * @returns {AutoTestShell} testShell
	 */function test175(){aeKernel.addAutomaticTest("175",{bestPractice:1626,testId:175,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more keygen elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute.",metaText:"The role attribute is not allowed on this element",metaTextDetail:"The role attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){return new AutoTestShell('keygen[data-ae_vis][data-ae_avat]','keygen[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-176
	 * @returns {AutoTestShell} testShell
	 */function test176(){aeKernel.addAutomaticTest("176",{bestPractice:1626,testId:176,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more label elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute.",metaText:"The role attribute is not allowed on this element",metaTextDetail:"The role attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){return new AutoTestShell('label[data-ae_vis][data-ae_avat]','label[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-180
	 * @returns {AutoTestShell} testShell
	 */function test180(){aeKernel.addAutomaticTest("180",{bestPractice:1626,testId:180,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more math elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute.",metaText:"The role attribute is not allowed on this math element",fixType:"",testFunc:function(){return new AutoTestShell('math[data-ae_vis][data-ae_avat]','math[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-182
	 * @returns {AutoTestShell} testShell
	 */function test182(){aeKernel.addAutomaticTest("182",{bestPractice:1626,testId:182,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more meta elements, in the head element in the DOM, has a role attribute and/or an ARIA attribute.",metaText:"This element should not have a role attribute and/or an ARIA attribute",metaTextDetail:"This {{tag-name}} should not have a role attribute and/or an ARIA attribute",fixType:"",testFunc:function(){const testShell=new AutoTestShell('head meta');testShell.set_candidateSetNodesFunc(function(){var notProper=[];var addIfFailed=function(meta){if(meta.hasAttribute("role")){notProper.push(meta);return 1;}var meta_attributes=meta.attributes;for(var i=meta_attributes.length;i--;){if(meta_attributes[i].name.substring(0,4).toLowerCase()==="aria"){notProper.push(meta);return 1;}}return 0;};var metaNodes=aeKernel.rootTestNode.querySelectorAll('meta');for(var x=metaNodes.length;x--;){addIfFailed(metaNodes[x]);}return notProper;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-185
	 * @returns {AutoTestShell} testShell
	 */function test185(){aeKernel.addAutomaticTest("185",{bestPractice:1626,testId:185,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more noscript elements, available to assistive technologies, has a role attribute.",metaText:"The role attribute is not allowed on this element",metaTextDetail:"The role attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){return new AutoTestShell('noscript[data-ae_avat]','noscript[data-ae_avat]:not([data-ae_ar="null"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-188
	 * @returns {AutoTestShell} testShell
	 */function test188(){aeKernel.addAutomaticTest("188",{bestPractice:1626,testId:188,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more param elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute.",metaText:"The role attribute is not allowed on this element",metaTextDetail:"The role attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){return new AutoTestShell('param[data-ae_vis][data-ae_avat]','param[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-191
	 * @returns {AutoTestShell} testShell
	 */function test191(){aeKernel.addAutomaticTest("191",{bestPractice:1626,testId:191,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more source elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute.",metaText:"The role attribute is not allowed on this element",metaTextDetail:"The role attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){return new AutoTestShell('source[data-ae_vis][data-ae_avat]','source[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-192
	 * @returns {AutoTestShell} testShell
	 */function test192(){aeKernel.addAutomaticTest("192",{bestPractice:1626,testId:192,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more style elements has a role attribute.",metaText:"The role attribute is not allowed on this element",metaTextDetail:"The role attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){return new AutoTestShell('style','style:not([data-ae_ar="null"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-199
	 * @returns {AutoTestShell} testShell
	 */function test199(){aeKernel.addAutomaticTest("199",{bestPractice:1626,testId:199,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more document title elements has a role attribute.",metaText:"The role attribute is not allowed on this element",metaTextDetail:"The role attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){const relevantCssSelector='title:not([data-ae_inshdw])';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const titles=aeKernel.rootTestNode.querySelectorAll(`${relevantCssSelector}:not([data-ae_ar="null"])`);for(let i=0;i<titles.length;i++){const title=titles[i];// title elements that are descendants of SVGs aren't document title elements
const svgAncestor=title.closest('svg');if(!svgAncestor){badNodes.push(title);}}return badNodes;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-203
	 * @returns {AutoTestShell} testShell
	 */function test203(){aeKernel.addAutomaticTest("203",{bestPractice:1626,testId:203,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more track elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute.",metaText:"The role attribute is not allowed on this element",metaTextDetail:"The role attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){return new AutoTestShell('track[data-ae_vis][data-ae_avat]','track[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-205
	 * @returns {AutoTestShell} testShell
	 */function test205(){aeKernel.addAutomaticTest("205",{bestPractice:1626,testId:205,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements with a disabled attribute, not intentionally hidden in the DOM and available to assistive technologies, has an aria-disabled attribute.",metaText:"This element does not need an aria-disabled attribute, as it has a disabled attribute",metaTextDetail:"This {{tag-name}} does not need an aria-disabled attribute, as it has a disabled attribute",fixType:"",testFunc:function(){return new AutoTestShell('body *[data-ae_vis][data-ae_avat]','body *[data-ae_vis][data-ae_avat][disabled][aria-disabled]');}});}/*
	 * Pre-Testing: Finds tables that appear to be data tables and adds an "data-ae_dtab" attribute
	 * @returns {Number}
	 */function identifiesDataTables(rootTestNode){try{const potentialTables=rootTestNode.querySelectorAll("table");for(let i=0,len_potTables=potentialTables.length;i<len_potTables;i++){if(isRunningTooLong()){throw getRuntimeExceededMessage();}const potentialTable=potentialTables[i];const cells=potentialTable.querySelectorAll("td");const numberOfCells=cells.length;let numberOfCellsWhichContainOnlyText=0;const cell=document.createElement("td");for(let j=numberOfCells;j--;){//Strip inline elements that might be wrapping text so they don't throw off nodeType/count checks
//Copy the cell so stripping doesn't mangle the original decorated DOM
cell.innerHTML=cells[j].innerHTML;//Grab any tags to be stripped - currently common inline elements
const thingsToStrip=cell.querySelectorAll("b,em,i,span,strong,br,a,bdi,label");//Rip each tag out but leave its innerHTML in place
for(let k=thingsToStrip.length;k--;){thingsToStrip[k].outerHTML=thingsToStrip[k].innerHTML;}//Firefox keeps adjacent text nodes as separate children so this fixes that
cell.normalize();if(cell.childNodes.length===1&&cell.childNodes[0].nodeType===3){// cell contains only text
numberOfCellsWhichContainOnlyText=numberOfCellsWhichContainOnlyText+1;}}const percTextOnlyCells=Math.round(numberOfCellsWhichContainOnlyText/numberOfCells*100);if(percTextOnlyCells>65){potentialTable.setAttribute("data-ae_dtab","true");}}return rootTestNode;}catch(err){logger.error(err);return null;}}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-214
	 * @returns {AutoTestShell} testShell
	 */function test214(){aeKernel.addStage4MarkdownFunc("identifiesDataTables",identifiesDataTables);aeKernel.addAutomaticTest("214",{bestPractice:590,testId:214,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more th elements in a data-table, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, contains at least one table element (without an ARIA-assigned role).",metaText:"Data table header cells should not contain a table element (without an ARIA-assigned role)",fixType:"",testFunc:function(){var testShell=new AutoTestShell('table[data-ae_vis][data-ae_avat][data-ae_ar="null"][data-ae_dtab="true"] th[data-ae_vis][data-ae_avat][data-ae_ar="null"]');testShell.set_candidateSetNodesFunc(function(){// clone body
// var clone = aeKernel.rootTestNode.querySelector("body").cloneNode(true);
const testNode=aeKernel.rootTestNode.querySelector("body")||aeKernel.rootTestNode;// fall back to using root node if body element is not a child of root node
var clone=cloneThisNode(testNode,true);// so we don't pick up th elements within the nested tables
var tableInTH=clone.querySelector("th[data-ae_vis][data-ae_avat][data-ae_ar='null'] table[data-ae_vis][data-ae_avat][data-ae_ar='null']:not(:empty)");while(tableInTH){tableInTH.innerHTML="";tableInTH=clone.querySelector("th[data-ae_vis][data-ae_avat][data-ae_ar='null'] table[data-ae_vis][data-ae_avat][data-ae_ar='null']:not(:empty)");}var thElementsThatContainTables=[];var res=clone.querySelectorAll('table[data-ae_vis][data-ae_avat][data-ae_ar="null"][data-ae_dtab="true"] th[data-ae_vis][data-ae_avat][data-ae_ar="null"]');for(var i=0,len_res=res.length;i<len_res;i++){var thOfInterest=res[i];var thContainsTables=thOfInterest.querySelectorAll('table[data-ae_vis][data-ae_avat][data-ae_ar="null"]');if(thContainsTables.length>0){thElementsThatContainTables.push(thOfInterest);}}return thElementsThatContainTables;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-221
	 * @returns {AutoTestShell} testShell
	 */function test221(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("221",{bestPractice:332,testId:221,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='radiogroup' attribute, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name to be calculated.",metaText:"This element (role=radiogroup) does not have a mechanism that allows an accessible name value to be calculated",metaTextDetail:"This {{tag-name}} (role=radiogroup) does not have a mechanism that allows an accessible name value to be calculated",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="radiogroup"]','*[data-ae_vis][data-ae_avat][data-ae_ar="radiogroup"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-224
	 * @returns {AutoTestShell} testShell
	 */function test224(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("224",{bestPractice:338,testId:224,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more button elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This button element does not have a mechanism that allows an accessible name value to be calculated",fixType:"",testFunc:function(){return new AutoTestShell('button[data-ae_vis][data-ae_avat][data-ae_ar="null"]','button[data-ae_vis][data-ae_avat][data-ae_ar="null"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-226
	 * @returns {AutoTestShell} testShell
	 */function test226(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("226",{bestPractice:338,testId:226,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more optgroup elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This optgroup element does not have a mechanism that allows an accessible name value to be calculated",fixType:"",testFunc:function(){return new AutoTestShell('optgroup[data-ae_vis][data-ae_avat][data-ae_ar="null"]','optgroup[data-ae_vis][data-ae_avat][data-ae_ar="null"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-226
	 * @returns {AutoTestShell} testShell
	 */function test227(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("227",{bestPractice:523,testId:227,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more frame elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This frame element does not have a mechanism that allows an accessible name value to be calculated",fixType:{"fixType":1,"domSpec":false,"helperText":"Specify a descriptive label for the frame.","fix":{"aria-label":null,"aria-labelledby":false}},testFunc:function(){return new AutoTestShell('frame[data-ae_vis][data-ae_avat][data-ae_ar="null"]','frame[data-ae_vis][data-ae_avat][data-ae_ar="null"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-228
	 * @returns {AutoTestShell} testShell
	 */function test228(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("228",{bestPractice:523,testId:228,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more iframe elements, excluding those elements with aria-hidden='true' or tabindex set to a negative value, not intentionally hidden in the DOM, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This iframe element does not have a mechanism that allows an accessible name value to be calculated",fixType:{"fixType":1,"domSpec":false,"helperText":"Specify a descriptive label for the iframe.","fix":{"aria-label":null,"aria-labelledby":false}},testFunc:function(){const relevantCssSelector='iframe:not([aria-hidden="true"]):not([tabindex^="-"])[data-ae_vis]';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}:not([data-ae_an])`);}});}/**
	 * To be superceded by Machine Learning
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-229
	 * @returns {AutoTestShell} testShell
	 */function test229(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("229",{bestPractice:523,testId:229,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more frame elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious calculated accessible name value.",metaText:"This frame element has a suspicious calculated accessible name value",metaTextDetail:"This frame element has a suspicious calculated accessible name value of: {{data-ae_an}}",fixType:{"fixType":1,"domSpec":false,"helperText":"Specify a descriptive label for the frame.","fix":{"aria-label":null,"aria-labelledby":false}},testFunc:function(){return new AutoTestShell('frame[data-ae_vis][data-ae_avat][data-ae_ar="null"][data-ae_an]','frame[data-ae_vis][data-ae_avat][data-ae_ar="null"][data-ae_an="title"], frame[data-ae_vis][data-ae_avat][data-ae_ar="null"][data-ae_an="frame"], frame[data-ae_vis][data-ae_avat][data-ae_ar="null"][data-ae_an="top"], frame[data-ae_vis][data-ae_avat][data-ae_ar="null"][data-ae_an="bottom"], frame[data-ae_vis][data-ae_avat][data-ae_ar="null"][data-ae_an="left"], frame[data-ae_vis][data-ae_avat][data-ae_ar="null"][data-ae_an="right"]');}});}/**
	 * NOTE: area element is available to screen-reader users and keyboard users even if it is "display:none", so the requirement on the area element in this test to be "not intentionally hidden in the DOM" is dropped.
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-230
	 * @returns {AutoTestShell} testShell
	 */function test230(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("230",{bestPractice:343,testId:230,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more area elements, without an ARIA-assigned role and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This area element does not have a mechanism that allows an accessible name value to be calculated",fixType:"",testFunc:function(){return new AutoTestShell('area[data-ae_avat][data-ae_ar="null"]','area[data-ae_avat][data-ae_ar="null"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-231
	 * @returns {AutoTestShell} testShell
	 */function test231(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("231",{bestPractice:2554,testId:231,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more progress elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This progress element does not have a mechanism that allows an accessible name value to be calculated",fixType:"",testFunc:function(){return new AutoTestShell('progress[data-ae_vis][data-ae_avat][data-ae_ar="null"]','progress[data-ae_vis][data-ae_avat][data-ae_ar="null"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-232
	 * @returns {AutoTestShell} testShell
	 */function test232(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("232",{bestPractice:2554,testId:232,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more meter elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This meter element does not have a mechanism that allows an accessible name value to be calculated",fixType:"",testFunc:function(){return new AutoTestShell('meter[data-ae_vis][data-ae_avat][data-ae_ar="null"]','meter[data-ae_vis][data-ae_avat][data-ae_ar="null"]:not([data-ae_an])');}});}/**
	 * This test will be superseded by a Machine Learning test
	 * text that contains the substring ".gif", ".jpg", ".png";
	 * text that is "img", "spacer", "alt", "blank", "_", "null", "photo", "image", "corner", "tag", "picture", "header", "histogram", "chart", or "undefined"
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-234
	 * @returns {AutoTestShell} testShell
	 */function test234(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("234",{bestPractice:370,testId:234,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more img elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious calculated accessible name value.",metaText:"This element has a suspicious calculated accessible name value",metaTextDetail:"This {{tag-name}} has a suspicious calculated accessible name value of: {{data-ae_an}}",fixType:{"fixType":1,"domSpec":false,"helperText":"Provide alternative text that describes the content conveyed by the image.","fix":{"alt":null,"aria-label":false,"aria-labelledby":false}},testFunc:function(){const baseCssSelector="img[data-ae_ar='null'][data-ae_vis][data-ae_avat]";const relevantCssSelector=`${baseCssSelector}[data-ae_an]`;let testCssSelector="";for(const postfix of IMG_SUSPICIOUS_TEXT_CSS_SELECTOR_POSTFIXES){testCssSelector+=`${baseCssSelector}${postfix},`;}testCssSelector=testCssSelector.substring(0,testCssSelector.length-1);return new AutoTestShell(relevantCssSelector,testCssSelector);}});}/**
	 * text that contains the substring ".gif", ".jpg", ".png";
	 * text that is "img", "spacer", "alt", "blank", "_", "null", "photo", "image", "corner", "tag", "picture", "header", "histogram", "chart", "undefined".
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-235
	 * @returns {AutoTestShell} testShell
	 */function test235(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("235",{bestPractice:370,testId:235,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements with a role attribute set to a value that starts with 'img' or 'graphics', not intentionally hidden in the DOM and available to assistive technologies, has a suspicious calculated accessible name value.",metaText:"This element (role=img, role=graphics-symbol or role=graphics-document) has a suspicious calculated accessible name value",metaTextDetail:"This {{tag-name}} (role={{data-ae_ar}}) has a suspicious calculated accessible name value of: {{data-ae_an}}",fixType:{"fixType":1,"domSpec":false,"helperText":'Provide alternative text that describes the content conveyed by the image.',"fix":{"aria-label":null,"aria-labelledby":false}},testFunc:function(){const prefixes=['*[data-ae_ar^="img"][data-ae_vis][data-ae_avat]','*[data-ae_ar^="graphics"][data-ae_vis][data-ae_avat]'];let relevantCssSelector="";let testCssSelector="";for(const prefix of prefixes){relevantCssSelector+=`${prefix}[data-ae_an],`;for(const postfix of IMG_SUSPICIOUS_TEXT_CSS_SELECTOR_POSTFIXES){testCssSelector+=`${prefix}${postfix},`;}}relevantCssSelector=relevantCssSelector.substring(0,relevantCssSelector.length-1);testCssSelector=testCssSelector.substring(0,testCssSelector.length-1);return new AutoTestShell(relevantCssSelector,testCssSelector);}});}/**
	 * text that contains the substring ".gif", ".jpg", ".png";
	 * text that is "img", "spacer", "alt", "blank", "_", "null", "photo", "image", "corner", "tag", "picture", "header", "histogram", "chart", "undefined".
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-236
	 * @returns {AutoTestShell} testShell
	 */function test236(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("236",{bestPractice:370,testId:236,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more svg elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious calculated accessible name value.",metaText:"This svg element has a suspicious calculated accessible name value",metaTextDetail:"This svg element has a suspicious calculated accessible name value of: {{data-ae_an}}",fixType:{"fixType":1,"domSpec":false,"helperText":"Provide text that describes the content conveyed by the SVG.","fix":{"alt":false,"aria-label":null,"aria-labelledby":false}},testFunc:function(){const baseCssSelector="svg[data-ae_ar='null'][data-ae_vis][data-ae_avat]";const relevantCssSelector=`${baseCssSelector}[data-ae_an]`;let testCssSelector="";for(const postfix of IMG_SUSPICIOUS_TEXT_CSS_SELECTOR_POSTFIXES){testCssSelector+=`${baseCssSelector}${postfix},`;}testCssSelector=testCssSelector.substring(0,testCssSelector.length-1);return new AutoTestShell(relevantCssSelector,testCssSelector);}});}/*
	 * Pre-Testing: Finds elements that have events added in code behind and adds an "data-ae_ev" attribute
	 * @returns {Number}
	 */function addAEEventAtts(rootTestNode){try{var els=rootTestNode.querySelectorAll("*[data-ae_styles*='events;'], *[onclick]:not([onclick='']), *[ondblclick]:not([ondblclick='']), *[onmouseout]:not([onmouseout='']), *[onmouseover]:not([onmouseover='']), *[onmousedown]:not([onmousedown='']), *[onmouseup]:not([onmouseup='']), *[onkeydown]:not([onkeydown='']), *[onkeypress]:not([onkeypress='']), *[onkeyup]:not([onkeyup='']), *[onblur]:not([onblur='']), *[onfocus]:not([onfocus=''])");for(var i=els.length;i--;){if(isRunningTooLong()){throw getRuntimeExceededMessage();}var el=els[i];var eventList=[];var originalInformation=retrieveComputedCssStyles(el);var events=originalInformation["events"];if(events){eventList=eventList.concat(events.split(','));}var intrinsicEvents=["onclick","ondblclick","onmouseout","onmouseover","onmousedown","onmouseup","onkeydown","onkeypress","onkeyup","onblur","onfocus"];for(var k=intrinsicEvents.length;k--;){var intEv=intrinsicEvents[k];var evntVal=el.getAttribute(intEv);if(evntVal){var shortIntEv=intEv.substr(2);if(eventList.indexOf(shortIntEv)===-1){eventList.push(shortIntEv);}}}el.setAttribute("data-ae_ev",eventList.toString());}return rootTestNode;}catch(err){logger.error(err);return null;}}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-237
	 * @returns {AutoTestShell} testShell
	 */function test237(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addStage4MarkdownFunc("addAEEventAtts",addAEEventAtts);aeKernel.addAutomaticTest("237",{bestPractice:1301,testId:237,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more A elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This A element does not have a mechanism that allows an accessible name value to be calculated",fixType:{"fixType":1,"domSpec":false,"helperText":"Provide a descriptive accessible name for the anchor element.","fix":{"aria-label":null}},testFunc:function(){return new AutoTestShell('a[data-ae_vis][data-ae_avat][data-ae_ar="null"][href], a[data-ae_vis][data-ae_avat][data-ae_ar="null"][data-ae_ev]:not([href])','a[data-ae_vis][data-ae_avat][data-ae_ar="null"][href]:not([data-ae_an]), a[data-ae_vis][data-ae_avat][data-ae_ar="null"][data-ae_ev]:not([href]):not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-238
	 * @returns {AutoTestShell} testShell
	 */function test238(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("238",{bestPractice:1301,testId:238,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements with a role='link' attribute, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This element (role=link) does not have a mechanism that allows an accessible name value to be calculated",metaTextDetail:"This {{tag-name}} (role=link) does not have a mechanism that allows an accessible name value to be calculated",fixType:{"fixType":1,"domSpec":false,"helperText":"Provide a descriptive accessible name for the anchor element.","fix":{"aria-label":null}},testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="link"]','*[data-ae_vis][data-ae_avat][data-ae_ar="link"]:not([data-ae_an])');}});}/**
	 * Determines if target nodes contain only the specified tag types withh role exceptions
	 * @param {String} tag of parent to test
	 * @param {*} allowed object literal list of allowed children for the particular tag where each allowed child type has an associated value of 1 ie {LI:1,TEMPLATE:1}
	 * @param {Object} rootTestNode DOM structure to fish the tag and children from for testing
	 * @param {function} testChildFilter optional function for filtering children inside a given parent identified by the tag param
	 * @returns {Node[]} Array of bad nodes
	 */function correctChildren(tag,allowed,rootTestNode,testChildFilter){const badnodes=[];const nodes=rootTestNode.querySelectorAll(tag+'[data-ae_vis][data-ae_avat][data-ae_ar="null"]:not(:empty)');for(let i=0;i<nodes.length;i++){const tagchildren=nodes[i].children;let fail=false;for(let j=0;j<tagchildren.length&&fail===false;j++){const testchild=tagchildren[j];const role=testchild.getAttribute("data-ae_ar");if(testchild.tagName==="LI"&&role==="listitem"){// redundant, but okay
continue;}if(allowed[testchild.tagName]){//Child type on the allowed list but needs to not have a role
if(role!=="null"||testChildFilter&&!testChildFilter(tagchildren,testchild)){badnodes.push(nodes[i]);fail=true;}}else{//Child type not on the allowed list but it's still legit of it has a listitem role
if(role!=="listitem"){badnodes.push(nodes[i]);fail=true;}}}}return badnodes;}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-244
	 * @returns {AutoTestShell} testShell
	 */function test244(){aeKernel.addAutomaticTest("244",{bestPractice:393,testId:244,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more dl elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not contain only elements with a role='listitem' attribute, dt, dd, script, template elements (without an ARIA-assigned role); or only div, script, template elements (without an ARIA-assigned role); as direct child elements.",metaText:"This element should only contain elements with a role='listitem' attribute, dt, dd, script, template elements (without an ARIA-assigned role); or only div, script, template elements (without an ARIA-assigned role); as direct child elements",metaTextDetail:"This {{tag-name}} should only contain elements with a role='listitem' attribute, dt, dd, script, template elements (without an ARIA-assigned role); or only div, script, template elements (without an ARIA-assigned role); as direct child elements",fixType:"",testFunc:function(){const testShell=new AutoTestShell('dl[data-ae_vis][data-ae_avat][data-ae_ar="null"]:not(:empty)');testShell.set_candidateSetNodesFunc(function(){return correctChildren("dl",{DT:1,DD:1,DIV:1,SCRIPT:1,TEMPLATE:1},aeKernel.rootTestNode,function(tagChildren,testChild){// under this test, dl can have dt and dd as children, or div as children, but not both, e.g. dt and/or dd and div as children
let hasDtDd=false;let hasDiv=false;for(let i=0;i<tagChildren.length;i++){let tagChild=tagChildren[i];if(tagChild.tagName==='DT'||tagChild.tagName==='DD'){hasDtDd=true;}else if(tagChild.tagName==='DIV'){hasDiv=true;}if(hasDtDd&&hasDiv){// optimization
break;}}if(hasDtDd&&hasDiv){return testChild.tagName!=='DT'&&testChild.tagName!=='DD'&&testChild.tagName!=='DIV';}else if(hasDtDd){return testChild.tagName!=='DIV';}else if(hasDiv){return testChild.tagName!=='DT'&&testChild.tagName!=='DD';}else{return true;}});});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-245
	 * @returns {AutoTestShell} testShell
	 */function test245(){aeKernel.addAutomaticTest("245",{bestPractice:393,testId:245,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more ol elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not contain only li, script or template elements (without an ARIA-assigned role); or elements with a role='listitem' attribute; as direct child elements.",metaText:"This element should only contain li elements (without an ARIA-assigned role), script elements (without an ARIA-assigned role), template elements (without an ARIA-assigned role); or elements with a role=listitem attribute; as direct child elements",metaTextDetail:"This {{tag-name}} should only contain li elements (without an ARIA-assigned role), script elements (without an ARIA-assigned role), template elements (without an ARIA-assigned role); or elements with a role=listitem attribute; as direct child elements",fixType:"",testFunc:function(){var testShell=new AutoTestShell('ol[data-ae_vis][data-ae_avat][data-ae_ar="null"]:not(:empty)');testShell.set_candidateSetNodesFunc(function(){return correctChildren("ol",{LI:1,SCRIPT:1,TEMPLATE:1},aeKernel.rootTestNode,null);});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-246
	 * @returns {AutoTestShell} testShell
	 */function test246(){aeKernel.addAutomaticTest("246",{bestPractice:393,testId:246,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more ul elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not contain only li, script or template elements (without an ARIA-assigned role); or elements with a role='listitem' attribute; as direct child elements.",metaText:"This element should only contain li elements (without an ARIA-assigned role), script elements (without an ARIA-assigned role) or template elements (without an ARIA-assigned role); or elements with a role=listitem attribute; as direct child elements",metaTextDetail:"This {{tag-name}} should only contain li elements (without an ARIA-assigned role), script elements (without an ARIA-assigned role) or template elements (without an ARIA-assigned role); or elements with a role=listitem attribute; as direct child elements",fixType:"",testFunc:function(){var testShell=new AutoTestShell('ul[data-ae_vis][data-ae_avat][data-ae_ar="null"]:not(:empty)');testShell.set_candidateSetNodesFunc(function(){return correctChildren("ul",{LI:1,SCRIPT:1,TEMPLATE:1},aeKernel.rootTestNode,null);});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-249
	 * @returns {AutoTestShell} testShell
	 */function test249(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("249",{bestPractice:363,testId:249,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more object elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This object element does not have a mechanism that allows an accessible name value to be calculated",fixType:"",testFunc:function(){return new AutoTestShell('object[data-ae_vis][data-ae_avat][data-ae_ar="null"]','object[data-ae_vis][data-ae_avat][data-ae_ar="null"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-251
	 * @returns {AutoTestShell} testShell
	 */function test251(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("251",{bestPractice:1143,testId:251,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more audio elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This audio element does not have a mechanism that allows an accessible name value to be calculated",fixType:"",testFunc:function(){return new AutoTestShell('audio[data-ae_vis][data-ae_avat][data-ae_ar="null"]','audio[data-ae_vis][data-ae_avat][data-ae_ar="null"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-252
	 * @returns {AutoTestShell} testShell
	 */function test252(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("252",{bestPractice:444,testId:252,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more video elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This video element does not have a mechanism that allows an accessible name value to be calculated",fixType:"",testFunc:function(){return new AutoTestShell('video[data-ae_vis][data-ae_avat][data-ae_ar="null"]','video[data-ae_vis][data-ae_avat][data-ae_ar="null"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-254
	 * @returns {AutoTestShell} testShell
	 */function test254(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("254",{bestPractice:476,testId:254,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more embed elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This embed element does not have a mechanism that allows an accessible name value to be calculated",fixType:"",testFunc:function(){return new AutoTestShell('embed[data-ae_vis][data-ae_avat][data-ae_ar="null"]','embed[data-ae_vis][data-ae_avat][data-ae_ar="null"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-257
	 * @returns {AutoTestShell} testShell
	 */function test257(){aeKernel.addAutomaticTest("257",{bestPractice:1626,testId:257,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, excluding input elements with type='checkbox | radio' and elements with a role='checkbox | option | radio | switch | menuitemcheckbox | menuitemradio | treeitem' attribute, not intentionally hidden in the DOM and available to assistive technologies, has an aria-checked attribute.",metaText:"The aria-checked attribute is not allowed on this element",metaTextDetail:"The aria-checked attribute is not allowed on this {{tag-name}}{{optional_data-ae_ar}}",fixType:"",testFunc:function(){const applicabilityExclusionSubSelectors=['input[type="checkbox"]','input[type="radio"]','*[data-ae_ar="checkbox"]','*[data-ae_ar="option"]','*[data-ae_ar="radio"]','*[data-ae_ar="switch"]','*[data-ae_ar="menuitemcheckbox"]','*[data-ae_ar="menuitemradio"]','*[data-ae_ar="treeitem"]'];const relevantCssSelector=`*${applicabilityExclusionSubSelectors.map(subSelector=>`:not(${subSelector})`).join('')}[data-ae_vis][data-ae_avat]`;return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[aria-checked]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-258
	 * @returns {AutoTestShell} testShell
	 */function test258(){aeKernel.addAutomaticTest("258",{bestPractice:1626,testId:258,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='menuitemcheckbox' attribute, not intentionally hidden in the DOM and available to assistive technologies, does not have an aria-checked attribute set to one of the text values: 'true | false | mixed'.",metaText:"This element (role=menuitemcheckbox) should have the aria-checked attribute set to one of the following text values: true | false | mixed",metaTextDetail:"This {{tag-name}} (role=menuitemcheckbox) should have the aria-checked attribute set to one of the following text values: true | false | mixed",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="menuitemcheckbox"]','*[data-ae_vis][data-ae_avat][data-ae_ar="menuitemcheckbox"]:not([aria-checked="true"]):not([aria-checked="false"]):not([aria-checked="mixed"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-259
	 * @returns {AutoTestShell} testShell
	 */function test259(){aeKernel.addAutomaticTest("259",{bestPractice:1626,testId:259,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='menuitemradio' attribute, not intentionally hidden in the DOM and available to assistive technologies, does not have an aria-checked attribute set to one of the text values: 'true | false'.",metaText:"This element (role=menuitemradio) needs an aria-checked attribute and that attribute needs to be the text value of either 'true' or 'false'",metaTextDetail:"This {{tag-name}} (role=menuitemradio) needs an aria-checked attribute and that attribute needs to be the text value of either 'true' or 'false'",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="menuitemradio"]','*[data-ae_vis][data-ae_avat][data-ae_ar="menuitemradio"]:not([aria-checked="true"]):not([aria-checked="false"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-260
	 * @returns {AutoTestShell} testShell
	 */function test260(){aeKernel.addAutomaticTest("260",{bestPractice:1626,testId:260,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, excluding heading elements and elements with a role='grid | heading | listitem | row | tablist | treeitem' attribute, not intentionally hidden in the DOM and available to assistive technologies, has an aria-level attribute.",metaText:"The aria-level attribute is not allowed on this element",metaTextDetail:"The aria-level attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){const applicabilityExclusionSubSelectors=['h1','h2','h3','h4','h5','h6','*[data-ae_ar="grid"]','*[data-ae_ar="heading"]','*[data-ae_ar="listitem"]','*[data-ae_ar="row"]','*[data-ae_ar="tablist"]','*[data-ae_ar="treeitem"]'];const relevantCssSelector=`*${applicabilityExclusionSubSelectors.map(subSelector=>`:not(${subSelector})`).join('')}[data-ae_vis][data-ae_avat]`;return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[aria-level]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-261
	 * @returns {AutoTestShell} testShell
	 */function test261(){aeKernel.addAutomaticTest("261",{bestPractice:1626,testId:261,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, excluding elements with a role='grid | listbox | tablist | tree | treegrid' attribute, not intentionally hidden in the DOM and available to assistive technologies, has an aria-multiselectable attribute.",metaText:"The aria-multiselectable attribute is not allowed on this element",metaTextDetail:"The aria-multiselectable attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){const applicabilityExclusionSubSelectors=['*[data-ae_ar="grid"]','*[data-ae_ar="listbox"]','*[data-ae_ar="tablist"]','*[data-ae_ar="tree"]','*[data-ae_ar="treegrid"]'];const relevantCssSelector=`*${applicabilityExclusionSubSelectors.map(subSelector=>`:not(${subSelector})`).join('')}[data-ae_vis][data-ae_avat]`;return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[aria-multiselectable]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-262
	 * @returns {AutoTestShell} testShell
	 */function test262(){aeKernel.addAutomaticTest("262",{bestPractice:1626,testId:262,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, excluding elements with a role='menu | menubar | scrollbar | separator | slider | tablist | tree | toolbar | radiogroup | treegrid | combobox | listbox' attribute and input type='range' elements, not intentionally hidden in the DOM and available to assistive technologies, has an aria-orientation attribute.",metaText:"The aria-orientation attribute is not allowed on this element",metaTextDetail:"The aria-orientation attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){const applicabilityExclusionSubSelectors=['*[data-ae_ar="menu"]','*[data-ae_ar="menubar"]','*[data-ae_ar="scrollbar"]','*[data-ae_ar="separator"]','*[data-ae_ar="slider"]','*[data-ae_ar="tablist"]','*[data-ae_ar="tree"]','*[data-ae_ar="toolbar"]','*[data-ae_ar="radiogroup"]','*[data-ae_ar="treegrid"]','*[data-ae_ar="combobox"]','*[data-ae_ar="listbox"]','input[type="range"]'];const relevantCssSelector=`*${applicabilityExclusionSubSelectors.map(subSelector=>`:not(${subSelector})`).join('')}[data-ae_vis][data-ae_avat]`;return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[aria-orientation]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-263
	 * @returns {AutoTestShell} testShell
	 */function test263(){aeKernel.addAutomaticTest("263",{bestPractice:1626,testId:263,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, excluding input elements (without an ARIA-assigned role), select elements (without an ARIA-assigned role), textarea elements (without an ARIA-assigned role) and elements with a role='checkbox | columnheader | combobox | grid | gridcell | listbox | menuitemcheckbox | menuitemradio | radiogroup | rowheader | searchbox | slider | spinbutton | switch | textbox | treegrid' attribute, not intentionally hidden in the DOM and available to assistive technologies, has an aria-readonly attribute.",metaText:"The aria-readonly attribute is not allowed on this element",metaTextDetail:"The aria-readonly attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){const applicabilityExclusionSubSelectors=['input[data-ae_ar="null"]','select[data-ae_ar="null"]','textarea[data-ae_ar="null"]','*[data-ae_ar="checkbox"]','*[data-ae_ar="columnheader"]','*[data-ae_ar="combobox"]','*[data-ae_ar="grid"]','*[data-ae_ar="gridcell"]','*[data-ae_ar="listbox"]','*[data-ae_ar="menuitemcheckbox"]','*[data-ae_ar="menuitemradio"]','*[data-ae_ar="radiogroup"]','*[data-ae_ar="rowheader"]','*[data-ae_ar="searchbox"]','*[data-ae_ar="slider"]','*[data-ae_ar="spinbutton"]','*[data-ae_ar="switch"]','*[data-ae_ar="textbox"]','*[data-ae_ar="treegrid"]'];const relevantCssSelector=`*${applicabilityExclusionSubSelectors.map(subSelector=>`:not(${subSelector})`).join('')}[data-ae_vis][data-ae_avat]`;return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[aria-readonly]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-264
	 * @returns {AutoTestShell} testShell
	 */function test264(){aeKernel.addAutomaticTest("264",{bestPractice:1626,testId:264,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, excluding excluding TR, TD and TH elements and elements with a role='gridcell | option | row | tab | columnheader | rowheader | treeitem' attribute, not intentionally hidden in the DOM and available to assistive technologies, has an aria-selected attribute.",metaText:"The aria-selected attribute is not allowed on this element",metaTextDetail:"The aria-selected attribute is not allowed on this {{tag-name}}{{optional_data-ae_ar}}",fixType:"",testFunc:function(){const applicabilityExclusionSubSelectors=['tr','td','th','*[data-ae_ar="gridcell"]','*[data-ae_ar="option"]','*[data-ae_ar="row"]','*[data-ae_ar="tab"]','*[data-ae_ar="columnheader"]','*[data-ae_ar="rowheader"]','*[data-ae_ar="treeitem"]'];const relevantCssSelector=`*${applicabilityExclusionSubSelectors.map(subSelector=>`:not(${subSelector})`).join('')}[data-ae_vis][data-ae_avat]`;return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[aria-selected]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-265
	 * @returns {AutoTestShell} testShell
	 */function test265(){aeKernel.addAutomaticTest("265",{bestPractice:1626,testId:265,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM and available to assistive technologies, has an aria-sort attribute that is not set to one of the following text values: 'ascending | descending | none | other'.",metaText:"This element's aria-sort attribute should be set to one of the following text values: ascending | descending | none | other",metaTextDetail:"This {{tag-name}}'s aria-sort attribute should be set to one of the following text values: ascending | descending | none | other",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][aria-sort]','*[data-ae_vis][data-ae_avat][aria-sort]:not([aria-sort="ascending"]):not([aria-sort="descending"]):not([aria-sort="none"]):not([aria-sort="other"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-266
	 * @returns {AutoTestShell} testShell
	 */function test266(){aeKernel.addAutomaticTest("266",{bestPractice:1626,testId:266,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, excluding elements with a role='progressbar | scrollbar | slider | spinbutton' attribute, input type='range' elements and input type='number' elements, not intentionally hidden in the DOM and available to assistive technologies, has an aria-valuemax attribute.",metaText:"The aria-valuemax attribute is not allowed on this element",metaTextDetail:"The aria-valuemax attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){const applicabilityExclusionSubSelectors=['*[data-ae_ar="progressbar"]','*[data-ae_ar="scrollbar"]','*[data-ae_ar="slider"]','*[data-ae_ar="spinbutton"]','input[type="range"]','input[type="number"]'];const relevantCssSelector=`*${applicabilityExclusionSubSelectors.map(subSelector=>`:not(${subSelector})`).join('')}[data-ae_vis][data-ae_avat]`;return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[aria-valuemax]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-267
	 * @returns {AutoTestShell} testShell
	 */function test267(){aeKernel.addAutomaticTest("267",{bestPractice:1626,testId:267,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, excluding elements with a role='progressbar | scrollbar | slider | spinbutton' attribute, input type='range' elements and input type='number' elements, not intentionally hidden in the DOM and available to assistive technologies, has an aria-valuenow attribute.",metaText:"The aria-valuenow attribute is not allowed on this element",metaTextDetail:"The aria-valuenow attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){const applicabilityExclusionSubSelectors=['*[data-ae_ar="progressbar"]','*[data-ae_ar="scrollbar"]','*[data-ae_ar="slider"]','*[data-ae_ar="spinbutton"]','input[type="range"]','input[type="number"]'];const relevantCssSelector=`*${applicabilityExclusionSubSelectors.map(subSelector=>`:not(${subSelector})`).join('')}[data-ae_vis][data-ae_avat]`;return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[aria-valuenow]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-268
	 * @returns {AutoTestShell} testShell
	 */function test268(){aeKernel.addAutomaticTest("268",{bestPractice:1626,testId:268,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, excluding elements with a role='progressbar | scrollbar | slider | spinbutton' attribute, input type='range' elements and input type='number' elements, not intentionally hidden in the DOM and available to assistive technologies, has an aria-valuemin attribute.",metaText:"The aria-valuemin attribute is not allowed on this element",metaTextDetail:"The aria-valuemin attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){const applicabilityExclusionSubSelectors=['*[data-ae_ar="progressbar"]','*[data-ae_ar="scrollbar"]','*[data-ae_ar="slider"]','*[data-ae_ar="spinbutton"]','input[type="range"]','input[type="number"]'];const relevantCssSelector=`*${applicabilityExclusionSubSelectors.map(subSelector=>`:not(${subSelector})`).join('')}[data-ae_vis][data-ae_avat]`;return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[aria-valuemin]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-269
	 * @returns {AutoTestShell} testShell
	 */function test269(){aeKernel.addAutomaticTest("269",{bestPractice:1626,testId:269,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, excluding elements with a role='progressbar | scrollbar | slider | spinbutton' attribute, input type='range' elements and input type='number' elements, not intentionally hidden in the DOM and available to assistive technologies, has an aria-valuetext attribute.",metaText:"The aria-valuetext attribute is not allowed on this element",metaTextDetail:"The aria-valuetext attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){const applicabilityExclusionSubSelectors=['*[data-ae_ar="progressbar"]','*[data-ae_ar="scrollbar"]','*[data-ae_ar="slider"]','*[data-ae_ar="spinbutton"]','input[type="range"]','input[type="number"]'];const relevantCssSelector=`*${applicabilityExclusionSubSelectors.map(subSelector=>`:not(${subSelector})`).join('')}[data-ae_vis][data-ae_avat]`;return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[aria-valuetext]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-270
	 * @returns {AutoTestShell} testShell
	 */function test270(){aeKernel.addAutomaticTest("270",{bestPractice:1626,testId:270,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, excluding datalist elements (without an ARIA-assigned role), select elements (without an ARIA-assigned role), textarea elements (without an ARIA-assigned role), input type='checkbox | color | date | datetime-local | email | file | month | number | password | radio | range | search | tel | text | time | url | week' elements (without an ARIA-assigned role), and elements with a role='searchbox | spinbutton | textbox | combobox | columnheader | rowheader | gridcell | listbox | radiogroup | tree | treegrid' attribute, not intentionally hidden in the DOM and available to assistive technologies, has an aria-required attribute.",metaText:"The aria-required attribute is not allowed on this element",metaTextDetail:"The aria-required attribute is not allowed on this {{tag-name}}",fixType:{"fixType":1,"domSpec":false,"helperText":'Select "Save" to remove the aria-required attribute, or "Cancel" if the fix should not be applied.',"fix":{"aria-required":false}},testFunc:function(){const applicabilityExclusionSubSelectors=['datalist[data-ae_ar="null"]','select[data-ae_ar="null"]','textarea[data-ae_ar="null"]','input[type="checkbox"][data-ae_ar="null"]','input[type="color"][data-ae_ar="null"]','input[type="date"][data-ae_ar="null"]','input[type="datetime-local"][data-ae_ar="null"]','input[type="email"][data-ae_ar="null"]','input[type="file"][data-ae_ar="null"]','input[type="month"][data-ae_ar="null"]','input[type="number"][data-ae_ar="null"]','input[type="password"][data-ae_ar="null"]','input[type="radio"][data-ae_ar="null"]','input[type="range"][data-ae_ar="null"]','input[type="search"][data-ae_ar="null"]','input[type="tel"][data-ae_ar="null"]','input[type="text"][data-ae_ar="null"]','input[type="time"][data-ae_ar="null"]','input[type="url"][data-ae_ar="null"]','input[type="week"][data-ae_ar="null"]','*[data-ae_ar="checkbox"]','*[data-ae_ar="combobox"]','*[data-ae_ar="columnheader"]','*[data-ae_ar="gridcell"]','*[data-ae_ar="listbox"]','*[data-ae_ar="radiogroup"]','*[data-ae_ar="rowheader"]','*[data-ae_ar="searchbox"]','*[data-ae_ar="spinbutton"]','*[data-ae_ar="textbox"]','*[data-ae_ar="tree"]','*[data-ae_ar="treegrid"]',//Input with no type is an implicit type=text
'input[data-ae_ar="null"]:not([type=null])'];const relevantCssSelector=`*${applicabilityExclusionSubSelectors.map(subSelector=>`:not(${subSelector})`).join('')}[data-ae_vis][data-ae_avat]`;return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[aria-required]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-271
	 * @returns {AutoTestShell} testShell
	 */function test271(){aeKernel.addAutomaticTest("271",{bestPractice:1626,testId:271,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, excluding table elements and elements with a role='table | grid | treegrid' attribute, not intentionally hidden in the DOM and available to assistive technologies, has an aria-rowcount attribute.",metaText:"The aria-rowcount attribute is not allowed on this element",metaTextDetail:"The aria-rowcount attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){const applicabilityExclusionSubSelectors=['table','[data-ae_ar="table"]','[data-ae_ar="grid"]','[data-ae_ar="treegrid"]'];const relevantCssSelector=`*${applicabilityExclusionSubSelectors.map(subSelector=>`:not(${subSelector})`).join('')}[data-ae_vis][data-ae_avat]`;return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[aria-rowcount]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-272
	 * @returns {AutoTestShell} testShell
	 */function test272(){aeKernel.addAutomaticTest("272",{bestPractice:1626,testId:272,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, excluding elements with a role='row | cell | gridcell | rowheader | columnheader' attribute, not intentionally hidden in the DOM and available to assistive technologies, has an aria-rowindex attribute.",metaText:"The aria-rowindex attribute is not allowed on this element",metaTextDetail:"The aria-rowindex attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){const applicabilityExclusionSubSelectors=['*[data-ae_ar="row"]','*[data-ae_ar="cell"]','*[data-ae_ar="gridcell"]','*[data-ae_ar="rowheader"]','*[data-ae_ar="columnheader"]'];const relevantCssSelector=`*${applicabilityExclusionSubSelectors.map(subSelector=>`:not(${subSelector})`).join('')}[data-ae_vis][data-ae_avat]`;return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[aria-rowindex]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-273
	 * @returns {AutoTestShell} testShell
	 */function test273(){aeKernel.addAutomaticTest("273",{bestPractice:1626,testId:273,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, excluding elements with a role='row | cell | gridcell | rowheader | columnheader' attribute, not intentionally hidden in the DOM and available to assistive technologies, has an aria-rowspan attribute.",metaText:"The aria-rowspan attribute is not allowed on this element",metaTextDetail:"The aria-rowspan attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){const applicabilityExclusionSubSelectors=['*[data-ae_ar="row"]','*[data-ae_ar="cell"]','*[data-ae_ar="gridcell"]','*[data-ae_ar="rowheader"]','*[data-ae_ar="columnheader"]'];const relevantCssSelector=`*${applicabilityExclusionSubSelectors.map(subSelector=>`:not(${subSelector})`).join('')}[data-ae_vis][data-ae_avat]`;return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[aria-rowspan]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-274
	 * @returns {AutoTestShell} testShell
	 */function test274(){aeKernel.addAutomaticTest("274",{bestPractice:1626,testId:274,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, excluding table elements and elements with a role='table | grid | treegrid' attribute, not intentionally hidden in the DOM and available to assistive technologies, has an aria-colcount attribute.",metaText:"The aria-colcount attribute is not allowed on this element",metaTextDetail:"The aria-colcount attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){const applicabilityExclusionSubSelectors=['table','[data-ae_ar="table"]','[data-ae_ar="grid"]','[data-ae_ar="treegrid"]'];const relevantCssSelector=`*${applicabilityExclusionSubSelectors.map(subSelector=>`:not(${subSelector})`).join('')}[data-ae_vis][data-ae_avat]`;return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[aria-colcount]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-275
	 * @returns {AutoTestShell} testShell
	 */function test275(){aeKernel.addAutomaticTest("275",{bestPractice:1626,testId:275,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, excluding elements with a role='row | cell | gridcell | rowheader | columnheader' attribute, not intentionally hidden in the DOM and available to assistive technologies, has an aria-colindex attribute.",metaText:"The aria-colindex attribute is not allowed on this element",metaTextDetail:"The aria-colindex attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){const applicabilityExclusionSubSelectors=['*[data-ae_ar="row"]','*[data-ae_ar="cell"]','*[data-ae_ar="gridcell"]','*[data-ae_ar="rowheader"]','*[data-ae_ar="columnheader"]'];const relevantCssSelector=`*${applicabilityExclusionSubSelectors.map(subSelector=>`:not(${subSelector})`).join('')}[data-ae_vis][data-ae_avat]`;return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[aria-colindex]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-276
	 * @returns {AutoTestShell} testShell
	 */function test276(){aeKernel.addAutomaticTest("276",{bestPractice:1626,testId:276,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, excluding elements with a role='cell | columnheader | gridcell | rowheader' attribute, not intentionally hidden in the DOM and available to assistive technologies, has an aria-colspan attribute.",metaText:"The aria-colspan attribute is not allowed on this element",metaTextDetail:"The aria-colspan attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){const applicabilityExclusionSubSelectors=['*[data-ae_ar="cell"]','*[data-ae_ar="columnheader"]','*[data-ae_ar="gridcell"]','*[data-ae_ar="rowheader"]'];const relevantCssSelector=`*${applicabilityExclusionSubSelectors.map(subSelector=>`:not(${subSelector})`).join('')}[data-ae_vis][data-ae_avat]`;return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[aria-colspan]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-277
	 * @returns {AutoTestShell} testShell
	 */function test277(){aeKernel.addAutomaticTest("277",{bestPractice:1626,testId:277,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, excluding textarea elements (without an ARIA-assigned role), input type='email | tel | search | text | url' elements with no list attribute (without an ARIA-assigned role), input type='password' elements (without an ARIA-assigned role), and elements with a role='combobox | textbox | searchbox' attribute, not intentionally hidden in the DOM and available to assistive technologies, has an aria-placeholder attribute.",metaText:"The aria-placeholder attribute is not allowed on this element",metaTextDetail:"The aria-placeholder attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){const applicabilityExclusionSubSelectors=['textarea[data-ae_ar="null"]','input[type="email"]:not([list])[data-ae_ar="null"]','input[type="tel"]:not([list])[data-ae_ar="null"]','input[type="search"]:not([list])[data-ae_ar="null"]','input[type="text"]:not([list])[data-ae_ar="null"]','input[type="url"]:not([list])[data-ae_ar="null"]','input[type="email"]:not([list])[data-ae_ar="null"]','input[type="password"][data-ae_ar="null"]','*[data-ae_ar="combobox"]','*[data-ae_ar="textbox"]','*[data-ae_ar="searchbox"]'];const relevantCssSelector=`*${applicabilityExclusionSubSelectors.map(subSelector=>`:not(${subSelector})`).join('')}[data-ae_vis][data-ae_avat]`;return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[aria-placeholder]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-278
	 * @returns {AutoTestShell} testShell
	 */function test278(){aeKernel.addAutomaticTest("278",{bestPractice:1626,testId:278,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, excluding dialog elements (without an ARIA-assigned role) and elements with a role='alertdialog | dialog' attribute, not intentionally hidden in the DOM and available to assistive technologies, has an aria-modal attribute.",metaText:"The aria-modal attribute is not allowed on this element",metaTextDetail:"The aria-modal attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){const applicabilityExclusionSubSelectors=['dialog[data-ae_ar="null"]','*[data-ae_ar="alertdialog"]','*[data-ae_ar="dialog"]'];const relevantCssSelector=`*${applicabilityExclusionSubSelectors.map(subSelector=>`:not(${subSelector})`).join('')}[data-ae_vis][data-ae_avat]`;return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[aria-modal]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-279
	 * @returns {AutoTestShell} testShell
	 */function test279(){aeKernel.addAutomaticTest("279",{bestPractice:1626,testId:279,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, excluding input type=radio elements and elements with a role='article | listitem | option | menuitem | menuitemcheckbox | menuitemradio | radio | tab | treeitem' attribute, not intentionally hidden in the DOM and available to assistive technologies, has an aria-posinset attribute.",metaText:"The aria-posinset attribute is not allowed on this element because it does not have the correct role attribute",metaTextDetail:"The aria-posinset attribute is not allowed on this {{tag-name}} because it does not have the correct role attribute",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][aria-posinset]','*[data-ae_vis][data-ae_avat]:not(input[type="radio"]):not([data-ae_ar="article"]):not([data-ae_ar="listitem"]):not([data-ae_ar="option"]):not([data-ae_ar="menuitem"]):not([data-ae_ar="menuitemcheckbox"]):not([data-ae_ar="menuitemradio"]):not([data-ae_ar="radio"]):not([data-ae_ar="tab"]):not([data-ae_ar="treeitem"])[aria-posinset]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-280
	 * @returns {AutoTestShell} testShell
	 */function test280(){aeKernel.addAutomaticTest("280",{bestPractice:1626,testId:280,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, excluding input type=radio elements and elements with a role='article | listitem | option | menuitem | menuitemcheckbox | menuitemradio | radio | tab | treeitem' attribute, not intentionally hidden in the DOM and available to assistive technologies, has both an aria-posinset and an aria-setsize attribute.",metaText:"The aria-posinset and aria-setsize attribute pair is not allowed on this element because it does not have the correct role attribute",metaTextDetail:"The aria-posinset and aria-setsize attribute pair is not allowed on this {{tag-name}} because it does not have the correct role attribute",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][aria-setsize][aria-posinset]','*[data-ae_vis][data-ae_avat]:not(input[type="radio"]):not([data-ae_ar="article"]):not([data-ae_ar="listitem"]):not([data-ae_ar="option"]):not([data-ae_ar="menuitem"]):not([data-ae_ar="menuitemcheckbox"]):not([data-ae_ar="menuitemradio"]):not([data-ae_ar="radio"]):not([data-ae_ar="tab"]):not([data-ae_ar="treeitem"])[aria-setsize][aria-posinset]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-281
	 * @returns {AutoTestShell} testShell
	 */function test281(){aeKernel.addAutomaticTest("281",{bestPractice:1626,testId:281,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, excluding button elements (without an ARIA-assigned role), input type='button | image | reset | submit' elements (without an ARIA-assigned role), summary elements (without an ARIA-assigned role), A elements with an href attribute (without an ARIA-assigned role), area elements with an href attribute (without an ARIA-assigned role), hr elements (without an ARIA-assigned role) and elements with a role='application | button | checkbox | columnheader | combobox | gridcell | link | listbox | menuitem | menuitemcheckbox | menuitemradio | row | rowheader | switch | tab | treeitem' attribute, not intentionally hidden in the DOM and available to assistive technologies, has an aria-expanded attribute.",metaText:"The aria-expanded attribute is not allowed on this element",metaTextDetail:"The aria-expanded attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){const applicabilityExclusionSubSelectors=['button[data-ae_ar="null"]','input[type="button"][data-ae_ar="null"]','input[type="image"][data-ae_ar="null"]','input[type="reset"][data-ae_ar="null"]','input[type="submit"][data-ae_ar="null"]','summary[data-ae_ar="null"]','a[href][data-ae_ar="null"]','area[href][data-ae_ar="null"]','hr[data-ae_ar="null"]','*[data-ae_ar="application"]','*[data-ae_ar="button"]','*[data-ae_ar="checkbox"]','*[data-ae_ar="columnheader"]','*[data-ae_ar="combobox"]','*[data-ae_ar="gridcell"]','*[data-ae_ar="link"]','*[data-ae_ar="listbox"]','*[data-ae_ar="menuitem"]','*[data-ae_ar="menuitemcheckbox"]','*[data-ae_ar="menuitemcheckradio"]','*[data-ae_ar="row"]','*[data-ae_ar="rowheader"]','*[data-ae_ar="switch"]','*[data-ae_ar="tab"]','*[data-ae_ar="treeitem"]'];const relevantCssSelector=`*${applicabilityExclusionSubSelectors.map(subSelector=>`:not(${subSelector})`).join('')}[data-ae_vis][data-ae_avat]`;return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[aria-expanded]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-282
	 * @returns {AutoTestShell} testShell
	 */function test282(){aeKernel.addAutomaticTest("282",{bestPractice:1626,testId:282,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, excluding input type='text | search' elements and elements with a role='combobox | searchbox | textbox' attribute, not intentionally hidden in the DOM and available to assistive technologies, has an aria-autocomplete attribute.",metaText:"The aria-autocomplete attribute is not allowed on this element",metaTextDetail:"The aria-autocomplete attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){const applicabilityExclusionSubSelectors=['*[data-ae_ar="combobox"]','*[data-ae_ar="searchbox"]','*[data-ae_ar="textbox"]','input[type="search"]','input[type="text"]'];const relevantCssSelector=`*${applicabilityExclusionSubSelectors.map(subSelector=>`:not(${subSelector})`).join('')}[data-ae_vis][data-ae_avat]`;return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[aria-autocomplete]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-283
	 * @returns {AutoTestShell} testShell
	 */function test283(){aeKernel.addAutomaticTest("283",{bestPractice:1626,testId:283,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, excluding details elements (without an ARIA-assigned role), optgroup elements (without an ARIA-assigned role), input type = 'password' elements, input type='email | tel | text | url' elements with no list attribute (without an ARIA-assigned role), textarea elements (without an ARIA-assigned role), and elements with a role='composite | group | textbox | application | combobox | grid | listbox | menu | menubar | radiogroup | row | searchbox | spinbutton | tablist | toolbar | tree | treegrid' attribute, not intentionally hidden in the DOM and available to assistive technologies, has an aria-activedescendant attribute.",metaText:"The aria-activedescendant attribute is not allowed on this element",metaTextDetail:"The aria-activedescendant attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){const applicabilityExclusionSubSelectors=['details[data-ae_ar="null"]','optgroup[data-ae_ar="null"]','input[type="password"][data-ae_ar="null"]','input[type="email"]:not([list])[data-ae_ar="null"]','input[type="tel"]:not([list])[data-ae_ar="null"]','input[type="text"]:not([list])[data-ae_ar="null"]','input[type="url"]:not([list])[data-ae_ar="null"]','textarea[data-ae_ar="null"]','*[data-ae_ar="composite"]','*[data-ae_ar="group"]','*[data-ae_ar="textbox"]','*[data-ae_ar="application"]','*[data-ae_ar="combobox"]','*[data-ae_ar="grid"]','*[data-ae_ar="listbox"]','*[data-ae_ar="menu"]','*[data-ae_ar="menubar"]','*[data-ae_ar="radiogroup"]','*[data-ae_ar="row"]','*[data-ae_ar="searchbox"]','*[data-ae_ar="spinbutton"]','*[data-ae_ar="tablist"]','*[data-ae_ar="toolbar"]','*[data-ae_ar="tree"]','*[data-ae_ar="treegrid"]'];const relevantCssSelector=`*${applicabilityExclusionSubSelectors.map(subSelector=>`:not(${subSelector})`).join('')}[data-ae_vis][data-ae_avat]`;return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[aria-activedescendant]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-284
	 * @returns {AutoTestShell} testShell
	 */function test284(){aeKernel.addAutomaticTest("284",{bestPractice:1626,testId:284,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more audio elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to a value that is not: 'application'.",metaText:"Role of 'application' on this audio element is usually inappropriate",fixType:"",testFunc:function(){return new AutoTestShell('audio[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"])','audio[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"]):not([data-ae_ar="application"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-285
	 * @returns {AutoTestShell} testShell
	 */function test285(){aeKernel.addAutomaticTest("285",{bestPractice:1626,testId:285,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more button elements, excluding those with a role='button' attribute, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to a value that is not: 'checkbox | link | menuitem | menuitemcheckbox | menuitemradio | radio | switch | tab | combobox | option | treeitem'.",metaText:"The role attribute value given to this element is not allowed.  This element's role attribute can be set to one of the following text values: checkbox | link |menuitem | menuitemcheckbox | menuitemradio | radio | switch | tab | combobox | option | treeitem; or the role attribute can be removed",metaTextDetail:"The role of '{{data-ae_ar}}' given to this {{tag-name}} is not allowed.  This element's role attribute can be set to one of the following text values: checkbox | link |menuitem | menuitemcheckbox | menuitemradio | radio | switch | tab | combobox | option | treeitem; or the role attribute can be removed",fixType:"",testFunc:function(){return new AutoTestShell('button[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"]):not([data-ae_ar="button"])','button[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"]):not([data-ae_ar="button"]):not([data-ae_ar="checkbox"]):not([data-ae_ar="link"]):not([data-ae_ar="menuitem"]):not([data-ae_ar="menuitemcheckbox"]):not([data-ae_ar="menuitemradio"]):not([data-ae_ar="radio"]):not([data-ae_ar="switch"]):not([data-ae_ar="tab"]):not([data-ae_ar="combobox"]):not([data-ae_ar="option"]):not([data-ae_ar="treeitem"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-286
	 * @returns {AutoTestShell} testShell
	 */function test286(){aeKernel.addAutomaticTest("286",{bestPractice:1626,testId:286,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more dl elements, excluding role='list' attributes, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to a value that is not: 'group'.",metaText:"The role attribute value given to this element is not allowed.  This element's role attribute can be set to the text value: group; or the role attribute can be removed",metaTextDetail:"The role of '{{data-ae_ar}}' given to this {{tag-name}} is not allowed.  This element's role attribute can be set to the text value: group; or the role attribute can be removed",fixType:"",testFunc:function(){return new AutoTestShell('dl[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"]):not([data-ae_ar="list"])','dl[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"]):not([data-ae_ar="list"]):not([data-ae_ar="group"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-287
	 * @returns {AutoTestShell} testShell
	 */function test287(){aeKernel.addAutomaticTest("287",{bestPractice:1626,testId:287,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more embed elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to a value that is not: 'application | document | img'.",metaText:"The role attribute value given to this element is not allowed.  This element's role attribute can be set to one of the following text values: application | document | img; or the role attribute can be removed",metaTextDetail:"The role attribute value given to this {{tag-name}} is not allowed.  This element's role attribute can be set to one of the following text values: application | document | img; or the role attribute can be removed",fixType:"",testFunc:function(){return new AutoTestShell('embed[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"])','embed[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"]):not([data-ae_ar="application"]):not([data-ae_ar="document"]):not([data-ae_ar="img"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-288
	 * @returns {AutoTestShell} testShell
	 */function test288(){aeKernel.addAutomaticTest("288",{bestPractice:1626,testId:288,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more fieldset elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to a value that is not: 'group | radiogroup'.",metaText:"The role attribute value given to this element is not allowed. This element's role attribute can be set to one of the following text values: group | radiogroup; or the role attribute can be removed",metaTextDetail:"The role '{{data-ae_ar}}' given to this {{tag-name}} is not allowed. This element's role attribute can be set to one of the following text values: group | radiogroup; or the role attribute can be removed",fixType:"",testFunc:function(){return new AutoTestShell('fieldset[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"])','fieldset[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"]):not([data-ae_ar="group"]):not([data-ae_ar="radiogroup"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-290
	 * @returns {AutoTestShell} testShell
	 */function test290(){aeKernel.addAutomaticTest("290",{bestPractice:1626,testId:290,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM and available to assistive technologies, has an aria-activedescendant attribute that is not set to a valid id.",metaText:"This element has an aria-activedescendant attribute that is set to an invalid or duplicate id",metaTextDetail:"This {{tag-name}} has an aria-activedescendant attribute that is set to an invalid or duplicate id of {{aria-activedescendant}}",fixType:"",testFunc:function(){var testShell=new AutoTestShell('*[data-ae_vis][data-ae_avat][aria-activedescendant]:not([aria-activedescendant=""])');testShell.set_candidateSetNodesFunc(function(){var nodesWithBadIds=[];var res=aeKernel.rootTestNode.querySelectorAll('*[data-ae_vis][data-ae_avat][aria-activedescendant]:not([aria-activedescendant=""])');for(var i=0,len_res=res.length;i<len_res;i++){var n=res[i];var detailsId=n.getAttribute("aria-activedescendant");if(isValidIdOrName(detailsId)===true){// "[id=" is used in place of "#" + id as ids can included colons which breaks css selector, although is valid html
var refNode=aeKernel.rootTestNode.querySelectorAll("[id='"+detailsId.replace(/'/g,"\\'")+"']");var refNodeLength=refNode.length;if(refNodeLength===0||refNodeLength>1){nodesWithBadIds.push(n);}}else{nodesWithBadIds.push(n);}}return nodesWithBadIds;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-291
	 * @returns {AutoTestShell} testShell
	 */function test291(){aeKernel.addAutomaticTest("291",{bestPractice:1626,testId:291,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM and available to assistive technologies, has an aria-activedescendant attribute that references an id of an element that has an aria-activedescendant attribute set to a non-null value.",metaText:"This element should not have an aria-activedescendant attribute that references an element that also has an aria-activedescendant attribute",metaTextDetail:"This {{tag-name}} should not have an aria-activedescendant attribute of {{aria-activedescendant}} that references an element that also has an aria-activedescendant attribute",fixType:"",testFunc:function(){var testShell=new AutoTestShell('*[data-ae_vis][data-ae_avat][aria-activedescendant]:not([aria-activedescendant=""])');testShell.set_candidateSetNodesFunc(function(){var nodesWithBadIds=[];var res=aeKernel.rootTestNode.querySelectorAll('*[data-ae_vis][data-ae_avat][aria-activedescendant]:not([aria-activedescendant=""])');for(var i=0,len_res=res.length;i<len_res;i++){var n=res[i];var detailsId=n.getAttribute("aria-activedescendant");if(isValidIdOrName(detailsId)===true){// "[id=" is used in place of "#" + id as ids can included colons which breaks css selector, although is valid html
var refNode=aeKernel.rootTestNode.querySelector("[id='"+detailsId.replace(/'/g,"\\'")+"']");if(refNode!==null){// aria-activedescendant is used to reference a single id
var desHasAriaDes=refNode.getAttribute("aria-activedescendant");if(desHasAriaDes!==null&&desHasAriaDes!==""){nodesWithBadIds.push(n);}}}}return nodesWithBadIds;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-292
	 * @returns {AutoTestShell} testShell
	 */function test292(){aeKernel.addAutomaticTest("292",{bestPractice:1626,testId:292,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM and available to assistive technologies, has an aria-activedescendant attribute that references the elements own id value.",metaText:"This element should not have an aria-activedescendant attribute that references its own id",metaTextDetail:"This {{tag-name}} should not have an aria-activedescendant attribute that references its own id of {{aria-activedescendant}}",fixType:"",testFunc:function(){var testShell=new AutoTestShell('*[data-ae_vis][data-ae_avat][id]:not([id=""])[aria-activedescendant]:not([aria-activedescendant=""])');testShell.set_candidateSetNodesFunc(function(){var nodesWithBadIds=[];var res=aeKernel.rootTestNode.querySelectorAll('*[data-ae_vis][data-ae_avat][id]:not([id=""])[aria-activedescendant]:not([aria-activedescendant=""])');for(var i=0,len_res=res.length;i<len_res;i++){var n=res[i];var detailsId=n.getAttribute("aria-activedescendant");if(isValidIdOrName(detailsId)===true){// "[id=" is used in place of "#" + id as ids can included colons which breaks css selector, although is valid html
var refNode=aeKernel.rootTestNode.querySelector("[id='"+detailsId.replace(/'/g,"\\'")+"']");if(refNode!==null){// aria-activedescendant is used to reference a single id
if(refNode.isEqualNode(n)===true){nodesWithBadIds.push(n);}}}}return nodesWithBadIds;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-294
	 * @returns {AutoTestShell} testShell
	 */function test294(){aeKernel.addAutomaticTest("294",{bestPractice:1626,testId:294,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more form elements, excluding those with a role='form' attribute, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to a value that is not: 'search | region'.",metaText:"The role attribute value given to this element is not allowed.  This element's role attribute can be set to one of the following text values: search | region; or the role attribute can be removed",metaTextDetail:"The role attribute value given to this {{tag-name}} is not allowed.  This element's role attribute can be set to one of the following text values: search | region; or the role attribute can be removed",fixType:"",testFunc:function(){return new AutoTestShell('form[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"]):not([data-ae_ar="form"])','form[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"]):not([data-ae_ar="form"]):not([data-ae_ar="search"]):not([data-ae_ar="region"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-297
	 * @returns {AutoTestShell} testShell
	 */function test297(){aeKernel.addAutomaticTest("297",{bestPractice:1626,testId:297,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more iframe elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to a value that is not: 'banner | complementary | contentinfo | form | main | navigation | search | application | document | img'.",metaText:"The role attribute value given to this element is not allowed.  This element's role attribute can be set to one of the following text values: banner | complementary | contentinfo | form | main | navigation | search | application | document | img; or the role attribute can be removed",metaTextDetail:"The role attribute value given to this {{tag-name}} is not allowed.  This element's role attribute can be set to one of the following text values: banner | complementary | contentinfo | form | main | navigation | search | application | document | img; or the role attribute can be removed",fixType:"",testFunc:function(){return new AutoTestShell('iframe[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"])','iframe[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"]):not([data-ae_ar="banner"]):not([data-ae_ar="complementary"]):not([data-ae_ar="contentinfo"]):not([data-ae_ar="form"]):not([data-ae_ar="main"]):not([data-ae_ar="navigation"]):not([data-ae_ar="search"]):not([data-ae_ar="application"]):not([data-ae_ar="document"]):not([data-ae_ar="img"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-298
	 * @returns {AutoTestShell} testShell
	 */function test298(){aeKernel.addAutomaticTest("298",{bestPractice:1626,testId:298,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='button' elements, excluding those with a role='button' attribute, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to a value that is not: 'link | menuitem | menuitemcheckbox | menuitemradio | radio | switch | tab | combobox | option | treeitem'.",metaText:"The role attribute value given to this element is not allowed.  This element's role attribute can be set to one of the following text values: link | menuitem | menuitemcheckbox | menuitemradio | radio | switch | tab | combobox | option | treeitem; or the role attribute can be removed",metaTextDetail:"The role attribute value given to this {{tag-name}} is not allowed.  This element's role attribute can be set to one of the following text values: link | menuitem | menuitemcheckbox | menuitemradio | radio | switch | tab | combobox | option | treeitem; or the role attribute can be removed",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_vis][data-ae_avat][type="button"]:not([data-ae_ar="null"]):not([data-ae_ar="button"])','input[data-ae_vis][data-ae_avat][type="button"]:not([data-ae_ar="null"]):not([data-ae_ar="button"]):not([data-ae_ar="link"]):not([data-ae_ar="menuitem"]):not([data-ae_ar="menuitemcheckbox"]):not([data-ae_ar="menuitemradio"]):not([data-ae_ar="option"]):not([data-ae_ar="radio"]):not([data-ae_ar="tab"]):not([data-ae_ar="switch"]):not([data-ae_ar="treeitem"]):not([data-ae_ar="combobox"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-299
	 * @returns {AutoTestShell} testShell
	 */function test299(){aeKernel.addAutomaticTest("299",{bestPractice:1626,testId:299,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='checkbox' elements that has an aria-pressed attribute, excluding those with a role='checkbox' attribute, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to a value that is not: 'button'.",metaText:"The role attribute value given to this element is not allowed.   This element's role attribute can be set to the following text value: button; or the role attribute can be removed",metaTextDetail:"The role attribute value given to this {{tag-name}} is not allowed. This element's role attribute can be set to the following text value: button; or the role attribute can be removed",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_vis][data-ae_avat][type="checkbox"][aria-pressed]:not([data-ae_ar="null"]):not([data-ae_ar="checkbox"])','input[data-ae_vis][data-ae_avat][type="checkbox"][aria-pressed]:not([data-ae_ar="null"]):not([data-ae_ar="checkbox"]):not([data-ae_ar="button"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-300
	 * @returns {AutoTestShell} testShell
	 */function test300(){aeKernel.addAutomaticTest("300",{bestPractice:1626,testId:300,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='checkbox' elements, excluding those with a role='checkbox' attribute; or an aria-pressed attribute, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to a value that is not: 'menuitemcheckbox | switch | option | treeitem'.",metaText:"The role attribute value given to this element is not allowed.  This element's role attribute can be set to one of the following text values: menuitemcheckbox | switch | option | treeitem; or the role attribute can be removed",metaTextDetail:"The role attribute value given to this {{tag-name}} is not allowed.  This element's role attribute can be set to one of the following text values: menuitemcheckbox | switch | option | treeitem; or the role attribute can be removed",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_vis][data-ae_avat][type="checkbox"]:not([aria-pressed]):not([data-ae_ar="null"]):not([data-ae_ar="checkbox"])','input[data-ae_vis][data-ae_avat][type="checkbox"]:not([aria-pressed]):not([data-ae_ar="null"]):not([data-ae_ar="checkbox"]):not([data-ae_ar="menuitemcheckbox"]):not([data-ae_ar="switch"]):not([data-ae_ar="option"]):not([data-ae_ar="treeitem"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-301
	 * @returns {AutoTestShell} testShell
	 */function test301(){aeKernel.addAutomaticTest("301",{bestPractice:1626,testId:301,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='email' elements, excluding those with a list attribute set to a non-null value, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'textbox'.",metaText:"The role attribute is not allowed on this element",metaTextDetail:"The role attribute is not allowed on this {{tag-name}} with type=email",fixType:"",testFunc:function(){const relevantCssSelector='input[type="email"]:not([list]:not([list=""]))[data-ae_vis][data-ae_avat]';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[data-ae_ar="textbox"]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-302
	 * @returns {AutoTestShell} testShell
	 */function test302(){aeKernel.addAutomaticTest("302",{bestPractice:1626,testId:302,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='image' elements, excluding those with a role='button' attribute, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to a value that is not: 'link | menuitem | menuitemcheckbox | menuitemradio | radio | switch | checkbox | option | treeitem'.",metaText:"The role attribute value given to this element is not allowed.  This element's role attribute can be set to one of the following text values: link | menuitem | menuitemcheckbox | menuitemradio | radio | switch | checkbox | option | treeitem; or the role attribute can be removed",metaTextDetail:"The role '{{data-ae_ar}}' given to this {{tag-name}} is not allowed.  This element's role attribute can be set to one of the following text values: link | menuitem | menuitemcheckbox | menuitemradio | radio | switch | checkbox | option | treeitem; or the role attribute can be removed",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_vis][data-ae_avat][type="image"]:not([data-ae_ar="null"]):not([data-ae_ar="button"])','input[data-ae_vis][data-ae_avat][type="image"]:not([data-ae_ar="null"]):not([data-ae_ar="button"]):not([data-ae_ar="link"]):not([data-ae_ar="menuitem"]):not([data-ae_ar="menuitemcheckbox"]):not([data-ae_ar="menuitemradio"]):not([data-ae_ar="option"]):not([data-ae_ar="radio"]):not([data-ae_ar="tab"]):not([data-ae_ar="switch"]):not([data-ae_ar="treeitem"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-303
	 * @returns {AutoTestShell} testShell
	 */function test303(){aeKernel.addAutomaticTest("303",{bestPractice:1626,testId:303,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='number' elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'spinbutton'.",metaText:"This element has a redundant role attribute",metaTextDetail:"This {{tag-name}} with type=number has a redundant role attribute of: {{data-ae_ar}}",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_vis][data-ae_avat][type="number"]','input[data-ae_vis][data-ae_avat][type="number"][data-ae_ar="spinbutton"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-304
	 * @returns {AutoTestShell} testShell
	 */function test304(){aeKernel.addAutomaticTest("304",{bestPractice:1626,testId:304,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='password' elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute.",metaText:"The role attribute is not allowed on this element",metaTextDetail:"The role attribute is not allowed on this input with type=password",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_vis][data-ae_avat][type="password"]','input[data-ae_vis][data-ae_avat][type="password"]:not([data-ae_ar="null"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-309
	 * @returns {AutoTestShell} testShell
	 */function test309(){aeKernel.addAutomaticTest("309",{bestPractice:1626,testId:309,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='tel' elements, excluding those with a list attribute set to a non-null value, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'textbox'.",metaText:"This element has a redundant role attribute",metaTextDetail:"This {{tag-name}} with type=tel has a redundant role attribute of: {{data-ae_ar}}",fixType:"",testFunc:function(){const relevantCssSelector='input[type="tel"]:not([list]:not([list=""]))[data-ae_vis][data-ae_avat]';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[data-ae_ar="textbox"]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-311
	 * @returns {AutoTestShell} testShell
	 */function test311(){aeKernel.addAutomaticTest("311",{bestPractice:1626,testId:311,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='text' elements with a list attribute set to a non-null value, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'combobox'.",metaText:"This element has a redundant role attribute",metaTextDetail:"This {{tag-name}} with type=text has a redundant role attribute of: {{data-ae_ar}}",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_vis][data-ae_avat][type="text"][list]:not([list=""])','input[data-ae_vis][data-ae_avat][type="text"][list]:not([list=""])[data-ae_ar="combobox"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-312
	 * @returns {AutoTestShell} testShell
	 */function test312(){aeKernel.addAutomaticTest("312",{bestPractice:1626,testId:312,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='search' elements with a list attribute set to a non-null value, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'combobox'.",metaText:"This element has a redundant role attribute",metaTextDetail:"This {{tag-name}} with type=search has a redundant role attribute of: {{data-ae_ar}}",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_vis][data-ae_avat][type="search"][list]:not([list=""])','input[data-ae_vis][data-ae_avat][type="search"][list]:not([list=""])[data-ae_ar="combobox"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-313
	 * @returns {AutoTestShell} testShell
	 */function test313(){aeKernel.addAutomaticTest("313",{bestPractice:1626,testId:313,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='tel' elements with a list attribute set to a non-null value, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'combobox'.",metaText:"This element has a redundant role attribute",metaTextDetail:"This {{tag-name}} with type=tel has a redundant role attribute of: {{data-ae_ar}}",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_vis][data-ae_avat][type="tel"][list]:not([list=""])','input[data-ae_vis][data-ae_avat][type="tel"][list]:not([list=""])[data-ae_ar="combobox"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-314
	 * @returns {AutoTestShell} testShell
	 */function test314(){aeKernel.addAutomaticTest("314",{bestPractice:1626,testId:314,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='url' elements with a list attribute set to a non-null value, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'combobox'.",metaText:"This element has a redundant role attribute",metaTextDetail:"This {{tag-name}} with type=url has a redundant role attribute of: {{data-ae_ar}}",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_vis][data-ae_avat][type="url"][list]:not([list=""])','input[data-ae_vis][data-ae_avat][type="url"][list]:not([list=""])[data-ae_ar="combobox"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-315
	 * @returns {AutoTestShell} testShell
	 */function test315(){aeKernel.addAutomaticTest("315",{bestPractice:1626,testId:315,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='email' elements with a list attribute set to a non-null value, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'combobox'.",metaText:"This element has a redundant role attribute",metaTextDetail:"This {{tag-name}} with type=email has a redundant role attribute of: {{data-ae_ar}}",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_vis][data-ae_avat][type="email"][list]:not([list=""])','input[data-ae_vis][data-ae_avat][type="email"][list]:not([list=""])[data-ae_ar="combobox"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-317
	 * @returns {AutoTestShell} testShell
	 */function test317(){aeKernel.addAutomaticTest("317",{bestPractice:1626,testId:317,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more link elements with an href attribute has a role attribute.",metaText:"The role attribute is not allowed on this element",metaTextDetail:"The role attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){return new AutoTestShell('link','link:not([data-ae_ar="null"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-318
	 * @returns {AutoTestShell} testShell
	 */function test318(){aeKernel.addAutomaticTest("318",{bestPractice:1626,testId:318,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more object elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to a value that is not: 'application | document | img'.",metaText:"The role attribute value given to this element is not allowed.  This element's role attribute can be set to one of the following text values: application | document | img; or the role attribute can be removed",metaTextDetail:"The role attribute value given to this {{tag-name}} is not allowed.  This element's role attribute can be set to one of the following text values: application | document | img; or the role attribute can be removed",fixType:"",testFunc:function(){return new AutoTestShell('object[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"])','object[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"]):not([data-ae_ar="application"]):not([data-ae_ar="document"]):not([data-ae_ar="img"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-319
	 * @returns {AutoTestShell} testShell
	 */function test319(){aeKernel.addAutomaticTest("319",{bestPractice:1626,testId:319,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more ol elements, excluding those with a role='list' attribute, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to a value that is not: 'directory | group | listbox | menu | menubar | radiogroup | tablist | toolbar | tree'.",metaText:"The role attribute value given to this element is not allowed.  This element's role attribute can be set to one of the following text values: directory | group | listbox | menu | menubar | radiogroup | tablist | toolbar | tree; or the role attribute can be removed",metaTextDetail:"The role attribute value given to this {{tag-name}} is not allowed.  This element's role attribute can be set to one of the following text values: directory | group | listbox | menu | menubar | radiogroup | tablist | toolbar | tree; or the role attribute can be removed",fixType:"",testFunc:function(){return new AutoTestShell('ol[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"]):not([data-ae_ar="list"])','ol[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"]):not([data-ae_ar="list"]):not([data-ae_ar="directory"]):not([data-ae_ar="group"]):not([data-ae_ar="listbox"]):not([data-ae_ar="menu"]):not([data-ae_ar="menubar"]):not([data-ae_ar="radiogroup"]):not([data-ae_ar="tablist"]):not([data-ae_ar="toolbar"]):not([data-ae_ar="tree"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-322
	 * @returns {AutoTestShell} testShell
	 */function test322(){aeKernel.addAutomaticTest("322",{bestPractice:1626,testId:322,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more picture elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute.",metaText:"The role attribute is not allowed on this element",metaTextDetail:"The role attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){return new AutoTestShell('picture[data-ae_vis][data-ae_avat]','picture[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-325
	 * @returns {AutoTestShell} testShell
	 */function test325(){aeKernel.addAutomaticTest("325",{bestPractice:1626,testId:325,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more ul elements, excluding those with a role='list' attribute, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to a value that is not: 'directory | group | listbox | menu | menubar | radiogroup | tablist | toolbar | tree'.",metaText:"The role attribute value given to this element is not allowed.  This element's role attribute can be set to one of the following text values: directory | group | listbox | menu | menubar | radiogroup | tablist | toolbar | tree; or the role attribute can be removed",metaTextDetail:"The role attribute value given to this {{tag-name}} is not allowed.  This element's role attribute can be set to one of the following text values: directory | group | listbox | menu | menubar | radiogroup | tablist | toolbar | tree; or the role attribute can be removed",fixType:"",testFunc:function(){const relevantCssSelector='ul[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"]):not([data-ae_ar="list"])';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}:not([data-ae_ar="directory"]):not([data-ae_ar="group"]):not([data-ae_ar="listbox"]):not([data-ae_ar="menu"]):not([data-ae_ar="menubar"]):not([data-ae_ar="radiogroup"]):not([data-ae_ar="tablist"]):not([data-ae_ar="toolbar"]):not([data-ae_ar="tree"])`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-326
	 * @returns {AutoTestShell} testShell
	 */function test326(){aeKernel.addAutomaticTest("326",{bestPractice:1626,testId:326,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more video elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to a value that is not: 'application'.",metaText:"Role of 'application' on this video element is usually inappropriate",fixType:"",testFunc:function(){return new AutoTestShell('video[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"])','video[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"]):not([data-ae_ar="application"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-328
	 * @returns {AutoTestShell} testShell
	 */function test328(){aeKernel.addAutomaticTest("328",{bestPractice:1626,testId:328,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements with a required attribute and an aria-required attribute, not intentionally hidden in the DOM and available to assistive technologies, has a different value in its required attribute from its aria-required attribute.",metaText:"This element has the 'required' attribute set while the aria-required attribute is set to false",metaTextDetail:"This {{tag-name}} has the 'required' attribute set while the aria-required attribute is set to false",fixType:"",testFunc:function(){var testShell=new AutoTestShell('*[data-ae_vis][data-ae_avat][required][aria-required]');testShell.set_candidateSetNodesFunc(function(){var diffAttValues=[];var res=aeKernel.rootTestNode.querySelectorAll('*[data-ae_vis][data-ae_avat][required][aria-required]');for(var i=0,len_res=res.length;i<len_res;i++){var n=res[i];var req=n.getAttribute("required");var ariareq=n.getAttribute("aria-required");if((req===""||req==="required")&&ariareq==="false"){diffAttValues.push(n);}}return diffAttValues;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-329
	 * @returns {AutoTestShell} testShell
	 */function test329(){aeKernel.addAutomaticTest("329",{bestPractice:1626,testId:329,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements with a readonly attribute and an aria-readonly attribute, not intentionally hidden in the DOM and available to assistive technologies, has a mismatch between the value of its readonly attribute and its aria-readonly attribute.",metaText:"This element has a mismatch between the value of its readonly and aria-readonly attributes",metaTextDetail:"This {{tag-name}} has a mismatch between the value of its readonly and aria-readonly attributes",fixType:"",testFunc:function(){var testShell=new AutoTestShell('*[data-ae_vis][data-ae_avat][readonly][aria-readonly]');testShell.set_candidateSetNodesFunc(function(){var diffAttValues=[];var res=aeKernel.rootTestNode.querySelectorAll('*[data-ae_vis][data-ae_avat][readonly][aria-readonly]');for(var i=0,len_res=res.length;i<len_res;i++){var n=res[i];var read=n.hasAttribute("readonly")?"true":"false";var ariaread=n.getAttribute("aria-readonly");if(read!==ariaread){diffAttValues.push(n);}}return diffAttValues;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-330
	 * @returns {AutoTestShell} testShell
	 */function test330(){aeKernel.addAutomaticTest("330",{bestPractice:1626,testId:330,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more img elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has an aria-owns attribute set to a non-null value.",metaText:"This element should not have an aria-owns attribute set to a non-null value",metaTextDetail:"This {{tag-name}} should not have an aria-owns attribute set to a non-null value",fixType:"",testFunc:function(){return new AutoTestShell('img[data-ae_vis][data-ae_avat][data-ae_ar="null"]','img[data-ae_vis][data-ae_avat][data-ae_ar="null"][aria-owns]:not([aria-owns=""])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-331
	 * @returns {AutoTestShell} testShell
	 */function test331(){aeKernel.addAutomaticTest("331",{bestPractice:1626,testId:331,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has an aria-owns attribute set to a non-null value.",metaText:"This element should not have an aria-owns attribute set to a non-null value",metaTextDetail:"This {{tag-name}} should not have an aria-owns attribute set to a non-null value",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_vis][data-ae_avat][data-ae_ar="null"]','input[data-ae_vis][data-ae_avat][data-ae_ar="null"][aria-owns]:not([aria-owns=""])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-333
	 * @returns {AutoTestShell} testShell
	 */function test333(){aeKernel.addAutomaticTest("333",{bestPractice:1626,testId:333,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements with a role='textbox' attribute, not intentionally hidden in the DOM and available to assistive technologies, has an aria-owns attribute set to a non-null value.",metaText:"This element (role=textbox) should not have an aria-owns attribute set to a non-null value",metaTextDetail:"This {{tag-name}} (role=textbox) should not have an aria-owns attribute set to a non-null value",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="textbox"]','*[data-ae_vis][data-ae_avat][data-ae_ar="textbox"][aria-owns]:not([aria-owns=""])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-334
	 * @returns {AutoTestShell} testShell
	 */function test334(){aeKernel.addAutomaticTest("334",{bestPractice:1626,testId:334,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements with a role='searchbox' attribute, not intentionally hidden in the DOM and available to assistive technologies, has an aria-owns attribute set to a non-null value.",metaText:"This element (role=searchbox) should not have an aria-owns attribute set to a non-null value",metaTextDetail:"This {{tag-name}} (role=searchbox) should not have an aria-owns attribute set to a non-null value",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="searchbox"]','*[data-ae_vis][data-ae_avat][data-ae_ar="searchbox"][aria-owns]:not([aria-owns=""])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-337
	 * @returns {AutoTestShell} testShell
	 */function test337(){aeKernel.addAutomaticTest("337",{bestPractice:1626,testId:337,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='button' excluding a, area, input, button and textarea elements, elements with a disabled attribute; or aria-disabled attribute set to a 'true' value; or referenced by aria-activedescendant, not intentionally hidden in the DOM and available to assistive technologies lacks a tabindex attribute set to 0 or a negative value.",metaText:"This element (role=button) does not have a tabindex attribute set to 0 or a negative value",metaTextDetail:"This {{tag-name}} (role=button) does not have a tabindex attribute set to 0 or a negative value",fixType:"",testFunc:function(){const baseCssSelector='*[data-ae_vis][data-ae_avat][data-ae_ar="button"]:not(a):not(area):not(input):not(button):not(textarea):not([disabled]):not([aria-disabled="true"])';const testShell=new AutoTestShell(baseCssSelector);testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const badNodeCandidates=aeKernel.rootTestNode.querySelectorAll(`${baseCssSelector}[tabindex]:not([tabindex=""]):not([tabindex="0"]):not([tabindex^="-"])`);for(let i=0;i<badNodeCandidates.length;i++){const badNodeCandidate=badNodeCandidates[i];let isBadNode=true;// nodes whose ID is referenced by another node's aria-activedescendant attribute are not bad nodes
const badNodeCandidateId=badNodeCandidate.getAttribute('id');if(badNodeCandidateId){const isBadNodeCandidateActiveDescendant=aeKernel.rootTestNode.querySelector("[aria-activedescendant='"+badNodeCandidateId.replace(/'/g,"\\'")+"']");if(isBadNodeCandidateActiveDescendant){isBadNode=false;}}if(isBadNode){badNodes.push(badNodeCandidate);}}return badNodes;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-338
	 * @returns {AutoTestShell} testShell
	 */function test338(){aeKernel.addAutomaticTest("338",{bestPractice:1626,testId:338,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='checkbox' attribute, excluding elements with a disabled attribute; or aria-disabled attribute set to a 'true' value, not intentionally hidden in the DOM and available to assistive technologies, does not have a tabindex attribute set to 0 or a negative value",metaText:"This element (role=checkbox) does not have a tabindex attribute set to 0 or a negative value",metaTextDetail:"This {{tag-name}} (role=checkbox) does not have a tabindex attribute set to 0 or a negative value",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="checkbox"]:not([disabled]):not([aria-disabled="true"])','*[data-ae_vis][data-ae_avat][data-ae_ar="checkbox"]:not([disabled]):not([aria-disabled="true"])[tabindex]:not([tabindex=""]):not([tabindex="0"]):not([tabindex^="-"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-341
	 * @returns {AutoTestShell} testShell
	 */function test341(){aeKernel.addAutomaticTest("341",{bestPractice:1626,testId:341,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='link' attribute, excluding elements with a disabled attribute; or aria-disabled attribute set to a 'true' value, not intentionally hidden in the DOM and available to assistive technologies, does not have a tabindex attribute set to 0 or a negative value",metaText:"This element (role=link) does not have a tabindex attribute set to 0 or a negative value",metaTextDetail:"This {{tag-name}} (role=link) does not have a tabindex attribute set to 0 or a negative value",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="link"]:not([disabled]):not([aria-disabled="true"])','*[data-ae_vis][data-ae_avat][data-ae_ar="link"]:not([disabled]):not([aria-disabled="true"])[tabindex]:not([tabindex=""]):not([tabindex="0"]):not([tabindex^="-"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-350
	 * @returns {AutoTestShell} testShell
	 */function test350(){aeKernel.addAutomaticTest("350",{bestPractice:1626,testId:350,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='scrollbar' attribute, excluding elements with a disabled attribute; or aria-disabled attribute set to a 'true' value, not intentionally hidden in the DOM and available to assistive technologies, does not have a tabindex attribute set to 0 or a negative value",metaText:"This element (role=scrollbar) does not have a tabindex attribute set to 0 or a negative value",metaTextDetail:"This {{tag-name}} (role=scrollbar) does not have a tabindex attribute set to 0 or a negative value",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="scrollbar"]:not([disabled]):not([aria-disabled="true"])','*[data-ae_vis][data-ae_avat][data-ae_ar="scrollbar"]:not([disabled]):not([aria-disabled="true"])[tabindex]:not([tabindex=""]):not([tabindex="0"]):not([tabindex^="-"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-351
	 * @returns {AutoTestShell} testShell
	 */function test351(){aeKernel.addAutomaticTest("351",{bestPractice:1626,testId:351,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='slider' attribute, excluding elements with a disabled attribute; or aria-disabled attribute set to a 'true' value, not intentionally hidden in the DOM and available to assistive technologies, does not have a tabindex attribute set to 0 or a negative value",metaText:"This element (role=slider) does not have a tabindex attribute set to 0 or a negative value",metaTextDetail:"This {{tag-name}} (role=slider) does not have a tabindex attribute set to 0 or a negative value",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="slider"]:not([disabled]):not([aria-disabled="true"])','*[data-ae_vis][data-ae_avat][data-ae_ar="slider"]:not([disabled]):not([aria-disabled="true"])[tabindex]:not([tabindex=""]):not([tabindex="0"]):not([tabindex^="-"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-352
	 * @returns {AutoTestShell} testShell
	 */function test352(){aeKernel.addAutomaticTest("352",{bestPractice:1626,testId:352,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='spinbutton' attribute, excluding elements with a disabled attribute; or aria-disabled attribute set to a 'true' value, not intentionally hidden in the DOM and available to assistive technologies, does not have a tabindex attribute set to 0 or a negative value",metaText:"This element (role=spinbutton) does not have a tabindex attribute set to 0 or a negative value",metaTextDetail:"This {{tag-name}} (role=spinbutton) does not have a tabindex attribute set to 0 or a negative value",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="spinbutton"]:not([disabled]):not([aria-disabled="true"])','*[data-ae_vis][data-ae_avat][data-ae_ar="spinbutton"]:not([disabled]):not([aria-disabled="true"])[tabindex]:not([tabindex=""]):not([tabindex="0"]):not([tabindex^="-"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-356
	 * @returns {AutoTestShell} testShell
	 */function test356(){aeKernel.addAutomaticTest("356",{bestPractice:1626,testId:356,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='textbox' attribute, excluding elements with a disabled attribute; or aria-disabled attribute set to a 'true' value, not intentionally hidden in the DOM and available to assistive technologies, does not have a tabindex attribute set to 0 or a negative value",metaText:"This element (role=textbox) does not have a tabindex attribute set to 0 or a negative value",metaTextDetail:"This {{tag-name}} (role=textbox) does not have a tabindex attribute set to 0 or a negative value",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="textbox"]:not([disabled]):not([aria-disabled="true"])','*[data-ae_vis][data-ae_avat][data-ae_ar="textbox"]:not([disabled]):not([aria-disabled="true"])[tabindex]:not([tabindex=""]):not([tabindex="0"]):not([tabindex^="-"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-360
	 * @returns {AutoTestShell} testShell
	 */function test360(){aeKernel.addAutomaticTest("360",{bestPractice:1626,testId:360,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM and available to assistive technologies, have an aria-owns attribute that references the same id value as appears in the aria-owns attribute of another element.",metaText:"This element should not have an aria-owns attribute that references the same id value as appears in the aria-owns attribute of another element",metaTextDetail:"This {{tag-name}} should not have an aria-owns attribute of {{aria-owns}} that references the same id value as appears in the aria-owns attribute of another element",fixType:"",testFunc:function(){var testShell=new AutoTestShell('*[data-ae_vis][data-ae_avat][aria-owns]:not([aria-owns=""])');testShell.set_candidateSetNodesFunc(function(){var sameOwnsValues=[];var res=aeKernel.rootTestNode.querySelectorAll('*[data-ae_vis][data-ae_avat][aria-owns]:not([aria-owns=""])');for(var i=0,len_res=res.length;i<len_res;i++){var n=res[i];var ariaowns=n.getAttribute("aria-owns");var resWithOwns=aeKernel.rootTestNode.querySelectorAll('*[data-ae_vis][data-ae_avat][aria-owns="'+ariaowns.replace(/"/g,'\\"')+'"]');if(resWithOwns.length>1){sameOwnsValues.push(n);}}return sameOwnsValues;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-361
	 * @returns {AutoTestShell} testShell
	 */function test361(){aeKernel.addAutomaticTest("361",{bestPractice:1626,testId:361,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'command | composite | input | landmark | range | roletype | section | sectionhead | select | structure | widget | window'.",metaText:"Abstract ARIA roles are not allowed on any element",fixType:"",testFunc:function(){const relevantCssSelector='*[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"])';const failConditionSubSelectors=['[data-ae_ar="command"]','[data-ae_ar="composite"]','[data-ae_ar="input"]','[data-ae_ar="landmark"]','[data-ae_ar="range"]','[data-ae_ar="roletype"]','[data-ae_ar="section"]','[data-ae_ar="sectionhead"]','[data-ae_ar="select"]','[data-ae_ar="structure"]','[data-ae_ar="widget"]','[data-ae_ar="window"]'];return new AutoTestShell(relevantCssSelector,failConditionSubSelectors.map(subSelector=>`${relevantCssSelector}${subSelector}`).join(', '));}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-363
	 * @returns {AutoTestShell} testShell
	 */function test363(){aeKernel.addAutomaticTest("363",{bestPractice:389,testId:363,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more track elements with a kind='chapters' attribute, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a srclang attribute set to a text value.",metaText:"This element does not have a srclang attribute set to a text value",metaTextDetail:"This {{tag-name}} does not have a srclang attribute set to a text value",fixType:"",testFunc:function(){return new AutoTestShell('track[data-ae_ar="null"][data-ae_vis][data-ae_avat][kind="chapters"]','track[data-ae_ar="null"][data-ae_vis][data-ae_avat][kind="chapters"]:not([srclang]),track[data-ae_ar="null"][data-ae_vis][data-ae_avat][kind="chapters"][srclang=""]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-364
	 * @returns {AutoTestShell} testShell
	 */function test364(){aeKernel.addAutomaticTest("364",{bestPractice:389,testId:364,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more track elements with a kind='descriptions' attribute, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a srclang attribute set to a text value.",metaText:"This element does not have a srclang attribute set to a text value",metaTextDetail:"This {{tag-name}} does not have a srclang attribute set to a text value",fixType:"",testFunc:function(){return new AutoTestShell('track[data-ae_ar="null"][data-ae_vis][data-ae_avat][kind="descriptions"]','track[data-ae_ar="null"][data-ae_vis][data-ae_avat][kind="descriptions"]:not([srclang]),track[data-ae_ar="null"][data-ae_vis][data-ae_avat][kind="descriptions"][srclang=""]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-365
	 * @returns {AutoTestShell} testShell
	 */function test365(){aeKernel.addAutomaticTest("365",{bestPractice:389,testId:365,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more track elements with a kind='subtitles' attribute, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a srclang attribute set to a text value.",metaText:"This element does not have a srclang attribute set to a text value",metaTextDetail:"This {{tag-name}} does not have a srclang attribute set to a text value",fixType:"",testFunc:function(){return new AutoTestShell('track[data-ae_ar="null"][data-ae_vis][data-ae_avat][kind="subtitles"]','track[data-ae_ar="null"][data-ae_vis][data-ae_avat][kind="subtitles"]:not([srclang]),track[data-ae_ar="null"][data-ae_vis][data-ae_avat][kind="subtitles"][srclang=""]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-366
	 * @returns {AutoTestShell} testShell
	 */function test366(){aeKernel.addAutomaticTest("366",{bestPractice:444,testId:366,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more track elements with a kind='chapters' attribute, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a label attribute set to a text value.",metaText:"This element does not have a label attribute set to a text value",metaTextDetail:"This {{tag-name}} does not have a label attribute set to a text value",fixType:"",testFunc:function(){return new AutoTestShell('track[data-ae_ar="null"][data-ae_vis][data-ae_avat][kind="chapters"]','track[data-ae_ar="null"][data-ae_vis][data-ae_avat][kind="chapters"]:not([label]),track[data-ae_ar="null"][data-ae_vis][data-ae_avat][kind="chapters"][label=""]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-367
	 * @returns {AutoTestShell} testShell
	 */function test367(){aeKernel.addAutomaticTest("367",{bestPractice:2895,testId:367,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more track elements with a kind='descriptions' attribute, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a label attribute set to a text value.",metaText:"This element does not have a label attribute set to a text value",metaTextDetail:"This {{tag-name}} does not have a label attribute set to a text value",fixType:"",testFunc:function(){return new AutoTestShell('track[data-ae_ar="null"][data-ae_vis][data-ae_avat][kind="descriptions"]','track[data-ae_ar="null"][data-ae_vis][data-ae_avat][kind="descriptions"]:not([label]),track[data-ae_ar="null"][data-ae_vis][data-ae_avat][kind="descriptions"][label=""]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-368
	 * @returns {AutoTestShell} testShell
	 */function test368(){aeKernel.addAutomaticTest("368",{bestPractice:2049,testId:368,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more track elements with a kind='subtitles' attribute, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a label attribute set to a text value.",metaText:"This element does not have a label attribute set to a text value",metaTextDetail:"This {{tag-name}} does not have a label attribute set to a text value",fixType:"",testFunc:function(){return new AutoTestShell('track[data-ae_ar="null"][data-ae_vis][data-ae_avat][kind="subtitles"]','track[data-ae_ar="null"][data-ae_vis][data-ae_avat][kind="subtitles"]:not([label]),track[data-ae_ar="null"][data-ae_vis][data-ae_avat][kind="subtitles"][label=""]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-369
	 * @returns {AutoTestShell} testShell
	 */function test369(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("369",{bestPractice:338,testId:369,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='password' elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This input (type=password) element does not have a mechanism that allows an accessible name value to be calculated",fixType:{"fixType":1,"domSpec":false,"helperText":"Provide a descriptive label for the password field. ","fix":{"aria-label":null,"aria-labelledby":false}},testFunc:function(){return new AutoTestShell('input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="password"]','input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="password"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-370
	 * @returns {AutoTestShell} testShell
	 */function test370(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("370",{bestPractice:338,testId:370,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='search' elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This input (type=search) element does not have a mechanism that allows an accessible name value to be calculated",fixType:{"fixType":1,"domSpec":false,"helperText":"Provide a descriptive label for the search field.","fix":{"aria-label":null,"aria-labelledby":false}},testFunc:function(){return new AutoTestShell('input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="search"]','input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="search"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-371
	 * @returns {AutoTestShell} testShell
	 */function test371(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("371",{bestPractice:338,testId:371,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='tel' elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This input (type=tel) element does not have a mechanism that allows an accessible name value to be calculated",fixType:{"fixType":1,"domSpec":false,"helperText":"Provide a descriptive label for this telephone input field.","fix":{"aria-label":null,"aria-labelledby":false}},testFunc:function(){return new AutoTestShell('input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="tel"]','input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="tel"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-372
	 * @returns {AutoTestShell} testShell
	 */function test372(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("372",{bestPractice:338,testId:372,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='email' elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This input (type=email) element does not have a mechanism that allows an accessible name value to be calculated",fixType:{"fixType":1,"domSpec":false,"helperText":"Provide a descriptive label for this email field.","fix":{"aria-label":null,"aria-labelledby":false}},testFunc:function(){return new AutoTestShell('input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="email"]','input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="email"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-373
	 * @returns {AutoTestShell} testShell
	 */function test373(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("373",{bestPractice:338,testId:373,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='url' elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This input (type=url) element does not have a mechanism that allows an accessible name value to be calculated",fixType:{"fixType":1,"domSpec":false,"helperText":"Provide a descriptive label for this url field.","fix":{"aria-label":null,"aria-labelledby":false}},testFunc:function(){return new AutoTestShell('input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="url"]','input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="url"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-374
	 * @returns {AutoTestShell} testShell
	 */function test374(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("374",{bestPractice:338,testId:374,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='radio' elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This input (type=radio) element does not have a mechanism that allows an accessible name value to be calculated",fixType:{"fixType":1,"domSpec":false,"helperText":"Provide a descriptive label for this radio field.","fix":{"aria-label":null,"aria-labelledby":false}},testFunc:function(){return new AutoTestShell('input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="radio"]','input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="radio"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-375
	 * @returns {AutoTestShell} testShell
	 */function test375(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("375",{bestPractice:338,testId:375,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='checkbox' elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This input (type=checkbox) element does not have a mechanism that allows an accessible name value to be calculated",fixType:{"fixType":1,"domSpec":false,"helperText":"Provide a descriptive label for this checkbox field.","fix":{"aria-label":null,"aria-labelledby":false}},testFunc:function(){return new AutoTestShell('input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="checkbox"]','input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="checkbox"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-376
	 * @returns {AutoTestShell} testShell
	 */function test376(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("376",{bestPractice:338,testId:376,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input elements with no type attribute; or a type attribute set to a null value, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This input (no type specified) element does not have a mechanism that allows an accessible name value to be calculated",fixType:{"fixType":1,"domSpec":false,"helperText":"Provide a descriptive label for this text input field.","fix":{"aria-label":null,"aria-labelledby":false}},testFunc:function(){return new AutoTestShell('input[data-ae_ar="null"][data-ae_vis][data-ae_avat]:not([type]), input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type=""]','input[data-ae_ar="null"][data-ae_vis][data-ae_avat]:not([type]):not([data-ae_an]),input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type=""]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-385
	 * @returns {AutoTestShell} testShell
	 */function test385(){aeKernel.addAutomaticTest("385",{bestPractice:464,testId:385,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements with a role attribute set to the value 'listitem', not intentionally hidden in the DOM and available to assistive technologies does not have a ul element (without an ARIA-assigned role); ol element (without an ARIA-assigned role); an element with a role set to the value 'list' as a parent; or a ul element (without an ARIA-assigned role), ol element (without an ARIA-assigned role) or element with a role set to the value 'list' with an aria-owns attribute set to the ID of the element in the same DOM.",metaText:"This element does not have a ul element (without an ARIA-assigned role); ol element (without an ARIA-assigned role); an element with a role set to the value: list as a parent; or a ul element (without an ARIA-assigned role), ol element (without an ARIA-assigned role) or element with a role set to the value 'list' with an aria-owns attribute set to the ID of the element in the same DOM",metaTextDetail:"This {{tag-name}} does not have a ul element (without an ARIA-assigned role); ol element (without an ARIA-assigned role); an element with a role set to the value: list as a parent; or a ul element (without an ARIA-assigned role), ol element (without an ARIA-assigned role) or element with a role set to the value 'list' with an aria-owns attribute set to the ID of the element in the same DOM",fixType:"",testFunc:function(){const listItemCssSelector='*[data-ae_ar="listitem"][data-ae_vis][data-ae_avat]';const testShell=new AutoTestShell(listItemCssSelector);testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const ulNoRoleCssSelector='ul[data-ae_ar="null"][data-ae_vis][data-ae_avat]';const olNoRoleCssSelector='ol[data-ae_ar="null"][data-ae_vis][data-ae_avat]';const elementListRoleCssSelector='*[data-ae_ar="list"][data-ae_vis][data-ae_avat]';const badNodeCandidates=testUtil_ifDiff(aeKernel.rootTestNode,[listItemCssSelector,`${ulNoRoleCssSelector} > ${listItemCssSelector}, ${olNoRoleCssSelector} > ${listItemCssSelector}, ${elementListRoleCssSelector} > ${listItemCssSelector}`]);for(let i=0;i<badNodeCandidates.length;i++){const badNodeCandidate=badNodeCandidates[i];let isBadNode=true;const badNodeCandidateId=badNodeCandidate.getAttribute('id');if(badNodeCandidateId){const isBadNodeNotInShadowDOM=aeKernel.rootTestNode.querySelector(`[id='${badNodeCandidateId.replace(/'/g,"\\'")}']:not([data-ae_inshdw])`);if(isBadNodeNotInShadowDOM){// value for aria-owns can include multiple IDs in a space-delimited list
const ariaOwnsStartCssSelectorPostfix=`[aria-owns^='${badNodeCandidateId.replace(/'/g,"\\'")}']`;const ariaOwnsMiddleCssSelectorPostfix=`[aria-owns*='${" "+badNodeCandidateId.replace(/'/g,"\\'")}']`;const ariaOwnsEndCssSelectorPostfix=`[aria-owns$='${badNodeCandidateId.replace(/'/g,"\\'")}']`;const ulNoRoleAriaOwnsCssSelector=`${ulNoRoleCssSelector}${ariaOwnsStartCssSelectorPostfix}, ${ulNoRoleCssSelector}${ariaOwnsMiddleCssSelectorPostfix}, ${ulNoRoleCssSelector}${ariaOwnsEndCssSelectorPostfix}`;const olNoRoleAriaOwnsCssSelector=`${olNoRoleCssSelector}${ariaOwnsStartCssSelectorPostfix}, ${olNoRoleCssSelector}${ariaOwnsMiddleCssSelectorPostfix}, ${olNoRoleCssSelector}${ariaOwnsEndCssSelectorPostfix}`;const elementListRoleAriaOwnsCssSelector=`${elementListRoleCssSelector}${ariaOwnsStartCssSelectorPostfix}, ${elementListRoleCssSelector}${ariaOwnsMiddleCssSelectorPostfix}, ${elementListRoleCssSelector}${ariaOwnsEndCssSelectorPostfix}`;const badNodeCandidateOwner=aeKernel.rootTestNode.querySelector(`${ulNoRoleAriaOwnsCssSelector}, ${olNoRoleAriaOwnsCssSelector}, ${elementListRoleAriaOwnsCssSelector}`);if(badNodeCandidateOwner){isBadNode=false;}}}if(isBadNode){badNodes.push(badNodeCandidate);}}return badNodes;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-386
	 * @returns {AutoTestShell} testShell
	 */function test386(){aeKernel.addAutomaticTest("386",{bestPractice:361,testId:386,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more td elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a headers attribute value that includes one or more invalid ids.",metaText:"This data cell has a headers attribute value that includes one or more invalid or duplicate ids",metaTextDetail:"This data cell has a headers attribute value of '{{headers}}' that includes one or more invalid or duplicate ids",fixType:"",testFunc:function(){var testShell=new AutoTestShell('td[data-ae_vis][data-ae_avat][headers]:not([headers=""])');testShell.set_candidateSetNodesFunc(function(){var nodesWithBadIds=[];var res=aeKernel.rootTestNode.querySelectorAll('td[data-ae_vis][data-ae_avat][headers]:not([headers=""])');for(var i=0,len_res=res.length;i<len_res;i++){var n=res[i];var headersIds=n.getAttribute("headers").trim().split(" ");for(var j=0,len_headersIds=headersIds.length;j<len_headersIds;j++){var headersId=headersIds[j];if(isValidIdOrName(headersId)===true){// "[id=" is used in place of "#" + id as ids can included colons which breaks css selector, although is valid html
var refNode=aeKernel.rootTestNode.querySelectorAll("[id='"+headersId.replace(/'/g,"\\'")+"']");var refNodeLength=refNode.length;if(refNodeLength===0||refNodeLength>1){nodesWithBadIds.push(n);break;}}else{nodesWithBadIds.push(n);break;}}}return nodesWithBadIds;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-387
	 * @returns {AutoTestShell} testShell
	 */function test387(){aeKernel.addStage4MarkdownFunc("identifiesDataTables",identifiesDataTables);aeKernel.addAutomaticTest("387",{bestPractice:375,testId:387,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more table elements that hold table data, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not contain one or more th elements (without an ARIA-assigned role); th elements with a role set to: 'rowheader | columnheader'; or td elements with a role set to: 'rowheader | columnheader'.",metaText:"This element does not contain one or more: th elements (without an ARIA-assigned role); th elements with a role set to: rowheader | columnheader; or td elements with a role set to: rowheader | columnheader",metaTextDetail:"This {{tag-name}} does not contain one or more: th elements (without an ARIA-assigned role); th elements with a role set to: rowheader | columnheader; or td elements with a role set to: rowheader | columnheader",fixType:"",testFunc:function(){var testShell=new AutoTestShell('table[data-ae_vis][data-ae_avat][data-ae_dtab][data-ae_ar="null"]');testShell.set_candidateSetNodesFunc(function(){var tables=[];var res=aeKernel.rootTestNode.querySelectorAll('table[data-ae_vis][data-ae_avat][data-ae_dtab][data-ae_ar="null"]');for(var i=0,len_res=res.length;i<len_res;i++){var n=res[i];var refNode=n.querySelectorAll("th[data-ae_vis][data-ae_avat][data-ae_ar='null'], th[data-ae_vis][data-ae_avat][data-ae_ar='rowheader'], th[data-ae_vis][data-ae_avat][data-ae_ar='columnheader'], td[data-ae_vis][data-ae_avat][data-ae_ar='rowheader'], td[data-ae_vis][data-ae_avat][data-ae_ar='columnheader']");var refNodeLength=refNode.length;if(refNodeLength===0){tables.push(n);}}return tables;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-388
	 * @returns {AutoTestShell} testShell
	 */function test388(){aeKernel.addStage4MarkdownFunc("identifiesDataTables",identifiesDataTables);aeKernel.addAutomaticTest("388",{bestPractice:590,testId:388,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more td elements in a data-table, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, contains at least one table element (without an ARIA-assigned role).",metaText:"Table data cells should not contain a table element (without an ARIA-assigned role)",fixType:"",testFunc:function(){var testShell=new AutoTestShell('table[data-ae_vis][data-ae_avat][data-ae_ar="null"][data-ae_dtab="true"] td[data-ae_vis][data-ae_avat][data-ae_ar="null"]');testShell.set_candidateSetNodesFunc(function(){// clone body
// var clone = aeKernel.rootTestNode.querySelector("body").cloneNode(true);
const testNode=aeKernel.rootTestNode.querySelector("body")||aeKernel.rootTestNode;// fall back to using root node if body element is not a child of root node
var clone=cloneThisNode(testNode,true);// so we don't pick up th elements within the nested tables
var tableInTD=clone.querySelector("td[data-ae_vis][data-ae_avat][data-ae_ar='null'] table[data-ae_vis][data-ae_avat][data-ae_ar='null']:not(:empty)");while(tableInTD){tableInTD.innerHTML="";tableInTD=clone.querySelector("td[data-ae_vis][data-ae_avat][data-ae_ar='null'] table[data-ae_vis][data-ae_avat][data-ae_ar='null']:not(:empty)");}var tdElementsThatContainTables=[];var res=clone.querySelectorAll('table[data-ae_vis][data-ae_avat][data-ae_ar="null"][data-ae_dtab="true"] td[data-ae_vis][data-ae_avat][data-ae_ar="null"]');for(var i=0,len_res=res.length;i<len_res;i++){var tdOfInterest=res[i];var tdContainsTables=tdOfInterest.querySelectorAll('table[data-ae_vis][data-ae_avat][data-ae_ar="null"]');if(tdContainsTables.length>0){tdElementsThatContainTables.push(tdOfInterest);}}return tdElementsThatContainTables;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-389
	 * @returns {AutoTestShell} testShell
	 */function test389(){aeKernel.addAutomaticTest("389",{bestPractice:361,testId:389,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more th elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, with a rowspan attribute set to a value of '0'; or '2' or more, does not have a scope attribute set to a text value of 'rowgroup'; or an id attribute set to a non-null value.",metaText:"This header cell does not have a scope attribute set to a text value of: rowgroup; or alternatively an id attribute set to a non-null text value",fixType:"",testFunc:function(){return new AutoTestShell('th[data-ae_vis][data-ae_avat][data-ae_ar="null"][rowspan]:not([rowspan="1"]):not([rowspan=""])','th[data-ae_vis][data-ae_avat][data-ae_ar="null"][rowspan]:not([rowspan="1"]):not([rowspan=""]):not([scope="rowgroup"]):not([id]), th[data-ae_vis][data-ae_avat][data-ae_ar="null"][rowspan]:not([rowspan="1"]):not([scope="rowgroup"])[id=""]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-390
	 * @returns {AutoTestShell} testShell
	 */function test390(){aeKernel.addAutomaticTest("390",{bestPractice:361,testId:390,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more th elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, with a colspan attribute set to a value of '0'; or '2' or more, does not have a scope attribute set to a text value of 'colgroup'; or an id attribute set to a non-null value.",metaText:"This header cell does not have a scope attribute set to a text value of: colgroup; or alternatively an id attribute set to a non-null text value",fixType:"",testFunc:function(){return new AutoTestShell('th[data-ae_vis][data-ae_avat][data-ae_ar="null"][colspan]:not([colspan="1"]):not([colspan=""])','th[data-ae_vis][data-ae_avat][data-ae_ar="null"][colspan]:not([colspan="1"]):not([colspan=""]):not([scope="colgroup"]):not([id]), th[data-ae_vis][data-ae_avat][data-ae_ar="null"][colspan]:not([colspan="1"]):not([scope="colgroup"])[id=""]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-391
	 * @returns {AutoTestShell} testShell
	 */function test391(){aeKernel.addAutomaticTest("391",{bestPractice:361,testId:391,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more th elements that is in a table in which other elements contain scope attributes set to non-null values, excluding those elements with a rowspan or colspan attribute set to a value of '0'; or '2' or more, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a scope attribute set to a text value of 'row' or 'col'.",metaText:"This header cell does not have a scope attribute set to a text value of: row; or col",fixType:"",testFunc:function(){const testShell=new AutoTestShell();testShell.set_applicabilityTest(function(){const res=this.querySelectorAll('table[data-ae_ar="null"][data-ae_vis][data-ae_avat]');for(let i=0;i<res.length;i++){const n=res[i];const refNodes=n.querySelectorAll('th[data-ae_ar="null"][data-ae_vis][data-ae_avat][scope]:not([scope=""]), td[data-ae_ar="null"][data-ae_vis][data-ae_avat][scope]:not([scope=""])');if(refNodes.length>0){return true;}}return false;});testShell.set_applicableNodesFunc(function(){const applicableNodes=[];const res=this.querySelectorAll('table[data-ae_ar="null"][data-ae_vis][data-ae_avat]');for(let i=0;i<res.length;i++){const n=res[i];const refNodes=n.querySelectorAll('th[data-ae_ar="null"][data-ae_vis][data-ae_avat][scope]:not([scope=""]), td[data-ae_ar="null"][data-ae_vis][data-ae_avat][scope]:not([scope=""])');for(let j=0;j<refNodes.length;j++){const refNode=refNodes[j];applicableNodes.push(refNode);}}return applicableNodes;});testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const failConditionSubSelectors=[':not([scope]):not([colspan]):not([rowspan])',':not([colspan])[rowspan=""]',':not([scope]):not([colspan])[rowspan="1"]',':not([scope]):not([rowspan])[colspan=""]',':not([scope]):not([rowspan])[colspan="1"]','[scope=""]:not([colspan]):not([rowspan])','[scope=""]:not([colspan])[rowspan=""]','[scope=""]:not([colspan])[rowspan="1"]','[scope=""]:not([rowspan])[colspan=""]','[scope=""]:not([rowspan])[colspan="1"]','[scope]:not([scope=""]):not([scope="row"]):not([scope="col"]):not([colspan]):not([rowspan])','[scope]:not([scope=""]):not([scope="row"]):not([scope="col"]):not([colspan])[rowspan=""]','[scope]:not([scope=""]):not([scope="row"]):not([scope="col"]):not([colspan])[rowspan="1"]','[scope]:not([scope=""]):not([scope="row"]):not([scope="col"]):not([rowspan])[colspan=""]','[scope]:not([scope=""]):not([scope="row"]):not([scope="col"]):not([rowspan])[colspan="1"]'];const res=this.querySelectorAll('table[data-ae_ar="null"][data-ae_vis][data-ae_avat]');for(let i=0;i<res.length;i++){const n=res[i];const refNodes=n.querySelectorAll(failConditionSubSelectors.map(subSelector=>`th[data-ae_ar="null"][data-ae_vis][data-ae_avat]${subSelector}`).join(', '));for(let j=0;j<refNodes.length;j++){const refNode=refNodes[j];badNodes.push(refNode);}}return badNodes;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-392
	 * @returns {AutoTestShell} testShell
	 */function test392(){aeKernel.addAutomaticTest("392",{bestPractice:361,testId:392,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more th elements that is in a table in which other elements contain scope attributes set to a non-null value, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, with a rowspan attribute set to a value of '0'; or '2' or more; does not have a scope attribute set to a text value of 'rowgroup'.",metaText:"This header cell does not have a scope attribute set to a text value of: rowgroup",fixType:"",testFunc:function(){const testShell=new AutoTestShell();testShell.set_applicabilityTest(function(){const res=this.querySelectorAll('table[data-ae_ar="null"][data-ae_vis][data-ae_avat]');for(let i=0;i<res.length;i++){const n=res[i];const refNodes=n.querySelectorAll('th[data-ae_ar="null"][data-ae_vis][data-ae_avat][scope]:not([scope=""]), td[data-ae_ar="null"][data-ae_vis][data-ae_avat][scope]:not([scope=""])');if(refNodes.length>0){return true;}}return false;});testShell.set_applicableNodesFunc(function(){const applicableNodes=[];const res=this.querySelectorAll('table[data-ae_ar="null"][data-ae_vis][data-ae_avat]');for(let i=0;i<res.length;i++){const n=res[i];const refNodes=n.querySelectorAll('th[data-ae_ar="null"][data-ae_vis][data-ae_avat][scope]:not([scope=""]), td[data-ae_ar="null"][data-ae_vis][data-ae_avat][scope]:not([scope=""])');for(let j=0;j<refNodes.length;j++){const refNode=refNodes[j];applicableNodes.push(refNode);}}return applicableNodes;});testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const failConditionSubSelectors=[':not([scope])[rowspan]:not([rowspan="1"])','[scope=""][rowspan]:not([rowspan="1"])','[scope]:not([scope=""]):not([scope="rowgroup"])[rowspan]:not([rowspan="1"])'];const res=this.querySelectorAll('table[data-ae_ar="null"][data-ae_vis][data-ae_avat]');for(let i=0;i<res.length;i++){const n=res[i];const refNodes=n.querySelectorAll(failConditionSubSelectors.map(subSelector=>`th[data-ae_ar="null"][data-ae_vis][data-ae_avat]${subSelector}`).join(', '));for(let j=0;j<refNodes.length;j++){const refNode=refNodes[j];badNodes.push(refNode);}}return badNodes;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-393
	 * @returns {AutoTestShell} testShell
	 */function test393(){aeKernel.addAutomaticTest("393",{bestPractice:361,testId:393,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more th elements that is in a table in which other elements contain scope attributes set to a non-null value, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, with a colspan attribute set to a value of '0'; or '2' or more; does not have a scope attribute set to a text value of 'colgroup'.",metaText:"This header cell does not have a scope attribute set to a text value of: colgroup",fixType:"",testFunc:function(){const testShell=new AutoTestShell();testShell.set_applicabilityTest(function(){const res=this.querySelectorAll('table[data-ae_ar="null"][data-ae_vis][data-ae_avat]');for(let i=0;i<res.length;i++){const n=res[i];const refNodes=n.querySelectorAll('th[data-ae_ar="null"][data-ae_vis][data-ae_avat][scope]:not([scope=""]), td[data-ae_ar="null"][data-ae_vis][data-ae_avat][scope]:not([scope=""])');if(refNodes.length>0){return true;}}return false;});testShell.set_applicableNodesFunc(function(){const applicableNodes=[];const res=this.querySelectorAll('table[data-ae_ar="null"][data-ae_vis][data-ae_avat]');for(let i=0;i<res.length;i++){const n=res[i];const refNodes=n.querySelectorAll('th[data-ae_ar="null"][data-ae_vis][data-ae_avat][scope]:not([scope=""]), td[data-ae_ar="null"][data-ae_vis][data-ae_avat][scope]:not([scope=""])');for(let j=0;j<refNodes.length;j++){const refNode=refNodes[j];applicableNodes.push(refNode);}}return applicableNodes;});testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const failConditionSubSelectors=[':not([scope])[colspan]:not([colspan="1"])','[scope=""][colspan]:not([colspan="1"])','[scope]:not([scope=""]):not([scope="colgroup"])[colspan]:not([colspan="1"])'];const res=this.querySelectorAll('table[data-ae_ar="null"][data-ae_vis][data-ae_avat]');for(let i=0;i<res.length;i++){const n=res[i];const refNodes=n.querySelectorAll(failConditionSubSelectors.map(subSelector=>`th[data-ae_ar="null"][data-ae_vis][data-ae_avat]${subSelector}`).join(', '));for(let j=0;j<refNodes.length;j++){const refNode=refNodes[j];badNodes.push(refNode);}}return badNodes;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-394
	 * @returns {AutoTestShell} testShell
	 */function test394(){aeKernel.addAutomaticTest("394",{bestPractice:361,testId:394,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more th elements that is in a table in which other elements contain headers attributes set to a non-null value, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have an id attribute set to a non-null value.",metaText:"This element does not have an id attribute set to a non-null text value",metaTextDetail:"This {{tag-name}} does not have an id attribute set to a non-null text value",fixType:"",testFunc:function(){const testShell=new AutoTestShell();testShell.set_applicabilityTest(function(){const res=this.querySelectorAll('table[data-ae_ar="null"][data-ae_vis][data-ae_avat]');for(let i=0;i<res.length;i++){const n=res[i];const refNodes=n.querySelectorAll('td[data-ae_ar="null"][data-ae_vis][data-ae_avat][headers]:not([headers=""])');if(refNodes.length>0){return true;}}return false;});testShell.set_applicableNodesFunc(function(){const applicableNodes=[];const res=this.querySelectorAll('table[data-ae_ar="null"][data-ae_vis][data-ae_avat]');for(let i=0;i<res.length;i++){const n=res[i];const refNodes=n.querySelectorAll('td[data-ae_ar="null"][data-ae_vis][data-ae_avat][headers]:not([headers=""])');for(let j=0;j<refNodes.length;j++){const refNode=refNodes[j];applicableNodes.push(refNode);}}return applicableNodes;});testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const failConditionSubSelectors=[':not([id])','[id=""]'];const res=this.querySelectorAll('table[data-ae_ar="null"][data-ae_vis][data-ae_avat]');for(let i=0;i<res.length;i++){const n=res[i];const refNodes=n.querySelectorAll(failConditionSubSelectors.map(subSelector=>`th[data-ae_ar="null"][data-ae_vis][data-ae_avat]${subSelector}`).join(', '));for(let j=0;j<refNodes.length;j++){const refNode=refNodes[j];badNodes.push(refNode);}}return badNodes;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-395
	 * @returns {AutoTestShell} testShell
	 */function test395(){aeKernel.addStage4MarkdownFunc("identifiesDataTables",identifiesDataTables);aeKernel.addAutomaticTest("395",{bestPractice:361,testId:395,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more td elements (excluding, if present, an empty td element which is found as the first element in the first row) in a data-table in which other elements contain headers attributes set to a non-null value, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a headers attribute set to a non-null value.",metaText:"This element does not have a headers attribute set to a non-null text value",metaTextDetail:"This {{tag-name}} does not have a headers attribute set to a non-null text value",fixType:"",testFunc:function(){const testShell=new AutoTestShell();testShell.set_applicabilityTest(function(){const res=this.querySelectorAll('table[data-ae_dtab="true"][data-ae_ar="null"][data-ae_vis][data-ae_avat]');for(let i=0;i<res.length;i++){const n=res[i];const refNodes=n.querySelectorAll('td[data-ae_ar="null"][data-ae_vis][data-ae_avat][headers]:not([headers=""])');if(refNodes.length>0){return true;}}return false;});testShell.set_applicableNodesFunc(function(){const applicableNodes=[];const res=this.querySelectorAll('table[data-ae_dtab="true"][data-ae_ar="null"][data-ae_vis][data-ae_avat]');for(let i=0;i<res.length;i++){const n=res[i];const refNodes=n.querySelectorAll('td[data-ae_ar="null"][data-ae_vis][data-ae_avat][headers]:not([headers=""])');for(let j=0;j<refNodes.length;j++){const refNode=refNodes[j];applicableNodes.push(refNode);}}return applicableNodes;});testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const res=this.querySelectorAll('table[data-ae_dtab="true"][data-ae_ar="null"][data-ae_vis][data-ae_avat]');for(let i=0;i<res.length;i++){const n=res[i];const refNodes=testUtil_ifDiff(n,["td[data-ae_vis][data-ae_avat][data-ae_ar='null']:not([headers]), td[data-ae_vis][data-ae_avat][data-ae_ar='null'][headers='']","tr[data-ae_vis][data-ae_avat][data-ae_ar='null']:first-child td:first-child:empty"]);for(let j=0;j<refNodes.length;j++){const refNode=refNodes[j];badNodes.push(refNode);}}return badNodes;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-396
	 * @returns {AutoTestShell} testShell
	 */function test396(){aeKernel.addAutomaticTest("396",{bestPractice:464,testId:396,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more dd elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a dl element (without an ARIA-assigned role) as a parent or a parent div's parent.",metaText:"This element does not have a dl element (without an ARIA-assigned role) as a parent or a parent div's parent",metaTextDetail:"This {{tag-name}} does not have a dl element (without an ARIA-assigned role) as a parent or a parent div's parent",fixType:"",testFunc:function(){const testShell=new AutoTestShell('dd[data-ae_ar="null"][data-ae_vis][data-ae_avat]');testShell.set_candidateSetNodesFunc(function(){return testUtil_ifDiff(aeKernel.rootTestNode,['dd[data-ae_ar="null"][data-ae_vis][data-ae_avat]','dl[data-ae_ar="null"][data-ae_vis][data-ae_avat] > dd[data-ae_ar="null"][data-ae_vis][data-ae_avat], dl[data-ae_ar="null"][data-ae_vis][data-ae_avat] > div[data-ae_ar="null"][data-ae_vis][data-ae_avat] > dd[data-ae_ar="null"][data-ae_vis][data-ae_avat]']);});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-397
	 * @returns {AutoTestShell} testShell
	 */function test397(){aeKernel.addAutomaticTest("397",{bestPractice:464,testId:397,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more dt elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a dl element (without an ARIA-assigned role) as a parent or a parent div's parent.",metaText:"This element does not have a dl element (without an ARIA-assigned role) as a parent or a parent div's parent",metaTextDetail:"This {{tag-name}} does not have a dl element (without an ARIA-assigned role) as a parent or a parent div's parent",fixType:"",testFunc:function(){const testShell=new AutoTestShell('dt[data-ae_ar="null"][data-ae_vis][data-ae_avat]');testShell.set_candidateSetNodesFunc(function(){return testUtil_ifDiff(aeKernel.rootTestNode,['dt[data-ae_ar="null"][data-ae_vis][data-ae_avat]','dl[data-ae_ar="null"][data-ae_vis][data-ae_avat] > dt[data-ae_ar="null"][data-ae_vis][data-ae_avat], dl[data-ae_ar="null"][data-ae_vis][data-ae_avat] > div[data-ae_ar="null"][data-ae_vis][data-ae_avat] > dt[data-ae_ar="null"][data-ae_vis][data-ae_avat]']);});return testShell;}});}/**
	 * Holds group type for input elements
	 */const testUtil_ariaRoleLookupTable={"command":true,"composite":true,"input":true,"landmark":true,"range":true,"roletype":true,"section":true,"sectionhead":true,"select":true,"structure":true,"widget":true,"window":true,"alert":true,"alertdialog":true,"button":true,"checkbox":true,"dialog":true,"gridcell":true,"link":true,"log":true,"marquee":true,"menuitem":true,"menuitemcheckbox":true,"menuitemradio":true,"option":true,"progressbar":true,"radio":true,"scrollbar":true,"searchbox":true,"slider":true,"spinbutton":true,"status":true,"switch":true,"tab":true,"tabpanel":true,"textbox":true,"timer":true,"tooltip":true,"treeitem":true,"combobox":true,"grid":true,"listbox":true,"menu":true,"menubar":true,"radiogroup":true,"tablist":true,"tree":true,"treegrid":true,"application":true,"article":true,"cell":true,"columnheader":true,"definition":true,"directory":true,"document":true,"feed":true,"figure":true,"group":true,"heading":true,"img":true,"list":true,"listitem":true,"math":true,"none":true,"note":true,"presentation":true,"region":true,"row":true,"rowgroup":true,"rowheader":true,"separator":true,"table":true,"term":true,"toolbar":true,"banner":true,"complementary":true,"contentinfo":true,"form":true,"main":true,"navigation":true,"search":true,"graphics-document":true,"graphics-object":true,"graphics-symbol":true,"doc-abstract":true,"doc-acknowledgments":true,"doc-afterword":true,"doc-appendix":true,"doc-backlink":true,"doc-biblioentry":true,"doc-bibliography":true,"doc-biblioref":true,"doc-chapter":true,"doc-colophon":true,"doc-conclusion":true,"doc-cover":true,"doc-credit":true,"doc-credits":true,"doc-dedication":true,"doc-endnote":true,"doc-endnotes":true,"doc-epigraph":true,"doc-epilogue":true,"doc-errata":true,"doc-example":true,"doc-footnote":true,"doc-foreword":true,"doc-glossary":true,"doc-glossref":true,"doc-index":true,"doc-introduction":true,"doc-noteref":true,"doc-notice":true,"doc-pagebreak":true,"doc-pagelist":true,"doc-part":true,"doc-preface":true,"doc-prologue":true,"doc-pullquote":true,"doc-qna":true,"doc-subtitle":true,"doc-tip":true,"doc-toc":true};/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-412
	 * @returns {AutoTestShell} testShell
	 */function test412(){aeKernel.addAutomaticTest("412",{bestPractice:1626,testId:412,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM, has a role attribute set to an invalid aria role value.",metaText:"The role attribute value given to this element is not allowed, as it is not a valid aria role.",metaTextDetail:"The role attribute of {{data-ae_ar}} given to this {{tag-name}} is not allowed, as it is not a valid aria role.",fixType:{"fixType":1,"domSpec":false,"helperText":"Specify a valid aria role for this element.","fix":{"role":null}},testFunc:function(){var testShell=new AutoTestShell('*[data-ae_ar]:not([data-ae_ar="null"])[data-ae_vis]');testShell.set_candidateSetNodesFunc(function(){var notProper=[];var res=aeKernel.rootTestNode.querySelectorAll('*[data-ae_ar]:not([data-ae_ar="null"])[data-ae_vis]');for(var i=0,len_res=res.length;i<len_res;i++){var result=res[i];var role=result.getAttribute("data-ae_ar").toLowerCase();if(testUtil_ariaRoleLookupTable[role]!==true){//ENG-412 Exception made for span, div and p to allow role="text"
var tag=result.tagName.toLowerCase();if(!(['span','div','p'].includes(tag)&&role==='text')){notProper[notProper.length]=result;}}}return notProper;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-477
	 * @returns {AutoTestShell} testShell
	 */function test477(){aeKernel.addAutomaticTest("477",{bestPractice:1626,testId:477,introduced:"2.1",mediaType:mediaTypes.WEB,description:"One or more body elements, not intentionally hidden in the DOM and available to assistive technologies, has an aria-hidden attribute set to the value: 'true'.",metaText:"This element has an aria-hidden attribute set to 'true'",metaTextDetail:"This {{tag-name}} has an aria-hidden attribute set to 'true'",fixType:"",testFunc:function(){const relevantCssSelector='body[data-ae_vis]';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[aria-hidden="true"]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-485
	 * @returns {AutoTestShell} testShell
	 */function test485(){aeKernel.addAutomaticTest("485",{bestPractice:2049,testId:485,introduced:"2.2",mediaType:mediaTypes.WEB,description:"One or more track elements with a kind='captions' attribute, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a src attribute set to a text value.",metaText:"This track element does not have a src attribute or it is set to a null text value",fixType:"",testFunc:function(){return new AutoTestShell('track[kind="captions"][data-ae_vis][data-ae_avat][data-ae_ar="null"]','track[kind="captions"][data-ae_vis][data-ae_avat][data-ae_ar="null"]:not([src]),track[kind="captions"][data-ae_vis][data-ae_avat][data-ae_ar="null"][src=""]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-509
	 * @returns {AutoTestShell} testShell
	 */function test509(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("509",{bestPractice:338,testId:509,introduced:"2.3",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='textbox' attribute, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name to be calculated.",metaText:"This element (role=textbox) does not have a mechanism that allows an accessible name value to be calculated",metaTextDetail:"This {{tag-name}} (role=textbox) does not have a mechanism that allows an accessible name value to be calculated",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="textbox"]','*[data-ae_vis][data-ae_avat][data-ae_ar="textbox"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-508
	 * @returns {AutoTestShell} testShell
	 */function test508(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("508",{bestPractice:338,testId:508,introduced:"2.3",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='slider' attribute, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name to be calculated.",metaText:"This element (role=slider) does not have a mechanism that allows an accessible name value to be calculated",metaTextDetail:"This {{tag-name}} (role=slider) does not have a mechanism that allows an accessible name value to be calculated",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="slider"]','*[data-ae_vis][data-ae_avat][data-ae_ar="slider"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-507
	 * @returns {AutoTestShell} testShell
	 */function test507(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("507",{bestPractice:338,testId:507,introduced:"2.3",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='radio' attribute, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name to be calculated.",metaText:"This element (role=radio) does not have a mechanism that allows an accessible name value to be calculated",metaTextDetail:"This {{tag-name}} (role=radio) does not have a mechanism that allows an accessible name value to be calculated",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="radio"]','*[data-ae_vis][data-ae_avat][data-ae_ar="radio"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-505
	 * @returns {AutoTestShell} testShell
	 */function test505(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("505",{bestPractice:338,testId:505,introduced:"2.3",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='checkbox' attribute, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name to be calculated.",metaText:"This element (role=checkbox) does not have a mechanism that allows an accessible name value to be calculated",metaTextDetail:"This {{tag-name}} (role=checkbox) does not have a mechanism that allows an accessible name value to be calculated",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="checkbox"]','*[data-ae_vis][data-ae_avat][data-ae_ar="checkbox"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-511
	 * @returns {AutoTestShell} testShell
	 */function test511(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("511",{bestPractice:338,testId:511,introduced:"2.3",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='button' attribute, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name to be calculated.",metaText:"This element (role=button) does not have a mechanism that allows an accessible name value to be calculated",metaTextDetail:"This {{tag-name}} (role=button) does not have a mechanism that allows an accessible name value to be calculated",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="button"]','*[data-ae_vis][data-ae_avat][data-ae_ar="button"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-510
	 * @returns {AutoTestShell} testShell
	 */function test510(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("510",{bestPractice:338,testId:510,introduced:"2.3",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='combobox' attribute, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name to be calculated.",metaText:"This element (role=combobox) does not have a mechanism that allows an accessible name value to be calculated",metaTextDetail:"This {{tag-name}} (role=combobox) does not have a mechanism that allows an accessible name value to be calculated",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="combobox"]','*[data-ae_vis][data-ae_avat][data-ae_ar="combobox"]:not([data-ae_an])');}});}/**
	 * To be superceded by Machine Learning
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-520
	 * @returns {AutoTestShell} testShell
	 */function test520(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("520",{bestPractice:523,testId:520,introduced:"2.3",mediaType:mediaTypes.WEB,description:"One or more iframe elements, excluding those elements with aria-hidden='true' or tabindex set to a negative value, not intentionally hidden in the DOM, has a suspicious calculated accessible name value.",metaText:"This iframe element has a suspicious calculated accessible name value",metaTextDetail:"This iframe element has a suspicious calculated accessible name value of: {{data-ae_an}}",fixType:{"fixType":1,"domSpec":true,"helperText":"Specify a descriptive label for the frame.","fix":{"aria-label":null,"aria-labelledby":false}},testFunc:function(){const relevantCssSelectorPrefix='iframe:not([aria-hidden="true"]):not([tabindex^="-"])[data-ae_vis]';return new AutoTestShell(`${relevantCssSelectorPrefix}[data-ae_an]`,`${relevantCssSelectorPrefix}[data-ae_an="title"], ${relevantCssSelectorPrefix}[data-ae_an="frame"], ${relevantCssSelectorPrefix}[data-ae_an="iframe"], ${relevantCssSelectorPrefix}[data-ae_an="top"], ${relevantCssSelectorPrefix}[data-ae_an="bottom"], ${relevantCssSelectorPrefix}[data-ae_an="left"], ${relevantCssSelectorPrefix}[data-ae_an="right"]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-527
	 * @returns {AutoTestShell} testShell
	 */function test527(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("527",{bestPractice:2519,testId:527,introduced:"2.3",mediaType:mediaTypes.WEB,description:"Two or more aside elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, have accessible names that are non-unique values",metaText:"This aside does not have a unique accessible name as required when two or more aside elements are present",fixType:"",testFunc:function(){const testShell=new AutoTestShell();const relevantCssSelector='aside[data-ae_ar="null"][data-ae_vis][data-ae_avat]';testShell.set_applicabilityTest(function(){return this.applicableNodesFunc(this._applicableNodesFuncArgs).length>0;});testShell.set_applicableNodesFunc(function(){// if there is more than 1 aside element then the test is applicable
const applicableNodes=this.querySelectorAll(relevantCssSelector);return applicableNodes.length>1?applicableNodes:[];});testShell.set_candidateSetNodesFunc(function(){var notUnique=[];var nodeStore={};var res=this.querySelectorAll(relevantCssSelector);if(res.length>1){for(var i=0;i<res.length;i++){var n=res[i];var thean=n.getAttribute("data-ae_an");try{nodeStore[thean].push(n);}catch(err){nodeStore[thean]=[];nodeStore[thean].push(n);}}var keys=Object.keys(nodeStore);for(var j=0;j<keys.length;j++){// collecting all duplicated keys
var nArray=nodeStore[keys[j]];if(nArray.length>1){notUnique=notUnique.concat(nArray);}}}return notUnique;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-533
	 * @returns {AutoTestShell} testShell
	 */function test533(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("533",{bestPractice:2519,testId:533,introduced:"2.4",mediaType:mediaTypes.WEB,description:"Two or more nav elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism for providing a calculated accessible name",metaText:"This nav does not have an accessible name as required when two or more nav elements are present",fixType:"",testFunc:function(){const relevantCssSelector='nav[data-ae_ar="null"][data-ae_vis][data-ae_avat]';const testShell=new AutoTestShell(null,`${relevantCssSelector}:not([data-ae_an])`);testShell.set_applicabilityTest(function(){return this.applicableNodesFunc(this._applicableNodesFuncArgs).length>0;});testShell.set_applicableNodesFunc(function(){// if there is more than 1 aside element then the test is applicable
const applicableNodes=this.querySelectorAll(relevantCssSelector);return applicableNodes.length>1?applicableNodes:[];});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-532
	 * @returns {AutoTestShell} testShell
	 */function test532(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("532",{bestPractice:2519,testId:532,introduced:"2.4",mediaType:mediaTypes.WEB,description:"Two or more aside elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism for providing a calculated accessible name",metaText:"This aside does not have an accessible name as required when two or more aside elements are present",fixType:"",testFunc:function(){const relevantCssSelector='aside[data-ae_ar="null"][data-ae_vis][data-ae_avat]';const testShell=new AutoTestShell(null,`${relevantCssSelector}:not([data-ae_an])`);testShell.set_applicabilityTest(function(){return this.applicableNodesFunc(this._applicableNodesFuncArgs).length>0;});testShell.set_applicableNodesFunc(function(){// if there is more than 1 aside element then the test is applicable
const applicableNodes=this.querySelectorAll(relevantCssSelector);return applicableNodes.length>1?applicableNodes:[];});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-531
	 * @returns {AutoTestShell} testShell
	 */function test531(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("531",{bestPractice:2519,testId:531,introduced:"2.4",mediaType:mediaTypes.WEB,description:"Two or more nav elements, excluding nav elements without an accessible name, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, have accessible names that are non-unique values",metaText:"This nav does not have a unique accessible name as required when two or more nav elements are present",fixType:"",testFunc:function(){const testShell=new AutoTestShell();const relevantCssSelector='nav[data-ae_ar="null"][data-ae_vis][data-ae_avat][data-ae_an]';testShell.set_applicabilityTest(function(){return this.applicableNodesFunc(this._applicableNodesFuncArgs).length>0;});testShell.set_applicableNodesFunc(function(){// if there is more than 1 aside element then the test is applicable
const applicableNodes=this.querySelectorAll(relevantCssSelector);return applicableNodes.length>1?applicableNodes:[];});testShell.set_candidateSetNodesFunc(function(){var notUnique=[];var nodeStore={};var res=this.querySelectorAll(relevantCssSelector);if(res.length>1){for(var i=0;i<res.length;i++){var n=res[i];var thean=n.getAttribute("data-ae_an");try{nodeStore[thean].push(n);}catch(err){nodeStore[thean]=[];nodeStore[thean].push(n);}}var keys=Object.keys(nodeStore);for(var j=0;j<keys.length;j++){// collecting all duplicated keys
var nArray=nodeStore[keys[j]];if(nArray.length>1){notUnique=notUnique.concat(nArray);}}}return notUnique;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-581
	 * @returns {AutoTestShell} testShell
	 */function test581(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("581",{bestPractice:338,testId:581,introduced:"2.5",mediaType:mediaTypes.WEB,description:"One or more input type='date' elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This input (type=date) element does not have a mechanism that allows an accessible name value to be calculated",fixType:{"fixType":1,"domSpec":false,"helperText":"Provide a descriptive label for this date input field.","fix":{"aria-label":null,"aria-labelledby":false}},testFunc:function(){return new AutoTestShell('input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="date"]','input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="date"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-582
	 * @returns {AutoTestShell} testShell
	 */function test582(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("582",{bestPractice:338,testId:582,introduced:"2.5",mediaType:mediaTypes.WEB,description:"One or more input type='number' elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This input (type=number) element does not have a mechanism that allows an accessible name value to be calculated",fixType:{"fixType":1,"domSpec":false,"helperText":"Provide a descriptive label for this number input field.","fix":{"aria-label":null,"aria-labelledby":false}},testFunc:function(){return new AutoTestShell('input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="number"]','input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="number"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-583
	 * @returns {AutoTestShell} testShell
	 */function test583(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("583",{bestPractice:338,testId:583,introduced:"2.5",mediaType:mediaTypes.WEB,description:"One or more input type='time' elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This input (type=time) element does not have a mechanism that allows an accessible name value to be calculated",fixType:{"fixType":1,"domSpec":false,"helperText":"Provide a descriptive label for this time input field.","fix":{"aria-label":null,"aria-labelledby":false}},testFunc:function(){return new AutoTestShell('input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="time"]','input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="time"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-609
	 * @returns {AutoTestShell} testShell
	 */function test609(){aeKernel.addAutomaticTest("609",{bestPractice:1626,testId:609,introduced:"2.6",mediaType:mediaTypes.WEB,description:"One or more elements with an aria-setsize attribute not intentionally hidden in the DOM and available to assistive technologies, does not have an aria-posinset attribute.",metaText:"This element has an aria-setsize attribute but does not have an aria-posinset attribute",metaTextDetail:"This {{tag-name}} has an aria-setsize attribute but does not have an aria-posinset attribute",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][aria-setsize]','*[data-ae_vis][data-ae_avat][aria-setsize]:not([aria-posinset])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-610
	 * @returns {AutoTestShell} testShell
	 */function test610(){aeKernel.addAutomaticTest("610",{bestPractice:1626,testId:610,introduced:"2.6",mediaType:mediaTypes.WEB,description:"One or more elements, excluding input type=radio elements and elements with a role='article | listitem | option | menuitem | menuitemcheckbox | menuitemradio | radio | tab | treeitem' attribute, not intentionally hidden in the DOM and available to assistive technologies, has an aria-setsize attribute.",metaText:"The aria-setsize attribute is not allowed on this element because it does not have the correct role attribute",metaTextDetail:"The aria-setsize attribute is not allowed on this {{tag-name}} because it does not have the correct role attribute",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][aria-setsize]','*[data-ae_vis][data-ae_avat]:not(input[type="radio"]):not([data-ae_ar="article"]):not([data-ae_ar="listitem"]):not([data-ae_ar="option"]):not([data-ae_ar="menuitem"]):not([data-ae_ar="menuitemcheckbox"]):not([data-ae_ar="menuitemradio"]):not([data-ae_ar="radio"]):not([data-ae_ar="tab"]):not([data-ae_ar="treeitem"])[aria-setsize]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-630
	 * @returns {AutoTestShell} testShell
	 */function test630(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("630",{bestPractice:338,testId:630,introduced:"2.7",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='spinbutton' attribute, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name to be calculated.",metaText:"This element (role=spinbutton) does not have a mechanism that allows an accessible name value to be calculated",metaTextDetail:"This {{tag-name}} (role=spinbutton) does not have a mechanism that allows an accessible name value to be calculated",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="spinbutton"]','*[data-ae_vis][data-ae_avat][data-ae_ar="spinbutton"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-647
	 * @returns {AutoTestShell} testShell
	 */function test647(){aeKernel.addAutomaticTest("647",{bestPractice:429,testId:647,introduced:"2.9",mediaType:mediaTypes.WEB,description:"The root html element does not have a lang attribute set to a text value that starts with an IANA primary language value.",metaText:"This element does not have a lang attribute set to a text value that start with an IANA Language Subtag Registry language value",metaTextDetail:'This {{tag-name}} has a lang attribute of "{{lang}}", which is not a text value that start with an IANA Language Subtag Registry language value',fixType:"",testFunc:function(){const testShell=new AutoTestShell('html[lang]');testShell.set_candidateSetNodesFunc(function(){const notProper=[];const htmlElement=this.querySelector('html[lang]');let lang=htmlElement.getAttribute("lang");if(lang!==""){//We only care about the main language attribute before the first dash, if there is one
lang=lang.split("-")[0].toLowerCase();if(lang.length<2||lang.length>3||!ACCEPTED_LANGUAGE_SUBTAGS[lang]){notProper.push(htmlElement);}}else{notProper.push(htmlElement);}return notProper;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-704
	 * @returns {AutoTestShell} testShell
	 */function test704(){aeKernel.addAutomaticTest("704",{bestPractice:1626,testId:704,introduced:"2.15",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM and available to assistive technologies, has an aria-setsize attribute set to a non-integer value.",metaText:"This element has an aria-setsize attribute that is set to a non-integer value",metaTextDetail:"This {{tag-name}} has an aria-setsize attribute that is set to a non-integer value",fixType:"",testFunc:function(){const relevantCssSelector='*[data-ae_vis][data-ae_avat][aria-setsize]:not([aria-setsize=""])';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const nodesWithBadValues=[];const res=aeKernel.rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0;i<res.length;i++){const node=res[i];const attributeValue=node.getAttribute("aria-setsize");if(!isInt(attributeValue)){nodesWithBadValues.push(node);}}return nodesWithBadValues;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-706
	 * @returns {AutoTestShell} testShell
	 */function test706(){aeKernel.addAutomaticTest("706",{bestPractice:1626,testId:706,introduced:"2.15",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM and available to assistive technologies, has an aria-posinset attribute that is not set to an integer value that is greater than, or equal to, 1.",metaText:"This element has an aria-posinset attribute that is not set to an integer value that is greater than, or equal to, 1",metaTextDetail:"This {{tag-name}} has an aria-posinset attribute of {{aria-posinset}}, which is not an integer value greater than, or equal to, 1",fixType:"",testFunc:function(){const relevantCssSelector='*[data-ae_vis][data-ae_avat][aria-posinset]:not([aria-posinset=""])';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const nodesWithBadValues=[];const res=aeKernel.rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0;i<res.length;i++){const node=res[i];const attributeValue=node.getAttribute("aria-posinset");if(!isInt(attributeValue)||attributeValue<1){nodesWithBadValues.push(node);}}return nodesWithBadValues;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-725
	 * @returns {AutoTestShell} testShell
	 */function test725(){aeKernel.addAutomaticTest("725",{bestPractice:456,testId:725,introduced:"2.15",mediaType:mediaTypes.WEB,description:"One or more FRAME elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a src attribute set to an image file reference.",metaText:"This element has a src attribute set to an image file reference",metaTextDetail:"This {{tag-name}} has a src attribute set to an image file reference",fixType:"",testFunc:function(){//Checks for various known image file endings.
//Will ignore src with url parameters so that src="viewer.html?foo=bob.png" will not be flagged
//Will not flag src="image.png?foo=bar" but we've not seen that in the wild
return new AutoTestShell("frame[data-ae_ar='null'][data-ae_vis][data-ae_avat][src]:not([src=''])","frame[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.gif']:not([src*='?']):not([src*='.htm']), frame[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.GIF']:not([src*='?']):not([src*='.htm']), frame[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.jpg']:not([src*='?']):not([src*='.htm']), frame[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.JPG']:not([src*='?']):not([src*='.htm']), frame[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.jpeg']:not([src*='?']):not([src*='.htm']), frame[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.JPEG']:not([src*='?']):not([src*='.htm']), frame[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.bmp']:not([src*='?']):not([src*='.htm']), frame[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.BMP']:not([src*='?']):not([src*='.htm']), frame[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.png']:not([src*='?']):not([src*='.htm']), frame[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.PNG']:not([src*='?']):not([src*='.htm']), frame[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.tiff']:not([src*='?']):not([src*='.htm']), frame[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.TIFF']:not([src*='?']):not([src*='.htm']), frame[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.eps']:not([src*='?']):not([src*='.htm']), frame[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.EPS']:not([src*='?']):not([src*='.htm']), frame[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.pcx']:not([src*='?']):not([src*='.htm']), frame[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.PCX']:not([src*='?']):not([src*='.htm'])");}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-726
	 * @returns {AutoTestShell} testShell
	 */function test726(){aeKernel.addAutomaticTest("726",{bestPractice:456,testId:726,introduced:"2.15",mediaType:mediaTypes.WEB,description:"One or more IFRAME elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a src attribute set to an image file reference.",metaText:"This element has a src attribute set to an image file reference",metaTextDetail:"This {{tag-name}} has a src attribute set to an image file reference",fixType:"",testFunc:function(){//Checks for various known image file endings.
//Will ignore src with url parameters so that src="viewer.html?foo=bob.png" will not be flagged
//Will not flag src="image.png?foo=bar" but we've not seen that in the wild
return new AutoTestShell("iframe[data-ae_ar='null'][data-ae_vis][data-ae_avat][src]:not([src=''])","iframe[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.gif']:not([src*='?']):not([src*='.htm']), iframe[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.GIF']:not([src*='?']):not([src*='.htm']), iframe[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.jpg']:not([src*='?']):not([src*='.htm']), iframe[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.JPG']:not([src*='?']):not([src*='.htm']), iframe[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.jpeg']:not([src*='?']):not([src*='.htm']), iframe[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.JPEG']:not([src*='?']):not([src*='.htm']), iframe[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.bmp']:not([src*='?']):not([src*='.htm']), iframe[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.BMP']:not([src*='?']):not([src*='.htm']), iframe[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.png']:not([src*='?']):not([src*='.htm']), iframe[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.PNG']:not([src*='?']):not([src*='.htm']), iframe[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.tiff']:not([src*='?']):not([src*='.htm']), iframe[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.TIFF']:not([src*='?']):not([src*='.htm']), iframe[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.eps']:not([src*='?']):not([src*='.htm']), iframe[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.EPS']:not([src*='?']):not([src*='.htm']), iframe[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.pcx']:not([src*='?']):not([src*='.htm']), iframe[data-ae_ar='null'][data-ae_vis][data-ae_avat][src$='.PCX']:not([src*='?']):not([src*='.htm'])");}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-734
	 * @returns {AutoTestShell} testShell
	 */function test734(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("734",{bestPractice:1893,testId:734,introduced:"2.13",mediaType:mediaTypes.WEB,description:"One or more dialog elements with an open attribute, without an ARIA-assigned role and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This dialog element does not have a mechanism that allows an accessible name value to be calculated",fixType:"",testFunc:function(){return new AutoTestShell('dialog[open][data-ae_ar="null"][data-ae_avat]','dialog[open][data-ae_ar="null"][data-ae_avat]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-736
	 * @returns {AutoTestShell} testShell
	 */function test736(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("736",{bestPractice:1893,testId:736,introduced:"2.13",mediaType:mediaTypes.WEB,description:"One or more elements with a role='dialog' attribute, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This element (role=dialog) does not have a mechanism that allows an accessible name value to be calculated",metaTextDetail:"This {{tag-name}} (role=dialog) does not have a mechanism that allows an accessible name value to be calculated",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="dialog"]','*[data-ae_vis][data-ae_avat][data-ae_ar="dialog"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-738
	 * @returns {AutoTestShell} testShell
	 */function test738(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("738",{bestPractice:1352,testId:738,introduced:"2.11",mediaType:mediaTypes.WEB,description:"One or more fieldset elements, not intentionally hidden in the DOM and available to assistive technologies, has a legend which is not the first child of the fieldset.",metaText:"This fieldset element has a legend element but it is not the first child of the fieldset",fixType:"",testFunc:function(){var testShell=new AutoTestShell('fieldset[data-ae_vis][data-ae_avat]');testShell.set_candidateSetNodesFunc(function(){var badNodes=[];var res=aeKernel.rootTestNode.querySelectorAll("fieldset[data-ae_vis][data-ae_avat]");for(var i=0,len_res=res.length;i<len_res;i++){var result=res[i];//Does the fieldset have a legend?
if(result.querySelectorAll("legend").length>0){//Is the legend not the first child
if(result.firstElementChild.tagName!="LEGEND"){badNodes.push(result);}}}return badNodes;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-779
	 * @returns {AutoTestShell} testShell
	 */function test779(){aeKernel.addAutomaticTest("779",{bestPractice:2900,testId:779,introduced:"2.13",mediaType:mediaTypes.WEB,description:"One or more meta elements, in the head element, has a content attribute set to a value that includes an instruction that stops a user rotating the viewport size i.e. 'autoRotate:disabled'.",metaText:"This element has a content attribute set to a value that includes an instruction that stops a user rotating the viewport",metaTextDetail:"This {{tag-name}} has a content attribute set to a value that includes an instruction that stops a user rotating the viewport",fixType:"",testFunc:function(){const relevantCssSelector='head meta[http-equiv="ScreenOrientation"]';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[content~="autoRotate:disabled"]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-790
	 * @returns {AutoTestShell} testShell
	 */function test790(){aeKernel.addAutomaticTest("790",{bestPractice:524,testId:790,introduced:"2.13",mediaType:mediaTypes.WEB,description:"One or more elements with a tabindex attribute set to a value greater than or equal to 0 and any implicitly focusable elements without a tabindex set to a negative value (a elements with an href attribute set to a text value, area elements with an href attribute set to a text value, input elements without a disabled attribute, select elements without a disabled attribute, textarea elements without a disabled attribute, button elements without a disabled attribute, details elements, audio elements with a controls attribute and video elements with a controls attribute), not intentionally hidden in the DOM, has an aria-hidden attribute set to a 'true' value directly on the element.",metaText:"This element is focusable and has an aria-hidden attribute set to true",metaTextDetail:"This {{tag-name}} is focusable and has an aria-hidden attribute set to true",fixType:"",testFunc:function(){const baseCssSelectors=["a[data-ae_vis][href]:not([href=''])","area[href]:not([href=''])","input[data-ae_vis]:not([disabled])","select[data-ae_vis]:not([disabled])","textarea[data-ae_vis]:not([disabled])","button[data-ae_vis]:not([disabled])","details[data-ae_vis]","audio[data-ae_vis][controls]","video[data-ae_vis][controls]"];const noTabIndexAttributeSuffix=':not([tabindex])';const blankTabIndexAttributeValueSuffix='[tabindex=""]';const positiveTabIndexAttributeValueSuffix=`[tabindex]:not([tabindex=""]):not([tabindex^="-"])`;const suffixes=[noTabIndexAttributeSuffix,blankTabIndexAttributeValueSuffix,positiveTabIndexAttributeValueSuffix];const genericCssSelector=`*[data-ae_vis]${positiveTabIndexAttributeValueSuffix}`;let relevantCssSelectorArray=[genericCssSelector];let cssSelectorArray=[`${genericCssSelector}[aria-hidden='true']`];for(let i=0;i<baseCssSelectors.length;i++){const baseCssSelector=baseCssSelectors[i];for(let l=0;l<suffixes.length;l++){const suffix=suffixes[l];relevantCssSelectorArray.push(`${baseCssSelector}${suffix}`);cssSelectorArray.push(`${baseCssSelector}${suffix}[aria-hidden='true']`);}}const relevantCssSelector=relevantCssSelectorArray.join(',');const cssSelector=cssSelectorArray.join(',');return new AutoTestShell(relevantCssSelector,cssSelector);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-809
	 * @returns {AutoTestShell} testShell
	 */function test809(){aeKernel.addAutomaticTest("809",{bestPractice:1352,testId:809,introduced:"2.14",mediaType:mediaTypes.WEB,description:"More than one main element, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies is located in the body element.",metaText:"This MAIN is one of multiple main elements that appear in the body element.",fixType:"",testFunc:function(){const testShell=new AutoTestShell();const relevantCssSelector='body main[data-ae_ar="null"][data-ae_vis][data-ae_avat]';testShell.set_applicabilityTest(function(){return this.applicableNodesFunc(this._applicableNodesFuncArgs).length>0;});testShell.set_applicableNodesFunc(function(){// if there is more than 1 main element then the test is applicable
const applicableNodes=this.querySelectorAll(relevantCssSelector);return applicableNodes.length>1?applicableNodes:[];});testShell.set_candidateSetNodesFunc(function(){return this.querySelectorAll(relevantCssSelector);});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-825
	 * @returns {AutoTestShell} testShell
	 */function test825(){aeKernel.addAutomaticTest("825",{bestPractice:361,testId:825,introduced:"2.15",mediaType:mediaTypes.WEB,description:"One or more elements in a table, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a headers attribute value that includes one or more valid ids that point to an element outside the table.",metaText:"This element has a headers attribute that includes one or more ids which point to an element outside the table",metaTextDetail:"This {{tag-name}} has a headers attribute value of {{headers}} that includes one or more ids which point to an element outside the table",fixType:"",testFunc:function(){const relevantCssSelector="td[data-ae_ar='null'][data-ae_vis][data-ae_avat][headers]:not([headers='']), th[data-ae_ar='null'][data-ae_vis][data-ae_avat][headers]:not([headers=''])";const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const tableElements=aeKernel.rootTestNode.querySelectorAll("table");for(let i=0;i<tableElements.length;i++){const relevantTableElement=tableElements[i];const relevantTableDataAndHeaderElements=relevantTableElement.querySelectorAll(relevantCssSelector);for(let j=0;j<relevantTableDataAndHeaderElements.length;j++){const relevantTableDataAndHeaderElement=relevantTableDataAndHeaderElements[j];const headerIds=relevantTableDataAndHeaderElement.getAttribute("headers").trim().split(" ");for(let k=0;k<headerIds.length;k++){const headerId=headerIds[k];// "[id=" is used in place of "#" + id as ids can included colons which breaks css selector, although is valid html
const headerReferenceNodeInTable=relevantTableElement.querySelector(`[id='${headerId.replace(/'/g,"\\'")}']`);if(!headerReferenceNodeInTable){// flag elements that reference a header (by ID) that exists, but is not an ancestor of this table;
// don't flag elements that reference a header that doesn't exist, as that's beyond the scope of this test
const headerReferenceNode=aeKernel.rootTestNode.querySelector(`[id='${headerId.replace(/'/g,"\\'")}']`);if(headerReferenceNode){badNodes.push(relevantTableDataAndHeaderElement);break;}}}}}return badNodes;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-828
	 * @returns {AutoTestShell} testShell
	 */function test828(){aeKernel.addAutomaticTest("828",{bestPractice:361,testId:828,introduced:"2.15",mediaType:mediaTypes.WEB,description:"One or more elements in a table, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a headers attribute value that includes one or more valid ids that point to an element that is not a td, th or element with a role of cell, gridcell, columnheader or rowheader.",metaText:"This element has a headers attribute that points to an element which is not a table cell",metaTextDetail:"This {{tag-name}} has a headers attribute that points to an element which is not a table cell",fixType:"",testFunc:function(){const relevantCssSelector="td[data-ae_ar='null'][data-ae_vis][data-ae_avat][headers]:not([headers='']), th[data-ae_ar='null'][data-ae_vis][data-ae_avat][headers]:not([headers=''])";const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const relevantTableDataAndHeaderElements=aeKernel.rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0;i<relevantTableDataAndHeaderElements.length;i++){const relevantTableDataAndHeaderElement=relevantTableDataAndHeaderElements[i];const headerIds=relevantTableDataAndHeaderElement.getAttribute("headers").trim().split(" ");for(let j=0;j<headerIds.length;j++){const headerId=headerIds[j];// "[id=" is used in place of "#" + id as ids can included colons which breaks css selector, although is valid html
const headerReferenceTableCellNode=aeKernel.rootTestNode.querySelector(`td[id='${headerId.replace(/'/g,"\\'")}'], th[id='${headerId.replace(/'/g,"\\'")}'], *[id='${headerId.replace(/'/g,"\\'")}'][data-ae_ar="cell"], *[id='${headerId.replace(/'/g,"\\'")}'][data-ae_ar="gridcell"], *[id='${headerId.replace(/'/g,"\\'")}'][data-ae_ar="columnheader"], *[id='${headerId.replace(/'/g,"\\'")}'][data-ae_ar="rowheader"]`);if(!headerReferenceTableCellNode){// flag elements that reference a header (by ID) that exists, but is not a table cell;
// don't flag elements that reference a header that doesn't exist, as that's beyond the scope of this test
const headerReferenceNode=aeKernel.rootTestNode.querySelector(`[id='${headerId.replace(/'/g,"\\'")}']`);if(headerReferenceNode){badNodes.push(relevantTableDataAndHeaderElement);break;}}}}return badNodes;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-863
	 * @returns {AutoTestShell} testShell
	 */function test863(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("863",{bestPractice:2440,testId:863,introduced:"2.15",mediaType:mediaTypes.WEB,description:"One or more input elements with a placeholder attribute, without an aria-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has an accessible name that depends on the placeholder attribute text.",metaText:"This input should not rely on text in the placeholder attribute for an accessible name",fixType:"",testFunc:function(){const relevantCssSelector="input[data-ae_ar='null'][data-ae_vis][data-ae_avat][data-ae_an]";return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[data-ae_anp]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-872
	 * @returns {AutoTestShell} testShell
	 */function test872(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("872",{bestPractice:2440,testId:872,introduced:"2.15",mediaType:mediaTypes.WEB,description:"One or more textarea elements with a placeholder attribute, without an aria-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has an accessible name that depends on the placeholder attribute text.",metaText:"This textarea should not rely on text in the placeholder attribute for an accessible name",fixType:"",testFunc:function(){const relevantCssSelector="textarea[data-ae_ar='null'][data-ae_vis][data-ae_avat][data-ae_an]";return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[data-ae_anp]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-874
	 * @returns {AutoTestShell} testShell
	 */function test874(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("874",{bestPractice:338,testId:874,introduced:"2.16",mediaType:mediaTypes.WEB,description:"One or more summary elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This summary element does not have a mechanism that allows an accessible name value to be calculated",fixType:"",testFunc:function(){const relevantCssSelector="summary[data-ae_ar='null'][data-ae_vis][data-ae_avat]";return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}:not([data-ae_an])`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-884
	 * @returns {AutoTestShell} testShell
	 */function test884(){aeKernel.addAutomaticTest("884",{bestPractice:364,testId:884,introduced:"2.16",mediaType:mediaTypes.WEB,description:"A document title element is not located in the DOM.",metaText:"This DOM does not contain a document title element",fixType:{"fixType":2,"domSpec":true,"helperText":"Specify a unique and descriptive title for the page.","fix":{"targetCssSelector":"head","create":"title","innerHTML":null}},testFunc:function(){const testShell=new AutoTestShell('html');testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const documentTitle=this.querySelector('html title:not([data-ae_inshdw]):not(svg title)');if(!documentTitle){badNodes.push(this.querySelector('html'));}return badNodes;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1010
	 * @returns {AutoTestShell} testShell
	 */function test1010(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("1010",{bestPractice:2519,testId:1010,introduced:"2.19",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='region' attribute, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name to be calculated.",metaText:"This element (role=region) does not have a mechanism that allows an accessible name value to be calculated",metaTextDetail:"This {{tag-name}} (role=region) does not have a mechanism that allows an accessible name value to be calculated",fixType:"",testFunc:function(){const relevantCssSelector="*[data-ae_ar='region'][data-ae_vis][data-ae_avat]";return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}:not([data-ae_an])`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1017
	 * @returns {AutoTestShell} testShell
	 */function test1017(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("1017",{bestPractice:2554,testId:1017,introduced:"2.19",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='progressbar' attribute, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name to be calculated.",metaText:"This element (role=progressbar) does not have a mechanism that allows an accessible name value to be calculated",metaTextDetail:"This {{tag-name}} (role=progressbar) does not have a mechanism that allows an accessible name value to be calculated",fixType:"",testFunc:function(){const relevantCssSelector='*[data-ae_vis][data-ae_avat][data-ae_ar="progressbar"]';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}:not([data-ae_an])`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1038
	 * @returns {AutoTestShell} testShell
	 */function test1038(){aeKernel.addAutomaticTest("1038",{bestPractice:1626,testId:1038,introduced:"2.20",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='switch' attribute, excluding input type='checkbox' elements, not intentionally hidden in the DOM and available to assistive technologies, does not have an aria-checked attribute.",metaText:"This element (role=switch) does not have an aria-checked attribute",metaTextDetail:"This {{tag-name}} (role=switch) does not have an aria-checked attribute",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="switch"]:not(input[type="checkbox"])','*[data-ae_vis][data-ae_avat][data-ae_ar="switch"]:not(input[type="checkbox"]):not([aria-checked])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1039
	 * @returns {AutoTestShell} testShell
	 */function test1039(){aeKernel.addAutomaticTest("1039",{bestPractice:1626,testId:1039,introduced:"2.20",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='checkbox' attribute, excluding input type='checkbox' elements, not intentionally hidden in the DOM and available to assistive technologies, does not have an aria-checked attribute.",metaText:"This element (role=checkbox) does not have an aria-checked attribute",metaTextDetail:"This {{tag-name}} (role=checkbox) does not have an aria-checked attribute",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="checkbox"]:not(input[type="checkbox"])','*[data-ae_vis][data-ae_avat][data-ae_ar="checkbox"]:not(input[type="checkbox"]):not([aria-checked])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1040
	 * @returns {AutoTestShell} testShell
	 */function test1040(){aeKernel.addAutomaticTest("1040",{bestPractice:1626,testId:1040,introduced:"2.20",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='combobox' attribute, not intentionally hidden in the DOM and available to assistive technologies, does not have an aria-controls attribute and an aria-expanded attribute.",metaText:"This element (role=combobox) must have all of the following attributes: aria-controls | aria-expanded",metaTextDetail:"This {{tag-name}} (role=combobox) must have all of the following attributes: aria-controls | aria-expanded",fixType:"",testFunc:function(){const relevantCssSelector='*[data-ae_vis][data-ae_avat][data-ae_ar="combobox"]';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}:not([aria-controls]), 
				${relevantCssSelector}:not([aria-expanded])`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1043
	 * @returns {AutoTestShell} testShell
	 */function test1043(){aeKernel.addAutomaticTest("1043",{bestPractice:1626,testId:1043,introduced:"2.20",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='radio' attribute, excluding input type='radio' elements, not intentionally hidden in the DOM and available to assistive technologies, does not have an aria-checked attribute.",metaText:"This element (role=radio) does not have an aria-checked attribute",metaTextDetail:"This {{tag-name}} (role=radio) does not have an aria-checked attribute",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="radio"]:not(input[type="radio"])','*[data-ae_vis][data-ae_avat][data-ae_ar="radio"]:not(input[type="radio"]):not([aria-checked])');}});}/**
	 * Holds valid aria attributes list
	 */const testUtil_ariaAttributeLookupTable={"aria-activedescendant":true,"aria-atomic":true,"aria-autocomplete":true,"aria-busy":true,"aria-checked":true,"aria-colcount":true,"aria-colindex":true,"aria-colspan":true,"aria-controls":true,"aria-current":true,"aria-describedby":true,"aria-details":true,"aria-disabled":true,"aria-errormessage":true,"aria-expanded":true,"aria-flowto":true,"aria-haspopup":true,"aria-hidden":true,"aria-invalid":true,"aria-keyshortcuts":true,"aria-label":true,"aria-labelledby":true,"aria-level":true,"aria-live":true,"aria-modal":true,"aria-multiline":true,"aria-multiselectable":true,"aria-orientation":true,"aria-owns":true,"aria-placeholder":true,"aria-posinset":true,"aria-pressed":true,"aria-readonly":true,"aria-relevant":true,"aria-required":true,"aria-roledescription":true,"aria-rowcount":true,"aria-rowindex":true,"aria-rowspan":true,"aria-selected":true,"aria-setsize":true,"aria-sort":true,"aria-valuemax":true,"aria-valuemin":true,"aria-valuenow":true,"aria-valuetext":true};/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1066
	 * @returns {AutoTestShell} testShell
	 */function test1066(){aeKernel.addAutomaticTest("1066",{bestPractice:1626,testId:1066,introduced:"2.20",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM, has an attribute starting with 'aria-' that is not valid.",metaText:"This element has one or more aria attributes which are not valid.",metaTextDetail:"This {{tag-name}} has one or more aria attributes which are not valid.",fixType:"",testFunc:function(){//Have body as part of applicability as there shouldn't be aria atributes in the head
const relevantCssSelector='body *[data-ae_vis]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const notProper=[];//Get all the visible nodes that have an aria- attribute. There currently is not a CSS way to do this
const res=[...aeKernel.rootTestNode.querySelectorAll(relevantCssSelector)].filter(tag=>[...tag.attributes].filter(_ref=>{let{name}=_ref;return name.startsWith("aria-");}).length>0);for(let i=0,len_res=res.length;i<len_res;i++){const result=res[i];//For each element that has aria attributes, verify all the aria-* attributes are on the allowed list
//TODO Might be possible to combine this filter with the above one and eliminate the loop
if([...result.attributes].filter(atr=>atr.name.startsWith("aria-")).filter(atr=>!testUtil_ariaAttributeLookupTable[atr.name]).length>0){notProper[notProper.length]=result;}}return notProper;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1077
	 * @returns {AutoTestShell} testShell
	 */function test1077(){aeKernel.addAutomaticTest("1077",{bestPractice:1626,testId:1077,introduced:"2.21",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='option' attribute, not intentionally hidden in the DOM and available to assistive technologies, does not have an aria-checked attribute set to one of the text values: 'true | false'.",metaText:"This element (role=option) does not have an aria-checked attribute set to one of the text values: true | false",metaTextDetail:"This {{tag-name}} (role=option) does not have an aria-checked attribute set to one of the text values: true | false",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="option"]','*[data-ae_vis][data-ae_avat][data-ae_ar="option"][aria-checked]:not([aria-checked="true"]):not([aria-checked="false"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1078
	 * @returns {AutoTestShell} testShell
	 */function test1078(){aeKernel.addAutomaticTest("1078",{bestPractice:1626,testId:1078,introduced:"2.21",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='treeitem' attribute, not intentionally hidden in the DOM and available to assistive technologies, does not have an aria-checked attribute set to one of the text values: 'true | false'.",metaText:"This element (role=treeitem) does not have an aria-checked attribute set to one of the text values: true | false",metaTextDetail:"This {{tag-name}} (role=treeitem) does not have an aria-checked attribute set to one of the text values: true | false",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="treeitem"]','*[data-ae_vis][data-ae_avat][data-ae_ar="treeitem"][aria-checked]:not([aria-checked="true"]):not([aria-checked="false"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1080
	 * @returns {AutoTestShell} testShell
	 */function test1080(){aeKernel.addAutomaticTest("1080",{bestPractice:1626,testId:1080,introduced:"2.21",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='treeitem' attribute, not intentionally hidden in the DOM and available to assistive technologies, does not have an aria-checked attribute.",metaText:"This element (role=treeitem) does not have an aria-checked attribute",metaTextDetail:"This {{tag-name}} (role=treeitem) does not have an aria-checked attribute",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="treeitem"]','*[data-ae_vis][data-ae_avat][data-ae_ar="treeitem"]:not([aria-checked])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1097
	 * @returns {AutoTestShell} testShell
	 */function test1097(){aeKernel.addAutomaticTest("1097",{bestPractice:1626,testId:1097,introduced:"2.21",mediaType:mediaTypes.WEB,description:"One or more A elements with an href attribute set to a text value, not intentionally hidden in the DOM, has a role='presentation' attribute.",metaText:"This A has a role attribute set to 'presentation'",fixType:"",testFunc:function(){const relevantCssSelector='a[href]:not([href=""])[data-ae_vis]';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[data-ae_ar="presentation"]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1098
	 * @returns {AutoTestShell} testShell
	 */function test1098(){aeKernel.addAutomaticTest("1098",{bestPractice:967,testId:1098,introduced:"2.21",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='listbox | menu | tree', not intentionally hidden in the DOM, contained one or more input elements (without an assigned ARIA role and not type=hidden), button elements (without an assigned ARIA role), textarea elements (without an assigned ARIA role), select elements (without an assigned ARIA role) or A elements (without an assigned ARIA role).",metaText:"This element (role=listbox, menu, or tree) should not contain input elements (without an assigned ARIA role), button elements (without an assigned ARIA role), textarea elements (without an assigned ARIA role) or A elements (without an assigned ARIA role)",metaTextDetail:"This {{tag-name}} with a role attribute value of '{{data-ae_ar}}' should not contain input elements (without an assigned ARIA role), button elements (without an assigned ARIA role), textarea elements (without an assigned ARIA role), select elements (without an assigned ARIA role) or A elements (without an assigned ARIA role)",fixType:"",testFunc:function(){const relevantCssSelector='*[data-ae_ar="listbox"][data-ae_vis],*[data-ae_ar="menu"][data-ae_vis],*[data-ae_ar="tree"][data-ae_vis]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const notProper=[];const res=aeKernel.rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0;i<res.length;i++){const result=res[i];const res1=result.querySelector('input[data-ae_ar="null"]:not([type="hidden"]), button[data-ae_ar="null"], textarea[data-ae_ar="null"], select[data-ae_ar="null"], a[data-ae_ar="null"]');if(res1){notProper.push(result);}}return notProper;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1099
	 * @returns {AutoTestShell} testShell
	 */function test1099(){aeKernel.addAutomaticTest("1099",{bestPractice:967,testId:1099,introduced:"2.21",mediaType:mediaTypes.WEB,description:"One or more button elements, without an ARIA-assigned role, not intentionally hidden in the DOM, contained one or more input elements (without an assigned ARIA role and not type=hidden), select elements (without an assigned ARIA role), textarea elements (without an assigned ARIA role) or A elements (without an assigned ARIA role).",metaText:"This button should not contain input elements (without an assigned ARIA role and not type=hidden), select elements (without an assigned ARIA role), textarea elements (without an assigned ARIA role) or A elements (without an assigned ARIA role).",fixType:"",testFunc:function(){const relevantCssSelector='button[data-ae_ar="null"][data-ae_vis]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const notProper=[];const res=aeKernel.rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0;i<res.length;i++){const result=res[i];const res1=result.querySelector('input[data-ae_ar="null"]:not([type="hidden"]), select[data-ae_ar="null"], textarea[data-ae_ar="null"], a[data-ae_ar="null"]');if(res1){notProper.push(result);}}return notProper;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1100
	 * @returns {AutoTestShell} testShell
	 */function test1100(){aeKernel.addAutomaticTest("1100",{bestPractice:967,testId:1100,introduced:"2.21",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='button' attribute, not intentionally hidden in the DOM, contained one or more input elements (without an assigned ARIA role and not type=hidden), select elements (without an assigned ARIA role), textarea elements (without an assigned ARIA role) or A elements (without an assigned ARIA role).",metaText:"This element (role=button) should not contain input elements (without an assigned ARIA role and not type=hidden), select elements (without an assigned ARIA role), textarea elements (without an assigned ARIA role) or A elements (without an assigned ARIA role).",metaTextDetail:"This {{tag-name}} (role=button) should not contain input elements (without an assigned ARIA role and not type=hidden), select elements (without an assigned ARIA role), textarea elements (without an assigned ARIA role) or A elements (without an assigned ARIA role).",fixType:"",testFunc:function(){const relevantCssSelector='*[data-ae_ar="button"][data-ae_vis]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const notProper=[];const res=aeKernel.rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0;i<res.length;i++){const result=res[i];const res1=result.querySelector('input[data-ae_ar="null"]:not([type="hidden"]), select[data-ae_ar="null"], textarea[data-ae_ar="null"], a[data-ae_ar="null"]');if(res1){notProper.push(result);}}return notProper;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1105
	 * @returns {AutoTestShell} testShell
	 */function test1105(){aeKernel.addAutomaticTest("1105",{bestPractice:733,testId:1105,introduced:"2.21",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='none' or role='presentation' attribute, excluding those elements with aria-hidden='true', not intentionally hidden in the DOM, has an aria-label attribute.",metaText:"This element (role=presentation or none) has an aria-label attribute.",metaTextDetail:"This {{tag-name}} (role={{data-ae_ar}}) has an aria-label attribute.",fixType:"",testFunc:function(){return new AutoTestShell('*[role="none"]:not([data-ae_at_hidden])[data-ae_vis],*[role="presentation"]:not([data-ae_at_hidden])[data-ae_vis]','*[role="none"]:not([data-ae_at_hidden])[data-ae_vis][aria-label],*[role="presentation"]:not([data-ae_at_hidden])[data-ae_vis][aria-label]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1106
	 * @returns {AutoTestShell} testShell
	 */function test1106(){aeKernel.addAutomaticTest("1106",{bestPractice:733,testId:1106,introduced:"2.21",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='none' or role='presentation' attribute, excluding those elements with aria-hidden='true', not intentionally hidden in the DOM, has an aria-labelledby attribute.",metaText:"This element (role=presentation or none) has an aria-labelledby attribute.",metaTextDetail:"This {{tag-name}} (role={{data-ae_ar}}) has an aria-labelledby attribute.",fixType:"",testFunc:function(){return new AutoTestShell('*[role="none"]:not([data-ae_at_hidden])[data-ae_vis],*[role="presentation"]:not([data-ae_at_hidden])[data-ae_vis]','*[role="none"]:not([data-ae_at_hidden])[data-ae_vis][aria-labelledby],*[role="presentation"]:not([data-ae_at_hidden])[data-ae_vis][aria-labelledby]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1107
	 * @returns {AutoTestShell} testShell
	 */function test1107(){aeKernel.addAutomaticTest("1107",{bestPractice:733,testId:1107,introduced:"2.21",mediaType:mediaTypes.WEB,description:"One or more img elements with an alt attribute set to a null value, without an ARIA-assigned role other than img, not intentionally hidden in the DOM and available to assistive technologies, has an aria-label attribute.",metaText:"This IMG with an alt='' has an aria-label attribute.",fixType:"",testFunc:function(){return new AutoTestShell('img[alt=""][data-ae_ar="null"][data-ae_vis][data-ae_avat],img[alt=""][data-ae_ar="img"][data-ae_vis][data-ae_avat]','img[alt=""][data-ae_ar="null"][data-ae_vis][data-ae_avat][aria-label],img[alt=""][data-ae_ar="img"][data-ae_vis][data-ae_avat][aria-label]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1108
	 * @returns {AutoTestShell} testShell
	 */function test1108(){aeKernel.addAutomaticTest("1108",{bestPractice:733,testId:1108,introduced:"2.21",mediaType:mediaTypes.WEB,description:"One or more img elements with an alt attribute set to a null value, without an ARIA-assigned role other than img, not intentionally hidden in the DOM and available to assistive technologies, has an aria-labelledby attribute.",metaText:"This IMG with an alt='' has an aria-labelledby attribute.",fixType:"",testFunc:function(){return new AutoTestShell('img[alt=""][data-ae_ar="null"][data-ae_vis][data-ae_avat],img[alt=""][data-ae_ar="img"][data-ae_vis][data-ae_avat]','img[alt=""][data-ae_ar="null"][data-ae_vis][data-ae_avat][aria-labelledby],img[alt=""][data-ae_ar="img"][data-ae_vis][data-ae_avat][aria-labelledby]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1121
	 * @returns {AutoTestShell} testShell
	 */function test1121(){aeKernel.addAutomaticTest("1121",{bestPractice:967,testId:1121,introduced:"2.21",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='radiogroup', not intentionally hidden in the DOM, contained one or more input elements (without an assigned ARIA role, not type=hidden and not type=radio), button elements (without an assigned ARIA role), textarea elements (without an assigned ARIA role), select elements (without an assigned ARIA role) or A elements (without an assigned ARIA role).",metaText:"This element with a role attribute value of radiogroup should not contain input elements (without an assigned ARIA role), button elements (without an assigned ARIA role), textarea elements (without an assigned ARIA role), select elements (without an assigned ARIA role) or A elements (without an assigned ARIA role)",metaTextDetail:"This {{tag-name}} with a role attribute value of radiogroup should not contain input elements (without an assigned ARIA role), button elements (without an assigned ARIA role), textarea elements (without an assigned ARIA role), select elements (without an assigned ARIA role) or A elements (without an assigned ARIA role)",fixType:"",testFunc:function(){const relevantCssSelector='*[data-ae_ar="radiogroup"][data-ae_vis]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const notProper=[];const res=aeKernel.rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0;i<res.length;i++){const result=res[i];const res1=result.querySelector('input[data-ae_ar="null"]:not([type="hidden"]):not([type="radio"]), button[data-ae_ar="null"], textarea[data-ae_ar="null"], select[data-ae_ar="null"], a[data-ae_ar="null"]');if(res1){notProper.push(result);}}return notProper;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-666
	 * @returns {AutoTestShell} testShell
	 */function test666(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("666",{bestPractice:1584,testId:666,introduced:"2.10",mediaType:mediaTypes.IOS,description:"One or more Picker elements does not have an accessible label.",metaText:"This [[object]] element does not have an accessible label",fixType:"",testFunc:function(){return new AutoTestShell('select[data-ae_ar="null"][data-ae_vis][data-ae_avat]','select[data-ae_ar="null"][data-ae_vis][data-ae_avat]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-684
	 * @returns {AutoTestShell} testShell
	 */function test684(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("684",{bestPractice:1584,testId:684,introduced:"2.10",mediaType:mediaTypes.IOS,description:"One or more Button elements does not have an accessible label.",metaText:"This [[object]] element does not have an accessible label",fixType:"",testFunc:function(){return new AutoTestShell('button[data-ae_vis][data-ae_avat][data-ae_ar="null"]','button[data-ae_vis][data-ae_avat][data-ae_ar="null"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-685
	 * @returns {AutoTestShell} testShell
	 */function test685(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("685",{bestPractice:1584,testId:685,introduced:"2.10",mediaType:mediaTypes.IOS,description:"One or more SecureTextField elements does not have an accessible label.",metaText:"This [[object]] element does not have an accessible label",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="password"]','input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="password"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-686
	 * @returns {AutoTestShell} testShell
	 */function test686(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("686",{bestPractice:1584,testId:686,introduced:"2.10",mediaType:mediaTypes.IOS,description:"One or more SearchField elements does not have an accessible label.",metaText:"This [[object]] element does not have an accessible label",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="search"]','input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="search"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-687
	 * @returns {AutoTestShell} testShell
	 */function test687(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("687",{bestPractice:1584,testId:687,introduced:"2.10",mediaType:mediaTypes.IOS,description:"One or more Switch elements does not have an accessible label.",metaText:"This [[object]] element does not have an accessible label",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="radio"]','input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="radio"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-688
	 * @returns {AutoTestShell} testShell
	 */function test688(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("688",{bestPractice:1584,testId:688,introduced:"2.10",mediaType:mediaTypes.IOS,description:"One or more Alert, ScrollView, TextField, or TextView elements does not have an accessible label.",metaText:"This [[object]] element does not have an accessible label",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="textbox"]','*[data-ae_vis][data-ae_avat][data-ae_ar="textbox"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-689
	 * @returns {AutoTestShell} testShell
	 */function test689(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("689",{bestPractice:1584,testId:689,introduced:"2.10",mediaType:mediaTypes.IOS,description:"One or more Slider elements does not have an accessible label.",metaText:"This [[object]] element does not have an accessible label",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="number"]','input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="number"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-690
	 * @returns {AutoTestShell} testShell
	 */function test690(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("690",{bestPractice:1584,testId:690,introduced:"2.10",mediaType:mediaTypes.IOS,description:"One or more Image elements does not have an accessible label.",metaText:"This [[object]] element does not have an accessible label",fixType:"",testFunc:function(){return new AutoTestShell('img[data-ae_ar="null"][data-ae_vis][data-ae_avat]:not([alt=""]), img[data-ae_ar="img"][data-ae_vis][data-ae_avat]:not([alt=""])','img[data-ae_ar="null"][data-ae_vis][data-ae_avat]:not([alt=""]):not([data-ae_an]), img[data-ae_ar="img"][data-ae_vis][data-ae_avat]:not([alt=""]):not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-677
	 * @returns {AutoTestShell} testShell
	 */function test677(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("677",{bestPractice:1963,testId:677,introduced:"2.10",mediaType:mediaTypes.ANDROID,description:"One or more Spinner elements does not have a content description.",metaText:"This [[object]] element does not have a content description",fixType:"",testFunc:function(){return new AutoTestShell('select[data-ae_ar="null"][data-ae_vis][data-ae_avat]','select[data-ae_ar="null"][data-ae_vis][data-ae_avat]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-691
	 * @returns {AutoTestShell} testShell
	 */function test691(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("691",{bestPractice:1963,testId:691,introduced:"2.10",mediaType:mediaTypes.ANDROID,description:"One or more EditText elements does not have hint or default text.",metaText:"This [[object]] element does not have hint or default text",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="text"]','input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="text"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-692
	 * @returns {AutoTestShell} testShell
	 */function test692(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("692",{bestPractice:1963,testId:692,introduced:"2.10",mediaType:mediaTypes.ANDROID,description:"One or more ImageButton elements does not have a content description.",metaText:"This [[object]] element does not have a content description",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="image"]',// using '[aria-label=""]' instead of ':not([data-ae_an])' here because the accessible name calculation
// for input elements of type 'image' produces an accessible name even when alt text isn't specified,
// and Continuum for Android always uses the aria-label attribute to specify alt text,
// so we can get away with this
'input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="image"][aria-label=""]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-693
	 * @returns {AutoTestShell} testShell
	 */function test693(){aeKernel.addAutomaticTest("693",{bestPractice:1973,testId:693,introduced:"2.10",mediaType:mediaTypes.ANDROID,description:"One or more CheckBox or ToggleButton elements is neither checked nor unchecked.",metaText:"This [[object]] element does not have a checked attribute set to one of the text values: 'true | false | mixed'.",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="checkbox"]','*[data-ae_vis][data-ae_avat][data-ae_ar="checkbox"][aria-checked]:not([aria-checked="true"]):not([aria-checked="false"]):not([aria-checked="mixed"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-694
	 * @returns {AutoTestShell} testShell
	 */function test694(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("694",{bestPractice:1963,testId:694,introduced:"2.10",mediaType:mediaTypes.ANDROID,description:"One or more RadioGroup elements does not have a content description.",metaText:"This [[object]] element does not have a content description",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="radiogroup"]','*[data-ae_vis][data-ae_avat][data-ae_ar="radiogroup"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-695
	 * @returns {AutoTestShell} testShell
	 */function test695(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("695",{bestPractice:1963,testId:695,introduced:"2.10",mediaType:mediaTypes.ANDROID,description:"One or more Button elements does not have text or a content description.",metaText:"This [[object]] element does not have text or a content description",fixType:"",testFunc:function(){return new AutoTestShell('button[data-ae_vis][data-ae_avat][data-ae_ar="null"]','button[data-ae_vis][data-ae_avat][data-ae_ar="null"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-696
	 * @returns {AutoTestShell} testShell
	 */function test696(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("696",{bestPractice:1963,testId:696,introduced:"2.10",mediaType:mediaTypes.ANDROID,description:"One or more VideoView elements does not have a content description.",metaText:"This [[object]] element does not have a content description",fixType:"",testFunc:function(){return new AutoTestShell('video[data-ae_vis][data-ae_avat][data-ae_ar="null"]','video[data-ae_vis][data-ae_avat][data-ae_ar="null"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-697
	 * @returns {AutoTestShell} testShell
	 */function test697(){aeKernel.addAutomaticTest("697",{bestPractice:1973,testId:697,introduced:"2.10",mediaType:mediaTypes.ANDROID,description:"One or more RadioButton elements is neither checked nor unchecked.",metaText:"This [[object]] element does not have a checked attribute set to one of the text values: 'true | false | mixed'.",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="radio"]','*[data-ae_vis][data-ae_avat][data-ae_ar="radio"][aria-checked]:not([aria-checked="true"]):not([aria-checked="false"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-698
	 * @returns {AutoTestShell} testShell
	 */function test698(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("698",{bestPractice:1963,testId:698,introduced:"2.10",mediaType:mediaTypes.ANDROID,description:"One or more SearchView elements does not have a content description.",metaText:"This [[object]] element does not have a content description",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="search"]','input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="search"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-699
	 * @returns {AutoTestShell} testShell
	 */function test699(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("699",{bestPractice:1963,testId:699,introduced:"2.10",mediaType:mediaTypes.ANDROID,description:"One or more CheckBox elements does not have text or a content description.",metaText:"This [[object]] element does not have text or a content description",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="checkbox"]','*[data-ae_vis][data-ae_avat][data-ae_ar="checkbox"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-700
	 * @returns {AutoTestShell} testShell
	 */function test700(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("700",{bestPractice:1963,testId:700,introduced:"2.10",mediaType:mediaTypes.ANDROID,description:"One or more RadioButton elements does not have text or a content description.",metaText:"This [[object]] element does not have text or a content description",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="radio"]','*[data-ae_vis][data-ae_avat][data-ae_ar="radio"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-701
	 * @returns {AutoTestShell} testShell
	 */function test701(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("701",{bestPractice:1963,testId:701,introduced:"2.10",mediaType:mediaTypes.ANDROID,description:"One or more DatePicker elements does not have a content description.",metaText:"This [[object]] element does not have a content description",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="date"]','input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="date"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-702
	 * @returns {AutoTestShell} testShell
	 */function test702(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("702",{bestPractice:1963,testId:702,introduced:"2.10",mediaType:mediaTypes.ANDROID,description:"One or more TimePicker elements does not have a content description.",metaText:"This [[object]] element does not have a content description",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="time"]','input[data-ae_ar="null"][data-ae_vis][data-ae_avat][type="time"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-703
	 * @returns {AutoTestShell} testShell
	 */function test703(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addAutomaticTest("703",{bestPractice:1963,testId:703,introduced:"2.10",mediaType:mediaTypes.ANDROID,description:"One or more ImageView elements does not have a content description.",metaText:"This [[object]] element does not have a content description",fixType:"",testFunc:function(){return new AutoTestShell('img[data-ae_ar="null"][data-ae_vis][data-ae_avat]:not([alt=""]), img[data-ae_ar="img"][data-ae_vis][data-ae_avat]:not([alt=""])','img[data-ae_ar="null"][data-ae_vis][data-ae_avat]:not([alt=""]):not([data-ae_an]), img[data-ae_ar="img"][data-ae_vis][data-ae_avat]:not([alt=""]):not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-740
	 * @returns {AutoTestShell} testShell
	 */function test740(){aeKernel.addAutomaticTest("740",{bestPractice:1962,testId:740,introduced:"2.11",mediaType:mediaTypes.ANDROID,description:"One or more Button, CheckBox, DatePicker, EditText, ImageView, RadioButton, RadioGroup, SearchView, Switch, ToggleButton, TimePicker, or VideoView elements is not focusable.",metaText:"This [[object]] is not focusable. The element may not be accessible by assistive technology.",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat]',//Button; CheckBox; DatePicker; EditText; ImageView; RadioButton; RadioGroup; SearchView; Switch; ToggleButton; TimePicker; VideoView.
'*[data-ae_vis][data-ae_avat][data-xmlnodetype="android.widget.Button"][disabled],*[data-ae_vis][data-ae_avat][data-xmlnodetype="android.widget.CheckBox"][disabled],*[data-ae_vis][data-ae_avat][data-xmlnodetype="android.widget.DatePicker"][disabled],*[data-ae_vis][data-ae_avat][data-xmlnodetype="android.widget.EditText"][disabled],*[data-ae_vis][data-ae_avat][data-xmlnodetype="android.widget.ImageView"][disabled],*[data-ae_vis][data-ae_avat][data-xmlnodetype="android.widget.RadioButton"][disabled],*[data-ae_vis][data-ae_avat][data-xmlnodetype="android.widget.RadioGroup"][disabled],*[data-ae_vis][data-ae_avat][data-xmlnodetype="android.widget.SearchView"][disabled],*[data-ae_vis][data-ae_avat][data-xmlnodetype="android.widget.Switch"][disabled],*[data-ae_vis][data-ae_avat][data-xmlnodetype="android.widget.ToggleButton"][disabled],*[data-ae_vis][data-ae_avat][data-xmlnodetype="android.widget.TimePicker"][disabled],*[data-ae_vis][data-ae_avat][data-xmlnodetype="android.widget.VideoView"][disabled]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-742
	 * @returns {AutoTestShell} testShell
	 */function test742(){aeKernel.addAutomaticTest("742",{bestPractice:2615,testId:742,introduced:"2.11",mediaType:mediaTypes.ANDROID,description:"One or more RadioButton elements is not an immediate child of a RadioGroup element.",metaText:"This [[object]] should be properly grouped within a RadioGroup element to establish a relationship between buttons and to the group container.",fixType:"",testFunc:function(){var testShell=new AutoTestShell('div[data-xmlnodetype="android.widget.RadioButton"][data-ae_vis][data-ae_avat]');testShell.set_candidateSetNodesFunc(function(){var bad=[];//Get all the radio buttons
var res=aeKernel.rootTestNode.querySelectorAll('div[data-xmlnodetype="android.widget.RadioButton"][data-ae_vis][data-ae_avat]');//Loop through each one and check if parent is a RadioGroup
for(var i=0,len_res=res.length;i<len_res;i++){var result=res[i];var nodeType=result.parentNode.getAttribute("data-xmlnodetype");if(nodeType!="android.widget.RadioGroup"){bad.push(result);}}return bad;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-788
	 * @returns {AutoTestShell} testShell
	 */function test788(){aeKernel.addAutomaticTest("788",{bestPractice:3112,testId:788,introduced:"2.13",mediaType:mediaTypes.ANDROID,description:"This application is locked from changing its orientation.",metaText:"This application is locked from changing its orientation",fixType:"",testFunc:function(){const relevantCssSelector="meta[http-equiv='ScreenOrientation']";return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[content='autoRotate:disabled']`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-17
	 * @returns {AutoTestShell} testShell
	 */function test17(){aeKernel.addGuidedAutomaticTest("17",{bestPractice:2042,testId:17,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more video elements without a muted attribute and with an autoplay attribute, without an ARIA-assigned role, not intentionally hidden in the DOM, has a duration greater than 3 seconds.",metaText:"This video element is not muted, has autoplay and has a duration that is greater than 3 seconds",fixType:"",testFunc:function(){const baseCssSelector='video:not([muted])[autoplay][data-ae_ar="null"][data-ae_vis]';return new AutoTestShell(baseCssSelector,`${baseCssSelector}[data-ae_durationtoolong]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-39
	 * @returns {AutoTestShell} testShell
	 */function test39(){aeKernel.addGuidedAutomaticTest("39",{bestPractice:2042,testId:39,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more audio elements without a muted attribute and with an autoplay attribute, without an ARIA-assigned role, not intentionally hidden in the DOM, has a duration greater than 3 seconds.",metaText:"This audio element is not muted, has autoplay and has a duration that is greater than 3 seconds",fixType:"",testFunc:function(){const baseCssSelector='audio:not([muted])[autoplay][data-ae_ar="null"][data-ae_vis]';return new AutoTestShell(baseCssSelector,`${baseCssSelector}[data-ae_durationtoolong]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-45
	 * @returns {AutoTestShell} testShell
	 */function test45(){aeKernel.addGuidedAutomaticTest("45",{bestPractice:480,testId:45,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more hr elements, without an ARIA-assigned role, not intentionally hidden in the DOM, has a width attribute.",metaText:"The width attribute is not allowed on this element",metaTextDetail:"The width attribute is not allowed on this {{tag-name}}",fixType:"",testFunc:function(){return new AutoTestShell('hr[data-ae_vis][data-ae_ar="null"]','hr[data-ae_vis][data-ae_ar="null"][width]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-52
	 * @returns {AutoTestShell} testShell
	 */function test52(){aeKernel.addGuidedAutomaticTest("52",{bestPractice:410,testId:52,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more iframe elements with a width > 1, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a width attribute set to a value that does not end with '%'.",metaText:"This element has a width attribute value in absolute units. Percentage based units need to be used",metaTextDetail:"This {{tag-name}} has a width attribute value in absolute units. Percentage based units need to be used",fixType:"",testFunc:function(){const relevantCssSelector='iframe[data-ae_vis][data-ae_avat][data-ae_ar="null"]';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[width]:not([width=""]):not([width="1"]):not([width="1px"]):not([width$="%"])`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-58
	 * @returns {AutoTestShell} testShell
	 */function test58(){aeKernel.addGuidedAutomaticTest("58",{bestPractice:410,testId:58,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more iframe elements with a height > 1, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a height attribute set to a value that does not end with '%'.",metaText:"This element has a height attribute value in absolute units. Percentage based units need to be used",metaTextDetail:"This {{tag-name}} has a height attribute value in absolute units. Percentage based units need to be used",fixType:"",testFunc:function(){const relevantCssSelector='iframe[data-ae_vis][data-ae_avat][data-ae_ar="null"]';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[height]:not([height=""]):not([height="1"]):not([height="1px"]):not([height$="%"])`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-91
	 * @returns {AutoTestShell} testShell
	 */function test91(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("91",{bestPractice:332,testId:91,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more fieldset elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This fieldset element does not have a mechanism that allows an accessible name value to be calculated",fixType:"",testFunc:function(){return new AutoTestShell('fieldset[data-ae_ar="null"][data-ae_vis][data-ae_avat]','fieldset[data-ae_ar="null"][data-ae_vis][data-ae_avat]:not([data-ae_an])');}});}/**
	 * Defines which input types should be excluded from color contrast testing
	 * @type {string[]}
	 */const INPUT_TYPE_EXCLUSIONS_FOR_COLOR_CONTRAST_TESTING=['checkbox','color','radio','range','hidden'];/**
	 * used in colour contrast calc
	 * @param {String} rgbCssStyle
	 * @returns {Number[]}
	 */function extractRGB(rgbCssStyle){const rgb=rgbCssStyle.replace(/rgba?\(/gi,"").replace(")","");const rgbArray=rgb.split(/, ?/);const alpha=rgbArray.length>=4?parseInt(rgbArray[3]):null;return[parseInt(rgbArray[0]),parseInt(rgbArray[1]),parseInt(rgbArray[2]),alpha];}/**
	 * used in colour contrast calc
	 * @param {Node} rgbIntValue
	 * @returns {Number}
	 */function calc_RGBvalues(rgbIntValue){var x=0;if(rgbIntValue<=0.04045){x=rgbIntValue/12.92;}else{x=Math.pow((rgbIntValue+0.055)/1.055,2.4);}return x;}/**
	 * used in colour contrast calc
	 * @param {Node} rgbIntValues
	 * @returns {Number}
	 */function cal_Luminance(rgbIntValues){var R_sRGB=rgbIntValues[0]/255;var G_sRGB=rgbIntValues[1]/255;var B_sRGB=rgbIntValues[2]/255;var R=calc_RGBvalues(R_sRGB);var G=calc_RGBvalues(G_sRGB);var B=calc_RGBvalues(B_sRGB);return 0.2126*R+0.7152*G+0.0722*B;}/**
	 * Calculates the Contrast ratio for a text node (nodeType 3) against the background color of the first ancestor with a non-transparent background.
	 * @param {Node} nodeUnderTest
	 * @returns {Number}
	 */function util_calc_ContrastRatio(nodeUnderTest){// if background is rgba it means transparent, so you have to go back to parent that has a background that is not rgba
var textColour=nodeUnderTest.getAttribute("data-ae_color");var text_extractedRGB=extractRGB(textColour);var L_text=cal_Luminance(text_extractedRGB);// gets ancestor with non-transparent color
var backgroundColour=nodeUnderTest.getAttribute("data-ae_bgcolor");// get background from parent
var bg_extractedRGB=extractRGB(backgroundColour);var L_bgColour=cal_Luminance(bg_extractedRGB);var contrastRatio=0;if(L_text<L_bgColour){contrastRatio=(L_bgColour+0.05)/(L_text+0.05);}else{contrastRatio=(L_text+0.05)/(L_bgColour+0.05);}return contrastRatio;}/**
	 * Finds text nodes (nodeType 3) which contain text.
	 * @returns {Array}
	 */function get_targetNodes(rootTestNode,relevantCssSelector){const nodesOfInterest=[];const containsCharsOfInterest=str=>{return str.match(/[0-9a-zA-Z]/g)!==null;};const nodes=rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0;i<nodes.length;i++){const node=nodes[i];if(node.nodeType===1&&node.tagName==="INPUT"&&!INPUT_TYPE_EXCLUSIONS_FOR_COLOR_CONTRAST_TESTING.includes(node.type)){if(node.value&&containsCharsOfInterest(node.value)){nodesOfInterest.push(node);}}else if(node.firstChild!==null){let n=node.firstChild;do{if(n.nodeType===3&&n.nodeValue&&containsCharsOfInterest(n.nodeValue)){nodesOfInterest.push(node);break;}}while(n=n.nextSibling);}}return nodesOfInterest;}/**
	 * Finds the set of nodes with a text / background color ratio under the specified luminosity.
	 * @returns {Node[]}
	 */function testUtil_colorContrast(rootTestNode,relevantCssSelector,relLumRatio){var nodesWithColourContrastIssues=[];// this is for text sizes - minimum contrast ratio for text that is less than 18 point (if not bold) and less than 14 point (if bold)
// TODO: will need to check if bold in css
// 1st - Text Node (i.e. rootTestNode.querySelector("#xTest").children.length === 0 && rootTestNode.querySelector("#xTest").firstChild !== null then firstChild.data.trim()!=""), then that is the target node
// 2nd - if background is rgba it means transparent, so you have to go back to parent that has a background that is not rgba
var textNodes=get_targetNodes(rootTestNode,relevantCssSelector);for(var i=0,len_textNodes=textNodes.length;i<len_textNodes;i++){var contrastRatio=util_calc_ContrastRatio(textNodes[i]);textNodes[i].setAttribute("data-ae_cr",Math.round(contrastRatio*1000)/1000);if(contrastRatio<relLumRatio){nodesWithColourContrastIssues[nodesWithColourContrastIssues.length]=textNodes[i];}}return nodesWithColourContrastIssues;}// defines which presentational SVG node types should should have their 'fill' attributes treated as foreground colors versus background colors
// (each array is a subset of tag names from the one in this list: https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/fill)
const PRESENTATIONAL_SVG_NODES_FILL_BACKGROUND=['circle','ellipse','path','polygon','polyline','rect'];const PRESENTATIONAL_SVG_NODES_FILL_FOREGROUND=['altglyph','text','textPath','tref','tspan'];/**
	 * Extract the information we need for color contrast testing from computed styles
	 */function getBackgroundColor_FromStyles(computedStyles,testWind,node){let backgroundColor;if(node&&node.tagName&&PRESENTATIONAL_SVG_NODES_FILL_BACKGROUND.indexOf(node.tagName.toLowerCase())!==-1){backgroundColor=computedStyles["fill"];}else{backgroundColor=computedStyles["background-color"];}return"background-color;"+backgroundColor;}/**
	 * Extract the information we need for color contrast testing from computed styles
	 */function getFontSize_FromStyles(computedStyles){return"font-size;"+computedStyles["font-size"];}/**
	 * Extract the information we need for color contrast testing from computed styles
	 */function getFontWeight_FromStyles(computedStyles){return"font-weight;"+computedStyles["font-weight"];}/**
	 * Extract the information we need for color contrast testing from computed styles
	 */function getColor_FromStyles(computedStyles,testWind,node){let color;if(node&&node.tagName&&PRESENTATIONAL_SVG_NODES_FILL_FOREGROUND.indexOf(node.tagName.toLowerCase())!==-1){color=computedStyles["fill"];}else{color=computedStyles["color"];}return"color;"+color;}function getHeight_FromStyles(computedStyles){return"height;"+computedStyles["height"];}function getWidth_FromStyles(computedStyles){return"width;"+computedStyles["width"];}// import this file through:
/*
	 * Pre-Testing: Finds elements that have events added in code behind and adds an "data-ae_ev" attribute
	 * @returns {Number}
	 */function colorContrast(rootTestNode){return rootTestNode;}function isRelevantBackgroundColor(backgroundColor){if(!backgroundColor){return false;}if(!backgroundColor.startsWith("rgb")){return false;}const alpha=extractRGB(backgroundColor)[3];if(alpha!==null&&alpha<=0){return false;}return true;}/**
	 * Sets the data-ae_bgcolor attribute to relevant background color
	 * @param {{}} originalInformation
	 * @param node
	 */function setBackgroundColor(originalInformation,node){try{let backgroundColor;const nodeBackgroundColor=originalInformation["background-color"];if(isRelevantBackgroundColor(nodeBackgroundColor)){// use the target node's background color
backgroundColor=nodeBackgroundColor;}else{// use the nearest relevant background color up the ancestry tree for the target node, if available
for(let ancestorNode=node;ancestorNode=getParent(ancestorNode);){let ancestorNodeBackgroundColor=ancestorNode.getAttribute("data-ae_bgcolor");if(ancestorNodeBackgroundColor){// reuse the background color we've already assigned to this ancestor node
backgroundColor=ancestorNodeBackgroundColor;break;}else{const ancestorNodeComputedStyles=retrieveComputedCssStyles(ancestorNode);ancestorNodeBackgroundColor=ancestorNodeComputedStyles["background-color"];if(isRelevantBackgroundColor(ancestorNodeBackgroundColor)){backgroundColor=ancestorNodeBackgroundColor;break;}}}if(!backgroundColor){// fall back to assuming a white background
backgroundColor="rgb(255, 255, 255)";}}return["data-ae_bgcolor",backgroundColor];}catch(err){logger.error("stage5_setBackgroundColor",err);return null;}}/**
	 * Sets the data-ae_color attribute to relevant text color
	 * @param {{}} originalInformation
	 * @param node
	 */function setTextColor(originalInformation,node){try{// Text contrast
if(hasText(node)){const fontWeight=originalInformation["font-weight"];const fontSize=originalInformation["font-size"];if(fontWeight!==undefined&&fontSize!==undefined){if(fontWeight==="normal"||parseInt(fontWeight)<700){// if text is less that 18 point then 4.5:1
// set color attributes for use in util_colourContrast
return["data-ae_color",originalInformation["color"]];}if(fontWeight==="bold"||parseInt(fontWeight)>=700){// if text is less that 14 point then 4.5:1
// set color attributes for use in util_colourContrast
return["data-ae_color",originalInformation["color"]];}}}return null;}catch(err){logger.error("stage5_setTextColor",err);return null;}}function inPoints(fontSize){let pt=18;if(fontSize.indexOf("px")!==-1){fontSize=parseFloat(fontSize.replace("px",""));pt=3/4*fontSize;}else if(fontSize.indexOf("em")!==-1){fontSize=parseFloat(fontSize.replace("em",""));pt=12*fontSize;}else if(fontSize.indexOf("%")!==-1){fontSize=parseFloat(fontSize.replace("%",""));pt=fontSize/(100/12);}return pt;}/**
	 * Calculates the relevant contrast ratio selector for text
	 * @param {{}} originalInformation
	 * @param node
	 */function setRelevantContrastRatioForText(originalInformation,node){try{// Text contrast
if(hasText(node)){const fontWeight=originalInformation["font-weight"];const fontSize=originalInformation["font-size"];if(fontWeight!==undefined&&fontSize!==undefined){const fontSizeInPoints=inPoints(fontSize);if(fontWeight==="normal"||parseInt(fontWeight)<700){// if text is less that 18 point then 4.5:1
// set color attributes for use in util_colourContrast
if(fontSizeInPoints<18){return["data-ae_ts","4.5"];}// if text is 18 point or larger then 3:1
else{return["data-ae_ts","3"];}}if(fontWeight==="bold"||parseInt(fontWeight)>=700){// if text is less that 14 point then 4.5:1
// set color attributes for use in util_colourContrast
if(fontSizeInPoints<14){return["data-ae_ts","4.5"];}// if text is 14 point or larger then 3:1
else{return["data-ae_ts","3"];}}}}return null;}catch(err){logger.error("stage5_setRelevantContrastRatioForText",err);return null;}}/**
	 * Returns true if the element has text, otherwise returns false.
	 *
	 * @param {Element} element
	 * @return {boolean}
	 */function hasText(element){const textContent=element.textContent;if(textContent&&textContent.trim()){return true;}const inputValueAttributeValue=element.tagName==="INPUT"&&!INPUT_TYPE_EXCLUSIONS_FOR_COLOR_CONTRAST_TESTING.includes(element.type)?element.value:null;if(inputValueAttributeValue&&inputValueAttributeValue.trim()){return true;}return false;}function setIsTiny(originalInformation,element){try{if(element.hasAttribute("data-ae_tiny")){// we've already determined this element is tiny
return null;}const heightString=originalInformation["height"];const widthString=originalInformation["width"];const isTiny=heightString==="1px"&&widthString==="1px";if(!isTiny){return null;}// element has just been determined to be tiny, so we now know all its descendents are tiny too
// set 'tiny' attribute on all this element's descendents
const descendentElements=element.querySelectorAll("*");for(let i=0;i<descendentElements.length;i++){const descendentElement=descendentElements[i];descendentElement.setAttribute("data-ae_tiny","true");}// set 'tiny' attribute on element
return["data-ae_tiny","true"];}catch(err){logger.error("stage5_setIsTiny",err);return null;}}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-107
	 * @returns {AutoTestShell} testShell
	 */function test107(){aeKernel.addStage1MarkdownFunc("getBackgroundColor_FromStyles",getBackgroundColor_FromStyles);aeKernel.addStage1MarkdownFunc("getFontSize_FromStyles",getFontSize_FromStyles);aeKernel.addStage1MarkdownFunc("getFontWeight_FromStyles",getFontWeight_FromStyles);aeKernel.addStage1MarkdownFunc("getColor_FromStyles",getColor_FromStyles);aeKernel.addStage1MarkdownFunc("getHeight_FromStyles",getHeight_FromStyles);aeKernel.addStage1MarkdownFunc("getWidth_FromStyles",getWidth_FromStyles);aeKernel.addStage4MarkdownFunc("colorContrast",colorContrast);aeKernel.addStage5MarkdownFunc("setBackgroundColor",setBackgroundColor);aeKernel.addStage5MarkdownFunc("setTextColor",setTextColor);aeKernel.addStage5MarkdownFunc("setRelevantContrastRatioForText",setRelevantContrastRatioForText);aeKernel.addStage5MarkdownFunc("setIsTiny",setIsTiny);aeKernel.addGuidedAutomaticTest("107",{bestPractice:368,testId:107,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements, excluding elements with a size of 1px by 1px, not intentionally hidden in the DOM, contains text less than 18 point in size; or bold text less than 14 point in size has a luminosity contrast ratio below 4.5:1.",metaText:"This element contains text that is less than 18 point in size; or bold text less than 14 point in size that has a luminosity contrast ratio below 4.5:1",metaTextDetail:"This {{tag-name}} contains text with a background color of {{data-ae_bgcolor}} and foreground color of {{data-ae_color}} that is less than 18 point in size; or bold text less than 14 point in size that has a luminosity contrast ratio of {{data-ae_cr}}, which is below 4.5:1",fixType:"",testFunc:function(){const relevantCssSelector='*:not(body):not(option):not([data-ae_tiny])[data-ae_vis]:not(script):not(noscript)[data-ae_ts="4.5"]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){return testUtil_colorContrast(aeKernel.rootTestNode,relevantCssSelector,4.5);});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-109
	 * @returns {AutoTestShell} testShell
	 */function test109(){aeKernel.addStage1MarkdownFunc("getBackgroundColor_FromStyles",getBackgroundColor_FromStyles);aeKernel.addStage1MarkdownFunc("getFontSize_FromStyles",getFontSize_FromStyles);aeKernel.addStage1MarkdownFunc("getFontWeight_FromStyles",getFontWeight_FromStyles);aeKernel.addStage1MarkdownFunc("getColor_FromStyles",getColor_FromStyles);aeKernel.addStage1MarkdownFunc("getHeight_FromStyles",getHeight_FromStyles);aeKernel.addStage1MarkdownFunc("getWidth_FromStyles",getWidth_FromStyles);aeKernel.addStage4MarkdownFunc("colorContrast",colorContrast);aeKernel.addStage5MarkdownFunc("setBackgroundColor",setBackgroundColor);aeKernel.addStage5MarkdownFunc("setTextColor",setTextColor);aeKernel.addStage5MarkdownFunc("setRelevantContrastRatioForText",setRelevantContrastRatioForText);aeKernel.addStage5MarkdownFunc("setIsTiny",setIsTiny);aeKernel.addGuidedAutomaticTest("109",{bestPractice:368,testId:109,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements, excluding elements with a size of 1px by 1px, not intentionally hidden in the DOM, contains text that is 18 point in size or larger; or bold text that is 14 point in size or larger has a luminosity contrast ratio below 3:1.",metaText:"This element contains text that is 18 point in size or larger; or bold text that is 14 point in size or larger that has a luminosity contrast ratio below 3:1",metaTextDetail:"This {{tag-name}} contains text with a background color of {{data-ae_bgcolor}} and foreground color of {{data-ae_color}} that is 18 point in size or larger; or bold text that is 14 point in size or larger that has a luminosity contrast ratio of {{data-ae_cr}}, which is below 3:1",fixType:"",testFunc:function(){const relevantCssSelector='*:not(body):not([data-ae_tiny])[data-ae_vis]:not(script):not(noscript)[data-ae_ts="3"]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){return testUtil_colorContrast(aeKernel.rootTestNode,relevantCssSelector,3);});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-112
	 * @returns {AutoTestShell} testShell
	 */function test112(){aeKernel.addStage4MarkdownFunc("addAEEventAtts",addAEEventAtts);aeKernel.addGuidedAutomaticTest("112",{bestPractice:359,testId:112,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements, excluding A elements with an href attribute set to a text value; input elements; area elements; or button elements, not intentionally hidden in the DOM, are only attached to a click event listener, rather than a click and key event ('keypress' | 'keydown' | 'keyup') listener pair.",metaText:"This element is only attached to a click event listener, rather than a click and key event (keypress | keydown | keyup) listener pair",metaTextDetail:"This {{tag-name}} is only attached to a click event listener, rather than a click and key event (keypress | keydown | keyup) listener pair",fixType:"",testFunc:function(){const testShell=new AutoTestShell('*:not(input):not(area):not(button)[data-ae_vis][data-ae_ev*="click"]');testShell.set_candidateSetNodesFunc(function(){const issues=[];const res=testUtil_ifDiff(aeKernel.rootTestNode,['*[data-ae_vis][data-ae_ev*="click"]','*[data-ae_vis][data-ae_ev*="click"][data-ae_ev*="keypress"], *[data-ae_vis][data-ae_ev*="click"][data-ae_ev*="keydown"], *[data-ae_vis][data-ae_ev*="click"][data-ae_ev*="keyup"]']);// remove exclusions - as it is too difficult to remove them by css alone
for(let i=0,len_res=res.length;i<len_res;i++){const result=res[i];switch(result.tagName){case"A":const href=result.getAttribute("href");if(href===null||href===""){issues.push(result);}break;case"INPUT":break;case"AREA":break;case"BUTTON":break;default:issues.push(result);break;}}return issues;});return testShell;}});}/**
	 * Finds the set of heading nodes (hx, or role="heading") within a node defined by a CSS selector that are not properly labelled.
	 * @param {String} cssSelector
	 * @returns {Array}
	 */function testUtil_approNestedHeadings(rootTestNode,cssSelector){var res=rootTestNode.querySelectorAll(cssSelector);var incorrectlyNestedHeaders=[];for(var i=0,len_res=res.length;i<len_res;i++){// var clone = res[i].cloneNode(true);
var clone=cloneThisNode(res[i],true);var articlesToRemove=clone.querySelectorAll("article");for(var j=0,len_articlesToRemove=articlesToRemove.length;j<len_articlesToRemove;j++){articlesToRemove[j].outerHTML="";// removes all contained article nodes
}// Also we need to include <div role="heading" aria-level="x"> what happens if aria-level is not there?
var headers=clone.querySelectorAll("h1[data-ae_ar='null'][data-ae_vis]:not(:empty), h2[data-ae_ar='null'][data-ae_vis]:not(:empty), h3[data-ae_ar='null'][data-ae_vis]:not(:empty), h4[data-ae_ar='null'][data-ae_vis]:not(:empty), h5[data-ae_ar='null'][data-ae_vis]:not(:empty), h6[data-ae_ar='null'][data-ae_vis]:not(:empty), *[data-ae_ar='heading'][data-ae_vis][aria-level]:not(:empty)");var headersAsInts=[];// turn header text values into a stack of integers
for(var k=0,len_headers=headers.length;k<len_headers;k++){var ariaLevel=headers[k].getAttribute("aria-level");if(ariaLevel!==null){headersAsInts[headersAsInts.length]=parseInt(ariaLevel);}else{headersAsInts[headersAsInts.length]=parseInt(headers[k].tagName[1]);}}// process the heading levels as integers
var numHeads=headersAsInts.length;if(numHeads===0){incorrectlyNestedHeaders[incorrectlyNestedHeaders.length]=res[i];return incorrectlyNestedHeaders;}if(numHeads===1){if(headersAsInts[0]!==1){incorrectlyNestedHeaders[incorrectlyNestedHeaders.length]=res[i];return incorrectlyNestedHeaders;}}if(numHeads>1){for(var m=0;m<numHeads;m++){var currHeadNum=headersAsInts[m];if(m===0){if(headersAsInts[m]!==1){incorrectlyNestedHeaders[incorrectlyNestedHeaders.length]=res[i];return incorrectlyNestedHeaders;}}else if(currHeadNum!==1&&(currHeadNum<=headersAsInts[m-1]||currHeadNum===headersAsInts[m-1]+1)){return[];}else{incorrectlyNestedHeaders[incorrectlyNestedHeaders.length]=res[i];return incorrectlyNestedHeaders;}}}}return incorrectlyNestedHeaders;}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-129
	 * @returns {AutoTestShell} testShell
	 */function test129(){aeKernel.addGuidedAutomaticTest("129",{bestPractice:387,testId:129,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more headings (created using h1 - h6 elements (without an ARIA-assigned role); or elements with a role attribute set to a value that starts with 'heading' and an aria-level attribute set to an appropriate value) inside an article element (excluding the contents of any further enclosed article elements) are not set to a heading level that can be considered as properly nested within the document heading hierarchy.",metaText:"This article element contains an incorrect or missing heading level which may cause improper nesting within the document heading hierarchy",fixType:"",testFunc:function(){var testShell=new AutoTestShell('article[data-ae_vis][data-ae_avat]');testShell.set_candidateSetNodesFunc(function(){return testUtil_approNestedHeadings(aeKernel.rootTestNode,"article");});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-135
	 * @returns {AutoTestShell} testShell
	 */function test135(){aeKernel.addStage4MarkdownFunc("addAEEventAtts",addAEEventAtts);aeKernel.addGuidedAutomaticTest("135",{bestPractice:359,testId:135,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM, are only attached to a dblclick event listener, rather than a dblclick and key event ('keypress' | 'keydown' | 'keyup') listener pair.",metaText:"This element is only attached to a dblclick event listener, rather than a dblclick and key event (keypress | keydown | keyup) listener pair",metaTextDetail:"This {{tag-name}} is only attached to a dblclick event listener, rather than a dblclick and key event (keypress | keydown | keyup) listener pair",fixType:"",testFunc:function(){var testShell=new AutoTestShell('*[data-ae_vis][data-ae_ev*="dblclick"]');testShell.set_candidateSetNodesFunc(function(){return testUtil_ifDiff(aeKernel.rootTestNode,['*[data-ae_vis][data-ae_ev*="dblclick"]','*[data-ae_vis][data-ae_ev*="dblclick"][data-ae_ev*="keypress"], *[data-ae_vis][data-ae_ev*="dblclick"][data-ae_ev*="keydown"], *[data-ae_vis][data-ae_ev*="dblclick"][data-ae_ev*="keyup"]']);});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-136
	 * @returns {AutoTestShell} testShell
	 */function test136(){aeKernel.addStage4MarkdownFunc("addAEEventAtts",addAEEventAtts);aeKernel.addGuidedAutomaticTest("136",{bestPractice:359,testId:136,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM, are only attached to a mouseout event listener, rather than a mouseout and blur event listener pair.",metaText:"This element is only attached to a mouseout event listener, rather than a mouseout and blur event listener pair",metaTextDetail:"This {{tag-name}} is only attached to a mouseout event listener, rather than a mouseout and blur event listener pair",fixType:"",testFunc:function(){var testShell=new AutoTestShell('*[data-ae_vis][data-ae_ev*="mouseout"]');testShell.set_candidateSetNodesFunc(function(){return testUtil_ifDiff(aeKernel.rootTestNode,['*[data-ae_vis][data-ae_ev*="mouseout"]','*[data-ae_vis][data-ae_ev*="mouseout"][data-ae_ev*="blur"]']);});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-137
	 * @returns {AutoTestShell} testShell
	 */function test137(){aeKernel.addStage4MarkdownFunc("addAEEventAtts",addAEEventAtts);aeKernel.addGuidedAutomaticTest("137",{bestPractice:359,testId:137,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM, are only attached to a mouse event listener (mousedown or mouseup), rather than a mouse event listener and key event ('keypress' | 'keydown' | 'keyup') listener pair.",metaText:"This element is only attached to a mouse event listener (mousedown or mouseup), rather than a mouse event listener and key event (keypress | keydown | keyup) listener pair",metaTextDetail:"This {{tag-name}} is only attached to a mouse event listener (mousedown or mouseup), rather than a mouse event listener and key event (keypress | keydown | keyup) listener pair",fixType:"",testFunc:function(){var testShell=new AutoTestShell('*[data-ae_vis][data-ae_ev*="mousedown"],*[data-ae_vis][data-ae_ev*="mouseup"]');testShell.set_candidateSetNodesFunc(function(){return testUtil_ifDiff(aeKernel.rootTestNode,['*[data-ae_vis][data-ae_ev*="mousedown"],*[data-ae_vis][data-ae_ev*="mouseup"]','*[data-ae_vis][data-ae_ev*="mousedown"][data-ae_ev*="keypress"], *[data-ae_vis][data-ae_ev*="mousedown"][data-ae_ev*="keydown"], *[data-ae_vis][data-ae_ev*="mousedown"][data-ae_ev*="keyup"],*[data-ae_vis][data-ae_ev*="mouseup"][data-ae_ev*="keypress"], *[data-ae_vis][data-ae_ev*="mouseup"][data-ae_ev*="keydown"], *[data-ae_vis][data-ae_ev*="mouseup"][data-ae_ev*="keyup"]']);});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-138
	 * @returns {AutoTestShell} testShell
	 */function test138(){aeKernel.addStage4MarkdownFunc("addAEEventAtts",addAEEventAtts);aeKernel.addGuidedAutomaticTest("138",{bestPractice:359,testId:138,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM, are only attached to a mouseover event listener, rather than a mouseover and focus event listener pair.",metaText:"This element is only attached to a mouseover event listener, rather than a mouseover and focus event listener pair",metaTextDetail:"This {{tag-name}} is only attached to a mouseover event listener, rather than a mouseover and focus event listener pair",fixType:"",testFunc:function(){var testShell=new AutoTestShell('*[data-ae_vis][data-ae_ev*="mouseover"]');testShell.set_candidateSetNodesFunc(function(){return testUtil_ifDiff(aeKernel.rootTestNode,['*[data-ae_vis][data-ae_ev*="mouseover"]','*[data-ae_vis][data-ae_ev*="mouseover"][data-ae_ev*="focus"]']);});return testShell;}});}/**
	 * NOTE: inclusion of the area element in the na / largecss group is important, as area is by default display:none in Firefox.  Also, it is still seen by screen-reader users / keyboard users if it is display:none in Chrome.
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-153
	 * @returns {AutoTestShell} testShell
	 */function test153(){aeKernel.addStage4MarkdownFunc("addAEEventAtts",addAEEventAtts);aeKernel.addGuidedAutomaticTest("153",{bestPractice:359,testId:153,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more elements (excluding interaction elements - A elements with an href attribute set to a text value (without an ARIA-assigned role), area elements with an href attribute set to a text value (without an ARIA-assigned role), input elements (without an ARIA-assigned role), select elements (without an ARIA-assigned role), textarea elements (without an ARIA-assigned role), button elements (without an ARIA-assigned role), summary elements (without an ARIA-assigned role), audio elements (without an ARIA-assigned role), video elements(without an ARIA-assigned role)) attached to a click or keyboard event listener, not intentionally hidden in the DOM, do not have a tabindex attribute set to a value greater than, or equal to 0.",metaText:"This element does not have a tabindex attribute set to a value greater than, or equal to 0",metaTextDetail:"This {{tag-name}} does not have a tabindex attribute set to a value greater than, or equal to 0",fixType:{"fixType":1,"domSpec":false,"helperText":"Select the tabindex number corresponding with the correct tab order for this element.","fix":{"tabindex":null}},testFunc:function(){var testShell=new AutoTestShell('body *[data-ae_vis][data-ae_ev*="click"], body *[data-ae_vis][data-ae_ev*="keypress"], body *[data-ae_vis][data-ae_ev*="keydown"], body *[data-ae_vis][data-ae_ev*="keyup"], area[data-ae_ev*="click"]:not([data-ae_ar="null"]), area[data-ae_ev*="keypress"]:not([data-ae_ar="null"]), area[data-ae_ev*="click"]:not([data-ae_ar="keydown"]), area[data-ae_ev*="click"]:not([data-ae_ar="keyup"])');testShell.set_candidateSetNodesFunc(function(){var notKeyboardAcc=[];var cssLargeGroup='body *[data-ae_vis][data-ae_ev*="click"], body *[data-ae_vis][data-ae_ev*="keypress"], body *[data-ae_vis][data-ae_ev*="keydown"], body *[data-ae_vis][data-ae_ev*="keyup"], area[data-ae_ev*="click"]:not([data-ae_ar="null"]), area[data-ae_ev*="keypress"]:not([data-ae_ar="null"]), area[data-ae_ev*="click"]:not([data-ae_ar="keydown"]), area[data-ae_ev*="click"]:not([data-ae_ar="keyup"])';var cssGroupToExcludeFromLargeGroup='a[href]:not([href=""])[data-ae_vis], area[href]:not([href=""])[data-ae_ar="null"], input[data-ae_ar="null"][data-ae_vis], select[data-ae_ar="null"][data-ae_vis], textarea[data-ae_ar="null"][data-ae_vis], button[data-ae_ar="null"][data-ae_vis], summary[data-ae_ar="null"][data-ae_vis], audio[data-ae_ar="null"][data-ae_vis], video[data-ae_ar="null"][data-ae_vis]';var remainingGroup=testUtil_ifDiff(aeKernel.rootTestNode,[cssLargeGroup,cssGroupToExcludeFromLargeGroup]);for(var i=remainingGroup.length;i--;){var el=remainingGroup[i];var tabIndex=el.getAttribute("tabindex");if(tabIndex===null||tabIndex===""||isNaN(tabIndex)===true||tabIndex<0){notKeyboardAcc.push(el);}}return notKeyboardAcc;});return testShell;}});}/**
	 * This test will be superseded by a Machine Learning test
	 * text that is less than 4 characters;
	 * text that is less than 10 characters long or less, and contains the substring "more", "click here", "...", "???";
	 * text that is longer than 150 characters
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-239
	 * @returns {AutoTestShell} testShell
	 */function test239(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("239",{bestPractice:1301,testId:239,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more A elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious calculated accessible name value.",metaText:"This A element has a suspicious (i.e. lacks purpose or is >150 characters) calculated accessible name value",metaTextDetail:"This A element has a suspicious (i.e. lacks purpose or is >150 characters) calculated accessible name value of: {{data-ae_an}}",fixType:{"fixType":1,"domSpec":false,"helperText":'Provide a descriptive accessible name for the anchor element.',"fix":{"aria-label":null,"aria-labelledby":false}},testFunc:function(){const relevantCssSelector='a[data-ae_vis][data-ae_avat][data-ae_ar="null"][data-ae_an]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const suspiciousText=[];const res=aeKernel.rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0;i<res.length;i++){const result=res[i];const accName=result.getAttribute("data-ae_an");const accNameLen=accName.length;if(accNameLen<4||accNameLen<11&&(accName.indexOf("more")!==-1/* includes "read more" */||accName.indexOf("click here")!==-1||accName.indexOf("...")!==-1||accName.indexOf("???")!==-1)||accNameLen>150){suspiciousText.push(result);}}return suspiciousText;});return testShell;}});}/**
	 * This test will be superseded by a Machine Learning test
	 * text that is less than 4 characters;
	 * text that is less than 10 characters long or less, and contains the substring "more", "click here", "...", "???";
	 * text that is longer than 150 characters
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-241
	 * @returns {AutoTestShell} testShell
	 */function test241(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("241",{bestPractice:1301,testId:241,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements that have a role='link' attribute, not intentionally hidden in the DOM and available to assistive technologies, has suspicious text as an accessible name value.",metaText:"This element (role=link) has a suspicious calculated accessible name value",metaTextDetail:"This {{tag-name}} (role=link) has a suspicious calculated accessible name value of: {{data-ae_an}}",fixType:{"fixType":1,"domSpec":false,"helperText":'Provide a descriptive accessible name for the anchor element.',"fix":{"aria-label":null,"aria-labelledby":false}},testFunc:function(){const relevantCssSelector='*[data-ae_vis][data-ae_avat][data-ae_ar="link"][data-ae_an]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const suspiciousText=[];const res=aeKernel.rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0;i<res.length;i++){const result=res[i];const accName=result.getAttribute("data-ae_an");const accNameLen=accName.length;if(accNameLen<4||accNameLen<11&&(accName.indexOf("more")!==-1/* includes "read more" */||accName.indexOf("click here")!==-1||accName.indexOf("...")!==-1||accName.indexOf("???")!==-1)||accNameLen>150){suspiciousText.push(result);}}return suspiciousText;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-242
	 * @returns {AutoTestShell} testShell
	 */function test242(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("242",{bestPractice:1301,testId:242,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more A elements with a unique href value, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, have an accessible name value that is not unique.",metaText:"This A element has an accessible name that is the same as another A element but doesn't go to the same HREF.",metaTextDetail:"This A element has an accessible name of '{{data-ae_an}}' that is the same as another A element but doesn't go to the same HREF.",fixType:{"fixType":1,"domSpec":false,"helperText":"Provide a descriptive accessible name for this element.","fix":{"aria-label":null,"aria-labelledby":false}},testFunc:function(){const testShell=new AutoTestShell();const relevantCssSelector='a[href]:not([href=""])[data-ae_ar="null"][data-ae_vis][data-ae_avat][data-ae_an]';testShell.set_applicabilityTest(function(){return this.applicableNodesFunc(this._applicableNodesFuncArgs).length>1;});testShell.set_applicableNodesFunc(function(){return this.querySelectorAll(relevantCssSelector);});testShell.set_candidateSetNodesFunc(function(){let suspiciousLinks=[];const linksByAccessibleName={};// group all relevant links by their accessible name
const links=this.querySelectorAll(relevantCssSelector);for(let i=0;i<links.length;i++){const link=links[i];const accessibleName=link.getAttribute("data-ae_an");if(linksByAccessibleName[accessibleName]==null){linksByAccessibleName[accessibleName]=[link];}else{linksByAccessibleName[accessibleName].push(link);}}// flag any relevant links that have a duplicative accessible name
const accessibleNames=Object.keys(linksByAccessibleName);for(let i=0;i<accessibleNames.length;i++){const accessibleName=accessibleNames[i];const links=linksByAccessibleName[accessibleName];if(links.length>1){let isSuspicious=false;// all links should have the same href, otherwise they're suspicious
let firstHref=null;for(let j=0;j<links.length;j++){const link=links[j];const href=link.getAttribute("href");if(firstHref===null){firstHref=href;}else if(href!==firstHref){isSuspicious=true;break;}}if(isSuspicious){suspiciousLinks=suspiciousLinks.concat(links);}}}return suspiciousLinks;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-248
	 * @returns {AutoTestShell} testShell
	 */function test248(){aeKernel.addGuidedAutomaticTest("248",{bestPractice:410,testId:248,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more frameset elements, not intentionally hidden in the DOM, has a cols attribute value that includes absolute units.",metaText:"This element has a cols attribute value in absolute units. Percentage based units need to be used",metaTextDetail:"This {{tag-name}} has a cols attribute value in absolute units. Percentage based units need to be used",fixType:"",testFunc:function(){var testShell=new AutoTestShell('frameset[data-ae_vis][cols]:not([cols=""])');testShell.set_candidateSetNodesFunc(function(){var framesetWithAbsolCols=[];var framesets=aeKernel.rootTestNode.querySelectorAll('frameset[data-ae_vis][cols]:not([cols=""])');for(var i=0,len_framesets=framesets.length;i<len_framesets;i++){var frameset=framesets[i];var cols=frameset.getAttribute("cols");if(cols){var colValues=cols.split(",");for(var j=colValues.length;j--;){var colValue=colValues[j].trim().toLowerCase();if(isNaN(colValue)===false||colValue.indexOf("px")!==-1){framesetWithAbsolCols.push(frameset);break;}}}}return framesetWithAbsolCols;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-253
	 * @returns {AutoTestShell} testShell
	 */function test253(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("253",{bestPractice:476,testId:253,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more canvas elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This canvas element does not have a mechanism that allows an accessible name value to be calculated",fixType:"",testFunc:function(){return new AutoTestShell('canvas[data-ae_vis][data-ae_avat][data-ae_ar="null"]','canvas[data-ae_vis][data-ae_avat][data-ae_ar="null"]:not([data-ae_an])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-400 (supercedes 139)
	 * @returns {AutoTestShell} testShell
	 */function test400(){aeKernel.addGuidedAutomaticTest("400",{bestPractice:457,testId:400,introduced:"1.0",mediaType:mediaTypes.WEB,description:"The body element does not contain elements that form a heading structure (i.e. h1-h6 elements; or elements given a role=heading).",metaText:"The body element does not contain elements that form a heading structure (i.e. h1-h6 elements; or elements given a role=heading)",fixType:"",testFunc:function(){const testShell=new AutoTestShell('body');testShell.set_candidateSetNodesFunc(function(){var res=this.querySelector('body');var incorrectlyNestedHeaders=[];if(res){// var clone = res.cloneNode(true);
var clone=cloneThisNode(res,true);var articlesToRemove=clone.querySelectorAll("article");for(var j=0,len_articlesToRemove=articlesToRemove.length;j<len_articlesToRemove;j++){articlesToRemove[j].outerHTML="";// removes all contained article nodes
}var headers=clone.querySelectorAll("h1[data-ae_ar='null'][data-ae_vis]:not(:empty), h2[data-ae_ar='null'][data-ae_vis]:not(:empty), h3[data-ae_ar='null'][data-ae_vis]:not(:empty), h4[data-ae_ar='null'][data-ae_vis]:not(:empty), h5[data-ae_ar='null'][data-ae_vis]:not(:empty), h6[data-ae_ar='null'][data-ae_vis]:not(:empty), *[data-ae_ar='heading'][data-ae_vis][aria-level]:not(:empty)");if(headers.length===0){incorrectlyNestedHeaders.push(res);}}return incorrectlyNestedHeaders;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-401 (supercedes 139)
	 * @returns {AutoTestShell} testShell
	 */function test401(){aeKernel.addGuidedAutomaticTest("401",{bestPractice:387,testId:401,introduced:"1.0",mediaType:mediaTypes.WEB,description:"The first element intended to create a visual heading (created using h1 - h6 elements (without an ARIA-assigned role); or elements with a role attribute set to a value that starts with 'heading' and an aria-level attribute set to an appropriate value), not intentionally hidden in the DOM and available to assistive technologies, does not create a level-1 heading.",metaText:"This element creates the only heading in this document, and, as such, should have been a level-1 heading",metaTextDetail:"This {{tag-name}} creates the only heading in this document, and, as such, should have been a level-1 heading",fixType:"",testFunc:function(){const testShell=new AutoTestShell();const headerCssSelector="h1[data-ae_ar='null'][data-ae_vis]:not(:empty), h2[data-ae_ar='null'][data-ae_vis]:not(:empty), h3[data-ae_ar='null'][data-ae_vis]:not(:empty), h4[data-ae_ar='null'][data-ae_vis]:not(:empty), h5[data-ae_ar='null'][data-ae_vis]:not(:empty), h6[data-ae_ar='null'][data-ae_vis]:not(:empty), *[data-ae_ar='heading'][data-ae_vis][aria-level]:not(:empty)";testShell.set_applicabilityTest(function(){return this.applicableNodesFunc(this._applicableNodesFuncArgs).length===1;});testShell.set_applicableNodesFunc(function(){const applicableNodes=[];const res=this.querySelector('body');if(res){const clone=cloneThisNode(res,true);const articlesToRemove=clone.querySelectorAll("article");for(let i=0;i<articlesToRemove.length;i++){articlesToRemove[i].outerHTML="";// removes all contained article nodes
}const headers=clone.querySelectorAll(headerCssSelector);if(headers.length===1){applicableNodes.push(headers[0]);}}return applicableNodes;});testShell.set_candidateSetNodesFunc(function(){var res=this.querySelector('body');var incorrectlyNestedHeaders=[];if(res){var clone=cloneThisNode(res,true);var articlesToRemove=clone.querySelectorAll("article");for(var j=0,len_articlesToRemove=articlesToRemove.length;j<len_articlesToRemove;j++){articlesToRemove[j].outerHTML="";// removes all contained article nodes
}var headers=clone.querySelectorAll(headerCssSelector);// turn header text values into a stack of integers
if(headers.length===1){var ariaLevel=headers[0].getAttribute("aria-level");if(ariaLevel!==null){if(parseInt(ariaLevel)!==1){incorrectlyNestedHeaders.push(res);}}else{if(parseInt(headers[0].tagName[1])!==1){incorrectlyNestedHeaders.push(res);}}}}return incorrectlyNestedHeaders;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-402 (supercedes 139)
	 * @returns {AutoTestShell} testShell
	 */function test402(){aeKernel.addGuidedAutomaticTest("402",{bestPractice:387,testId:402,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements (created using h1 - h6 elements (without an ARIA-assigned role); or elements with a role attribute set to a value that starts with 'heading' and an aria-level attribute set to an appropriate value), not intentionally hidden in the DOM and available to assistive technologies, creates an inappropriate jump in heading levels within the document heading hierarchy.",metaText:"This element creates an inappropriate jump in heading levels within the document heading hierarchy",metaTextDetail:"This {{tag-name}} creates an inappropriate jump in heading levels within the document heading hierarchy",fixType:{"fixType":1,"domSpec":false,"helperText":"Specify the correct heading level for this element.","fix":{"aria-level":null}},testFunc:function(){const testShell=new AutoTestShell();const headerCssSelector="h1[data-ae_ar='null'][data-ae_vis]:not(:empty), h2[data-ae_ar='null'][data-ae_vis]:not(:empty), h3[data-ae_ar='null'][data-ae_vis]:not(:empty), h4[data-ae_ar='null'][data-ae_vis]:not(:empty), h5[data-ae_ar='null'][data-ae_vis]:not(:empty), h6[data-ae_ar='null'][data-ae_vis]:not(:empty), *[data-ae_ar='heading'][data-ae_vis][aria-level]:not(:empty)";testShell.set_applicabilityTest(function(){return this.applicableNodesFunc(this._applicableNodesFuncArgs).length===1;});testShell.set_applicableNodesFunc(function(){const applicableNodes=[];const res=this.querySelector('body');if(res){const clone=cloneThisNode(res,true);const articlesToRemove=clone.querySelectorAll("article");for(let i=0;i<articlesToRemove.length;i++){articlesToRemove[i].outerHTML="";// removes all contained article nodes
}const headers=clone.querySelectorAll(headerCssSelector);if(headers.length>0){applicableNodes.push(headers[0]);}}return applicableNodes;});testShell.set_candidateSetNodesFunc(function(){var res=this.querySelector('body');var incorrectlyNestedHeaders=[];if(res){var clone=cloneThisNode(res,true);var articlesToRemove=clone.querySelectorAll("article");for(var j=0,len_articlesToRemove=articlesToRemove.length;j<len_articlesToRemove;j++){articlesToRemove[j].outerHTML="";// removes all contained article nodes
}var headers=clone.querySelectorAll(headerCssSelector);var numHeads=headers.length;var headersAsInts=[];// turn header text values into a stack of integers
for(var i=0,len_headers=headers.length;i<len_headers;i++){var ariaLevel=headers[i].getAttribute("aria-level");if(ariaLevel!==null){headersAsInts[headersAsInts.length]=parseInt(ariaLevel);}else{headersAsInts[headersAsInts.length]=parseInt(headers[i].tagName[1]);}}var k=0;// process the heading levels as integers
if(headersAsInts[0]!==1){incorrectlyNestedHeaders[incorrectlyNestedHeaders.length]=headers[k];}for(k=1;k<numHeads;k++){var currHeadNum=headersAsInts[k];if(currHeadNum===1||currHeadNum>headersAsInts[k-1]&&currHeadNum!==headersAsInts[k-1]+1){incorrectlyNestedHeaders[incorrectlyNestedHeaders.length]=headers[k];}}}return incorrectlyNestedHeaders;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-445
	 * @returns {AutoTestShell} testShell
	 */function test445(){aeKernel.addGuidedAutomaticTest("445",{bestPractice:1871,testId:445,introduced:"2.1",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM and available to assitive technologies were found to have a role set to application.",metaText:"Role of 'application' on this element is usually inappropriate",metaTextDetail:"Role of 'application' on this {{tag-name}} is usually inappropriate",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][role]','*[data-ae_vis][data-ae_avat][role="application"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-479
	 * @returns {AutoTestShell} testShell
	 */function test479(){aeKernel.addGuidedAutomaticTest("479",{bestPractice:2049,testId:479,introduced:"2.1",mediaType:mediaTypes.WEB,description:"One or more track elements, without an ARIA-assigned role, not intentionally hidden in the DOM, with a kind attribute set to 'captions' were found to have a src attribute that does not end in .vtt.",metaText:"This captions track element src should end in .vtt",fixType:"",testFunc:function(){return new AutoTestShell('track[data-ae_vis][data-ae_avat][data-ae_ar="null"][kind="captions"][src]:not([src=""])','track[data-ae_vis][data-ae_avat][data-ae_ar="null"][kind="captions"][src]:not([src=""]):not([src$=".vtt"]):not([src$=".Vtt"]):not([src$=".vTt"]):not([src$=".vtt"]):not([src$=".VTt"]):not([src$=".vtT"]):not([src$=".VtT"]):not([src$=".vTT"]):not([src$=".VTT"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-483
	 * @returns {AutoTestShell} testShell
	 */function test483(){aeKernel.addGuidedAutomaticTest("483",{bestPractice:2049,testId:483,introduced:"2.1",mediaType:mediaTypes.WEB,description:"One or more video elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a track element with kind set to captions as a direct child element",metaText:"This video element does not have a track with kind=captions",fixType:"",testFunc:function(){var testShell=new AutoTestShell('video[data-ae_vis][data-ae_avat][data-ae_ar="null"]');testShell.set_candidateSetNodesFunc(function(){var bad=[];var nodes=aeKernel.rootTestNode.querySelectorAll('video[data-ae_vis][data-ae_avat][data-ae_ar="null"]');for(var i=0;i<nodes.length;i++){var node=nodes[i];if(node.querySelector('track[kind="captions"]')===null){bad.push(node);}}return bad;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-490
	 * @returns {AutoTestShell} testShell
	 */function test490(){aeKernel.addGuidedAutomaticTest("490",{bestPractice:551,testId:490,introduced:"2.2",mediaType:mediaTypes.WEB,description:"Two or more label elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, have a for attribute with a non-unique value.",metaText:"This label element has a for attribute set to a non-unique value, which is not well supported by assistive technologies.",metaTextDetail:"This label element has a for attribute set to a non-unique value of {{for}}, which is not well supported by assistive technologies.",fixType:"",testFunc:function(){const testShell=new AutoTestShell();const relevantCssSelector='label[data-ae_vis][data-ae_avat][data-ae_ar="null"][for]:not([for=""])';testShell.set_applicabilityTest(function(){return this.applicableNodesFunc(this._applicableNodesFuncArgs).length>0;});testShell.set_applicableNodesFunc(function(){// if there is more than 1 label element then the test is applicable
const applicableNodes=this.querySelectorAll(relevantCssSelector);return applicableNodes.length>1?applicableNodes:[];});testShell.set_candidateSetNodesFunc(function(){var notUnique=[];var nodeStore={};var res=this.querySelectorAll(relevantCssSelector);if(res.length>1){for(var i=0;i<res.length;i++){var n=res[i];var thefor=n.getAttribute("for");try{nodeStore[thefor].push(n);}catch(err){nodeStore[thefor]=[];nodeStore[thefor].push(n);}}var keys=Object.keys(nodeStore);for(var j=0;j<keys.length;j++){// collecting all duplicated keys
var nArray=nodeStore[keys[j]];if(nArray.length>1){notUnique=notUnique.concat(nArray);}}}return notUnique;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-522
	 * @returns {AutoTestShell} testShell
	 */function test522(){aeKernel.addGuidedAutomaticTest("522",{bestPractice:542,testId:522,introduced:"2.3",mediaType:mediaTypes.WEB,description:"The first document title element located has a suspicious value.",metaText:"This title has a suspicious value",fixType:{"fixType":3,"domSpec":true,"helperText":'Specify a descriptive title for the page.',"fix":{"innerHTML":null}},testFunc:function(){const testShell=new AutoTestShell();const relevantCssSelector='html title:not([data-ae_inshdw]), body title:not([data-ae_inshdw]), head title:not([data-ae_inshdw])';testShell.set_applicabilityTest(function(){return this.applicableNodesFunc(this._applicableNodesFuncArgs).length===1;});testShell.set_applicableNodesFunc(function(){const t=this.querySelector(relevantCssSelector);return!!t&&t.childNodes.length>0?[t]:[];});testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const titles=this.querySelectorAll(relevantCssSelector);for(let i=0;i<titles.length;i++){const title=titles[i];// title elements that are descendants of SVGs aren't document title elements
const svgAncestor=title.closest('svg');if(!svgAncestor){const s=title.innerHTML.toLowerCase();if(s.length<11||s.length>150||s.indexOf('enter the title of your html document here')!==-1||s.indexOf('untitled document')!==-1||s.indexOf('untitled page')!==-1){badNodes.push(title);}break;}}return badNodes;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-523
	 * @returns {AutoTestShell} testShell
	 */function test523(){aeKernel.addStage4MarkdownFunc("identifiesDataTables",identifiesDataTables);aeKernel.addGuidedAutomaticTest("523",{bestPractice:1775,testId:523,introduced:"2.3",mediaType:mediaTypes.WEB,description:"One or more table elements seemingly used to layout content, rather than present data, not intentionally hidden in the DOM and available to assistive technologies, does not have a role='presentation' attribute.",metaText:"This table appears to be used for layout but does not have a role of presentation",fixType:{"fixType":1,"domSpec":false,"helperText":'Select "Save" to add the role of "presentation" if this table is used for layout purposes, or "Cancel" if the fix should not be applied.',"fix":{"role":"presentation"}},testFunc:function(){return new AutoTestShell('table[data-ae_vis][data-ae_avat]:not([data-ae_dtab])','table[data-ae_vis][data-ae_avat]:not([data-ae_dtab]):not([data-ae_ar="presentation"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-590
	 * @returns {AutoTestShell} testShell
	 */function test590(){aeKernel.addGuidedAutomaticTest("590",{bestPractice:1871,testId:590,introduced:"2.6",mediaType:mediaTypes.WEB,description:"One or more button elements, excluding input [type=hidden] elements and those elements with a tabindex set to a negative value, without an ARIA-assigned role, not intentionally hidden in the DOM, has an aria-hidden attribute set to the text value: true",metaText:"The button element has an aria-hidden attribute set to the value: true",fixType:"",testFunc:function(){return new AutoTestShell('button[data-ae_vis][data-ae_ar="null"]','button[data-ae_vis][data-ae_ar="null"]:not([type="hidden"]):not([tabindex^="-"])[aria-hidden="true"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-591
	 * @returns {AutoTestShell} testShell
	 */function test591(){aeKernel.addGuidedAutomaticTest("591",{bestPractice:1871,testId:591,introduced:"2.6",mediaType:mediaTypes.WEB,description:"One or more A elements, excluding those elements with a tabindex set to a negative value, without an ARIA-assigned role, not intentionally hidden in the DOM, has an aria-hidden attribute set to the text value: true",metaText:"The A element has an aria-hidden attribute set to the value: true",fixType:"",testFunc:function(){return new AutoTestShell('a[data-ae_vis][data-ae_ar="null"]','a[data-ae_vis][data-ae_ar="null"]:not([tabindex^="-"])[aria-hidden="true"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-592
	 * @returns {AutoTestShell} testShell
	 */function test592(){aeKernel.addGuidedAutomaticTest("592",{bestPractice:1871,testId:592,introduced:"2.6",mediaType:mediaTypes.WEB,description:"One or more input elements, excluding input [type=hidden] elements and those elements with a tabindex set to a negative value, without an ARIA-assigned role, not intentionally hidden in the DOM, has an aria-hidden attribute set to the text value: true",metaText:"This input element has an aria-hidden attribute set to the value: true",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_vis][data-ae_ar="null"]','input[data-ae_vis][data-ae_ar="null"]:not([type="hidden"]):not([tabindex^="-"])[aria-hidden="true"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-593
	 * @returns {AutoTestShell} testShell
	 */function test593(){aeKernel.addGuidedAutomaticTest("593",{bestPractice:1871,testId:593,introduced:"2.7",mediaType:mediaTypes.WEB,description:"One or more select elements, excluding those elements with a tabindex set to a negative value, without an ARIA-assigned role, not intentionally hidden in the DOM, has an aria-hidden attribute set to the text value: true.",metaText:"This select element has an aria-hidden attribute set to the value: true",fixType:"",testFunc:function(){return new AutoTestShell('select:not([tabindex^="-"])[data-ae_ar="null"][data-ae_vis]','select:not([tabindex^="-"])[data-ae_ar="null"][data-ae_vis][aria-hidden="true"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-594
	 * @returns {AutoTestShell} testShell
	 */function test594(){aeKernel.addGuidedAutomaticTest("594",{bestPractice:1871,testId:594,introduced:"2.7",mediaType:mediaTypes.WEB,description:"One or more textarea elements, excluding those elements with a tabindex set to a negative value, without an ARIA-assigned role, not intentionally hidden in the DOM, has an aria-hidden attribute set to the text value: true.",metaText:"This textarea element has an aria-hidden attribute set to the value: true",fixType:"",testFunc:function(){return new AutoTestShell('textarea:not([tabindex^="-"])[data-ae_ar="null"][data-ae_vis]','textarea:not([tabindex^="-"])[data-ae_ar="null"][data-ae_vis][aria-hidden="true"]');}});}/**
	 * (To be replaced with Machine Learning)
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-595
	 * @returns {AutoTestShell} testShell
	 */function test595(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("595",{bestPractice:423,testId:595,introduced:"2.7",mediaType:mediaTypes.WEB,description:"One or more object elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious accessible name value (i.e. under 4 characters long).",metaText:"This object element has a suspicious calculated accessible name value",metaTextDetail:"This object element has a suspicious calculated accessible name value of: {{data-ae_an}}",fixType:"",testFunc:function(){var testShell=new AutoTestShell('object[data-ae_vis][data-ae_avat][data-ae_ar="null"][data-ae_an]');testShell.set_candidateSetNodesFunc(function(){var suspiciousText=[];var res=aeKernel.rootTestNode.querySelectorAll("object[data-ae_vis][data-ae_avat][data-ae_ar='null'][data-ae_an]");for(var i=0,len_res=res.length;i<len_res;i++){var result=res[i];var accName=result.getAttribute("data-ae_an");if(accName.length<4){suspiciousText.push(result);}}return suspiciousText;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-596
	 * @returns {AutoTestShell} testShell
	 */function test596(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("596",{bestPractice:363,testId:596,introduced:"2.11",mediaType:mediaTypes.WEB,description:"One or more audio elements, without an aria-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious accessible name value (i.e. under 4 characters long).",metaText:"This audio element has a suspicious calculated accessible name value",metaTextDetail:"This audio element has a suspicious calculated accessible name value of: {{data-ae_an}}",fixType:"",testFunc:function(){var testShell=new AutoTestShell('audio[data-ae_vis][data-ae_avat][data-ae_ar="null"][data-ae_an]');testShell.set_candidateSetNodesFunc(function(){var suspiciousText=[];var res=aeKernel.rootTestNode.querySelectorAll("audio[data-ae_vis][data-ae_avat][data-ae_ar='null'][data-ae_an]");for(var i=0,len_res=res.length;i<len_res;i++){var result=res[i];var accName=result.getAttribute("data-ae_an");if(accName.length<4){suspiciousText.push(result);}}return suspiciousText;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-597
	 * @returns {AutoTestShell} testShell
	 */function test597(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("597",{bestPractice:363,testId:597,introduced:"2.11",mediaType:mediaTypes.WEB,description:"One or more video elements, without an aria-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious accessible name value (i.e. under 4 characters long).",metaText:"This video element has a suspicious calculated accessible name value",metaTextDetail:"This video element has a suspicious calculated accessible name value of: {{data-ae_an}}",fixType:"",testFunc:function(){var testShell=new AutoTestShell('video[data-ae_vis][data-ae_avat][data-ae_ar="null"][data-ae_an]');testShell.set_candidateSetNodesFunc(function(){var suspiciousText=[];var res=aeKernel.rootTestNode.querySelectorAll("video[data-ae_vis][data-ae_avat][data-ae_ar='null'][data-ae_an]");for(var i=0,len_res=res.length;i<len_res;i++){var result=res[i];var accName=result.getAttribute("data-ae_an");if(accName.length<4){suspiciousText.push(result);}}return suspiciousText;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-598
	 * @returns {AutoTestShell} testShell
	 */function test598(){aeKernel.addGuidedAutomaticTest("598",{bestPractice:2895,testId:598,introduced:"2.11",mediaType:mediaTypes.WEB,description:"One or more track elements, without an ARIA-assigned role, not intentionally hidden in the DOM, with a kind attribute set to 'descriptions' were found to have a src attribute that does not end in .vtt.",metaText:"This descriptions track element src should end in .vtt",fixType:"",testFunc:function(){return new AutoTestShell('track[data-ae_vis][data-ae_avat][data-ae_ar="null"][kind="descriptions"][src]:not([src=""])','track[data-ae_vis][data-ae_avat][data-ae_ar="null"][kind="descriptions"][src]:not([src=""]):not([src$=".vtt"]):not([src$=".Vtt"]):not([src$=".vTt"]):not([src$=".vtt"]):not([src$=".VTt"]):not([src$=".vtT"]):not([src$=".VtT"]):not([src$=".vTT"]):not([src$=".VTT"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-599
	 * @returns {AutoTestShell} testShell
	 */function test599(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("599",{bestPractice:476,testId:599,introduced:"2.11",mediaType:mediaTypes.WEB,description:"One or more canvas elements, without an aria-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious accessible name value (i.e. under 4 characters long).",metaText:"This canvas element has a suspicious calculated accessible name value",metaTextDetail:"This canvas element has a suspicious calculated accessible name value of: {{data-ae_an}}",fixType:"",testFunc:function(){var testShell=new AutoTestShell('canvas[data-ae_vis][data-ae_avat][data-ae_ar="null"][data-ae_an]');testShell.set_candidateSetNodesFunc(function(){var suspiciousText=[];var res=aeKernel.rootTestNode.querySelectorAll("canvas[data-ae_vis][data-ae_avat][data-ae_ar='null'][data-ae_an]");for(var i=0,len_res=res.length;i<len_res;i++){var result=res[i];var accName=result.getAttribute("data-ae_an");if(accName.length<4){suspiciousText.push(result);}}return suspiciousText;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-600
	 * @returns {AutoTestShell} testShell
	 */function test600(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("600",{bestPractice:476,testId:600,introduced:"2.11",mediaType:mediaTypes.WEB,description:"One or more embed elements, without an aria-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious accessible name value (i.e. under 4 characters long).",metaText:"This embed element has a suspicious calculated accessible name value",metaTextDetail:"This embed element has a suspicious calculated accessible name value of: {{data-ae_an}}",fixType:"",testFunc:function(){var testShell=new AutoTestShell('embed[data-ae_vis][data-ae_avat][data-ae_ar="null"][data-ae_an]');testShell.set_candidateSetNodesFunc(function(){var suspiciousText=[];var res=aeKernel.rootTestNode.querySelectorAll("embed[data-ae_vis][data-ae_avat][data-ae_ar='null'][data-ae_an]");for(var i=0,len_res=res.length;i<len_res;i++){var result=res[i];var accName=result.getAttribute("data-ae_an");if(accName.length<4){suspiciousText.push(result);}}return suspiciousText;});return testShell;}});}/**
	 * (To be replaced with Machine Learning)
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-707
	 * @returns {AutoTestShell} testShell
	 */function test707(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("707",{bestPractice:332,testId:707,introduced:"2.15",mediaType:mediaTypes.WEB,description:"One or more fieldset elements, without an aria-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious accessible name value (i.e. under 4 characters long).",metaText:"This fieldset element has a suspicious accessible name",metaTextDetail:"This fieldset element has a suspicious accessible name value of: {{data-ae_an}}",fixType:"",testFunc:function(){const relevantCssSelector='fieldset[data-ae_ar="null"][data-ae_vis][data-ae_avat][data-ae_an]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const relevantNodes=aeKernel.rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0;i<relevantNodes.length;i++){const relevantNode=relevantNodes[i];const accName=relevantNode.getAttribute('data-ae_an');if(accName.length<4){badNodes.push(relevantNode);}}return badNodes;});return testShell;}});}/**
	 * (To be replaced with Machine Learning)
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-709
	 * @returns {AutoTestShell} testShell
	 */function test709(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("709",{bestPractice:332,testId:709,introduced:"2.15",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='radiogroup' attribute, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious accessible name value (i.e. under 4 characters long).",metaText:"This element (role=radiogroup) has a suspicious accessible name",metaTextDetail:"This {{tag-name}} (role=radiogroup) has a suspicious accessible name value of: {{data-ae_an}}",fixType:"",testFunc:function(){const relevantCssSelector='*[data-ae_ar="radiogroup"][data-ae_vis][data-ae_avat][data-ae_an]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const relevantNodes=aeKernel.rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0;i<relevantNodes.length;i++){const relevantNode=relevantNodes[i];const accName=relevantNode.getAttribute('data-ae_an');if(accName.length<4){badNodes.push(relevantNode);}}return badNodes;});return testShell;}});}/**
	 * (To be replaced with Machine Learning)
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-710
	 * @returns {AutoTestShell} testShell
	 */function test710(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("710",{bestPractice:338,testId:710,introduced:"2.13",mediaType:mediaTypes.WEB,description:"One or more input type='text' elements, without an aria-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious accessible name value (i.e. under 4 characters long).",metaText:"This input (type=text) element has a suspicious calculated accessible name value",metaTextDetail:"This input (type=text) element has a suspicious calculated accessible name value of: {{data-ae_an}}",fixType:"",testFunc:function(){var testShell=new AutoTestShell('input[type="text"][data-ae_ar="null"][data-ae_vis][data-ae_avat][data-ae_an]');testShell.set_candidateSetNodesFunc(function(){var suspiciousText=[];var res=aeKernel.rootTestNode.querySelectorAll('input[type="text"][data-ae_ar="null"][data-ae_vis][data-ae_avat][data-ae_an]');for(var i=0,len_res=res.length;i<len_res;i++){var result=res[i];var accName=result.getAttribute("data-ae_an");if(accName.length<4){suspiciousText.push(result);}}return suspiciousText;});return testShell;}});}/**
	 * (To be replaced with Machine Learning)
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-711
	 * @returns {AutoTestShell} testShell
	 */function test711(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("711",{bestPractice:338,testId:711,introduced:"2.15",mediaType:mediaTypes.WEB,description:"One or more input type='password' elements, without an aria-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious accessible name value (i.e. under 4 characters long).",metaText:"This input (type=password) element has a suspicious calculated accessible name",metaTextDetail:"This input (type=password) element has a suspicious calculated accessible name value of: {{data-ae_an}}",fixType:"",testFunc:function(){const relevantCssSelector='input[type="password"][data-ae_ar="null"][data-ae_vis][data-ae_avat][data-ae_an]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const relevantNodes=aeKernel.rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0;i<relevantNodes.length;i++){const relevantNode=relevantNodes[i];const accName=relevantNode.getAttribute('data-ae_an');if(accName.length<4){badNodes.push(relevantNode);}}return badNodes;});return testShell;}});}/**
	 * (To be replaced with Machine Learning)
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-712
	 * @returns {AutoTestShell} testShell
	 */function test712(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("712",{bestPractice:338,testId:712,introduced:"2.15",mediaType:mediaTypes.WEB,description:"One or more input type='search' elements, without an aria-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious accessible name value (i.e. under 4 characters long).",metaText:"This input (type=search) element has a suspicious calculated accessible name",metaTextDetail:"This input (type=search) element has a suspicious calculated accessible name value of: {{data-ae_an}}",fixType:"",testFunc:function(){const relevantCssSelector='input[type="search"][data-ae_ar="null"][data-ae_vis][data-ae_avat][data-ae_an]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const relevantNodes=aeKernel.rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0;i<relevantNodes.length;i++){const relevantNode=relevantNodes[i];const accName=relevantNode.getAttribute('data-ae_an');if(accName.length<4){badNodes.push(relevantNode);}}return badNodes;});return testShell;}});}/**
	 * (To be replaced with Machine Learning)
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-713
	 * @returns {AutoTestShell} testShell
	 */function test713(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("713",{bestPractice:338,testId:713,introduced:"2.15",mediaType:mediaTypes.WEB,description:"One or more input type='tel' elements, without an aria-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious accessible name value (i.e. under 4 characters long).",metaText:"This input (type=tel) element has a suspicious calculated accessible name",metaTextDetail:"This input (type=tel) element has a suspicious calculated accessible name value of: {{data-ae_an}}",fixType:"",testFunc:function(){const relevantCssSelector='input[type="tel"][data-ae_ar="null"][data-ae_vis][data-ae_avat][data-ae_an]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const relevantNodes=aeKernel.rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0;i<relevantNodes.length;i++){const relevantNode=relevantNodes[i];const accName=relevantNode.getAttribute('data-ae_an');if(accName.length<4){badNodes.push(relevantNode);}}return badNodes;});return testShell;}});}/**
	 * (To be replaced with Machine Learning)
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-714
	 * @returns {AutoTestShell} testShell
	 */function test714(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("714",{bestPractice:338,testId:714,introduced:"2.15",mediaType:mediaTypes.WEB,description:"One or more input type='email' elements, without an aria-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious accessible name value (i.e. under 4 characters long).",metaText:"This input (type=email) element has a suspicious calculated accessible name",metaTextDetail:"This input (type=email) element has a suspicious calculated accessible name value of: {{data-ae_an}}",fixType:"",testFunc:function(){const relevantCssSelector='input[type="email"][data-ae_ar="null"][data-ae_vis][data-ae_avat][data-ae_an]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const relevantNodes=aeKernel.rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0;i<relevantNodes.length;i++){const relevantNode=relevantNodes[i];const accName=relevantNode.getAttribute('data-ae_an');if(accName.length<4){badNodes.push(relevantNode);}}return badNodes;});return testShell;}});}/**
	 * (To be replaced with Machine Learning)
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-715
	 * @returns {AutoTestShell} testShell
	 */function test715(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("715",{bestPractice:338,testId:715,introduced:"2.15",mediaType:mediaTypes.WEB,description:"One or more input type='url' elements, without an aria-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious accessible name value (i.e. under 4 characters long).",metaText:"This input (type=url) element has a suspicious calculated accessible name",metaTextDetail:"This input (type=url) element has a suspicious calculated accessible name value of: {{data-ae_an}}",fixType:"",testFunc:function(){const relevantCssSelector='input[type="url"][data-ae_ar="null"][data-ae_vis][data-ae_avat][data-ae_an]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const relevantNodes=aeKernel.rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0;i<relevantNodes.length;i++){const relevantNode=relevantNodes[i];const accName=relevantNode.getAttribute('data-ae_an');if(accName.length<4){badNodes.push(relevantNode);}}return badNodes;});return testShell;}});}/**
	 * (To be replaced with Machine Learning)
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-716
	 * @returns {AutoTestShell} testShell
	 */function test716(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("716",{bestPractice:338,testId:716,introduced:"2.15",mediaType:mediaTypes.WEB,description:"One or more input type='radio' elements, without an aria-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious accessible name value (i.e. under 4 characters long).",metaText:"This input (type=radio) element has a suspicious calculated accessible name",metaTextDetail:"This input (type=radio) element has a suspicious calculated accessible name value of: {{data-ae_an}}",fixType:"",testFunc:function(){const relevantCssSelector='input[type="radio"][data-ae_ar="null"][data-ae_vis][data-ae_avat][data-ae_an]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const relevantNodes=aeKernel.rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0;i<relevantNodes.length;i++){const relevantNode=relevantNodes[i];const accName=relevantNode.getAttribute('data-ae_an');if(accName.length<4){badNodes.push(relevantNode);}}return badNodes;});return testShell;}});}/**
	 * (To be replaced with Machine Learning)
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-717
	 * @returns {AutoTestShell} testShell
	 */function test717(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("717",{bestPractice:338,testId:717,introduced:"2.15",mediaType:mediaTypes.WEB,description:"One or more input type='checkbox' elements, without an aria-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious accessible name value (i.e. under 4 characters long).",metaText:"This input (type=checkbox) element has a suspicious calculated accessible name",metaTextDetail:"This input (type=checkbox) element has a suspicious calculated accessible name value of: {{data-ae_an}}",fixType:"",testFunc:function(){const relevantCssSelector='input[type="checkbox"][data-ae_ar="null"][data-ae_vis][data-ae_avat][data-ae_an]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const relevantNodes=aeKernel.rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0;i<relevantNodes.length;i++){const relevantNode=relevantNodes[i];const accName=relevantNode.getAttribute('data-ae_an');if(accName.length<4){badNodes.push(relevantNode);}}return badNodes;});return testShell;}});}/**
	 * (To be replaced with Machine Learning)
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-718
	 * @returns {AutoTestShell} testShell
	 */function test718(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("718",{bestPractice:338,testId:718,introduced:"2.15",mediaType:mediaTypes.WEB,description:"One or more input elements with no type attribute; or a type attribute set to a null value, without an aria-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious accessible name value (i.e. under 4 characters long).",metaText:"This input (no type specified) element has a suspicious calculated accessible name",metaTextDetail:"This input (no type specified) element has a suspicious calculated accessible name value of: {{data-ae_an}}",fixType:"",testFunc:function(){const relevantCssSelector='input:not([type])[data-ae_ar="null"][data-ae_vis][data-ae_avat][data-ae_an], input[type=""][data-ae_ar="null"][data-ae_vis][data-ae_avat][data-ae_an]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const relevantNodes=aeKernel.rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0;i<relevantNodes.length;i++){const relevantNode=relevantNodes[i];const accName=relevantNode.getAttribute('data-ae_an');if(accName.length<4){badNodes.push(relevantNode);}}return badNodes;});return testShell;}});}/**
	 * (To be replaced with Machine Learning)
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-719
	 * @returns {AutoTestShell} testShell
	 */function test719(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("719",{bestPractice:338,testId:719,introduced:"2.15",mediaType:mediaTypes.WEB,description:"One or more input type='button' elements, without an aria-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious accessible name value (i.e. under 4 characters long).",metaText:"This input (type=button) element has a suspicious calculated accessible name",metaTextDetail:"This input (type=button) element has a suspicious calculated accessible name value of: {{data-ae_an}}",fixType:"",testFunc:function(){const relevantCssSelector='input[type="button"][data-ae_ar="null"][data-ae_vis][data-ae_avat][data-ae_an]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const relevantNodes=aeKernel.rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0;i<relevantNodes.length;i++){const relevantNode=relevantNodes[i];const accName=relevantNode.getAttribute('data-ae_an');if(accName.length<4){badNodes.push(relevantNode);}}return badNodes;});return testShell;}});}/**
	 * (To be replaced with Machine Learning)
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-720
	 * @returns {AutoTestShell} testShell
	 */function test720(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("720",{bestPractice:338,testId:720,introduced:"2.15",mediaType:mediaTypes.WEB,description:"One or more select elements, without an aria-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious accessible name value (i.e. under 4 characters long).",metaText:"This select element has a suspicious accessible name",metaTextDetail:"This select element has a suspicious accessible name value of: {{data-ae_an}}",fixType:"",testFunc:function(){const relevantCssSelector='select[data-ae_ar="null"][data-ae_vis][data-ae_avat][data-ae_an]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const relevantNodes=aeKernel.rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0;i<relevantNodes.length;i++){const relevantNode=relevantNodes[i];const accName=relevantNode.getAttribute('data-ae_an');if(accName.length<4){badNodes.push(relevantNode);}}return badNodes;});return testShell;}});}/**
	 * (To be replaced with Machine Learning)
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-721
	 * @returns {AutoTestShell} testShell
	 */function test721(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("721",{bestPractice:338,testId:721,introduced:"2.15",mediaType:mediaTypes.WEB,description:"One or more textarea elements, without an aria-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious accessible name value (i.e. under 4 characters long).",metaText:"This textarea element has a suspicious accessible name",metaTextDetail:"This textarea element has a suspicious accessible name value of: {{data-ae_an}}",fixType:"",testFunc:function(){const relevantCssSelector='textarea[data-ae_ar="null"][data-ae_vis][data-ae_avat][data-ae_an]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const relevantNodes=aeKernel.rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0;i<relevantNodes.length;i++){const relevantNode=relevantNodes[i];const accName=relevantNode.getAttribute('data-ae_an');if(accName.length<4){badNodes.push(relevantNode);}}return badNodes;});return testShell;}});}/**
	 * (To be replaced with Machine Learning)
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-722
	 * @returns {AutoTestShell} testShell
	 */function test722(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("722",{bestPractice:338,testId:722,introduced:"2.15",mediaType:mediaTypes.WEB,description:"One or more optgroup elements, without an aria-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious accessible name value (i.e. under 4 characters long).",metaText:"This optgroup element has a suspicious accessible name",metaTextDetail:"This optgroup element has a suspicious accessible name value of: {{data-ae_an}}",fixType:"",testFunc:function(){const relevantCssSelector='optgroup[data-ae_ar="null"][data-ae_vis][data-ae_avat][data-ae_an]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const relevantNodes=aeKernel.rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0;i<relevantNodes.length;i++){const relevantNode=relevantNodes[i];const accName=relevantNode.getAttribute('data-ae_an');if(accName.length<4){badNodes.push(relevantNode);}}return badNodes;});return testShell;}});}/**
	 * (To be replaced with Machine Learning)
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-723
	 * @returns {AutoTestShell} testShell
	 */function test723(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("723",{bestPractice:373,testId:723,introduced:"2.15",mediaType:mediaTypes.WEB,description:"One or more input type='image' elements, without an aria-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious accessible name value (i.e. under 4 characters long).",metaText:"This input (type=image) element has a suspicious calculated accessible name",metaTextDetail:"This input (type=image) element has a suspicious calculated accessible name value of: {{data-ae_an}}",fixType:"",testFunc:function(){const relevantCssSelector='input[type="image"][data-ae_ar="null"][data-ae_vis][data-ae_avat][data-ae_an]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const relevantNodes=aeKernel.rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0;i<relevantNodes.length;i++){const relevantNode=relevantNodes[i];const accName=relevantNode.getAttribute('data-ae_an');if(accName.length<4){badNodes.push(relevantNode);}}return badNodes;});return testShell;}});}/**
	 * (To be replaced with Machine Learning)
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-724
	 * @returns {AutoTestShell} testShell
	 */function test724(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("724",{bestPractice:373,testId:724,introduced:"2.15",mediaType:mediaTypes.WEB,description:"One or more button elements, without an aria-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious accessible name value (i.e. under 4 characters long).",metaText:"This button element has a suspicious accessible name",metaTextDetail:"This button element has a suspicious accessible name value of: {{data-ae_an}}",fixType:"",testFunc:function(){const relevantCssSelector='button[data-ae_ar="null"][data-ae_vis][data-ae_avat][data-ae_an]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const relevantNodes=aeKernel.rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0;i<relevantNodes.length;i++){const relevantNode=relevantNodes[i];const accName=relevantNode.getAttribute('data-ae_an');if(accName.length<4){badNodes.push(relevantNode);}}return badNodes;});return testShell;}});}/**
	 * (To be replaced with Machine Learning)
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-727
	 * @returns {AutoTestShell} testShell
	 */function test727(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("727",{bestPractice:343,testId:727,introduced:"2.14",mediaType:mediaTypes.WEB,description:"One or more area elements, without an aria-assigned role, available to assistive technologies, has a suspicious calculated accessible name value (i.e. under 4 characters long)",metaText:"This area has a suspicious calculated accessible name value",metaTextDetail:"This area has a suspicious calculated accessible name value of: {{data-ae_an}}",fixType:"",//We don't check for visibility because Firefox secretly hides all area elements
//The visibility requirement would render this test unusable for Spiders (currently runs on Firefox) or AST on Firefox
testFunc:function(){var testShell=new AutoTestShell('area[data-ae_ar="null"][data-ae_avat][data-ae_an]');testShell.set_candidateSetNodesFunc(function(){var suspiciousText=[];var res=aeKernel.rootTestNode.querySelectorAll('area[data-ae_ar="null"][data-ae_avat][data-ae_an]');for(var i=0,len_res=res.length;i<len_res;i++){var result=res[i];var accName=result.getAttribute("data-ae_an");if(accName.length<4){suspiciousText.push(result);}}return suspiciousText;});return testShell;}});}/**
	 * (To be replaced with Machine Learning)
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-728
	 * @returns {AutoTestShell} testShell
	 */function test728(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("728",{bestPractice:2554,testId:728,introduced:"2.14",mediaType:mediaTypes.WEB,description:"One or more progress elements, without an aria-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious calculated accessible name value (i.e. under 4 characters long)",metaText:"This progress has a suspicious calculated accessible name value",metaTextDetail:"This progress has a suspicious calculated accessible name value of: {{data-ae_an}}",fixType:"",testFunc:function(){var testShell=new AutoTestShell('progress[data-ae_ar="null"][data-ae_vis][data-ae_avat][data-ae_an]');testShell.set_candidateSetNodesFunc(function(){var suspiciousText=[];var res=aeKernel.rootTestNode.querySelectorAll('progress[data-ae_ar="null"][data-ae_vis][data-ae_avat][data-ae_an]');for(var i=0,len_res=res.length;i<len_res;i++){var result=res[i];var accName=result.getAttribute("data-ae_an");if(accName.length<4){suspiciousText.push(result);}}return suspiciousText;});return testShell;}});}/**
	 * (To be replaced with Machine Learning)
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-729
	 * @returns {AutoTestShell} testShell
	 */function test729(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("729",{bestPractice:2554,testId:729,introduced:"2.14",mediaType:mediaTypes.WEB,description:"One or more meter elements, without an aria-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious calculated accessible name value (i.e. under 4 characters long)",metaText:"This meter has a suspicious calculated accessible name value",metaTextDetail:"This meter has a suspicious calculated accessible name value of: {{data-ae_an}}",fixType:"",testFunc:function(){var testShell=new AutoTestShell('meter[data-ae_ar="null"][data-ae_vis][data-ae_avat][data-ae_an]');testShell.set_candidateSetNodesFunc(function(){var suspiciousText=[];var res=aeKernel.rootTestNode.querySelectorAll('meter[data-ae_ar="null"][data-ae_vis][data-ae_avat][data-ae_an]');for(var i=0,len_res=res.length;i<len_res;i++){var result=res[i];var accName=result.getAttribute("data-ae_an");if(accName.length<4){suspiciousText.push(result);}}return suspiciousText;});return testShell;}});}/**
	 * (To be replaced with Machine Learning)
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-735
	 * @returns {AutoTestShell} testShell
	 */function test735(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("735",{bestPractice:1893,testId:735,introduced:"2.13",mediaType:mediaTypes.WEB,description:"One or more dialog elements with an open attribute, without an aria-assigned role and available to assistive technologies, has a suspicious calculated accessible name value (i.e. under 4 characters long).",metaText:"This dialog has a suspicious calculated accessible name value",metaTextDetail:"This dialog has a suspicious calculated accessible name value of: {{data-ae_an}}",fixType:"",testFunc:function(){const relevantCssSelector='dialog[open][data-ae_ar="null"][data-ae_avat][data-ae_an]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const suspiciousText=[];const res=aeKernel.rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0,len_res=res.length;i<len_res;i++){const result=res[i];const accName=result.getAttribute("data-ae_an");if(accName.length<4){suspiciousText.push(result);}}return suspiciousText;});return testShell;}});}/**
	 * (To be replaced with Machine Learning)
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-737
	 * @returns {AutoTestShell} testShell
	 */function test737(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("737",{bestPractice:1893,testId:737,introduced:"2.13",mediaType:mediaTypes.WEB,description:"One or more elements with a role='dialog' attribute, without an aria-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious calculated accessible name value (i.e. under 4 characters long).",metaText:"This element (role=dialog) has a suspicious calculated accessible name value",metaTextDetail:"This {{tag-name}} (role=dialog) has a suspicious calculated accessible name value of: {{data-ae_an}}",fixType:"",testFunc:function(){var testShell=new AutoTestShell('*[data-ae_ar="dialog"][data-ae_vis][data-ae_avat][data-ae_an]');testShell.set_candidateSetNodesFunc(function(){var suspiciousText=[];var res=aeKernel.rootTestNode.querySelectorAll('*[data-ae_ar="dialog"][data-ae_vis][data-ae_avat][data-ae_an]');for(var i=0,len_res=res.length;i<len_res;i++){var result=res[i];var accName=result.getAttribute("data-ae_an");if(accName.length<4){suspiciousText.push(result);}}return suspiciousText;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-848
	 * @returns {AutoTestShell} testShell
	 */function test848(){aeKernel.addGuidedAutomaticTest("848",{bestPractice:1626,testId:848,introduced:"2.15",mediaType:mediaTypes.WEB,description:"One or more input type='checkbox' elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has an aria-checked attribute.",metaText:"This input (type=checkbox) should not have an aria-checked attribute, as it has a native semantic checked state",metaTextDetail:"This input (type=checkbox) should not have an aria-checked attribute, as it has a native semantic checked state",fixType:"",testFunc:function(){const relevantCssSelector='input[type="checkbox"][data-ae_ar="null"][data-ae_vis][data-ae_avat]';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[aria-checked]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-849
	 * @returns {AutoTestShell} testShell
	 */function test849(){aeKernel.addGuidedAutomaticTest("849",{bestPractice:1626,testId:849,introduced:"2.15",mediaType:mediaTypes.WEB,description:"One or more input type='radio' elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has an aria-checked attribute.",metaText:"This input (type=radio) should not have an aria-checked attribute, as it has a native semantic checked state",metaTextDetail:"This input (type=radio) should not have an aria-checked attribute, as it has a native semantic checked state",fixType:"",testFunc:function(){const relevantCssSelector='input[type="radio"][data-ae_ar="null"][data-ae_vis][data-ae_avat]';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[aria-checked]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-871
	 * @returns {AutoTestShell} testShell
	 */function test871(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("871",{bestPractice:523,testId:871,introduced:"2.16",mediaType:mediaTypes.WEB,description:"Two or more iframe elements with a unique src value, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, have an accessible name value that is not unique.",metaText:"This iframe element has an accessible name that is not unique",metaTextDetail:"This iframe element has an accessible name of '{{data-ae_an}}' that is not unique",fixType:{"fixType":1,"domSpec":false,"helperText":"Provide a descriptive accessible name for this element.","fix":{"aria-label":null,"aria-labelledby":false}},testFunc:function(){const testShell=new AutoTestShell();const baseCssSelector='iframe[data-ae_ar="null"][data-ae_vis][data-ae_avat][data-ae_an]';testShell.set_applicabilityTest(function(){return this.applicableNodesFunc(this._applicableNodesFuncArgs).length>0;});testShell.set_applicableNodesFunc(function(){const applicableNodes=[];// if there is more than 1 applicable iframe element then the test is applicable
const iframes=this.querySelectorAll(baseCssSelector);if(iframes.length>1){for(let i=0;i<iframes.length;i++){const iframe=iframes[i];let src=iframe.getAttribute('src');if(src){src=src.replace(/\/$/g,"");// src attribute values whose only difference is a trailing forward slash are effectively equal
if(this.querySelectorAll(`${baseCssSelector}[src="${src.replace(/"/g,'\\"')}"], ${baseCssSelector}[src="${src.replace(/"/g,'\\"')}/"]`).length===1){applicableNodes.push(iframe);}}}}return applicableNodes.length>1?applicableNodes:[];});testShell.set_candidateSetNodesFunc(function(){const suspiciousIframes=[];const iframes=this.querySelectorAll(baseCssSelector);if(iframes.length>1){// you create an object to hold accNames
// you run through all the iframes, and each time the same accName is found you add 1 so accName1 = 1; accName2 = 4, etc...
// then you run through the iframes again, and for any that have > 1 you have the issues
const accNames={};for(let i=iframes.length;i--;){const accName=iframes[i].getAttribute('data-ae_an');if(accNames[accName]===undefined){accNames[accName]=1;}else{accNames[accName]=accNames[accName]+1;}}for(let i=0;i<iframes.length;i++){const iframe=iframes[i];let src=iframe.getAttribute('src');if(src){src=src.replace(/\/$/g,"");// src attribute values whose only difference is a trailing forward slash are effectively equal
if(this.querySelectorAll(`${baseCssSelector}[src="${src.replace(/"/g,'\\"')}"], ${baseCssSelector}[src="${src.replace(/"/g,'\\"')}/"]`).length===1){// for each iframe element with a unique src value
const accName=iframe.getAttribute('data-ae_an');if(accNames[accName]>1){suspiciousIframes.push(iframe);}}}}}return suspiciousIframes;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-875
	 * @returns {AutoTestShell} testShell
	 */function test875(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("875",{bestPractice:338,testId:875,introduced:"2.16",mediaType:mediaTypes.WEB,description:"One or more summary elements, without an aria-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has a suspicious calculated accessible name value (i.e. under 4 characters long).",metaText:"This summary has a suspicious calculated accessible name value",metaTextDetail:"This summary has a suspicious calculated accessible name value of: {{data-ae_an}}",fixType:"",testFunc:function(){const relevantCssSelector='summary[data-ae_ar="null"][data-ae_vis][data-ae_avat][data-ae_an]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const suspiciousText=[];const res=aeKernel.rootTestNode.querySelectorAll(relevantCssSelector);for(let i=0,len_res=res.length;i<len_res;i++){const result=res[i];const accName=result.getAttribute("data-ae_an");if(accName.length<4){suspiciousText.push(result);}}return suspiciousText;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-879
	 * @returns {AutoTestShell} testShell
	 */function test879(){aeKernel.addGuidedAutomaticTest("879",{bestPractice:389,testId:879,introduced:"2.16",mediaType:mediaTypes.WEB,description:"One or more elements with a lang attribute set to a text value that starts with an IANA primary language value, excluding the root html node, not intentionally hidden in the DOM and available to assistive technologies, has an extended language or region subtag.",metaText:"This element has a lang attribute containing an extended language or region subtag which needs to be verified as valid",metaTextDetail:'This {{tag-name}} has a lang attribute of "{{lang}}" containing an extended language or region subtag which needs to be verified as valid',fixType:"",testFunc:function(){const testShell=new AutoTestShell();const relevantCssSelector='*[lang]:not([lang=""]):not(html)[data-ae_vis][data-ae_avat]';testShell.set_applicabilityTest(function(){const nodesWithLang=this.querySelectorAll(relevantCssSelector);for(let i=0;i<nodesWithLang.length;i++){const nodeWithLang=nodesWithLang[i];let lang=nodeWithLang.getAttribute("lang");if(lang&&!ACCEPTED_LANGUAGE_FULLTAGS[lang.toLowerCase()]){const splitLang=lang.split("-");const subtag=splitLang[0].toLowerCase();if(ACCEPTED_LANGUAGE_SUBTAGS[subtag]){return true;}}}return false;});testShell.set_applicableNodesFunc(function(){const applicableNodes=[];const nodesWithLang=this.querySelectorAll(relevantCssSelector);for(let i=0;i<nodesWithLang.length;i++){const nodeWithLang=nodesWithLang[i];let lang=nodeWithLang.getAttribute("lang");if(lang&&!ACCEPTED_LANGUAGE_FULLTAGS[lang.toLowerCase()]){const splitLang=lang.split("-");const subtag=splitLang[0].toLowerCase();if(ACCEPTED_LANGUAGE_SUBTAGS[subtag]){applicableNodes.push(nodeWithLang);}}}return applicableNodes;});testShell.set_candidateSetNodesFunc(function(){const nodes=[];const nodesWithLang=this.querySelectorAll(relevantCssSelector);for(let i=0;i<nodesWithLang.length;i++){const nodeWithLang=nodesWithLang[i];let lang=nodeWithLang.getAttribute("lang");if(lang&&!ACCEPTED_LANGUAGE_FULLTAGS[lang.toLowerCase()]){let langExtendedPart="";const splitLang=lang.split("-");const subtag=splitLang[0].toLowerCase();if(splitLang.length>1){langExtendedPart=splitLang[1].toLowerCase();}if(ACCEPTED_LANGUAGE_SUBTAGS[subtag]&&langExtendedPart.length>0){nodes.push(nodeWithLang);}}}return nodes;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-880
	 * @returns {AutoTestShell} testShell
	 */function test880(){aeKernel.addGuidedAutomaticTest("880",{bestPractice:429,testId:880,introduced:"2.16",mediaType:mediaTypes.WEB,description:"The root html element with a lang attribute set to a text value that starts with an IANA primary language has an extended language or region subtag.",metaText:"This element has a lang attribute containing an extended language or region subtag which needs to be verified as valid",metaTextDetail:'This {{tag-name}} has a lang attribute of "{{lang}}" containing an extended language or region subtag which needs to be verified as valid',fixType:"",testFunc:function(){const testShell=new AutoTestShell();const relevantCssSelector='html[lang]:not([lang=""])';testShell.set_applicabilityTest(function(){return this.applicableNodesFunc(this._applicableNodesFuncArgs).length>0;});testShell.set_applicableNodesFunc(function(){const applicableNodes=[];const nodesWithLang=this.querySelectorAll(relevantCssSelector);for(let i=0;i<nodesWithLang.length;i++){const nodeWithLang=nodesWithLang[i];let lang=nodeWithLang.getAttribute("lang");if(lang&&!ACCEPTED_LANGUAGE_FULLTAGS[lang.toLowerCase()]){const splitLang=lang.split("-");const subtag=splitLang[0].toLowerCase();if(ACCEPTED_LANGUAGE_SUBTAGS[subtag]){applicableNodes.push(nodeWithLang);}}}return applicableNodes;});testShell.set_candidateSetNodesFunc(function(){const nodes=[];const nodesWithLang=this.querySelectorAll(relevantCssSelector);for(let i=0;i<nodesWithLang.length;i++){const nodeWithLang=nodesWithLang[i];let lang=nodeWithLang.getAttribute("lang");if(lang&&!ACCEPTED_LANGUAGE_FULLTAGS[lang.toLowerCase()]){let langExtendedPart="";const splitLang=lang.split("-");const subtag=splitLang[0].toLowerCase();if(splitLang.length>1){langExtendedPart=splitLang[1].toLowerCase();}if(ACCEPTED_LANGUAGE_SUBTAGS[subtag]&&langExtendedPart.length>0){nodes.push(nodeWithLang);}}}return nodes;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-925
	 * @returns {AutoTestShell} testShell
	 */function test925(){aeKernel.addGuidedAutomaticTest("925",{bestPractice:1301,testId:925,introduced:"2.17",mediaType:mediaTypes.WEB,description:"One or more A elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has an alt attribute.",metaText:"This A element has an alt attribute",fixType:"",testFunc:function(){const relevantCssSelector='a[data-ae_ar="null"][data-ae_vis][data-ae_avat]';return new AutoTestShell(`${relevantCssSelector}`,`${relevantCssSelector}[alt]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-933
	 * @returns {AutoTestShell} testShell
	 */function test933(){aeKernel.addGuidedAutomaticTest("933",{bestPractice:886,testId:933,introduced:"2.17",mediaType:mediaTypes.WEB,description:"One or more dialog elements with an open attribute, without a hidden attribute and available to assistive technologies is empty.",metaText:"This dialog is open but empty",fixType:"",testFunc:function(){const relevantCssSelector="dialog[open]:not([hidden])[data-ae_avat]";// note that some of the relevant CSS selector above is redundant with the implementation of the 'data-ae_emptyopendialog' attribute
return new AutoTestShell(`${relevantCssSelector}`,`${relevantCssSelector}[data-ae_emptyopendialog]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-934
	 * @returns {AutoTestShell} testShell
	 */function test934(){aeKernel.addGuidedAutomaticTest("934",{bestPractice:886,testId:934,introduced:"2.17",mediaType:mediaTypes.WEB,description:"One or more elements with a role='dialog' attribute, without a hidden attribute, not intentionally hidden in the DOM and available to assistive technologies, is empty.",metaText:"This element (role=dialog) is open but empty",metaTextDetail:"This {{tag-name}} (role=dialog) is open but empty",fixType:"",testFunc:function(){const relevantCssSelector="*[data-ae_ar='dialog']:not([hidden])[data-ae_vis][data-ae_avat]";// note that some of the relevant CSS selector above is redundant with the implementation of the 'data-ae_emptyopendialog' attribute
return new AutoTestShell(`${relevantCssSelector}`,`${relevantCssSelector}[data-ae_emptyopendialog]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-993
	 * @returns {AutoTestShell} testShell
	 */function test993(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addGuidedAutomaticTest("993",{bestPractice:523,testId:993,introduced:"2.21",mediaType:mediaTypes.WEB,description:"One or more iframe elements with tabindex set to a negative value, excluding those elements with aria-hidden='true', not intentionally hidden in the DOM, does not have a mechanism that allows an accessible name value to be calculated.",metaText:"This iframe element does not have a mechanism that allows an accessible name value to be calculated",fixType:"",testFunc:function(){const relevantCssSelector='iframe[tabindex^="-"]:not([aria-hidden="true"])[data-ae_vis]';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}:not([data-ae_an])`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1014
	 * @returns {AutoTestShell} testShell
	 */function test1014(){aeKernel.addGuidedAutomaticTest("1014",{bestPractice:389,testId:1014,introduced:"2.19",mediaType:mediaTypes.WEB,description:"One or more elements, excluding the root html node, not intentionally hidden in the DOM and available to assistive technologies, has a lang attribute set to a null value.",metaText:"This element has a lang attribute set to a null value",metaTextDetail:'This {{tag-name}} has a lang attribute set to a null value',fixType:"",testFunc:function(){const relevantCssSelector='*:not(:root)[data-ae_vis][data-ae_avat][lang]';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[lang=""]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1092
	 * @returns {AutoTestShell} testShell
	 */function test1092(){aeKernel.addGuidedAutomaticTest("1092",{bestPractice:1626,testId:1092,introduced:"2.21",mediaType:mediaTypes.WEB,description:"One or more span elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has an aria-label attribute.",metaText:"This span element has an aria-label attribute",fixType:"",testFunc:function(){return new AutoTestShell('span[data-ae_ar="null"][data-ae_vis]','span[data-ae_ar="null"][data-ae_vis][aria-label]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1093
	 * @returns {AutoTestShell} testShell
	 */function test1093(){aeKernel.addGuidedAutomaticTest("1093",{bestPractice:1626,testId:1093,introduced:"2.21",mediaType:mediaTypes.WEB,description:"One or more span elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has an aria-labelledby attribute.",metaText:"This span element has an aria-labelledby attribute",fixType:"",testFunc:function(){return new AutoTestShell('span[data-ae_ar="null"][data-ae_vis]','span[data-ae_ar="null"][data-ae_vis][aria-labelledby]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1094
	 * @returns {AutoTestShell} testShell
	 */function test1094(){aeKernel.addGuidedAutomaticTest("1094",{bestPractice:1626,testId:1094,introduced:"2.21",mediaType:mediaTypes.WEB,description:"One or more div elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has an aria-labelledby attribute.",metaText:"This div element has an aria-labelledby attribute",fixType:"",testFunc:function(){return new AutoTestShell('div[data-ae_ar="null"][data-ae_vis]','div[data-ae_ar="null"][data-ae_vis][aria-labelledby]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1109
	 * @returns {AutoTestShell} testShell
	 */function test1109(){aeKernel.addGuidedAutomaticTest("1109",{bestPractice:602,testId:1109,introduced:"2.21",mediaType:mediaTypes.WEB,description:"One or more SPAN elements with a tabindex set to 0 or a positive value, excluding those elements with aria-hidden='true', not intentionally hidden in the DOM, lacks a role attribute.",metaText:"This SPAN is focusable but lacks a role attribute.",fixType:"",testFunc:function(){const baseCssSelector='span[tabindex]:not([tabindex=""]):not([tabindex^="-"]):not([aria-hidden="true"])[data-ae_vis]';return new AutoTestShell(baseCssSelector,`${baseCssSelector}[data-ae_ar="null"]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1110
	 * @returns {AutoTestShell} testShell
	 */function test1110(){aeKernel.addGuidedAutomaticTest("1110",{bestPractice:602,testId:1110,introduced:"2.21",mediaType:mediaTypes.WEB,description:"One or more DIV elements with a tabindex set to 0 or a positive value, excluding those elements with aria-hidden='true', not intentionally hidden in the DOM, lacks a role attribute.",metaText:"This DIV is focusable but lacks a role attribute.",fixType:"",testFunc:function(){const baseCssSelector='div[tabindex]:not([tabindex=""]):not([tabindex^="-"]):not([aria-hidden="true"])[data-ae_vis]';return new AutoTestShell(baseCssSelector,`${baseCssSelector}[data-ae_ar="null"]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1111
	 * @returns {AutoTestShell} testShell
	 */function test1111(){aeKernel.addGuidedAutomaticTest("1111",{bestPractice:602,testId:1111,introduced:"2.21",mediaType:mediaTypes.WEB,description:"One or more P elements with a tabindex set to 0 or a positive value, excluding those elements with aria-hidden='true', not intentionally hidden in the DOM, lacks a role attribute.",metaText:"This P is focusable but lacks a role attribute.",fixType:"",testFunc:function(){const baseCssSelector='p[tabindex]:not([tabindex=""]):not([tabindex^="-"]):not([aria-hidden="true"])[data-ae_vis]';return new AutoTestShell(baseCssSelector,`${baseCssSelector}[data-ae_ar="null"]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1112
	 * @returns {AutoTestShell} testShell
	 */function test1112(){aeKernel.addGuidedAutomaticTest("1112",{bestPractice:602,testId:1112,introduced:"2.21",mediaType:mediaTypes.WEB,description:"One or more A elements with a tabindex set to 0 or a positive value, excluding those elements with aria-hidden='true' or an href attribute set to a text value, not intentionally hidden in the DOM, lacks a role attribute.",metaText:"This A is focusable but lacks either an href or role attribute.",fixType:"",testFunc:function(){//All the anchors with a non-negative tabindex and an href that is empty
const baseCssSelector1='a[tabindex]:not([tabindex=""]):not([tabindex^="-"]):not([aria-hidden="true"])[href][href=""][data-ae_vis]';//All the anchors with a non-negative tabindex and no href
const baseCssSelector2='a[tabindex]:not([tabindex=""]):not([tabindex^="-"]):not([aria-hidden="true"]):not([href])[data-ae_vis]';return new AutoTestShell(`${baseCssSelector1},${baseCssSelector2}`,`${baseCssSelector1}[data-ae_ar="null"],${baseCssSelector2}[data-ae_ar="null"]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1113
	 * @returns {AutoTestShell} testShell
	 */function test1113(){aeKernel.addGuidedAutomaticTest("1113",{bestPractice:602,testId:1113,introduced:"2.21",mediaType:mediaTypes.WEB,description:"One or more Custom elements with a tabindex set to 0 or a positive value, excluding those elements with aria-hidden='true', not intentionally hidden in the DOM, lacks a role attribute.",metaText:"This element is focusable but lacks a role attribute.",metaTextDetail:"This {{tag-name}} is focusable but lacks a role attribute.",fixType:"",testFunc:function(){//Anything not on this list is a custom element - 122 standard elements as of 1/14/2022
//See https://developer.mozilla.org/en-US/docs/Web/HTML/Element
//Excludes most deprecated elements. Hopefully nobody is using a "spacer" element make an interactive custom control
const tags=['html','head','title','base','link','meta','style','body','article','section','nav','aside','h1','h2','h3','h4','h5','h6','hgroup','header','footer','address','p','hr','pre','blockquote','ol','ul','li','dl','dt','dd','figure','figcaption','div','main','a','em','strong','small','s','cite','q','dfn','abbr','data','time','code','var','samp','kbd','sub','sup','i','b','u','mark','ruby','rb','rt','rtc','rp','bdi','bdo','span','br','wbr','ins','del','picture','img','iframe','embed','object','param','video','audio','source','track','map','area','math','svg','table','caption','colgroup','col','tbody','thead','tfoot','tr','td','th','form','label','input','button','select','datalist','optgroup','option','textarea','keygen','output','progress','meter','fieldset','legend','script','noscript','template','canvas','rect','g','path','circle','details','summary','dialog','slot','portal','menu'];const customElementCssSelector=tags.map(tagName=>`:not(${tagName})`).join('');const baseCssSelector=`*[tabindex]:not([tabindex=""]):not([tabindex^="-"]):not([aria-hidden="true"])[data-ae_vis]${customElementCssSelector}`;return new AutoTestShell(baseCssSelector,`${baseCssSelector}[data-ae_ar="null"]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1119
	 * @returns {AutoTestShell} testShell
	 */function test1119(){aeKernel.addGuidedAutomaticTest("1119",{bestPractice:490,testId:1119,introduced:"2.21",mediaType:mediaTypes.WEB,description:"One or more A elements, not intentionally hidden in the DOM and available to assistive technologies, lacks an href or a tabindex set to 0 or a positive value.",metaText:"This A lacks an href or a tabindex set to 0 or a positive value.",fixType:"",testFunc:function(){const baseCssSelector='a[data-ae_avat][data-ae_vis]';//Fail Cases
//No href and has negative tabindex
//No href and has empty tabindex
//No href and has no tabindex
return new AutoTestShell(`${baseCssSelector}`,`${baseCssSelector}:not([href])[tabindex^="-"],${baseCssSelector}:not([href])[tabindex=""],${baseCssSelector}:not([href]):not([tabindex])`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-744
	 * @returns {AutoTestShell} testShell
	 */function test744(){aeKernel.addGuidedAutomaticTest("744",{bestPractice:1927,testId:744,introduced:"2.11",mediaType:mediaTypes.IOS,description:"One or more UINavigationBar elements does not contain text.",metaText:"All UINavigationBar components should contain text.",fixType:"",testFunc:function(){var testShell=new AutoTestShell('div[data-ae_vis][data-ae_avat][data-xmlnodetype="XCUIElementTypeNavigationBar"]');testShell.set_candidateSetNodesFunc(function(){var bad=[];var nodes=aeKernel.rootTestNode.querySelectorAll('div[data-ae_vis][data-ae_avat][data-xmlnodetype="XCUIElementTypeNavigationBar"]');for(var i=0;i<nodes.length;i++){var node=nodes[i];if(node.hasChildNodes()==false){bad.push(node);}}return bad;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-746
	 * @returns {AutoTestShell} testShell
	 */function test746(){aeKernel.addGuidedAutomaticTest("746",{bestPractice:1918,testId:746,introduced:"2.11",mediaType:mediaTypes.IOS,description:"One or more Button, Switch, TextField, SecureTextField, SearchField, TextView, or Slider elements is less than 44 points in height and/or width.",metaText:"This [[object]] is less than 44 points in height and/or width",fixType:"",testFunc:function(){var cssSelector='[data-xmlnodetype="XCUIElementTypeButton"],[data-xmlnodetype="XCUIElementTypeSwitch"],[data-xmlnodetype="XCUIElementTypeTextField"],[data-xmlnodetype="XCUIElementTypeSecureTextField"],[data-xmlnodetype="XCUIElementTypeSearchField"],[data-xmlnodetype="XCUIElementTypeTextView"],[data-xmlnodetype="XCUIElementTypeSlider"]';var testShell=new AutoTestShell(cssSelector);testShell.set_candidateSetNodesFunc(function(){var bad=[];var nodes=aeKernel.rootTestNode.querySelectorAll(cssSelector);for(var i=0;i<nodes.length;i++){var node=nodes[i];if(node.getAttribute('data-xmlnodetype')==='XCUIElementTypeSearchField'){// XCUIElementTypeSearchField elements are wrapped in XCUIElementTypeOther elements that define the focus area for the search field,
// so when determining if a search field is large enough, use the XCUIElementTypeOther element's size, not the size of the XCUIElementTypeSearchField inside of it
node=node.parentNode;}var width=parseInt(node.style.width,10);var height=parseInt(node.style.height,10);if(width<44||height<44){bad.push(node);}}return bad;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-789
	 * @returns {AutoTestShell} testShell
	 */function test789(){aeKernel.addGuidedAutomaticTest("789",{bestPractice:3115,testId:789,introduced:"2.13",mediaType:mediaTypes.IOS,description:"This application, or the device it was tested on, is locked from changing its orientation.",metaText:"This application, or the device it was tested on, is locked from changing its orientation",fixType:"",testFunc:function(){const relevantCssSelector="meta[http-equiv='ScreenOrientation']";return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[content='autoRotate:disabled']`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-743
	 * @returns {AutoTestShell} testShell
	 */function test743(){aeKernel.addGuidedAutomaticTest("743",{bestPractice:2617,testId:743,introduced:"2.11",mediaType:mediaTypes.ANDROID,description:"One or more ListView elements is focusable.",metaText:"This [[object]] element is focusable, and this may cause its sub-elements to not be focusable by a keyboard. Ignore this error if the element is used as a compound control.",fixType:"",testFunc:function(){return new AutoTestShell('div[data-xmlnodetype="android.widget.ListView"]','div[data-xmlnodetype="android.widget.ListView"]:not([disabled])');}});}/**
	 * NOTE: area element is available to screen-reader users and keyboard users even if it is "display:none"
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-2
	 * @returns {AutoTestShell} testShell
	 */function test2(){aeKernel.addAdvisoryAutomaticTest("2",{bestPractice:1626,testId:2,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more area elements with an href attribute, available to assistive technologies, has a role attribute set to the value: 'link...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('area[data-ae_avat][href]','area[data-ae_avat][href][data-ae_ar="link"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-4
	 * datalist is display:none by default
	 * @returns {AutoTestShell} testShell
	 */function test4(){aeKernel.addAdvisoryAutomaticTest("4",{bestPractice:1626,testId:4,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more dd elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'definition...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('dd[data-ae_vis][data-ae_avat]','dd[data-ae_vis][data-ae_avat][data-ae_ar="definition"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-5
	 * datalist is display:none by default
	 * @returns {AutoTestShell} testShell
	 */function test5(){aeKernel.addAdvisoryAutomaticTest("5",{bestPractice:1626,testId:5,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more datalist elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'listbox...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('datalist[data-ae_vis][data-ae_avat]','datalist[data-ae_vis][data-ae_avat][data-ae_ar="listbox"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-6
	 * @returns {AutoTestShell} testShell
	 */function test6(){aeKernel.addAdvisoryAutomaticTest("6",{bestPractice:1626,testId:6,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more article elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'article...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('article[data-ae_vis][data-ae_avat]','article[data-ae_vis][data-ae_avat][data-ae_ar="article"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-9
	 * @returns {AutoTestShell} testShell
	 */function test9(){aeKernel.addAdvisoryAutomaticTest("9",{bestPractice:1626,testId:9,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more aside elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'complementary...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('aside[data-ae_vis][data-ae_avat]','aside[data-ae_vis][data-ae_avat][data-ae_ar="complementary"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-7
	 * @returns {AutoTestShell} testShell
	 */function test7(){aeKernel.addAdvisoryAutomaticTest("7",{bestPractice:1626,testId:7,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more body elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'document...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){const relevantCssSelector='body[data-ae_vis][data-ae_avat]';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[data-ae_ar="document"]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-10
	 * @returns {AutoTestShell} testShell
	 */function test10(){aeKernel.addAdvisoryAutomaticTest("10",{bestPractice:1626,testId:10,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more button elements with a type='menu' attribute, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'button...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('button[data-ae_vis][data-ae_avat][type="menu"]','button[data-ae_vis][data-ae_avat][type="menu"][data-ae_ar="button"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-11
	 * @returns {AutoTestShell} testShell
	 */function test11(){aeKernel.addAdvisoryAutomaticTest("11",{bestPractice:1626,testId:11,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more button elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'button...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('button[data-ae_vis][data-ae_avat]','button[data-ae_vis][data-ae_avat][data-ae_ar="button"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-13
	 * @returns {AutoTestShell} testShell
	 */function test13(){aeKernel.addAdvisoryAutomaticTest("13",{bestPractice:2042,testId:13,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more audio elements, without an ARIA-assigned role, not intentionally hidden in the DOM, were found to have a loop attribute.",metaText:"This audio element should not have a loop attribute",fixType:"",testFunc:function(){return new AutoTestShell('audio[data-ae_vis][data-ae_ar="null"]','audio[data-ae_vis][data-ae_ar="null"][loop]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-19
	 * @returns {AutoTestShell} testShell
	 */function test19(){aeKernel.addAdvisoryAutomaticTest("19",{bestPractice:1626,testId:19,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more A elements with an href attribute, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'link...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('a[data-ae_vis][data-ae_avat][href]','a[data-ae_vis][data-ae_avat][href][data-ae_ar="link"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-20
	 * @returns {AutoTestShell} testShell
	 */function test20(){aeKernel.addAdvisoryAutomaticTest("20",{bestPractice:343,testId:20,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more map elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, were found to be empty.",metaText:"This map element should not be empty",fixType:"",testFunc:function(){return new AutoTestShell('map[data-ae_vis][data-ae_avat][data-ae_ar="null"]','map[data-ae_vis][data-ae_avat][data-ae_ar="null"]:empty');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-23
	 * @returns {AutoTestShell} testShell
	 */function test23(){aeKernel.addAdvisoryAutomaticTest("23",{bestPractice:2042,testId:23,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more video elements, without an ARIA-assigned role, not intentionally hidden in the DOM, were found to have a loop attribute.",metaText:"This video element should not have a loop attribute",fixType:"",testFunc:function(){return new AutoTestShell('video[data-ae_vis][data-ae_ar="null"]','video[data-ae_vis][data-ae_ar="null"][loop]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-24
	 * @returns {AutoTestShell} testShell
	 */function test24(){aeKernel.addAdvisoryAutomaticTest("24",{bestPractice:343,testId:24,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more map elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not contain a name attribute set to a text value.",metaText:"This element does not contain a name attribute set to a text value",fixType:"",testFunc:function(){return new AutoTestShell('map[data-ae_vis][data-ae_avat][data-ae_ar="null"]','map[data-ae_vis][data-ae_avat][data-ae_ar="null"]:not([name]),map[data-ae_vis][data-ae_avat][data-ae_ar="null"][name=""]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-25
	 * @returns {AutoTestShell} testShell
	 */function test25(){aeKernel.addAdvisoryAutomaticTest("25",{bestPractice:343,testId:25,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more map elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not contain only area elements (without an ARIA-assigned role) as direct child elements.",metaText:"This element should only contain area elements (without an ARIA-assigned role) as direct child elements",fixType:"",testFunc:function(){var testShell=new AutoTestShell('map[data-ae_vis][data-ae_avat][data-ae_ar="null"]:not(:empty)');testShell.set_candidateSetNodesFunc(function(){var badnodes=[];var maps=aeKernel.rootTestNode.querySelectorAll('map[data-ae_vis][data-ae_avat][data-ae_ar="null"]:not(:empty)');for(var i=0;i<maps.length;i++){var mapchildren=maps[i].children,fail=false;for(var j=0;j<mapchildren.length&&fail==false;j++){var testchild=mapchildren[j];if(testchild.tagName!="AREA"||testchild.getAttribute("data-ae_ar")!="null"){//Child type not on the allowed list or has a role
badnodes.push(maps[i]);fail=true;}}}return badnodes;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-29
	 * @returns {AutoTestShell} testShell
	 */function test29(){aeKernel.addAdvisoryAutomaticTest("29",{bestPractice:907,testId:29,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more tr elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, were found to be empty.",metaText:"This tr element should not be empty",fixType:"",testFunc:function(){return new AutoTestShell('tr[data-ae_vis][data-ae_avat][data-ae_ar="null"]','tr[data-ae_vis][data-ae_avat][data-ae_ar="null"]:empty');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-36
	 * @returns {AutoTestShell} testShell
	 */function test36(){aeKernel.addAdvisoryAutomaticTest("36",{bestPractice:332,testId:36,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more legend elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, were found to be empty.",metaText:"This legend element should not be empty",fixType:"",testFunc:function(){return new AutoTestShell('legend[data-ae_ar="null"][data-ae_vis][data-ae_avat]','legend[data-ae_ar="null"][data-ae_vis][data-ae_avat]:empty');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-46
	 * datalist is display:none by default
	 * @returns {AutoTestShell} testShell
	 */function test46(){aeKernel.addAdvisoryAutomaticTest("46",{bestPractice:1626,testId:46,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more img elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'img...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('img[data-ae_vis][data-ae_avat]','img[data-ae_vis][data-ae_avat][data-ae_ar="img"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-49
	 * @returns {AutoTestShell} testShell
	 */function test49(){aeKernel.addAdvisoryAutomaticTest("49",{bestPractice:415,testId:49,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more caption elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, were found to be empty.",metaText:"This caption element should not be empty",fixType:"",testFunc:function(){return new AutoTestShell('caption[data-ae_vis][data-ae_avat][data-ae_ar="null"]','caption[data-ae_vis][data-ae_avat][data-ae_ar="null"]:empty');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-56
	 * @returns {AutoTestShell} testShell
	 */function test56(){aeKernel.addAdvisoryAutomaticTest("56",{bestPractice:393,testId:56,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more ul elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, were found to be empty.",metaText:"This ul element should not be empty",fixType:"",testFunc:function(){return new AutoTestShell('ul[data-ae_vis][data-ae_avat][data-ae_ar="null"]','ul[data-ae_vis][data-ae_avat][data-ae_ar="null"]:empty');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-62
	 * @returns {AutoTestShell} testShell
	 */function test62(){aeKernel.addAdvisoryAutomaticTest("62",{bestPractice:1626,testId:62,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more header elements, excluding children of article elements or section elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'banner'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){const relevantCssSelector='header:not(article header):not(section header)[data-ae_vis][data-ae_avat]';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[data-ae_ar="banner"]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-63
	 * @returns {AutoTestShell} testShell
	 */function test63(){aeKernel.addAdvisoryAutomaticTest("63",{bestPractice:433,testId:63,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more acronym elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, is present.",metaText:"The acronym element is obsolete and should not be used",fixType:"",testFunc:function(){const relevantCssSelector='acronym';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[data-ae_ar="null"][data-ae_vis][data-ae_avat]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-68
	 * @returns {AutoTestShell} testShell
	 */function test68(){aeKernel.addAdvisoryAutomaticTest("68",{bestPractice:482,testId:68,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more s elements, without an ARIA-assigned role, not intentionally hidden in the DOM, are present in the DOM.",metaText:"The s element is obsolete and should not be used",fixType:"",testFunc:function(){const relevantCssSelector='s';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[data-ae_ar="null"][data-ae_vis]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-70
	 * @returns {AutoTestShell} testShell
	 */function test70(){aeKernel.addAdvisoryAutomaticTest("70",{bestPractice:482,testId:70,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more b elements, without an ARIA-assigned role, not intentionally hidden in the DOM, are present in the DOM.",metaText:"The b element is obsolete and should not be used",fixType:"",testFunc:function(){const relevantCssSelector='b';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[data-ae_ar="null"][data-ae_vis]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-72
	 * @returns {AutoTestShell} testShell
	 */function test72(){aeKernel.addAdvisoryAutomaticTest("72",{bestPractice:482,testId:72,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more u elements, without an ARIA-assigned role, not intentionally hidden in the DOM, are present in the DOM.",metaText:"The u element is obsolete and should not be used",fixType:"",testFunc:function(){const relevantCssSelector='u';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[data-ae_ar="null"][data-ae_vis]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-73
	 * @returns {AutoTestShell} testShell
	 */function test73(){aeKernel.addAdvisoryAutomaticTest("73",{bestPractice:482,testId:73,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more small elements, without an ARIA-assigned role, not intentionally hidden in the DOM, are present in the DOM.",metaText:"The small element is obsolete and should not be used",fixType:"",testFunc:function(){const relevantCssSelector='small';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[data-ae_ar="null"][data-ae_vis]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-75
	 * @returns {AutoTestShell} testShell
	 */function test75(){aeKernel.addAdvisoryAutomaticTest("75",{bestPractice:393,testId:75,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more ol elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, were found to be empty.",metaText:"This ol element should not be empty",fixType:"",testFunc:function(){return new AutoTestShell('ol[data-ae_vis][data-ae_avat][data-ae_ar="null"]','ol[data-ae_vis][data-ae_avat][data-ae_ar="null"]:empty');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-76
	 * @returns {AutoTestShell} testShell
	 */function test76(){aeKernel.addAdvisoryAutomaticTest("76",{bestPractice:393,testId:76,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more dl elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, were found to be empty.",metaText:"This dl element should not be empty",fixType:"",testFunc:function(){return new AutoTestShell('dl[data-ae_vis][data-ae_avat][data-ae_ar="null"]','dl[data-ae_vis][data-ae_avat][data-ae_ar="null"]:empty');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-77
	 * datalist is display:none by default
	 * @returns {AutoTestShell} testShell
	 */function test77(){aeKernel.addAdvisoryAutomaticTest("77",{bestPractice:1626,testId:77,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more details elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'group...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('details[data-ae_vis][data-ae_avat]','details[data-ae_vis][data-ae_avat][data-ae_ar="group"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-79
	 * datalist is display:none by default
	 * @returns {AutoTestShell} testShell
	 */function test79(){aeKernel.addAdvisoryAutomaticTest("79",{bestPractice:1626,testId:79,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more dialog elements with an open attribute, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'dialog...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('dialog[data-ae_vis][data-ae_avat][open]','dialog[data-ae_vis][data-ae_avat][open][data-ae_ar="dialog"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-82
	 * datalist is display:none by default
	 * @returns {AutoTestShell} testShell
	 */function test82(){aeKernel.addAdvisoryAutomaticTest("82",{bestPractice:1626,testId:82,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more figure elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'figure...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('figure[data-ae_vis][data-ae_avat]','figure[data-ae_vis][data-ae_avat][data-ae_ar="figure"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-84
	 * datalist is display:none by default
	 * @returns {AutoTestShell} testShell
	 */function test84(){aeKernel.addAdvisoryAutomaticTest("84",{bestPractice:1626,testId:84,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more dl elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'list...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('dl[data-ae_vis][data-ae_avat]','dl[data-ae_vis][data-ae_avat][data-ae_ar="list"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-86
	 * @returns {AutoTestShell} testShell
	 */function test86(){aeKernel.addAdvisoryAutomaticTest("86",{bestPractice:343,testId:86,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more map elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, are not preceded by an img element (without an ARIA-assigned role) that has a usemap attribute set to the same text value as their name attribute preceded by the '#' character.",metaText:"This element is not preceded by an img element (without an ARIA-assigned role) that has a usemap attribute set to the same text value as their name attribute preceded by the character: #",fixType:"",testFunc:function(){var testShell=new AutoTestShell('map[data-ae_vis][data-ae_avat][data-ae_ar="null"][name]:not([name=""])');testShell.set_candidateSetNodesFunc(function(){var notPreceded=[];var res=aeKernel.rootTestNode.querySelectorAll('map[data-ae_vis][data-ae_avat][data-ae_ar="null"][name]:not([name=""])');for(var i=0,len_res=res.length;i<len_res;i++){var name=res[i].getAttribute("name");var addRes=aeKernel.rootTestNode.querySelector('img[data-ae_ar="null"][usemap="#'+name.replace(/"/g,'\\"')+'"][data-ae_vis][data-ae_avat]~map[data-ae_ar="null"][name="'+name.replace(/"/g,'\\"')+'"]');// due to querySelector, rather than querySelectorAll.  querySelector is null if nothing is found
if(addRes===null){notPreceded[notPreceded.length]=res[i];}}return notPreceded;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-111
	 * @returns {AutoTestShell} testShell
	 */function test111(){aeKernel.addAdvisoryAutomaticTest("111",{bestPractice:338,testId:111,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more label elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, has an A element as a child.",metaText:"This element should not have an A element as a child",fixType:"",testFunc:function(){var testShell=new AutoTestShell('label[data-ae_ar="null"][data-ae_vis][data-ae_avat]');testShell.set_candidateSetNodesFunc(function(){var notProper=[];var res=aeKernel.rootTestNode.querySelectorAll('label[data-ae_ar="null"][data-ae_vis][data-ae_avat]');for(var i=0,len_res=res.length;i<len_res;i++){var result=res[i];var res1=result.querySelector('a');if(res1!==null){notProper[notProper.length]=res[i];}}return notProper;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-130
	 * @returns {AutoTestShell} testShell
	 */function test130(){aeKernel.addAdvisoryAutomaticTest("130",{bestPractice:387,testId:130,introduced:"0.9",mediaType:mediaTypes.WEB,description:"One or more h1 elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, is empty.",metaText:"This h1 element should not be empty",fixType:"",testFunc:function(){return new AutoTestShell('h1[data-ae_ar="null"][data-ae_vis][data-ae_avat]','h1[data-ae_ar="null"][data-ae_vis][data-ae_avat]:empty');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-161
	 * @returns {AutoTestShell} testShell
	 */function test161(){aeKernel.addAdvisoryAutomaticTest("161",{bestPractice:2518,testId:161,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements, not intentionally hidden in the DOM, has a color; bgcolor; alink; link; vlink; or text attribute set to a text value.",metaText:"This element should not have a color; bgcolor; alink; link; vlink; or text attribute set to a text value",fixType:"",testFunc:function(){// TODO: test applicability seems too broad here
const relevantCssSelector='*[data-ae_vis]';const failConditionSubSelectors=['[color]:not([color=""])','[bgcolor]:not([bgcolor=""])','[alink]:not([alink=""])','[link]:not([link=""])','[vlink]:not([vlink=""])','[text]:not([text=""])'];return new AutoTestShell(relevantCssSelector,failConditionSubSelectors.map(subSelector=>`${relevantCssSelector}${subSelector}`).join(', '));}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-163
	 * @returns {AutoTestShell} testShell
	 */function test163(){aeKernel.addAdvisoryAutomaticTest("163",{bestPractice:1626,testId:163,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='button' elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'button'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('button[data-ae_vis][data-ae_avat][type="button"]','button[data-ae_vis][data-ae_avat][type="button"][data-ae_ar="button"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-164
	 * @returns {AutoTestShell} testShell
	 */function test164(){aeKernel.addAdvisoryAutomaticTest("164",{bestPractice:1626,testId:164,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='checkbox' elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'checkbox...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_vis][data-ae_avat][type="checkbox"]','input[data-ae_vis][data-ae_avat][type="checkbox"][data-ae_ar="checkbox"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-172
	 * @returns {AutoTestShell} testShell
	 */function test172(){aeKernel.addAdvisoryAutomaticTest("172",{bestPractice:1626,testId:172,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='radio' elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'radio...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_vis][data-ae_avat][type="radio"]','input[data-ae_vis][data-ae_avat][type="radio"][data-ae_ar="radio"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-177
	 * @returns {AutoTestShell} testShell
	 */function test177(){aeKernel.addAdvisoryAutomaticTest("177",{bestPractice:1626,testId:177,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more li elements with a parent that is an ol element (without an ARIA-assigned role) or a ul element (without an ARIA-assigned role), not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'listitem...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('ol[data-ae_vis][data-ae_avat][data-ae_ar="null"] > li[data-ae_vis][data-ae_avat], ul[data-ae_vis][data-ae_avat][data-ae_ar="null"] > li[data-ae_vis][data-ae_avat]','ol[data-ae_vis][data-ae_avat][data-ae_ar="null"] > li[data-ae_vis][data-ae_avat][data-ae_ar="listitem"], ul[data-ae_vis][data-ae_avat][data-ae_ar="null"] > li[data-ae_vis][data-ae_avat][data-ae_ar="listitem"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-178
	 * @returns {AutoTestShell} testShell
	 */function test178(){aeKernel.addAdvisoryAutomaticTest("178",{bestPractice:1626,testId:178,introduced:"1.0",mediaType:mediaTypes.WEB,description:"A main element, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute.",metaText:"The role attribute is not allowed on this element",fixType:"",testFunc:function(){return new AutoTestShell('main[data-ae_vis][data-ae_avat]','main[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-179
	 * @returns {AutoTestShell} testShell
	 */function test179(){aeKernel.addAdvisoryAutomaticTest("179",{bestPractice:1626,testId:179,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more map elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute.",metaText:"The role attribute is not allowed on this element",fixType:"",testFunc:function(){return new AutoTestShell('map[data-ae_vis][data-ae_avat]','map[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-181
	 * @returns {AutoTestShell} testShell
	 */function test181(){aeKernel.addAdvisoryAutomaticTest("181",{bestPractice:1626,testId:181,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more menu type='toolbar' elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value:'toolbar...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('menu[data-ae_vis][data-ae_avat][type="toolbar"]','menu[data-ae_vis][data-ae_avat][type="toolbar"][data-ae_ar="toolbar"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-183
	 * @returns {AutoTestShell} testShell
	 */function test183(){aeKernel.addAdvisoryAutomaticTest("183",{bestPractice:1626,testId:183,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more meter elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'progressbar...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('meter[data-ae_vis][data-ae_avat]','meter[data-ae_vis][data-ae_avat][data-ae_ar="progressbar"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-184
	 * @returns {AutoTestShell} testShell
	 */function test184(){aeKernel.addAdvisoryAutomaticTest("184",{bestPractice:1626,testId:184,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more nav elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'navigation...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('nav[data-ae_vis][data-ae_avat]','nav[data-ae_vis][data-ae_avat][data-ae_ar="navigation"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-186
	 * @returns {AutoTestShell} testShell
	 */function test186(){aeKernel.addAdvisoryAutomaticTest("186",{bestPractice:1626,testId:186,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more ol elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'list...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('ol[data-ae_vis][data-ae_avat]','ol[data-ae_vis][data-ae_avat][data-ae_ar="list"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-187
	 * @returns {AutoTestShell} testShell
	 */function test187(){aeKernel.addAdvisoryAutomaticTest("187",{bestPractice:1626,testId:187,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more output elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'status...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('output[data-ae_vis][data-ae_avat]','output[data-ae_vis][data-ae_avat][data-ae_ar="status"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-190
	 * @returns {AutoTestShell} testShell
	 */function test190(){aeKernel.addAdvisoryAutomaticTest("190",{bestPractice:1626,testId:190,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more section elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'region...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('section[data-ae_vis][data-ae_avat]','section[data-ae_vis][data-ae_avat][data-ae_ar="region"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-193
	 * @returns {AutoTestShell} testShell
	 */function test193(){aeKernel.addAdvisoryAutomaticTest("193",{bestPractice:1626,testId:193,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more summary elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'button...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('summary[data-ae_vis][data-ae_avat]','summary[data-ae_vis][data-ae_avat][data-ae_ar="button"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-194
	 * @returns {AutoTestShell} testShell
	 */function test194(){aeKernel.addAdvisoryAutomaticTest("194",{bestPractice:1626,testId:194,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more table elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'table...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('table[data-ae_vis][data-ae_avat]','table[data-ae_vis][data-ae_avat][data-ae_ar="table"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-196
	 * @returns {AutoTestShell} testShell
	 */function test196(){aeKernel.addAdvisoryAutomaticTest("196",{bestPractice:1626,testId:196,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more tbody elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'rowgroup...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('tbody[data-ae_vis][data-ae_avat]','tbody[data-ae_vis][data-ae_avat][data-ae_ar="rowgroup"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-197
	 * @returns {AutoTestShell} testShell
	 */function test197(){aeKernel.addAdvisoryAutomaticTest("197",{bestPractice:1626,testId:197,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more thead elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'rowgroup...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('thead[data-ae_vis][data-ae_avat]','thead[data-ae_vis][data-ae_avat][data-ae_ar="rowgroup"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-198
	 * @returns {AutoTestShell} testShell
	 */function test198(){aeKernel.addAdvisoryAutomaticTest("198",{bestPractice:1626,testId:198,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more tfoot elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'rowgroup...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('tfoot[data-ae_vis][data-ae_avat]','tfoot[data-ae_vis][data-ae_avat][data-ae_ar="rowgroup"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-200
	 * @returns {AutoTestShell} testShell
	 */function test200(){aeKernel.addAdvisoryAutomaticTest("200",{bestPractice:1626,testId:200,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more td elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'cell...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('td[data-ae_vis][data-ae_avat]','td[data-ae_vis][data-ae_avat][data-ae_ar="cell"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-201
	 * @returns {AutoTestShell} testShell
	 */function test201(){aeKernel.addAdvisoryAutomaticTest("201",{bestPractice:1626,testId:201,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more th elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'columnheader... | rowheader...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('th[data-ae_vis][data-ae_avat]','th[data-ae_vis][data-ae_avat][data-ae_ar="columnheader"], th[data-ae_vis][data-ae_avat][data-ae_ar="rowheader"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-202
	 * @returns {AutoTestShell} testShell
	 */function test202(){aeKernel.addAdvisoryAutomaticTest("202",{bestPractice:1626,testId:202,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more tr elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'row'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('tr[data-ae_vis][data-ae_avat]','tr[data-ae_vis][data-ae_avat][data-ae_ar="row"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-204
	 * @returns {AutoTestShell} testShell
	 */function test204(){aeKernel.addAdvisoryAutomaticTest("204",{bestPractice:1626,testId:204,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more ul elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'list...'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('ul[data-ae_vis][data-ae_avat]','ul[data-ae_vis][data-ae_avat][data-ae_ar="list"]');}});}/**
	 * (To be replaced with Machine Learning)
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-220
	 * @returns {AutoTestShell} testShell
	 */function test220(){aeKernel.addStage1MarkdownFunc("getBefore_FromStyles",getBefore_FromStyles);aeKernel.addStage1MarkdownFunc("getAfter_FromStyles",getAfter_FromStyles);aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addStage4MarkdownFunc("calcAccDescriptionsForCertainElements",calcAccDescriptionsForCertainElements);aeKernel.addAdvisoryAutomaticTest("220",{bestPractice:398,testId:220,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more A elements that have a target attribute set to the text value '_blank', without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have a calculated accessible name or description value that includes a warning to the user that a new window will open i.e. 'new window'.",metaText:"This element does not have a calculated accessible name or description value that includes a warning to the user that it will cause a new window to be opened when activated i.e. new window",metaTextDetail:"This {{tag-name}} does not have a calculated accessible name or description value that includes a warning to the user that it will cause a new window to be opened when activated i.e. new window",fixType:"",testFunc:function(){const relevantCssSelector='a[data-ae_vis][data-ae_avat][data-ae_ar="null"][target="_blank"]';const testShell=new AutoTestShell(relevantCssSelector);testShell.set_candidateSetNodesFunc(function(){const badNodes=[];const pattern=/new window|tab/;const badNodeCandidates=aeKernel.rootTestNode.querySelectorAll(`${relevantCssSelector}[data-ae_an], ${relevantCssSelector}[data-ae_ad]`);for(let i=0;i<badNodeCandidates.length;i++){const badNodeCandidate=badNodeCandidates[i];if(!pattern.test(badNodeCandidate.getAttribute('data-ae_an'))&&!pattern.test(badNodeCandidate.getAttribute('data-ae_ad'))){badNodes.push(badNodeCandidate);}}return badNodes;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-225
	 * @returns {AutoTestShell} testShell
	 */function test225(){aeKernel.addAdvisoryAutomaticTest("225",{bestPractice:338,testId:225,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more label elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, were found to be empty.",metaText:"This label element should not be empty",fixType:"",testFunc:function(){return new AutoTestShell('label[data-ae_vis][data-ae_avat][data-ae_ar="null"]','label[data-ae_vis][data-ae_avat][data-ae_ar="null"]:empty');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-247
	 * @returns {AutoTestShell} testShell
	 */function test247(){aeKernel.addAdvisoryAutomaticTest("247",{bestPractice:482,testId:247,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more i elements, without an ARIA-assigned role, not intentionally hidden in the DOM, are present in the DOM.",metaText:"The i element is obsolete and should not be used",fixType:"",testFunc:function(){const relevantCssSelector='i';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[data-ae_ar="null"][data-ae_vis]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-293
	 * @returns {AutoTestShell} testShell
	 */function test293(){aeKernel.addAdvisoryAutomaticTest("293",{bestPractice:1626,testId:293,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more footer elements, excluding children of article elements (without an ARIA-assigned role) or section elements (without an ARIA-assigned role), not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'contentinfo'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){const relevantCssSelector='footer:not(article[data-ae_ar="null"] footer):not(section[data-ae_ar="null"] footer)[data-ae_vis][data-ae_avat]';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[data-ae_ar="contentinfo"]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-295
	 * @returns {AutoTestShell} testShell
	 */function test295(){aeKernel.addAdvisoryAutomaticTest("295",{bestPractice:1626,testId:295,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more form elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'form'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('form[data-ae_vis][data-ae_avat]','form[data-ae_vis][data-ae_avat][data-ae_ar="form"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-296
	 * @returns {AutoTestShell} testShell
	 */function test296(){aeKernel.addAdvisoryAutomaticTest("296",{bestPractice:1626,testId:296,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more header elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to a value that is not: 'group'.",metaText:"This element has a role attribute that is not set to the following text value: group",fixType:"",testFunc:function(){return new AutoTestShell('header[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"])','header[data-ae_vis][data-ae_avat]:not([data-ae_ar="null"]):not([data-ae_ar="group"])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-305
	 * @returns {AutoTestShell} testShell
	 */function test305(){aeKernel.addAdvisoryAutomaticTest("305",{bestPractice:1626,testId:305,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='range' elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'slider'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_vis][data-ae_avat][type="range"]','input[data-ae_vis][data-ae_avat][type="range"][data-ae_ar="slider"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-306
	 * @returns {AutoTestShell} testShell
	 */function test306(){aeKernel.addAdvisoryAutomaticTest("306",{bestPractice:1626,testId:306,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='reset' element, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'button'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_vis][data-ae_avat][type="reset"]','input[data-ae_vis][data-ae_avat][type="reset"][data-ae_ar="button"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-307
	 * @returns {AutoTestShell} testShell
	 */function test307(){aeKernel.addAdvisoryAutomaticTest("307",{bestPractice:1626,testId:307,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='search' element, excluding those with a list attribute set to a non-null value, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'searchbox'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){const relevantCssSelector='input[type="search"]:not([list]:not([list=""]))[data-ae_vis][data-ae_avat]';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[data-ae_ar="searchbox"]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-308
	 * @returns {AutoTestShell} testShell
	 */function test308(){aeKernel.addAdvisoryAutomaticTest("308",{bestPractice:1626,testId:308,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='submit' elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'button'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('input[data-ae_vis][data-ae_avat][type="submit"]','input[data-ae_vis][data-ae_avat][type="submit"][data-ae_ar="button"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-310
	 * @returns {AutoTestShell} testShell
	 */function test310(){aeKernel.addAdvisoryAutomaticTest("310",{bestPractice:1626,testId:310,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='text', excluding those with a list attribute set to a non-null value, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'textbox | combobox'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){const relevantCssSelector='input[type="text"]:not([list]:not([list=""]))[data-ae_vis][data-ae_avat]';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[data-ae_ar="textbox"], ${relevantCssSelector}[data-ae_ar="combobox"]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-316
	 * @returns {AutoTestShell} testShell
	 */function test316(){aeKernel.addAdvisoryAutomaticTest("316",{bestPractice:1626,testId:316,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more input type='url', excluding those with a list attribute set to a non-null value, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'textbox | combobox'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){const relevantCssSelector='input[type="url"]:not([list]:not([list=""]))[data-ae_vis][data-ae_avat]';return new AutoTestShell(relevantCssSelector,`${relevantCssSelector}[data-ae_ar="textbox"], ${relevantCssSelector}[data-ae_ar="combobox"]`);}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-320
	 * @returns {AutoTestShell} testShell
	 */function test320(){aeKernel.addAdvisoryAutomaticTest("320",{bestPractice:1626,testId:320,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more optgroup elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'group'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('optgroup[data-ae_vis][data-ae_avat]','optgroup[data-ae_vis][data-ae_avat][data-ae_ar="group"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-321
	 * @returns {AutoTestShell} testShell
	 */function test321(){aeKernel.addAdvisoryAutomaticTest("321",{bestPractice:1626,testId:321,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more option elements, that is a child of a select element (without an ARIA-assigned role); an optgroup element (without an ARIA-assigned role) or a datalist element (without an ARIA-assigned role), not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'option'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('select[data-ae_ar="null"] option[data-ae_vis][data-ae_avat], optgroup[data-ae_ar="null"] option[data-ae_vis][data-ae_avat], datalist[data-ae_ar="null"] option[data-ae_vis][data-ae_avat]','select[data-ae_ar="null"] option[data-ae_vis][data-ae_avat][data-ae_ar="option"], optgroup[data-ae_ar="null"] option[data-ae_vis][data-ae_avat][data-ae_ar="option"], datalist[data-ae_ar="null"] option[data-ae_vis][data-ae_avat][data-ae_ar="option"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-323
	 * @returns {AutoTestShell} testShell
	 */function test323(){aeKernel.addAdvisoryAutomaticTest("323",{bestPractice:1626,testId:323,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more select elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'listbox'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('select[data-ae_vis][data-ae_avat]','select[data-ae_vis][data-ae_avat][data-ae_ar="listbox"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-324
	 * @returns {AutoTestShell} testShell
	 */function test324(){aeKernel.addAdvisoryAutomaticTest("324",{bestPractice:1626,testId:324,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more textarea elements, not intentionally hidden in the DOM and available to assistive technologies, has a role attribute set to the value: 'textbox'.",metaText:"This element has a redundant role attribute",fixType:"",testFunc:function(){return new AutoTestShell('textarea[data-ae_vis][data-ae_avat]','textarea[data-ae_vis][data-ae_avat][data-ae_ar="textbox"]');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-327
	 * @returns {AutoTestShell} testShell
	 */function test327(){aeKernel.addAdvisoryAutomaticTest("327",{bestPractice:362,testId:327,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more picture elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, does not have an img element as its last child element.",metaText:"This element does not have an img element as its last child element",fixType:"",testFunc:function(){var testShell=new AutoTestShell('picture[data-ae_vis][data-ae_avat][data-ae_ar="null"]');testShell.set_candidateSetNodesFunc(function(){var pictureNoImgLastChild=[];var res=aeKernel.rootTestNode.querySelectorAll('picture[data-ae_vis][data-ae_avat][data-ae_ar="null"]');for(var i=0,len_res=res.length;i<len_res;i++){var n=res[i];var imgLastChild=n.querySelector('img[data-ae_ar="null"]:last-child');if(imgLastChild===null){pictureNoImgLastChild.push(n);}}return pictureNoImgLastChild;});return testShell;}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-379
	 * @returns {AutoTestShell} testShell
	 */function test379(){aeKernel.addAdvisoryAutomaticTest("379",{bestPractice:387,testId:379,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more h2 elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, is empty.",metaText:"This h2 element should not be empty",fixType:"",testFunc:function(){return new AutoTestShell('h2[data-ae_ar="null"][data-ae_vis][data-ae_avat]','h2[data-ae_ar="null"][data-ae_vis][data-ae_avat]:empty');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-380
	 * @returns {AutoTestShell} testShell
	 */function test380(){aeKernel.addAdvisoryAutomaticTest("380",{bestPractice:387,testId:380,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more h3 elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, is empty.",metaText:"This h3 element should not be empty",fixType:"",testFunc:function(){return new AutoTestShell('h3[data-ae_ar="null"][data-ae_vis][data-ae_avat]','h3[data-ae_ar="null"][data-ae_vis][data-ae_avat]:empty');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-381
	 * @returns {AutoTestShell} testShell
	 */function test381(){aeKernel.addAdvisoryAutomaticTest("381",{bestPractice:387,testId:381,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more h4 elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, is empty.",metaText:"This h4 element should not be empty",fixType:"",testFunc:function(){return new AutoTestShell('h4[data-ae_ar="null"][data-ae_vis][data-ae_avat]','h4[data-ae_ar="null"][data-ae_vis][data-ae_avat]:empty');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-382
	 * @returns {AutoTestShell} testShell
	 */function test382(){aeKernel.addAdvisoryAutomaticTest("382",{bestPractice:387,testId:382,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more h5 elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, is empty.",metaText:"This h5 element should not be empty",fixType:"",testFunc:function(){return new AutoTestShell('h5[data-ae_ar="null"][data-ae_vis][data-ae_avat]','h5[data-ae_ar="null"][data-ae_vis][data-ae_avat]:empty');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-383
	 * @returns {AutoTestShell} testShell
	 */function test383(){aeKernel.addAdvisoryAutomaticTest("383",{bestPractice:387,testId:383,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more h6 elements, without an ARIA-assigned role, not intentionally hidden in the DOM and available to assistive technologies, is empty.",metaText:"This h6 element should not be empty",fixType:"",testFunc:function(){return new AutoTestShell('h6[data-ae_ar="null"][data-ae_vis][data-ae_avat]','h6[data-ae_ar="null"][data-ae_vis][data-ae_avat]:empty');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-384
	 * @returns {AutoTestShell} testShell
	 */function test384(){aeKernel.addAdvisoryAutomaticTest("384",{bestPractice:387,testId:384,introduced:"1.0",mediaType:mediaTypes.WEB,description:"One or more elements with a role attribute set to a value that starts with 'heading', not intentionally hidden in the DOM and available to assistive technologies, is empty.",metaText:"Elements given a role=heading should not be empty",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_ar="heading"][data-ae_vis][data-ae_avat]','*[data-ae_ar="heading"][data-ae_vis][data-ae_avat]:empty');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-1042
	 * @returns {AutoTestShell} testShell
	 */function test1042(){aeKernel.addAdvisoryAutomaticTest("1042",{bestPractice:1626,testId:1042,introduced:"2.20",mediaType:mediaTypes.WEB,description:"One or more elements that has a role='option' attribute, excluding option elements, not intentionally hidden in the DOM and available to assistive technologies, does not have an aria-selected attribute.",metaText:"This element (role=option) does not have an aria-selected attribute",metaTextDetail:"This {{tag-name}} (role=option) does not have an aria-selected attribute",fixType:"",testFunc:function(){return new AutoTestShell('*[data-ae_vis][data-ae_avat][data-ae_ar="option"]:not(option)','*[data-ae_vis][data-ae_avat][data-ae_ar="option"]:not(option):not([aria-selected])');}});}/**
	 * Jira: https://ssbbart.atlassian.net/browse/ENG-745
	 * @returns {AutoTestShell} testShell
	 */function test745(){aeKernel.addAdvisoryAutomaticTest("745",{bestPractice:1917,testId:745,introduced:"2.11",mediaType:mediaTypes.IOS,description:"One or more Button, Switch, TextField, SecureTextField, SearchField, TextView, or Slider elements is less than 6 points away from (and may be overlapping) another element.",metaText:"This [[object]] is less than 6 points away from (and may be overlapping) another element",fixType:"",testFunc:function(){var cssScrollView='[data-xmlnodetype="XCUIElementTypeScrollView"]';var cssLargeGroup='[data-xmlnodetype="XCUIElementTypeButton"],[data-xmlnodetype="XCUIElementTypeSwitch"],[data-xmlnodetype="XCUIElementTypeTextField"],[data-xmlnodetype="XCUIElementTypeSecureTextField"],[data-xmlnodetype="XCUIElementTypeSearchField"],[data-xmlnodetype="XCUIElementTypeTextView"],[data-xmlnodetype="XCUIElementTypeSlider"]';// compute a CSS selector that is identical to cssLargeGroup, but prefixes each sub-selector with a scroll view (cssScrollView) using a space as a combinator;
// this resulting CSS selector selects nodes we're interested in testing that are inside/descendants of scroll views
var cssGroupToExcludeFromLargeGroup=cssScrollView+' '+cssLargeGroup.split(',').join(','+cssScrollView+' ');var testShell=new AutoTestShell(cssLargeGroup);testShell.set_candidateSetNodesFunc(function(){var bucketedNodesToTest=[];// for each scroll view, bucket all its descendant nodes that we want to test
var scrollViewNodes=aeKernel.rootTestNode.querySelectorAll(cssScrollView);for(var i=0;i<scrollViewNodes.length;i++){var scrollViewNode=scrollViewNodes[i];bucketedNodesToTest.push(scrollViewNode.querySelectorAll(cssLargeGroup));}// bucket all nodes that we want to test that are not descendants of scroll views
var nodesToTestNotInScrollView=testUtil_ifDiff(aeKernel.rootTestNode,[cssLargeGroup,cssGroupToExcludeFromLargeGroup]);bucketedNodesToTest.push(nodesToTestNotInScrollView);var badNodes=[];var ALLOWED_MARGIN=3;// equates to 6 points of separation between elements (3 + 3)
for(i=0;i<bucketedNodesToTest.length;i++){var nodes=bucketedNodesToTest[i];var badNodeIndices={};// keep track of which nodes we flag so that we don't create duplicate violations
for(var j=0;j<nodes.length;j++){if(badNodeIndices[j]){// node1 has already been flagged
continue;}for(var k=0;k<nodes.length;k++){if(j===k){// don't compare a node to itself
continue;}if(badNodeIndices[k]){// node2 has already been flagged
continue;}var node1=nodes[j];var node1Top=parseInt(node1.style.top,10);var node1Left=parseInt(node1.style.left,10);var node1Width=parseInt(node1.style.width,10);var node1Height=parseInt(node1.style.height,10);var node1TopWithMargin=node1Top-ALLOWED_MARGIN;var node1LeftWithMargin=node1Left-ALLOWED_MARGIN;var node1RightWithMargin=node1Left+node1Width+ALLOWED_MARGIN;var node1BottomWithMargin=node1Top+node1Height+ALLOWED_MARGIN;var node2=nodes[k];var node2Top=parseInt(node2.style.top,10);var node2Left=parseInt(node2.style.left,10);var node2Width=parseInt(node2.style.width,10);var node2Height=parseInt(node2.style.height,10);var node2TopWithMargin=node2Top-ALLOWED_MARGIN;var node2LeftWithMargin=node2Left-ALLOWED_MARGIN;var node2RightWithMargin=node2Left+node2Width+ALLOWED_MARGIN;var node2BottomWithMargin=node2Top+node2Height+ALLOWED_MARGIN;var isOverlapping=!(node1RightWithMargin<=node2LeftWithMargin||node1LeftWithMargin>=node2RightWithMargin||node1BottomWithMargin<=node2TopWithMargin||node1TopWithMargin>=node2BottomWithMargin);if(isOverlapping){if(!badNodeIndices[j]){badNodes.push(node1);badNodeIndices[j]=true;}if(!badNodeIndices[k]){badNodes.push(node2);badNodeIndices[k]=true;}}}}}return badNodes;});return testShell;}});}/**
	 * Extract the information we need for background image testing from computed styles
	 */function getBackgroundImage_FromStyles(computedStyles){let backImg=computedStyles["background-image"];if(backImg!=="none"){return"background-image;yes";}return null;}/**
	 * Sets the data-ae_bckimg attribute to relevant value
	 * @param {*} originalInformation
	 */function setBackgroundImage(originalInformation,node){// addBackgroundImage
try{var bgImage=originalInformation["background-image"];if(bgImage!==undefined){return["data-ae_bckimg","true"];}else{return null;}}catch(err){logger.error("stage5_setBackgroundImage",err);return null;}}/**
	 * Mode: Background Images
	 * @returns Array Nodes
	 */function previewMode_backgroundImages(){aeKernel.addStage1MarkdownFunc("getBackgroundImage_FromStyles",getBackgroundImage_FromStyles);aeKernel.addStage5MarkdownFunc("setBackgroundImage",setBackgroundImage);aeKernel.addPreviewMode("previewMode_backgroundImages",{bestPractice:0,testId:"previewMode_backgroundImages",testFunc:function(){return aeKernel.rootTestNode.querySelectorAll('*[data-ae_vis][data-ae_bckimg="true"]');}});}/**
	 * Mode: Elements with lang attributes
	 * @returns Array Nodes
	 */function previewMode_elsWithLangAttribs(){aeKernel.addPreviewMode("previewMode_elsWithLangAttribs",{bestPractice:0,testId:"previewMode_elsWithLangAttribs",testFunc:function(){return aeKernel.rootTestNode.querySelectorAll('*[data-ae_vis][data-ae_avat][lang]');}});}/**
	 * Mode: Headings
	 * @returns Array Nodes
	 */function previewMode_headings(){aeKernel.addPreviewMode("previewMode_headings",{bestPractice:0,testId:"previewMode_headings",testFunc:function(){return aeKernel.rootTestNode.querySelectorAll('h1[data-ae_vis][data-ae_avat][data-ae_ar="null"], h2[data-ae_vis][data-ae_avat][data-ae_ar="null"], h3[data-ae_vis][data-ae_avat][data-ae_ar="null"], h4[data-ae_vis][data-ae_avat][data-ae_ar="null"], h5[data-ae_vis][data-ae_avat][data-ae_ar="null"], h6[data-ae_vis][data-ae_avat][data-ae_ar="null"], *[data-ae_vis][data-ae_avat][data-ae_ar="heading"]');}});}/**
	 * Mode: Iframes
	 * @returns Array Nodes
	 */function previewMode_iframes(){aeKernel.addStage4MarkdownFunc("calcAccNamesForCertainElements",calcAccNamesForCertainElements);aeKernel.addPreviewMode("previewMode_iframes",{bestPractice:0,testId:"previewMode_iframes",collectMetaDataFunc:function(instance){return{"accNameMechanism":null,"accNameValue":instance.getAttribute("data-ae_an")};},testFunc:function(){return aeKernel.rootTestNode.querySelectorAll('iframe[data-ae_vis][data-ae_avat][data-ae_ar="null"][data-ae_an]');}});}/**
	 * Mode: Foreground Images - with accessible names
	 * @returns Array Nodes
	 */function previewMode_images(){aeKernel.addPreviewMode("previewMode_images",{bestPractice:0,testId:"previewMode_images",testFunc:function(){return aeKernel.rootTestNode.querySelectorAll('img[data-ae_vis], *[data-ae_vis][data-ae_ar="img"], input[type="image"][data-ae_vis], svg[data-ae_vis]');}});}/**
	 * Mode: Lists
	 * @returns Array Nodes
	 */function previewMode_lists(){aeKernel.addPreviewMode("previewMode_lists",{bestPractice:0,testId:"previewMode_lists",testFunc:function(){return aeKernel.rootTestNode.querySelectorAll('ul[data-ae_vis][data-ae_avat][data-ae_ar="null"], ol[data-ae_vis][data-ae_avat][data-ae_ar="null"], dl[data-ae_vis][data-ae_avat][data-ae_ar="null"]');}});}/**
	 * Mode: Table Data
	 * @returns Array Nodes
	 */function previewMode_tableCells(){aeKernel.addPreviewMode("previewMode_tableCells",{bestPractice:0,testId:"previewMode_tableData",testFunc:function(){return aeKernel.rootTestNode.querySelectorAll('td[data-ae_vis][data-ae_avat][data-ae_ar="null"], *[data-ae_vis][data-ae_avat][data-ae_ar="cell"], *[data-ae_vis][data-ae_avat][data-ae_ar="gridcell"]');}});}/**
	 * Mode: Table Headings
	 * @returns Array Nodes
	 */function previewMode_tableHeadings(){aeKernel.addPreviewMode("previewMode_tableHeadings",{bestPractice:0,testId:"previewMode_tableHeadings",testFunc:function(){return aeKernel.rootTestNode.querySelectorAll('th[data-ae_vis][data-ae_avat][data-ae_ar="null"], *[data-ae_vis][data-ae_avat][data-ae_ar="rowheader"], *[data-ae_vis][data-ae_avat][data-ae_ar="columnheader"], *[data-ae_vis][data-ae_avat][data-ae_ar="grid"]');}});}/**
	 * Mode: Table Headings
	 * @returns Array Nodes
	 */function previewMode_tables(){aeKernel.addStage4MarkdownFunc("identifiesDataTables",identifiesDataTables);aeKernel.addPreviewMode("previewMode_tables",{bestPractice:0,testId:"previewMode_tables",testFunc:function(){return aeKernel.rootTestNode.querySelectorAll('table[data-ae_vis][data-ae_avat][data-ae_ar="null"][data-ae_dtab="true"], *[data-ae_vis][data-ae_avat][data-ae_ar="grid"][data-ae_dtab="true"], *[data-ae_vis][data-ae_avat][data-ae_ar="table"][data-ae_dtab="true"]');}});}/**
	 * Public interface to the Access Engine version
	 */function getVersion(){return aeKernel.version;}/**
	 * Public interface to get a catalog of data about the tests
	 */function getTestInfo(columns){//If no columns then they want everything so set it to empty for robustness
if(typeof columns==='undefined'){var columns={};}//If we are not passed any testTypes then accumulate a list of all of them
//Except negative values as there are no AMP equivalents for those
//In practice, there are only test types 4 and 5.
if(columns.testType==null){columns.testType=[];for(var i in testTypes){if(testTypes[i]>-2){columns.testType.push(testTypes[i]);}}}//If we are not passed any columns to display, show all of them
if(columns.columns==null){columns.columns=["bestPractice","metaText","testType","description","fixType","mediaType","introduced"];}var results={};//Accumulate all the available tests for each requested testType
for(var k=0;k<columns.testType.length;k++){var testType=columns.testType[k];var tests=aeKernel.getTestsToRun(testType);//Walk through each test available for a particular testType
for(var i in tests){var row={};//Accumulate the requested attributes for a particular test
for(var j=0;j<columns.columns.length;j++){//Special handling of column testType as it's not actually an attribute of the test javascript struct
if(columns.columns[j]=="testType"){row["testType"]=testType;}else{row[columns.columns[j]]=tests[i][columns.columns[j]];}}results[tests[i]["testId"]]=row;}}return results;}/**
	 * Public interface to the Access Engine Kernel object, to enable settings to be changed
	 * @param {Window} testWind
	 */function setWindowUnderTest(testWind){aeKernel.setWindowUnderTest(testWind);}var sha256$1={exports:{}};var core={exports:{}};(function(module,exports){(function(root,factory){{// CommonJS
module.exports=factory();}})(commonjsGlobal,function(){/*globals window, global, require*/ /**
		 * CryptoJS core components.
		 */var CryptoJS=CryptoJS||function(Math,undefined$1){var crypto;// Native crypto from window (Browser)
if(typeof window!=='undefined'&&window.crypto){crypto=window.crypto;}// Native crypto in web worker (Browser)
if(typeof self!=='undefined'&&self.crypto){crypto=self.crypto;}// Native crypto from worker
if(typeof globalThis!=='undefined'&&globalThis.crypto){crypto=globalThis.crypto;}// Native (experimental IE 11) crypto from window (Browser)
if(!crypto&&typeof window!=='undefined'&&window.msCrypto){crypto=window.msCrypto;}// Native crypto from global (NodeJS)
if(!crypto&&typeof commonjsGlobal!=='undefined'&&commonjsGlobal.crypto){crypto=commonjsGlobal.crypto;}// Native crypto import via require (NodeJS)
if(!crypto&&typeof commonjsRequire==='function'){try{crypto=require('crypto');}catch(err){}}/*
		     * Cryptographically secure pseudorandom number generator
		     *
		     * As Math.random() is cryptographically not safe to use
		     */var cryptoSecureRandomInt=function(){if(crypto){// Use getRandomValues method (Browser)
if(typeof crypto.getRandomValues==='function'){try{return crypto.getRandomValues(new Uint32Array(1))[0];}catch(err){}}// Use randomBytes method (NodeJS)
if(typeof crypto.randomBytes==='function'){try{return crypto.randomBytes(4).readInt32LE();}catch(err){}}}throw new Error('Native crypto module could not be used to get secure random number.');};/*
		     * Local polyfill of Object.create

		     */var create=Object.create||function(){function F(){}return function(obj){var subtype;F.prototype=obj;subtype=new F();F.prototype=null;return subtype;};}();/**
		     * CryptoJS namespace.
		     */var C={};/**
		     * Library namespace.
		     */var C_lib=C.lib={};/**
		     * Base object for prototypal inheritance.
		     */var Base=C_lib.Base=function(){return{/**
		             * Creates a new object that inherits from this object.
		             *
		             * @param {Object} overrides Properties to copy into the new object.
		             *
		             * @return {Object} The new object.
		             *
		             * @static
		             *
		             * @example
		             *
		             *     var MyType = CryptoJS.lib.Base.extend({
		             *         field: 'value',
		             *
		             *         method: function () {
		             *         }
		             *     });
		             */extend:function(overrides){// Spawn
var subtype=create(this);// Augment
if(overrides){subtype.mixIn(overrides);}// Create default initializer
if(!subtype.hasOwnProperty('init')||this.init===subtype.init){subtype.init=function(){subtype.$super.init.apply(this,arguments);};}// Initializer's prototype is the subtype object
subtype.init.prototype=subtype;// Reference supertype
subtype.$super=this;return subtype;},/**
		             * Extends this object and runs the init method.
		             * Arguments to create() will be passed to init().
		             *
		             * @return {Object} The new object.
		             *
		             * @static
		             *
		             * @example
		             *
		             *     var instance = MyType.create();
		             */create:function(){var instance=this.extend();instance.init.apply(instance,arguments);return instance;},/**
		             * Initializes a newly created object.
		             * Override this method to add some logic when your objects are created.
		             *
		             * @example
		             *
		             *     var MyType = CryptoJS.lib.Base.extend({
		             *         init: function () {
		             *             // ...
		             *         }
		             *     });
		             */init:function(){},/**
		             * Copies properties into this object.
		             *
		             * @param {Object} properties The properties to mix in.
		             *
		             * @example
		             *
		             *     MyType.mixIn({
		             *         field: 'value'
		             *     });
		             */mixIn:function(properties){for(var propertyName in properties){if(properties.hasOwnProperty(propertyName)){this[propertyName]=properties[propertyName];}}// IE won't copy toString using the loop above
if(properties.hasOwnProperty('toString')){this.toString=properties.toString;}},/**
		             * Creates a copy of this object.
		             *
		             * @return {Object} The clone.
		             *
		             * @example
		             *
		             *     var clone = instance.clone();
		             */clone:function(){return this.init.prototype.extend(this);}};}();/**
		     * An array of 32-bit words.
		     *
		     * @property {Array} words The array of 32-bit words.
		     * @property {number} sigBytes The number of significant bytes in this word array.
		     */var WordArray=C_lib.WordArray=Base.extend({/**
		         * Initializes a newly created word array.
		         *
		         * @param {Array} words (Optional) An array of 32-bit words.
		         * @param {number} sigBytes (Optional) The number of significant bytes in the words.
		         *
		         * @example
		         *
		         *     var wordArray = CryptoJS.lib.WordArray.create();
		         *     var wordArray = CryptoJS.lib.WordArray.create([0x00010203, 0x04050607]);
		         *     var wordArray = CryptoJS.lib.WordArray.create([0x00010203, 0x04050607], 6);
		         */init:function(words,sigBytes){words=this.words=words||[];if(sigBytes!=undefined$1){this.sigBytes=sigBytes;}else{this.sigBytes=words.length*4;}},/**
		         * Converts this word array to a string.
		         *
		         * @param {Encoder} encoder (Optional) The encoding strategy to use. Default: CryptoJS.enc.Hex
		         *
		         * @return {string} The stringified word array.
		         *
		         * @example
		         *
		         *     var string = wordArray + '';
		         *     var string = wordArray.toString();
		         *     var string = wordArray.toString(CryptoJS.enc.Utf8);
		         */toString:function(encoder){return(encoder||Hex).stringify(this);},/**
		         * Concatenates a word array to this word array.
		         *
		         * @param {WordArray} wordArray The word array to append.
		         *
		         * @return {WordArray} This word array.
		         *
		         * @example
		         *
		         *     wordArray1.concat(wordArray2);
		         */concat:function(wordArray){// Shortcuts
var thisWords=this.words;var thatWords=wordArray.words;var thisSigBytes=this.sigBytes;var thatSigBytes=wordArray.sigBytes;// Clamp excess bits
this.clamp();// Concat
if(thisSigBytes%4){// Copy one byte at a time
for(var i=0;i<thatSigBytes;i++){var thatByte=thatWords[i>>>2]>>>24-i%4*8&0xff;thisWords[thisSigBytes+i>>>2]|=thatByte<<24-(thisSigBytes+i)%4*8;}}else{// Copy one word at a time
for(var j=0;j<thatSigBytes;j+=4){thisWords[thisSigBytes+j>>>2]=thatWords[j>>>2];}}this.sigBytes+=thatSigBytes;// Chainable
return this;},/**
		         * Removes insignificant bits.
		         *
		         * @example
		         *
		         *     wordArray.clamp();
		         */clamp:function(){// Shortcuts
var words=this.words;var sigBytes=this.sigBytes;// Clamp
words[sigBytes>>>2]&=0xffffffff<<32-sigBytes%4*8;words.length=Math.ceil(sigBytes/4);},/**
		         * Creates a copy of this word array.
		         *
		         * @return {WordArray} The clone.
		         *
		         * @example
		         *
		         *     var clone = wordArray.clone();
		         */clone:function(){var clone=Base.clone.call(this);clone.words=this.words.slice(0);return clone;},/**
		         * Creates a word array filled with random bytes.
		         *
		         * @param {number} nBytes The number of random bytes to generate.
		         *
		         * @return {WordArray} The random word array.
		         *
		         * @static
		         *
		         * @example
		         *
		         *     var wordArray = CryptoJS.lib.WordArray.random(16);
		         */random:function(nBytes){var words=[];for(var i=0;i<nBytes;i+=4){words.push(cryptoSecureRandomInt());}return new WordArray.init(words,nBytes);}});/**
		     * Encoder namespace.
		     */var C_enc=C.enc={};/**
		     * Hex encoding strategy.
		     */var Hex=C_enc.Hex={/**
		         * Converts a word array to a hex string.
		         *
		         * @param {WordArray} wordArray The word array.
		         *
		         * @return {string} The hex string.
		         *
		         * @static
		         *
		         * @example
		         *
		         *     var hexString = CryptoJS.enc.Hex.stringify(wordArray);
		         */stringify:function(wordArray){// Shortcuts
var words=wordArray.words;var sigBytes=wordArray.sigBytes;// Convert
var hexChars=[];for(var i=0;i<sigBytes;i++){var bite=words[i>>>2]>>>24-i%4*8&0xff;hexChars.push((bite>>>4).toString(16));hexChars.push((bite&0x0f).toString(16));}return hexChars.join('');},/**
		         * Converts a hex string to a word array.
		         *
		         * @param {string} hexStr The hex string.
		         *
		         * @return {WordArray} The word array.
		         *
		         * @static
		         *
		         * @example
		         *
		         *     var wordArray = CryptoJS.enc.Hex.parse(hexString);
		         */parse:function(hexStr){// Shortcut
var hexStrLength=hexStr.length;// Convert
var words=[];for(var i=0;i<hexStrLength;i+=2){words[i>>>3]|=parseInt(hexStr.substr(i,2),16)<<24-i%8*4;}return new WordArray.init(words,hexStrLength/2);}};/**
		     * Latin1 encoding strategy.
		     */var Latin1=C_enc.Latin1={/**
		         * Converts a word array to a Latin1 string.
		         *
		         * @param {WordArray} wordArray The word array.
		         *
		         * @return {string} The Latin1 string.
		         *
		         * @static
		         *
		         * @example
		         *
		         *     var latin1String = CryptoJS.enc.Latin1.stringify(wordArray);
		         */stringify:function(wordArray){// Shortcuts
var words=wordArray.words;var sigBytes=wordArray.sigBytes;// Convert
var latin1Chars=[];for(var i=0;i<sigBytes;i++){var bite=words[i>>>2]>>>24-i%4*8&0xff;latin1Chars.push(String.fromCharCode(bite));}return latin1Chars.join('');},/**
		         * Converts a Latin1 string to a word array.
		         *
		         * @param {string} latin1Str The Latin1 string.
		         *
		         * @return {WordArray} The word array.
		         *
		         * @static
		         *
		         * @example
		         *
		         *     var wordArray = CryptoJS.enc.Latin1.parse(latin1String);
		         */parse:function(latin1Str){// Shortcut
var latin1StrLength=latin1Str.length;// Convert
var words=[];for(var i=0;i<latin1StrLength;i++){words[i>>>2]|=(latin1Str.charCodeAt(i)&0xff)<<24-i%4*8;}return new WordArray.init(words,latin1StrLength);}};/**
		     * UTF-8 encoding strategy.
		     */var Utf8=C_enc.Utf8={/**
		         * Converts a word array to a UTF-8 string.
		         *
		         * @param {WordArray} wordArray The word array.
		         *
		         * @return {string} The UTF-8 string.
		         *
		         * @static
		         *
		         * @example
		         *
		         *     var utf8String = CryptoJS.enc.Utf8.stringify(wordArray);
		         */stringify:function(wordArray){try{return decodeURIComponent(escape(Latin1.stringify(wordArray)));}catch(e){throw new Error('Malformed UTF-8 data');}},/**
		         * Converts a UTF-8 string to a word array.
		         *
		         * @param {string} utf8Str The UTF-8 string.
		         *
		         * @return {WordArray} The word array.
		         *
		         * @static
		         *
		         * @example
		         *
		         *     var wordArray = CryptoJS.enc.Utf8.parse(utf8String);
		         */parse:function(utf8Str){return Latin1.parse(unescape(encodeURIComponent(utf8Str)));}};/**
		     * Abstract buffered block algorithm template.
		     *
		     * The property blockSize must be implemented in a concrete subtype.
		     *
		     * @property {number} _minBufferSize The number of blocks that should be kept unprocessed in the buffer. Default: 0
		     */var BufferedBlockAlgorithm=C_lib.BufferedBlockAlgorithm=Base.extend({/**
		         * Resets this block algorithm's data buffer to its initial state.
		         *
		         * @example
		         *
		         *     bufferedBlockAlgorithm.reset();
		         */reset:function(){// Initial values
this._data=new WordArray.init();this._nDataBytes=0;},/**
		         * Adds new data to this block algorithm's buffer.
		         *
		         * @param {WordArray|string} data The data to append. Strings are converted to a WordArray using UTF-8.
		         *
		         * @example
		         *
		         *     bufferedBlockAlgorithm._append('data');
		         *     bufferedBlockAlgorithm._append(wordArray);
		         */_append:function(data){// Convert string to WordArray, else assume WordArray already
if(typeof data=='string'){data=Utf8.parse(data);}// Append
this._data.concat(data);this._nDataBytes+=data.sigBytes;},/**
		         * Processes available data blocks.
		         *
		         * This method invokes _doProcessBlock(offset), which must be implemented by a concrete subtype.
		         *
		         * @param {boolean} doFlush Whether all blocks and partial blocks should be processed.
		         *
		         * @return {WordArray} The processed data.
		         *
		         * @example
		         *
		         *     var processedData = bufferedBlockAlgorithm._process();
		         *     var processedData = bufferedBlockAlgorithm._process(!!'flush');
		         */_process:function(doFlush){var processedWords;// Shortcuts
var data=this._data;var dataWords=data.words;var dataSigBytes=data.sigBytes;var blockSize=this.blockSize;var blockSizeBytes=blockSize*4;// Count blocks ready
var nBlocksReady=dataSigBytes/blockSizeBytes;if(doFlush){// Round up to include partial blocks
nBlocksReady=Math.ceil(nBlocksReady);}else{// Round down to include only full blocks,
// less the number of blocks that must remain in the buffer
nBlocksReady=Math.max((nBlocksReady|0)-this._minBufferSize,0);}// Count words ready
var nWordsReady=nBlocksReady*blockSize;// Count bytes ready
var nBytesReady=Math.min(nWordsReady*4,dataSigBytes);// Process blocks
if(nWordsReady){for(var offset=0;offset<nWordsReady;offset+=blockSize){// Perform concrete-algorithm logic
this._doProcessBlock(dataWords,offset);}// Remove processed words
processedWords=dataWords.splice(0,nWordsReady);data.sigBytes-=nBytesReady;}// Return processed words
return new WordArray.init(processedWords,nBytesReady);},/**
		         * Creates a copy of this object.
		         *
		         * @return {Object} The clone.
		         *
		         * @example
		         *
		         *     var clone = bufferedBlockAlgorithm.clone();
		         */clone:function(){var clone=Base.clone.call(this);clone._data=this._data.clone();return clone;},_minBufferSize:0});/**
		     * Abstract hasher template.
		     *
		     * @property {number} blockSize The number of 32-bit words this hasher operates on. Default: 16 (512 bits)
		     */C_lib.Hasher=BufferedBlockAlgorithm.extend({/**
		         * Configuration options.
		         */cfg:Base.extend(),/**
		         * Initializes a newly created hasher.
		         *
		         * @param {Object} cfg (Optional) The configuration options to use for this hash computation.
		         *
		         * @example
		         *
		         *     var hasher = CryptoJS.algo.SHA256.create();
		         */init:function(cfg){// Apply config defaults
this.cfg=this.cfg.extend(cfg);// Set initial values
this.reset();},/**
		         * Resets this hasher to its initial state.
		         *
		         * @example
		         *
		         *     hasher.reset();
		         */reset:function(){// Reset data buffer
BufferedBlockAlgorithm.reset.call(this);// Perform concrete-hasher logic
this._doReset();},/**
		         * Updates this hasher with a message.
		         *
		         * @param {WordArray|string} messageUpdate The message to append.
		         *
		         * @return {Hasher} This hasher.
		         *
		         * @example
		         *
		         *     hasher.update('message');
		         *     hasher.update(wordArray);
		         */update:function(messageUpdate){// Append
this._append(messageUpdate);// Update the hash
this._process();// Chainable
return this;},/**
		         * Finalizes the hash computation.
		         * Note that the finalize operation is effectively a destructive, read-once operation.
		         *
		         * @param {WordArray|string} messageUpdate (Optional) A final message update.
		         *
		         * @return {WordArray} The hash.
		         *
		         * @example
		         *
		         *     var hash = hasher.finalize();
		         *     var hash = hasher.finalize('message');
		         *     var hash = hasher.finalize(wordArray);
		         */finalize:function(messageUpdate){// Final message update
if(messageUpdate){this._append(messageUpdate);}// Perform concrete-hasher logic
var hash=this._doFinalize();return hash;},blockSize:512/32,/**
		         * Creates a shortcut function to a hasher's object interface.
		         *
		         * @param {Hasher} hasher The hasher to create a helper for.
		         *
		         * @return {Function} The shortcut function.
		         *
		         * @static
		         *
		         * @example
		         *
		         *     var SHA256 = CryptoJS.lib.Hasher._createHelper(CryptoJS.algo.SHA256);
		         */_createHelper:function(hasher){return function(message,cfg){return new hasher.init(cfg).finalize(message);};},/**
		         * Creates a shortcut function to the HMAC's object interface.
		         *
		         * @param {Hasher} hasher The hasher to use in this HMAC helper.
		         *
		         * @return {Function} The shortcut function.
		         *
		         * @static
		         *
		         * @example
		         *
		         *     var HmacSHA256 = CryptoJS.lib.Hasher._createHmacHelper(CryptoJS.algo.SHA256);
		         */_createHmacHelper:function(hasher){return function(message,key){return new C_algo.HMAC.init(hasher,key).finalize(message);};}});/**
		     * Algorithm namespace.
		     */var C_algo=C.algo={};return C;}(Math);return CryptoJS;});})(core);(function(module,exports){(function(root,factory){{// CommonJS
module.exports=factory(core.exports);}})(commonjsGlobal,function(CryptoJS){(function(Math){// Shortcuts
var C=CryptoJS;var C_lib=C.lib;var WordArray=C_lib.WordArray;var Hasher=C_lib.Hasher;var C_algo=C.algo;// Initialization and round constants tables
var H=[];var K=[];// Compute constants
(function(){function isPrime(n){var sqrtN=Math.sqrt(n);for(var factor=2;factor<=sqrtN;factor++){if(!(n%factor)){return false;}}return true;}function getFractionalBits(n){return(n-(n|0))*0x100000000|0;}var n=2;var nPrime=0;while(nPrime<64){if(isPrime(n)){if(nPrime<8){H[nPrime]=getFractionalBits(Math.pow(n,1/2));}K[nPrime]=getFractionalBits(Math.pow(n,1/3));nPrime++;}n++;}})();// Reusable object
var W=[];/**
		     * SHA-256 hash algorithm.
		     */var SHA256=C_algo.SHA256=Hasher.extend({_doReset:function(){this._hash=new WordArray.init(H.slice(0));},_doProcessBlock:function(M,offset){// Shortcut
var H=this._hash.words;// Working variables
var a=H[0];var b=H[1];var c=H[2];var d=H[3];var e=H[4];var f=H[5];var g=H[6];var h=H[7];// Computation
for(var i=0;i<64;i++){if(i<16){W[i]=M[offset+i]|0;}else{var gamma0x=W[i-15];var gamma0=(gamma0x<<25|gamma0x>>>7)^(gamma0x<<14|gamma0x>>>18)^gamma0x>>>3;var gamma1x=W[i-2];var gamma1=(gamma1x<<15|gamma1x>>>17)^(gamma1x<<13|gamma1x>>>19)^gamma1x>>>10;W[i]=gamma0+W[i-7]+gamma1+W[i-16];}var ch=e&f^~e&g;var maj=a&b^a&c^b&c;var sigma0=(a<<30|a>>>2)^(a<<19|a>>>13)^(a<<10|a>>>22);var sigma1=(e<<26|e>>>6)^(e<<21|e>>>11)^(e<<7|e>>>25);var t1=h+sigma1+ch+K[i]+W[i];var t2=sigma0+maj;h=g;g=f;f=e;e=d+t1|0;d=c;c=b;b=a;a=t1+t2|0;}// Intermediate hash value
H[0]=H[0]+a|0;H[1]=H[1]+b|0;H[2]=H[2]+c|0;H[3]=H[3]+d|0;H[4]=H[4]+e|0;H[5]=H[5]+f|0;H[6]=H[6]+g|0;H[7]=H[7]+h|0;},_doFinalize:function(){// Shortcuts
var data=this._data;var dataWords=data.words;var nBitsTotal=this._nDataBytes*8;var nBitsLeft=data.sigBytes*8;// Add padding
dataWords[nBitsLeft>>>5]|=0x80<<24-nBitsLeft%32;dataWords[(nBitsLeft+64>>>9<<4)+14]=Math.floor(nBitsTotal/0x100000000);dataWords[(nBitsLeft+64>>>9<<4)+15]=nBitsTotal;data.sigBytes=dataWords.length*4;// Hash final blocks
this._process();// Return final computed hash
return this._hash;},clone:function(){var clone=Hasher.clone.call(this);clone._hash=this._hash.clone();return clone;}});/**
		     * Shortcut function to the hasher's object interface.
		     *
		     * @param {WordArray|string} message The message to hash.
		     *
		     * @return {WordArray} The hash.
		     *
		     * @static
		     *
		     * @example
		     *
		     *     var hash = CryptoJS.SHA256('message');
		     *     var hash = CryptoJS.SHA256(wordArray);
		     */C.SHA256=Hasher._createHelper(SHA256);/**
		     * Shortcut function to the HMAC's object interface.
		     *
		     * @param {WordArray|string} message The message to hash.
		     * @param {WordArray|string} key The secret key.
		     *
		     * @return {WordArray} The HMAC.
		     *
		     * @static
		     *
		     * @example
		     *
		     *     var hmac = CryptoJS.HmacSHA256(message, key);
		     */C.HmacSHA256=Hasher._createHmacHelper(SHA256);})(Math);return CryptoJS.SHA256;});})(sha256$1);var sha256=sha256$1.exports;function getCommonBlockData(element,assetId,subject,scanId){if(!element){return null;}if(!aeKernel.rootTestNode.contains(element)){// this element is not a descendant of our root test node
return null;}if(!element.hasAttribute("data-ae_uid")){// this element does not have a unique ID assigned to it yet
// make sure nextgen_runAllTests_returnInstances_JSON was run prior to invoking this function
return null;}const tagName=element.getAttribute("data-ae_origTagName")||element.tagName.toLowerCase();if(tagName==='body'||!element.closest('body')){// we don't calculate clustering data for body elements nor elements that are not descendants of body elements
return null;}const clusteringData=getClusteringData(element);let commonBlockKey;if(assetId){commonBlockKey=sha256(`${assetId} ${JSON.stringify(clusteringData,['tagName','children'],null)}`).toString();}let commonBlockInstanceKey;if(assetId&&subject&&scanId&&clusteringData.uniqueId){commonBlockInstanceKey=sha256(`${assetId} ${subject} ${scanId} ${clusteringData.uniqueId}`).toString();}return{clusteringData,commonBlockKey,commonBlockInstanceKey};}function getClusteringData(element){let parentClusteringData=arguments.length>1&&arguments[1]!==undefined?arguments[1]:null;const clusteringData={uniqueId:element.getAttribute("data-ae_uid"),tagName:element.getAttribute("data-ae_origTagName")||element.tagName.toLowerCase(),attributes:{},text:null,children:[]};// format attributes
const elementAttributes=element.attributes;for(let i=0;i<elementAttributes.length;i++){const attribute=elementAttributes[i];const{name,value}=attribute;if(name.startsWith("data-ae_")){// don't include attributes added by Access Engine
continue;}clusteringData.attributes[name]=value;}// format text
clusteringData.text=Array.prototype.filter.call(element.childNodes,child=>child.nodeType===Node.TEXT_NODE).map(child=>child.textContent).join('');if(parentClusteringData){parentClusteringData.children.push(clusteringData);}// recursively format the element's children
let childElement=element.firstElementChild;do{if(!childElement){break;}getClusteringData(childElement,clusteringData);}while(childElement=childElement.nextElementSibling);return clusteringData;}/**
	 * Public interface to the Access Engine Kernel object, to enable settings to be changed
	 */function getRootTestNode(){if(aeKernel.rootTestNode){aeKernel.rootTestNode.getCommonBlockData=getCommonBlockData;}return aeKernel.rootTestNode;}/**
	 * Returns variant 1 test result (na | pass | fail) for a specified test
	 * For use in BDD testing
	 * @param {string} testId The specified test id
	 * @returns {string} The outcome of the specified test
	 */function runTest_returnOutcome(testId){aeKernel.markdownFuncs_failedToRun={};// for resilience
aeKernel.tests_failedToRun={};// for resilience
var result="failed to run";if(aeKernel.captureDOM()){try{var testFunc=aeKernel.getTestToRun(testId).testFunc();// added in break-up
testFunc.rootTestNode=aeKernel.rootTestNode;var testRes=testFunc.getResult();result=testRes.outcome;}catch(err){logger.error('Error',err);aeKernel.tests_failedToRun[testId]=err;}}return result;}/**
	 * Returns variant 2 test result for a specified test
	 * @param {string} testId The specified test id
	 * @returns {string} The outcome of the specified test in terms of "bp id, number of issues located"
	 */function runTest_returnNumbers(testId){aeKernel.markdownFuncs_failedToRun={};// for resilience
aeKernel.tests_failedToRun={};// for resilience
var result="failed to run";if(aeKernel.captureDOM()){try{var outcome=[];var testFunc=aeKernel.getTestToRun(testId).testFunc();// added in break-up
testFunc.rootTestNode=aeKernel.rootTestNode;var testRes=testFunc.getResult("numbers");if(testRes.outcome==="fail"){outcome[0]=testId;outcome[1]=testRes.numberOfIssues;}result=outcome.toString();}catch(err){aeKernel.tests_failedToRun[testId]=err;}}return result;}/**
	 * Returns variant 3 test result for a specified test
	 * @param {string} testId The specified test id
	 * @returns {Array} The outcome of the specified test in terms of "bp id, [array of failing node instances]"
	 */function runTest_returnInstances(testId){aeKernel.markdownFuncs_failedToRun={};// for resilience
aeKernel.tests_failedToRun={};// for resilience
var result=["failed to run"];if(aeKernel.captureDOM()){try{var outcome=null;var testFunc=aeKernel.getTestToRun(testId).testFunc();// added in break-up
testFunc.rootTestNode=aeKernel.rootTestNode;var testRes=testFunc.getResult("instances");if(testRes.outcome==="fail"){outcome=testRes;}result=outcome;}catch(err){logger.error('Error',err);aeKernel.tests_failedToRun[testId]=err;}}return result;}/**
	 * Returns variant 3 test result for all tests
	 * @param {Number} testType
	 * @returns {Object} The outcomes object holds results for each "fail" test that has instances in terms of "bp id, [array of failing node instances]", with test name as a key
	 */function runAllTests_returnInstances(){let testType=arguments.length>0&&arguments[0]!==undefined?arguments[0]:testTypes.AUTOMATIC;// need getters and setters
aeKernel.markdownFuncs_failedToRun={};aeKernel.tests_failedToRun={};var outcomes={};if(aeKernel.captureDOM()){var tests=aeKernel.getTestsToRun(testType);var keys=Object.keys(tests);for(var i=0,len_keys=keys.length;i<len_keys;i++){var outcome=[];var key=keys[i];var test=tests[key];//This is a web API so only run tests of type WEB
if(test.mediaType==mediaTypes.WEB){try{var testFunc=test.testFunc();// needed to add this
testFunc.rootTestNode=aeKernel.rootTestNode;var testRes=testFunc.getResult("instances");if(testRes.outcome==="fail"){outcome[0]=test.testId;outcome[1]=testRes.instances;outcomes[key]=outcome;}}catch(err){logger.error('Error',err);aeKernel.tests_failedToRun[key]=err;}}}}return outcomes;}/**
	 * Returns variant 1 test result (na | pass | fail) for a specified test
	 * For use in BDD testing
	 * @param {string} testType
	 * @returns {string} The outcome of the specified test
	 */function runAllTests_returnFailedTests(){let testType=arguments.length>0&&arguments[0]!==undefined?arguments[0]:testTypes.AUTOMATIC;aeKernel.markdownFuncs_failedToRun={};// for resilience
aeKernel.tests_failedToRun={};// for resilience
var outcomes=[];if(aeKernel.captureDOM()){var tests=aeKernel.getTestsToRun(testType);var keys=Object.keys(tests);for(var i=0,len_keys=keys.length;i<len_keys;i++){var key=keys[i];var test=tests[key];//This is a web API so only run tests of type WEB
if(test.mediaType==mediaTypes.WEB){try{var testFunc=test.testFunc();// added in break-up
testFunc.rootTestNode=aeKernel.rootTestNode;var testRes=testFunc.getResult("instances");if(testRes.outcome==="fail"){var len=outcomes.length;outcomes[len]=key;}}catch(err){logger.error('Error',err);aeKernel.tests_failedToRun[key]=err;}}}}return outcomes;}/**
	 * Returns variant 2 test result for all failing tests
	 * For use in analytics
	 * @param {Number} testType
	 * @returns {Object} The outcomes object holds results for each test that has one or more failing instances in terms of "bp id, number of issues", with check name as a key
	 */function runAllTests_returnFailedNumbers(){let testType=arguments.length>0&&arguments[0]!==undefined?arguments[0]:testTypes.AUTOMATIC;aeKernel.markdownFuncs_failedToRun={};// for resilience
aeKernel.tests_failedToRun={};// for resilience
var outcomes={};if(aeKernel.captureDOM()){var tests=aeKernel.getTestsToRun(testType);var keys=Object.keys(tests);for(var i=0,len_keys=keys.length;i<len_keys;i++){var key=keys[i];var test=tests[key];//This is a web API so only run tests of type WEB
if(test.mediaType==mediaTypes.WEB){try{var testFunc=test.testFunc();// needed to add this
testFunc.rootTestNode=aeKernel.rootTestNode;var testRes=testFunc.getResult("numbers");if(testRes.outcome==="fail"){outcomes[key]=testRes.numberOfIssues;}}catch(err){aeKernel.tests_failedToRun[key]=err;}}}}return outcomes;}/**
	 * Returns variant 3 test result for all tests
	 * @param {Number} testType
	 * @param {function} processTest
	 * @param {function} callback
	 * @returns {Object} The outcomes object holds results for each "fail" test that has instances in terms of "bp id, [array of failing node instances]", with test name as a key
	 */function timer_api_shell(testType,processTest,callback){function runTestsWithTimers(){// deal with tests
// extract tests in batches of 8
const tests=aeKernel.getTestsToRun(testType);const testIds=Object.keys(tests);const testsLength=testIds.length;let batchOne_iterations=testsLength%8;let remainingIterations=Math.floor(testsLength/8);let i=testsLength-1;function timer_tests(){// deal with remainders first e.g. if number of tests is not perfectly divisible by 8
if(batchOne_iterations>0){while(batchOne_iterations){processTest(tests[testIds[i--]]);batchOne_iterations--;}setTimeout(timer_tests,25);}// run remaining tests in batches of 8
if(batchOne_iterations===0&&remainingIterations>0){processTest(tests[testIds[i--]]);processTest(tests[testIds[i--]]);processTest(tests[testIds[i--]]);processTest(tests[testIds[i--]]);processTest(tests[testIds[i--]]);processTest(tests[testIds[i--]]);processTest(tests[testIds[i--]]);processTest(tests[testIds[i--]]);remainingIterations--;if(remainingIterations>0){setTimeout(timer_tests,25);}else{aeKernel.updateSuccess();if(callback){callback();}}}}if(batchOne_iterations>0&&remainingIterations===0){// there are less than 8 tests
while(batchOne_iterations){processTest(tests[testIds[i--]]);batchOne_iterations--;}aeKernel.updateSuccess();if(callback){callback();}}else{// there are more than 8 tests
setTimeout(timer_tests,25);}}// proxy functions needed otherwise object variables become divorced 
function stage0_capture_proxy(){return aeKernel.stage0_capture();}function stage1_capture_proxy(){return aeKernel.stage1_capture();}function stage2_capture_proxy(){return aeKernel.stage2_capture();}function stage3a_capture_proxy(){return aeKernel.stage3a_capture();}function stage3c_capture_proxy(){return aeKernel.stage3c_capture();}function stage3d_capture_proxy(){return aeKernel.stage3d_capture();}function stage3_capture_proxy(){return aeKernel.stage3_capture();}function stage4_capture_proxy(){return aeKernel.stage4_capture();}function stage5_capture_proxy(){return aeKernel.stage5_capture();}function stage6_capture_proxy(){return aeKernel.stage6_capture();}aeKernel.updateSuccess(null);const tasks=[stage0_capture_proxy,stage1_capture_proxy,stage2_capture_proxy,stage3a_capture_proxy,stage3c_capture_proxy,stage3d_capture_proxy,stage3_capture_proxy,stage4_capture_proxy,stage5_capture_proxy,stage6_capture_proxy,runTestsWithTimers];// deal with tasks
function timer_tasks(){try{const task=tasks.shift();if(task()&&tasks.length>0){setTimeout(timer_tasks,25);}}finally{if(aeKernel.isLiveDOMMarkedUp){stage3_capture_proxy();}}}setTimeout(timer_tasks,25);return 1;}/**
	 * We need to expose test_outcomes through the api, but not by importing and exposing aeKernel
	 * Hence this step
	 */exports.test_outcomes=aeKernel.test_outcomes;/**
	 * Returns variant 3 test result for all tests
	 * @param {Number} testType
	 * @param {function} callback
	 * @returns {Object} The outcomes object holds results for each "fail" test that has instances in terms of "bp id, [array of failing node instances]", with test name as a key
	 */function runAllTests_returnFailedNumbers_timers(){let testType=arguments.length>0&&arguments[0]!==undefined?arguments[0]:testTypes.AUTOMATIC;let callback=arguments.length>1?arguments[1]:undefined;aeKernel.markdownFuncs_failedToRun={};// for resilience
aeKernel.tests_failedToRun={};// for resilience
exports.test_outcomes={};var processTest=function(test){//This is a web API so only run tests of type WEB
if(test.mediaType==mediaTypes.WEB){var testId=test.testId;try{var testFunc=test.testFunc();// needed to add this
testFunc.rootTestNode=aeKernel.rootTestNode;var testRes=testFunc.getResult("numbers");if(testRes.outcome==="fail"){exports.test_outcomes[testId]=testRes.numberOfIssues;}}catch(err){aeKernel.tests_failedToRun[testId]=err;// for resilience
}}};timer_api_shell(testType,processTest,callback);}/**
	 * Returns variant 3 test result for all tests - tests are run asyncronously using timers so they are non-blocking
	 * @param {Number} testType
	 * @param {function} callback
	 * @returns {Object} The outcomes object holds results for each "fail" test that has instances in terms of "bp id, [array of failing node instances]", with test name as a key
	 */function runAllTests_returnInstances_timers(){let testType=arguments.length>0&&arguments[0]!==undefined?arguments[0]:testTypes.AUTOMATIC;let callback=arguments.length>1?arguments[1]:undefined;aeKernel.markdownFuncs_failedToRun={};// for resilience
aeKernel.tests_failedToRun={};// for resilience
exports.test_outcomes={};var processTest=function(test){//This is a web API so only run tests of type WEB
if(test.mediaType==mediaTypes.WEB){var outcome=[];var testId=test.testId;try{var testFunc=test.testFunc();// needed to add this
testFunc.rootTestNode=aeKernel.rootTestNode;var testRes=testFunc.getResult("instances");if(testRes.outcome==="fail"){var len=outcome.length;outcome[len]=testId;outcome[len+1]=testRes.instances;exports.test_outcomes[testId]=outcome;}}catch(err){aeKernel.tests_failedToRun[testId]=err;// for resilience
}}};timer_api_shell(testType,processTest,callback);}function test_stages(stageNumber){aeKernel.markdownFuncs_failedToRun=[];// for resilience
aeKernel.tests_failedToRun=[];// for resilience
function stage0_capture_proxy(){return aeKernel.stage0_capture();}function stage1_capture_proxy(){return aeKernel.stage1_capture();}function stage2_capture_proxy(){return aeKernel.stage2_capture();}function stage3a_capture_proxy(){return aeKernel.stage3a_capture();}function stage3c_capture_proxy(){return aeKernel.stage3c_capture();}function stage3d_capture_proxy(){return aeKernel.stage3d_capture();}function stage3_capture_proxy(){return aeKernel.stage3_capture();}function stage4_capture_proxy(){return aeKernel.stage4_capture();}function stage5_capture_proxy(){return aeKernel.stage5_capture();}function stage6_capture_proxy(){return aeKernel.stage6_capture();}aeKernel.updateSuccess(null);const tasks=[];switch(stageNumber){case"stage0":tasks.push(stage0_capture_proxy);break;case"stage1":tasks.push(stage0_capture_proxy,stage1_capture_proxy);break;case"stage2":tasks.push(stage0_capture_proxy,stage1_capture_proxy,stage2_capture_proxy);break;case"stage3a":tasks.push(stage0_capture_proxy,stage1_capture_proxy,stage2_capture_proxy,stage3a_capture_proxy);break;case"stage3c":tasks.push(stage0_capture_proxy,stage1_capture_proxy,stage2_capture_proxy,stage3a_capture_proxy,stage3c_capture_proxy);break;case"stage3d":tasks.push(stage0_capture_proxy,stage1_capture_proxy,stage2_capture_proxy,stage3a_capture_proxy,stage3c_capture_proxy,stage3d_capture_proxy);break;case"stage3":tasks.push(stage0_capture_proxy,stage1_capture_proxy,stage2_capture_proxy,stage3a_capture_proxy,stage3c_capture_proxy,stage3d_capture_proxy,stage3_capture_proxy);break;case"stage4":tasks.push(stage0_capture_proxy,stage1_capture_proxy,stage2_capture_proxy,stage3a_capture_proxy,stage3c_capture_proxy,stage3d_capture_proxy,stage3_capture_proxy,stage4_capture_proxy);break;case"stage5":tasks.push(stage0_capture_proxy,stage1_capture_proxy,stage2_capture_proxy,stage3a_capture_proxy,stage3c_capture_proxy,stage3d_capture_proxy,stage3_capture_proxy,stage4_capture_proxy,stage5_capture_proxy);break;case"stage6":tasks.push(stage0_capture_proxy,stage1_capture_proxy,stage2_capture_proxy,stage3a_capture_proxy,stage3c_capture_proxy,stage3d_capture_proxy,stage3_capture_proxy,stage4_capture_proxy,stage5_capture_proxy,stage6_capture_proxy);break;}try{for(let i=0;i<tasks.length;i++){const task=tasks[i];if(!task()){return null;}}}finally{if(aeKernel.isLiveDOMMarkedUp){stage3_capture_proxy();}}aeKernel.updateSuccess();return aeKernel.rootTestNode;}/**
	 *
	 * @param {Node} el
	 * @param {Number} fixType
	 * @param {boolean} domSpec
	 * @param {Number} version
	 * @returns {{css:string, attNo: number, encoding: array, url: string, version: string, wordArray: array}}
	 */function getFingerprint(el,fixType,domSpec,version){var fingerprint={"version":version||"1"};if(domSpec===true||fixType===fixTypes.TAG){fingerprint.url=window.location.href;}fingerprint.css=getCssForFingerprint(el);fingerprint.attNo=el.attributes.length;fingerprint.encoding=charFreq(el.innerHTML);if(fingerprint.version==="1"){var innerHTML=el.tagName.toLowerCase()==='html'?'':el.innerHTML;fingerprint.wordArray=toWordsArray(innerHTML);}return fingerprint;}/**
	 *
	 * @param {String} val
	 * @returns {String[]}
	 */function toWordsArray(val){if(val===''){return[];}val=val.replace(/[^a-zA-Z]/g,' ');val=val.replace(/\s\s+/g,' ');return val.trim().split(' ');}function getCssForFingerprint(element){const characterBudget=2000;const qualifyingAttributes=[];let numCharactersInQualifyingAttributes=0;const attributes=element.attributes;for(let i=attributes.length;i--;){const attribute=attributes[i];const shouldAppendAttribute=// ignore the style attribute as it can be altered without changing the accessibility components of an element
attribute.name!=='style'&&// ignore Engine attributes
attribute.name.indexOf('data-ae_')===-1&&// ignore AST attributes
attribute.name.indexOf('data-la-')===-1&&// ignore attributes with a name that contains invalid characters relative to JavaScript's querySelector method
attribute.name.indexOf(':')===-1&&// ignore attributes with a value that contains CR or LF characters
attribute.value.indexOf('\r')===-1&&attribute.value.indexOf('\n')===-1&&// ignore 'xmlns' attributes; JavaScript's querySelector method doesn't consistently handle this attribute well
attribute.name!=='xmlns';if(shouldAppendAttribute){const qualifyingAttribute={name:attribute.name,value:attribute.value.replace(/"/g,'\\"')};numCharactersInQualifyingAttributes+=qualifyingAttribute.name.length+qualifyingAttribute.value.length;qualifyingAttributes.push(qualifyingAttribute);}}let css=element.tagName.toLowerCase();if(numCharactersInQualifyingAttributes<=characterBudget){css=qualifyingAttributes.reduce((accumulator,attribute)=>`${accumulator}[${attribute.name}="${attribute.value}"]`,css);}else{// not all qualifying attributes will fit into our CSS selector given our character budget, so we need to exclude some
// sort attributes by name + value, shortest first;
// we want to prioritize inclusion of shorter attributes in our CSS selector over longer ones
qualifyingAttributes.sort((a,b)=>a.name.length+a.value.length-(b.name.length+b.value.length));let characterBudgetRemaining=characterBudget;for(let i=0;i<qualifyingAttributes.length;i++){const attribute=qualifyingAttributes[i];const attributeLength=attribute.name.length+attribute.value.length;if(characterBudgetRemaining<attributeLength){// there's no more room for any more of our attributes
break;}css+=`[${attribute.name}="${attribute.value}"]`;characterBudgetRemaining-=attributeLength;}}return css;}function toPercentage(number,total){return total!==0?parseInt((number/total*100).toFixed(0)):0;}function charFreq(text){//If the node is empty return an array of zeros
if(text.trim()===""){var arr=[];for(var d=38;d--;){arr.push(0);}return arr;}var freq={"a":0,"b":0,"c":0,"d":0,"e":0,"f":0,"g":0,"h":0,"i":0,"j":0,"k":0,"l":0,"m":0,"n":0,"o":0,"p":0,"q":0,"r":0,"s":0,"t":0,"u":0,"v":0,"w":0,"x":0,"y":0,"z":0,"0":0,"1":0,"2":0,"3":0,"4":0,"5":0,"6":0,"7":0,"8":0,"9":0,"<":0,">":0};var charArray=text.toLowerCase().split('');for(var i=0,len=charArray.length;i<len;i++){var char=charArray[i];try{if(typeof freq[char]!=="undefined"){freq[char]=freq[char]+(i+1);// simple change from + 1 to plus i - to add position.  Additional + 1 corrects array pos to actual pos.
}}catch(err){//do nothing}
}}var keys=Object.keys(freq);var total=0;for(var k=keys.length;k--;){total=total+freq[keys[k]];}var onehotencoding=[];for(var a=0,len_keys=keys.length;a<len_keys;a++){onehotencoding[a]=toPercentage(freq[keys[a]],total);}return onehotencoding;}/**
	 * AMP data model
	 * 0 instance_id - bind later
	 * 1 module_id - bind later
	 * 2 violation_id - bestPractice
	 * 3 line_number - cannot supply
	 * 4 element - outerHTML up to 3000 characters
	 * 5 attribute - ??? Access Engine Test ID: initially plain English of Access Engine Test. getRichData() {} key id returns functions for getting text?
	 * 6 test_result - VIOLATION_FAILED = 0; VIOLATION_PASSED = 1; VIOLATION_NOT_APPLICABLE = 2
	 * 7 css cssSelector
	 * @param instance
	 * @param test
	 * @param type
	 * @param {Number} version
	 * @returns {{}}
	 */function formatOutcomeForToolbar(instance,test,type,version){let outcome={};//If version not specified set it to latest one, currently 2.
let ver=version||2;outcome.engineTestId=test.testId;outcome.bestPracticeId=test.bestPractice;let note=test.metaText;if(note!==undefined){outcome.attribute=note;}else{outcome.attribute="";}let detail=test.metaTextDetail;//If there isn't a detailed attribute then use the static one
if(detail==undefined){outcome.attributeDetail=outcome.attribute;}else{//If detail contains tokens, replace them with actual values
// the replacement is done before we remove ae or la attributes - see removeAEAttributes below.
if(detail.indexOf("{{")!==-1){// {{tag-name}} is replaced with the tag name
detail=detail.replace("{{tag-name}}",instance.tagName);// {{optional_data-ae_ar}} is replaced with the aria role and some prefixing text, if applicable, otherwise it's replaced with an empty string
detail=detail.replace("{{optional_data-ae_ar}}",instance.getAttribute("data-ae_ar")!=="null"?` with a role of ${instance.getAttribute("data-ae_ar")}`:"");// {{attribute-name}} is replaced with the attribute-name value e.g. {{alt}} would be replaced with alt value
var pattern=/({{([^}}]+)}})/gi;var matches=detail.match(pattern);if(matches!==null){for(var i=0;i<matches.length;i++){var attributeNameToken=matches[i];var attributeName=attributeNameToken.replace("{{","").replace("}}","");var attributeValue=instance.getAttribute(attributeName);if(attributeValue!==null){if(attributeName==="data-ae_color"||attributeName==="data-ae_bgcolor"){// replace RGB color values with hex color values
attributeValue=`${rgbToHex(attributeValue)} ${attributeValue}`;}detail=detail.replace(attributeNameToken,attributeValue);}}}}outcome.attributeDetail=detail.substring(0,2998);}let element=removeAEAttributes(instance);outcome.element=element.outerHTML.substring(0,2998);// data-base takes string to 3000
switch(type){case testTypes.GUIDED_AUTOMATIC:outcome.testResult=3;break;case testTypes.AUTOMATIC:case testTypes.ADVISORY:case testTypes.MANUAL:case testTypes.PREVIEW_MODE:default:// Everything besides GA should be considered a failure
outcome.testResult=0;break;}// outcome.path = instance.getAttribute("data-ae_uel");
// change due to building in support for web components
outcome.path=instance.getAttribute("data-ae_domuel");if(outcome.path===null){outcome.path=instance.getAttribute("data-ae_uel");}// handles fixes
let fixType=test.fixType;if(fixType!==undefined){outcome.fixType=fixType;//Special handling for fixType 4
if(fixType.fixType==4){//No need for encoding or attNo in this case, always runs against html of whole page
outcome.fingerprint={attNo:0,css:"html",version:2,encoding:[1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1]};}else{outcome.fingerprint=getFingerprint(element,fixType.fixType!==undefined?fixType.fixType:-1,fixType.domSpec!==undefined?fixType.domSpec:false,ver);}}else{outcome.fixType="";}return outcome;}/**
	 *
	 */let resultsAsJSON={/**
		 *
		 * @param {String} text
		 * @param {Function} reviver {optional}
		 * @returns {*}
		 */parse:function(text,reviver){if(typeof text==='string'&&typeof text.evalJSON==='function'){return text.evalJSON();}return JSON.parseJSON(text,reviver);},/**
		 *
		 * @param {Object} value The value to convert to a JSON string.
		 * @param {Function} replacer {optional}
		 * @param {String|Number} space {optional}
		 * @returns {*|string}
		 */stringify:function(value,replacer,space){if(typeof value!=='undefined'&&typeof value.toJSON==='function'){return value.toJSON();}return JSON.stringify(value,replacer,space);}};/**
	 * Returns variant 3 test result for failed tests - as a JSON object
	 * @param {Number} testType
	 * @returns {Object} The outcomes object holds results for each "fail" in accordance with AMP data model
	 */function runAllTests_returnInstances_JSON(){let testType=arguments.length>0&&arguments[0]!==undefined?arguments[0]:testTypes.AUTOMATIC;aeKernel.markdownFuncs_failedToRun={};// for resilience
aeKernel.tests_failedToRun={};// for resilience
var outcomes=[];if(aeKernel.captureDOM()){var tests=aeKernel.getTestsToRun(testType);var keys=Object.keys(tests);for(var i=0,len_keys=keys.length;i<len_keys;i++){var key=keys[i];var test=tests[key];//This is a web API so only run tests of type 1
if(test.mediaType==mediaTypes.WEB){try{var testFunc=test.testFunc();// added in break-up
testFunc.rootTestNode=aeKernel.rootTestNode;var testRes=testFunc.getResult("instances");if(testRes.outcome==="fail"){var instances=testRes.instances;for(var j=0,len_instances=instances.length;j<len_instances;j++){var outcome=formatOutcomeForToolbar(instances[j],test,testType);outcomes.push(outcome);}}}catch(err){aeKernel.tests_failedToRun[key]=err;logger.error('Error',err);}}}}return resultsAsJSON.stringify(outcomes);}/**
	 * Returns variant 3 test result for failed tests - as a JSON object
	 * @param {Node} node to test
	 * @param {Number} testType to use for gathering tests
	 * @returns {Object} The outcomes object holds results for each "fail" in accordance with AMP data model
	 */function runAllTests_returnInstances_JSON_NodeCapture(node){let testType=arguments.length>1&&arguments[1]!==undefined?arguments[1]:testTypes.AUTOMATIC;testType=testType||testTypes.AUTOMATIC;aeKernel.markdownFuncs_failedToRun={};// for resilience
aeKernel.tests_failedToRun={};// for resilience
var outcomes=[];if(aeKernel.captureDOM()){// css selectors don't work on :root e.g. an A element placed as rootTestNode would not be able to be located.
// This means we have to get the parent element; then remove all other contents apart from captured node
// The selector done in this manner works in environments that have non-html tags e.g. <tile:importantattribute
var nodeCapture_UEL=uelAccurate_FromRoot(node);// it is important to use the quote types as shown to avoid syntax errors e.g. [data-ae_uel='[id='main']>*:nth-child(2)']
// IMPORTANT: (need to add to test writing doc) we have to use [data-ae_uel=] format in selectors, as nodeCapture will change the DOM structure and stop the ids from working directly
var cssSelectorForCapturedNode='[data-ae_uel="'+nodeCapture_UEL.replace(/"/g,'\\"')+'"]';var capturedNode=aeKernel.rootTestNode.querySelector(cssSelectorForCapturedNode);var parentElementOfCapturedNode=getParent(capturedNode);var children=parentElementOfCapturedNode.children;for(var j=children.length;j--;){if(children[j]!==capturedNode){parentElementOfCapturedNode.removeChild(children[j]);}}aeKernel.rootTestNode=parentElementOfCapturedNode;var tests=aeKernel.getTestsToRun(testType);var keys=Object.keys(tests);for(var i=0,len_keys=keys.length;i<len_keys;i++){var key=keys[i];var test=tests[key];//This is a web API so only run tests of type WEB
if(test.mediaType==mediaTypes.WEB){try{var testFunc=test.testFunc();// added in break-up
testFunc.rootTestNode=aeKernel.rootTestNode;var testRes=testFunc.getResult("instances");if(testRes.outcome==="fail"){var instances=testRes.instances;for(var j=0,len_instances=instances.length;j<len_instances;j++){var outcome=formatOutcomeForToolbar(instances[j],test,testType);outcomes.push(outcome);}}}catch(err){aeKernel.tests_failedToRun[key]=err;}}}}return resultsAsJSON.stringify(outcomes);}/**
	 * Public interface to the Access Engine Kernel object, to enable markdownFuncs_failedToRun to be collected
	 */function getMarkdownFuncs_failedToRun(){return aeKernel.markdownFuncs_failedToRun;}/**
	 * Public interface to the Access Engine Kernel object, to enable tests_failedToRun to be collected
	 */function getTests_failedToRun(){return aeKernel.tests_failedToRun;}/**
	 * Returns JSON stringified array of nmr tests with content to check
	 * For use with ACCESS Assistant
	 * @returns {Array} of nmr tests with content to check
	 */function ast_runPreviewMode_returnInstances_JSON(testId){let testType=arguments.length>1&&arguments[1]!==undefined?arguments[1]:testTypes.PREVIEW_MODE;aeKernel.markdownFuncs_failedToRun={};// for resilience
aeKernel.tests_failedToRun={};// for resilience
var outcomes=[];if(aeKernel.captureDOM()){try{var test=aeKernel.getTestToRun(testId);var instances=test.testFunc();for(var j=0,len_instances=instances.length;j<len_instances;j++){var instance=instances[j];var outcome=formatOutcomeForToolbar(instance,test,testType);outcomes.push(outcome);}}catch(err){logger.error('Error',err);aeKernel.tests_failedToRun[testId]=err;}}return JSON.stringify(outcomes);}/**
	 * Returns variant 3 test result for failed tests - as a JSON object
	 * @param {Array} testTypeArray
	 * @returns {Object} The outcomes object holds results for each "fail" in accordance with AMP data model
	 */function ast_runAllTests_returnInstances_JSON(){let testTypeArray=arguments.length>0&&arguments[0]!==undefined?arguments[0]:[testTypes.AUTOMATIC,testTypes.GUIDED_AUTOMATIC];aeKernel.markdownFuncs_failedToRun={};// for resilience
aeKernel.tests_failedToRun={};// for resilience
var outcomes=[];if(aeKernel.captureDOM()){var tests=null;// runs through each test type specified - for toolbar [testTypes.AUTOMATIC, testTypes.GUIDED_AUTOMATIC]
for(var k=testTypeArray.length;k--;){let testType=testTypeArray[k];tests=aeKernel.getTestsToRun(testType);var keys=Object.keys(tests);for(var i=0,len_keys=keys.length;i<len_keys;i++){var key=keys[i];var test=tests[key];//This is a web API so only run tests of type WEB
if(test.mediaType==mediaTypes.WEB){try{var testFunc=test.testFunc();// added in break-up
testFunc.rootTestNode=aeKernel.rootTestNode;var testRes=testFunc.getResult("instances");if(testRes.outcome==="fail"){var instances=testRes.instances;for(var j=0,len_instances=instances.length;j<len_instances;j++){try{var outcome=formatOutcomeForToolbar(instances[j],test,testType);outcomes.push(outcome);}catch(err){logger.error(err);}}}}catch(err){aeKernel.tests_failedToRun[key]=err;}}}}}return resultsAsJSON.stringify(outcomes);}/**
	 * Returns variant 3 test result for failed tests - as a JSON object
	 * @param {Element} node to test
	 * @param {Array} testTypeArray to use for gathering tests
	 * @returns {Object} The outcomes object holds results for each "fail" in accordance with AMP data model
	 */function ast_runAllTests_returnInstances_JSON_NodeCapture(node){let testTypeArray=arguments.length>1&&arguments[1]!==undefined?arguments[1]:[testTypes.AUTOMATIC,testTypes.GUIDED_AUTOMATIC];// testType = testType || testTypes.AUTOMATIC;
aeKernel.markdownFuncs_failedToRun={};// for resilience
aeKernel.tests_failedToRun={};// for resilience
var outcomes=[];if(aeKernel.captureDOM()){// css selectors don't work on :root e.g. an A element placed as rootTestNode would not be able to be located.
// This means we have to get the parent element; then remove all other contents apart from captured node
// The selector done in this manner works in environments that have non-html tags e.g. <tile:importantattribute
var cssSelectorForCapturedNode="";if(node.hasAttribute("data-ae_domuel")){var domuel=node.getAttribute("data-ae_domuel");var cssSelectorForCapturedNode='[data-ae_domuel="'+domuel.replace(/"/g,'\\"')+'"]';}else{var nodeCapture_UEL=uelAccurate_FromRoot(node);var cssSelectorForCapturedNode='[data-ae_uel="'+nodeCapture_UEL.replace(/"/g,'\\"')+'"]';}// it is important to use the quote types as shown to avoid syntax errors e.g. [data-ae_uel='[id='main']>*:nth-child(2)']
// IMPORTANT: (need to add to test writing doc) we have to use [data-ae_uel=] format in selectors, as nodeCapture will change the DOM structure and stop the ids from working directly
var capturedNode=aeKernel.rootTestNode.querySelector(cssSelectorForCapturedNode);var parentElementOfCapturedNode=getParent(capturedNode);var children=parentElementOfCapturedNode.children;for(var j=children.length;j--;){if(children[j]!==capturedNode){parentElementOfCapturedNode.removeChild(children[j]);}}aeKernel.rootTestNode=parentElementOfCapturedNode;// runs through each test type specified - for toolbar [testTypes.AUTOMATIC, testTypes.GUIDED_AUTOMATIC]
for(var k=testTypeArray.length;k--;){let testType=testTypeArray[k];var tests=aeKernel.getTestsToRun(testType);var keys=Object.keys(tests);for(var i=0,len_keys=keys.length;i<len_keys;i++){var key=keys[i];var test=tests[key];//This is a web API so only run tests of type WEB
if(test.mediaType==mediaTypes.WEB){try{var testFunc=test.testFunc();// added in break-up
testFunc.rootTestNode=aeKernel.rootTestNode;var testRes=testFunc.getResult("instances");if(testRes.outcome==="fail"){var instances=testRes.instances;for(var j=0,len_instances=instances.length;j<len_instances;j++){var outcome=formatOutcomeForToolbar(instances[j],test,testType);outcomes.push(outcome);}}}catch(err){aeKernel.tests_failedToRun[key]=err;}}}}}return resultsAsJSON.stringify(outcomes);}/**
	 * Returns variant 3 test result for failed tests - as a JSON object
	 * @param {Node} node to test - this should only be web components which are in the DOM (e.g. on the top level = BODY Web Component)
	 * @returns {Node} The node (representing the web component) with its expanded - properly formed - content
	 */function ast_runMarkDown_GetExpandedShadDOM_ForNode(node){// testType = testType || testTypes.AUTOMATIC;
aeKernel.markdownFuncs_failedToRun=[];// for resilience
aeKernel.tests_failedToRun=[];// for resilience
// build the Virtualised DOM
if(aeKernel.captureDOM()){// this should ideally be a stripped version of captureDOM, but the additional overhead in the toolbar context is small.  Also, maintainability is is improved, and code reduced.
// css selectors don't work on :root e.g. an A element placed as rootTestNode would not be able to be located.
// This means we have to get the parent element; then remove all other contents apart from captured node
// The selector done in this manner works in environments that have non-html tags e.g. <tile:importantattribute
var nodeCapture_DOMUEL=uelAccurate_FromRoot(node);// it is important to use the quote types as shown to avoid syntax errors e.g. [data-ae_uel='[id='main']>*:nth-child(2)']
// IMPORTANT: (need to add to test writing doc) we have to use [data-ae_uel=] format in selectors, as nodeCapture will change the DOM structure and stop the ids from working directly
var cssSelectorForCapturedNode='[data-ae_domuel="'+nodeCapture_DOMUEL.replace(/"/g,'\\"')+'"]';return aeKernel.rootTestNode.querySelector(cssSelectorForCapturedNode);}else{return null;}}/**
	 * Gets the liveDOM element using the locator - supporting Web Components
	 * @param {*} locator
	 */function ast_GetLiveDOMElementFromDOMUEL(locator){return getElementFromOrigShadowRoot(locator,aeKernel.document);}function ast_getTestById(testId){return aeKernel.getTestToRun(testId);}/**
	 * Returns JSON stringified array of nmr tests with content to check
	 * For use with ACCESS Assistant
	 * @returns {Array} of nmr tests with content to check
	 */function ast_nodeCapture_markdown(){stage0_markDown_liveDOM_domuels(window.document);}/**
	 * Returns JSON stringified array of nmr tests with content to check
	 * For use with ACCESS Assistant
	 * @returns {Array} of nmr tests with content to check
	 */function ast_nodeCapture_cleanup(){stage3_cleanUpLiveDOM(window.document);}/**
	 * API needed in AMP Firefox extension
	 * not using an amp_ for api prefix as it is refd in AMP
	 * runAllTests_returnXPathFilteredInstances_JSON is only called if an exclusion is registered in AMP
	 * @param {number[]} testTypeArray
	 * @param {string} exclusionsJSON
	 */function runAllTests_returnXPathFilteredInstances_JSON(){let testTypeArray=arguments.length>0&&arguments[0]!==undefined?arguments[0]:[testTypes.AUTOMATIC,testTypes.GUIDED_AUTOMATIC];let exclusionsJSON=arguments.length>1?arguments[1]:undefined;const excludes=JSON.parse(exclusionsJSON);aeKernel.markdownFuncs_failedToRun={};// for resilience
aeKernel.tests_failedToRun={};// for resilience
const outcomes=[];//Get a list of all elements matching all supplied xpaths
if(aeKernel.captureDOM()){const xpathSet=[];for(let i=0;i<excludes.length;i++){const x=aeKernel.rootTestNode.ownerDocument.evaluate(excludes[i],aeKernel.rootTestNode.ownerDocument,null,XPathResult.UNORDERED_NODE_ITERATOR_TYPE,null);try{let node=x.iterateNext();while(node){//Does the set already contain the element?
if(!xpathSet.some(cv=>cv===node)){xpathSet.push(node);}node=x.iterateNext();}}catch(e){logger.error(e);}}for(let i=0;i<testTypeArray.length;i++){const testType=testTypeArray[i];const tests=aeKernel.getTestsToRun(testType);const keys=Object.keys(tests);for(let j=0;j<keys.length;j++){const key=keys[j];const test=tests[key];//This is a web API so only run tests of type WEB
if(test.mediaType===mediaTypes.WEB){try{const testFunc=test.testFunc();// added in break-up
testFunc.rootTestNode=aeKernel.rootTestNode;const testRes=testFunc.getResult("instances");if(testRes.outcome==="fail"){const instances=testRes.instances;for(let k=0;k<instances.length;k++){const instance=instances[k];try{let shouldExclude=false;for(let l=0;l<xpathSet.length;l++){const comparableInstance=xpathSet[l];const path_1=LevelAccess_AccessEngine.uelAccurate_FromRoot(instance);const path_2=LevelAccess_AccessEngine.uelAccurate_FromRoot(comparableInstance);if(path_1.includes(path_2)){shouldExclude=true;break;}}if(!shouldExclude){const outcome=formatOutcomeForToolbar(instances[k],test,testType);outcomes.push(outcome);}}catch(err){logger.error(err);}}}}catch(err){aeKernel.tests_failedToRun[key]=err;}}}}}return resultsAsJSON.stringify(outcomes);}/**
	* Returns opening xml tag only of instance with original attributes + values restored
	* @returns {String}
	*/function origAndroidXmlFromHTML(node){var origXml="<"+node.getAttribute("data-xmlnodetype")+' resource-id="';if(node.hasAttribute("data-resource-id")){origXml+=node.getAttribute("data-resource-id").replace(/"/g,'\\"');}origXml+='" ';//Only process xmlattmapping if it actually exists
if(node.hasAttribute("data-xmlattmapping")){var attMappingPieces=node.getAttribute("data-xmlattmapping").split(";");for(var i=0,len_mapping=attMappingPieces.length;i<len_mapping;i++){var attSet=attMappingPieces[i];if(attSet.indexOf(":")!==-1){var atts=attSet.split(":");var xmlAtt=atts[0];// gets html attribute
var htmlAtt=atts[1];// gets html attribute
var xmlAttValue=node.getAttribute(htmlAtt);if(xmlAttValue!==null){origXml=origXml+xmlAtt+'="'+xmlAttValue.replace(/"/g,'\\"')+'" ';}}}}var result=origPositionAttsFromStyle(node.getAttribute("style"));origXml=origXml+'bounds="['+result[0]+','+result[1]+']['+result[2]+','+result[3]+']"'+" ";// don't close the tag as it could be self-closing or not self-closing.
return origXml.trimRight()+"...";}/**
	 * AMP data model
	 * 0 instance_id - bind later
	 * 1 module_id - bind later
	 * 2 violation_id - bestPractice
	 * 3 line_number - cannot supply
	 * 4 element - outerHTML up to 3000 characters
	 * 5 attribute - ??? Access Engine Test ID: initially plain English of Access Engine Test. getRichData() {} key id returns functions for getting text?
	 * 6 test_result - VIOLATION_FAILED = 0; VIOLATION_PASSED = 1; VIOLATION_NOT_APPLICABLE = 2
	 * 7 css cssSelector
	 * @param instance
	 * @param test
	 * @param type
	 * @returns {{}}
	 */function formatOutcomeForAccessContinuumMobileAndroid(instance,test,type){var outcome={};outcome.engineTestId=test.testId;outcome.bestPracticeId=test.bestPractice;// HTML from XML will contain the mapping of xml attributes to HTML attributes used when creating the HTML from the XML
// original xml component name is in data-xmlNodeType attribute
// mapping is in data-xmlAttMapping attribute
// mapping data looks like label:aria-label;name:src;width:css-width;height:css-height;x:css-top;y:css-left;
// So we just have to reverse the mapping
let element;if(instance.hasAttribute("data-xmlnodetype")){element=origAndroidXmlFromHTML(instance).substring(0,2998);// data-base takes string to 3000
}else{element=null;}outcome.element=element;switch(type){case testTypes.GUIDED_AUTOMATIC:outcome.testResult=3;break;case testTypes.AUTOMATIC:case testTypes.ADVISORY:case testTypes.MANUAL:case testTypes.PREVIEW_MODE:default:// Everything besides GA should be considered a failure
outcome.testResult=0;break;}// Note - might want XPATH - but xpath is costly to generate; and to use
// outcome.path = "";
var note=test.metaText;let attributeDetail;if(note!==undefined){if(instance.hasAttribute("data-xmlnodetype")){attributeDetail=note.replace("[[object]]",instance.getAttribute("data-xmlnodetype").split("android.widget.")[1]);}else{attributeDetail=note;}}else{attributeDetail="";}outcome.attributeDetail=attributeDetail;return outcome;}/**
	 * Returns variant 3 test result for failed tests - as a JSON object
	 * @param {Number} testType
	 * @returns {Object} The outcomes object holds results for each "fail" in accordance with AMP data model
	 */function runAllTests_nativeAndroidUI_returnInstances_JSON(){let testTypeArray=arguments.length>0&&arguments[0]!==undefined?arguments[0]:[testTypes.AUTOMATIC];aeKernel.markdownFuncs_failedToRun={};// for resilience
aeKernel.tests_failedToRun={};// for resilience
var tests=null;var outcomes=[];if(aeKernel.captureDOM()){// runs through each test type specified - for mobile just [testTypes.AUTOMATIC]
for(var k=testTypeArray.length;k--;){let testType=testTypeArray[k];tests=aeKernel.getTestsToRun(testType);var keys=Object.keys(tests);for(var i=0,len_keys=keys.length;i<len_keys;i++){var key=keys[i];var test=tests[key];//This is an Android API so only run tests of type 3
if(test.mediaType==mediaTypes.ANDROID){try{var testFunc=test.testFunc();// added in break-up
testFunc.rootTestNode=aeKernel.rootTestNode;var testRes=testFunc.getResult("instances");if(testRes.outcome==="fail"){var instances=testRes.instances;for(var j=0,len_instances=instances.length;j<len_instances;j++){try{var outcome=formatOutcomeForAccessContinuumMobileAndroid(instances[j],test,testType);outcomes.push(outcome);}catch(err){logger.error(err);}}}}catch(err){aeKernel.tests_failedToRun[key]=err;logger.error('Error',err);}}}}}return resultsAsJSON.stringify(outcomes);}/**
	* Returns opening xml tag only of instance with original attributes + values restored
	* @returns {String}
	*/function origiOSXmlFromHTML(node){var origXml="<"+node.getAttribute("data-xmlnodetype")+' resource-id="';if(node.hasAttribute("data-resource-id")){origXml+=node.getAttribute("data-resource-id").replace(/"/g,'\\"');}origXml+='" ';//Only process xmlattmapping if it actually exists
if(node.hasAttribute("data-xmlattmapping")){var attMappingPieces=node.getAttribute("data-xmlattmapping").split(";");for(var i=0,len_mapping=attMappingPieces.length;i<len_mapping;i++){var attSet=attMappingPieces[i];if(attSet.indexOf(":")!==-1){var atts=attSet.split(":");var xmlAtt=atts[0];// gets html attribute
var htmlAtt=atts[1];// gets html attribute
var xmlAttValue=node.getAttribute(htmlAtt);if(xmlAttValue!==null){origXml=origXml+xmlAtt+'="'+xmlAttValue.replace(/"/g,'\\"')+'" ';}}}}var result=origPositionAttsFromStyle(node.getAttribute("style"));origXml=origXml+'width="'+result[0].replace(/"/g,'\\"')+'" height="'+result[1].replace(/"/g,'\\"')+'" x="'+result[2].replace(/"/g,'\\"')+'" y="'+result[3].replace(/"/g,'\\"')+'"'+" ";// don't close the tag as it could be self-closing or not self-closing.
return origXml.trimRight()+"...";}/**
	 * AMP data model
	 * 0 instance_id - bind later
	 * 1 module_id - bind later
	 * 2 violation_id - bestPractice
	 * 3 line_number - cannot supply
	 * 4 element - outerHTML up to 3000 characters
	 * 5 attribute - ??? Access Engine Test ID: initially plain English of Access Engine Test. getRichData() {} key id returns functions for getting text?
	 * 6 test_result - VIOLATION_FAILED = 0; VIOLATION_PASSED = 1; VIOLATION_NOT_APPLICABLE = 2
	 * 7 css cssSelector
	 * @param instance
	 * @param test
	 * @param type
	 * @returns {{}}
	 */function formatOutcomeForAccessContinuumMobileiOS(instance,test,type){var outcome={};outcome.engineTestId=test.testId;outcome.bestPracticeId=test.bestPractice;// HTML from XML will contain the mapping of xml attributes to HTML attributes used when creating the HTML from the XML
// original xml component name is in data-xmlNodeType attribute
// mapping is in data-xmlAttMapping attribute
// mapping data looks like label:aria-label;name:src;width:css-width;height:css-height;x:css-top;y:css-left;
// So we just have to reverse the mapping
let element;if(instance.hasAttribute("data-xmlnodetype")){element=origiOSXmlFromHTML(instance).substring(0,2998);// data-base takes string to 3000
}else{element=null;}outcome.element=element;switch(type){case testTypes.GUIDED_AUTOMATIC:outcome.testResult=3;break;case testTypes.AUTOMATIC:case testTypes.ADVISORY:case testTypes.MANUAL:case testTypes.PREVIEW_MODE:default:// Everything besides GA should be considered a failure
outcome.testResult=0;break;}// Note - might want XPATH - but xpath is costly to generate; and to use
// outcome.path = "";
var note=test.metaText;let attributeDetail;if(note!==undefined){if(instance.hasAttribute("data-xmlnodetype")){attributeDetail=note.replace("[[object]]",instance.getAttribute("data-xmlnodetype").split("XCUIElementType")[1]);}else{attributeDetail=note;}}else{attributeDetail="";}outcome.attributeDetail=attributeDetail;return outcome;}/**
	 * Returns variant 3 test result for failed tests - as a JSON object
	 * @param {Number} testType
	 * @returns {Object} The outcomes object holds results for each "fail" in accordance with AMP data model
	 */function runAllTests_nativeiOSUI_returnInstances_JSON(){let testTypeArray=arguments.length>0&&arguments[0]!==undefined?arguments[0]:[testTypes.AUTOMATIC];aeKernel.markdownFuncs_failedToRun={};// for resilience
aeKernel.tests_failedToRun={};// for resilience
var tests=null;// label:aria-label;width:css-width;height:css-height;x:css-top;y:css-left;
// css- implies its part of the style attribute value
var outcomes=[];if(aeKernel.captureDOM()){// runs through each test type specified - for mobile just [testTypes.AUTOMATIC]
for(var k=testTypeArray.length;k--;){let testType=testTypeArray[k];tests=aeKernel.getTestsToRun(testType);var keys=Object.keys(tests);for(var i=0,len_keys=keys.length;i<len_keys;i++){var key=keys[i];var test=tests[key];//This is an iOS API so only run tests of type 2
if(test.mediaType==mediaTypes.IOS){try{var testFunc=test.testFunc();// added in break-up
testFunc.rootTestNode=aeKernel.rootTestNode;var testRes=testFunc.getResult("instances");if(testRes.outcome==="fail"){var instances=testRes.instances;for(var j=0,len_instances=instances.length;j<len_instances;j++){try{var outcome=formatOutcomeForAccessContinuumMobileiOS(instances[j],test,testType);outcomes.push(outcome);}catch(err){logger.error(err);}}}}catch(err){aeKernel.tests_failedToRun[key]=err;logger.error('Error',err);}}}}}return resultsAsJSON.stringify(outcomes);}/**
	 * AMP data model
	 * 0 instance_id - bind later
	 * 1 module_id - bind later
	 * 2 violation_id - bestPractice
	 * 3 line_number - cannot supply
	 * 4 element - outerHTML up to 3000 characters
	 * 5 attribute - ??? Access Engine Test ID: initially plain English of Access Engine Test. getRichData() {} key id returns functions for getting text?
	 * 6 test_result - VIOLATION_FAILED = 0; VIOLATION_PASSED = 1; VIOLATION_NOT_APPLICABLE = 2
	 * 7 css cssSelector
	 * @param instance
	 * @param test
	 * @param type
	 * @returns {{}}
	 */function formatOutcomeForAnalytics(instance,test,type){var outcome={};outcome.t=test.testId;outcome.b=test.bestPractice;//Special handling for fixType 4
if(test.fixType.fixType==4){//No need for encoding or attNo in this case, always runs against html of whole page
outcome.css="html";outcome.attNo=0;outcome.version=2;outcome.encoding=[1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1];}else{var fingerprint=getFingerprint(removeAEAttributes(instance),undefined,undefined,2);outcome.css=fingerprint.css;outcome.attNo=fingerprint.attNo;outcome.encoding=fingerprint.encoding;outcome.version=fingerprint.version;}// Always automatic
outcome.r=0;outcome.uel=instance.getAttribute("data-ae_domuel");if(outcome.uel===null){outcome.uel=instance.getAttribute("data-ae_uel");}// handles fixes
outcome.fixable=test.fixType==""?0:1;return outcome;}/**
	 * Returns variant 3 test result for failed tests - as a JSON object
	 * @param {Array} testTypeArray
	 * @returns {Object} The outcomes object holds results for each "fail" in accordance with AMP data model
	 */function analytics_runAllTests_returnInstances_JSON(){let testTypeArray=arguments.length>0&&arguments[0]!==undefined?arguments[0]:[testTypes.AUTOMATIC];aeKernel.updateSuccess(null);aeKernel.markdownFuncs_failedToRun={};// for resilience
aeKernel.tests_failedToRun={};// for resilience
var tests=null;var outcomes=[];if(aeKernel.captureDOM()){// runs through each test type specified - for toolbar [testTypes.AUTOMATIC, testTypes.GUIDED_AUTOMATIC]
for(var k=testTypeArray.length;k--;){let testType=testTypeArray[k];tests=aeKernel.getTestsToRun(testType);var keys=Object.keys(tests);for(var i=0,len_keys=keys.length;i<len_keys;i++){var key=keys[i];var test=tests[key];//This is a web API so only run tests of type WEB
if(test.mediaType==mediaTypes.WEB){try{var testFunc=test.testFunc();// added in break-up
testFunc.rootTestNode=aeKernel.rootTestNode;var testRes=testFunc.getResult("instances");if(testRes.outcome==="fail"){var instances=testRes.instances;for(var j=0,len_instances=instances.length;j<len_instances;j++){var outcome=formatOutcomeForAnalytics(instances[j],test,testType);outcomes.push(outcome);}}}catch(err){logger.error(err);aeKernel.tests_failedToRun[key]=err;}}}}}aeKernel.updateSuccess();return resultsAsJSON.stringify(outcomes);}/**
	 * We need to expose test_analytics_outcomes through the api, but not by importing and exposing aeKernel
	 * Hence this step
	 */exports.test_analytics_outcomes=aeKernel.test_analytics_outcomes;/**
	 * Returns variant 3 test result for all tests - tests are run asyncronously using timers so they are non-blocking
	 * @param {Number} testType
	 * @param {function} callback
	 * @returns {Object} The outcomes object holds results for each "fail" test that has instances in terms of "bp id, [array of failing node instances]", with test name as a key
	 */function analytics_runAllTests_returnInstances_JSON_timer(callback){aeKernel.markdownFuncs_failedToRun={};// for resilience
aeKernel.tests_failedToRun={};// for resilience
exports.test_analytics_outcomes=[];var testType=testTypes.AUTOMATIC;var processTest=function(test){//This is a web API so only run tests of type WEB
if(test.mediaType==mediaTypes.WEB){//var outcome = [];
var testId=test.testId;try{var testFunc=test.testFunc();// needed to add this
testFunc.rootTestNode=aeKernel.rootTestNode;var testRes=testFunc.getResult("instances");if(testRes.instances){var instances=testRes.instances;for(var j=0,len_instances=instances.length;j<len_instances;j++){try{var outcome=formatOutcomeForAnalytics(instances[j],test,testType);exports.test_analytics_outcomes.push(outcome);}catch(err){logger.error(err);}}}}catch(err){aeKernel.tests_failedToRun[testId]=err;// for resilience
}}};timer_api_shell(testType,processTest,callback);}function formatResultForElevin(instance,fixType,shouldIncludeElement){let numAttributes=0;const instanceAttributes=instance.attributes;for(let i=0;i<instanceAttributes.length;i++){const attribute=instanceAttributes[i];const attributeName=attribute.name;if(!attributeName.startsWith("data-ae_")){numAttributes++;}}const result={};if(fixType&&fixType.fixType===4){result.attNo=0;result.css="html";result.encoding=[1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1];}else{result.attNo=numAttributes;result.css=getCssForFingerprint(instance);result.encoding=charFreq(instance.innerHTML);}result.uel=instance.getAttribute("data-ae_domuel")||instance.getAttribute("data-ae_uel");result.timestamp=new Date().toISOString();const tagName=instance.getAttribute("data-ae_origTagName")||instance.tagName.toLowerCase();if(shouldIncludeElement&&!['html','head','body'].includes(tagName)){const element=removeAEAttributes(instance);result.element=element.outerHTML;}const commonBlockData=getCommonBlockData(instance);result.clusteringData=commonBlockData?commonBlockData.clusteringData:null;return result;}let nextUniqueId$1;/**
	 *
	 * @param {Array} testTypeArray
	 * @param {boolean?} shouldIncludeElement
	 * @return {*|string}
	 */function nextgen_runAllTests_returnInstances_JSON(){let testTypeArray=arguments.length>0&&arguments[0]!==undefined?arguments[0]:[testTypes.AUTOMATIC];let shouldIncludeElement=arguments.length>1&&arguments[1]!==undefined?arguments[1]:false;aeKernel.updateSuccess(null);aeKernel.markdownFuncs_failedToRun={};aeKernel.tests_failedToRun={};const assertions=[];if(!aeKernel.captureDOM()){return resultsAsJSON.stringify(assertions);}// assign unique IDs to all elements prior to building results
nextUniqueId$1=0;// reset unique ID generator from any previous invocation of this function
const allElements=aeKernel.rootTestNode.querySelectorAll("*");for(const element of allElements){let uniqueId=element.getAttribute("data-ae_uid");if(!uniqueId){// element has not yet been assigned a unique ID, so assign one
uniqueId=nextUniqueId$1++;element.setAttribute("data-ae_uid",uniqueId);}}for(let i=0;i<testTypeArray.length;i++){const testType=testTypeArray[i];const testTypeName=Object.keys(testTypes).find(key=>testTypes[key]===testType);const tests=aeKernel.getTestsToRun(testType);const testIds=Object.keys(tests);for(let j=0;j<testIds.length;j++){const testId=testIds[j];const test=tests[testId];if(test.mediaType!==mediaTypes.WEB){continue;}let outcome,failingNodes;try{const testFunc=test.testFunc();testFunc.rootTestNode=aeKernel.rootTestNode;const results=testFunc.getNextGenResultsMetadata();outcome=results.outcome;failingNodes=results.failingNodes;}catch(err){aeKernel.tests_failedToRun[testId]=err;continue;}const results=[];for(let k=0;k<failingNodes.length;k++){const failingNode=failingNodes[k];let result;try{result=formatResultForElevin(failingNode,test.fixType,shouldIncludeElement);}catch(err){logger.error(err);continue;}results.push(result);}const assertion={testId:testId,testType:testTypeName,outcome:outcome,results:results};assertions.push(assertion);}}aeKernel.updateSuccess();return resultsAsJSON.stringify(assertions);}let nextUniqueId;/**
	 * @param {Element} node
	 */function winnowTestNode(node){// css selectors don't work on :root e.g. an A element placed as rootTestNode would not be able to be located.
// This means we have to get the parent element; then remove all other contents apart from captured node
// The selector done in this manner works in environments that have non-html tags e.g. <tile:importantattribute
let cssSelectorForCapturedNode="";if(node.hasAttribute("data-ae_domuel")){const domUel=node.getAttribute("data-ae_domuel");cssSelectorForCapturedNode='[data-ae_domuel="'+domUel.replace(/"/g,'\\"')+'"]';}else{const nodeCapture_UEL=uelAccurate_FromRoot(node);cssSelectorForCapturedNode='[data-ae_uel="'+nodeCapture_UEL.replace(/"/g,'\\"')+'"]';}// it is important to use the quote types as shown to avoid syntax errors e.g. [data-ae_uel='[id='main']>*:nth-child(2)']
// IMPORTANT: (need to add to test writing doc) we have to use [data-ae_uel=] format in selectors, as nodeCapture will change the DOM structure and stop the ids from working directly
const capturedNode=aeKernel.rootTestNode.querySelector(cssSelectorForCapturedNode);const parentElementOfCapturedNode=getParent(capturedNode);const children=parentElementOfCapturedNode.children;for(let j=children.length;j--;){if(children[j]!==capturedNode){parentElementOfCapturedNode.removeChild(children[j]);}}return parentElementOfCapturedNode;}/**
	 * @param {Element} node to test
	 * @param {Array} testTypeArray
	 * @param {boolean?} shouldIncludeElement
	 * @return {*|string}
	 */function nextgen_runAllTests_returnInstances_JSON_NodeCapture(node){let testTypeArray=arguments.length>1&&arguments[1]!==undefined?arguments[1]:[testTypes.AUTOMATIC,testTypes.GUIDED_AUTOMATIC];let shouldIncludeElement=arguments.length>2&&arguments[2]!==undefined?arguments[2]:false;aeKernel.updateSuccess(null);aeKernel.markdownFuncs_failedToRun={};aeKernel.tests_failedToRun={};const assertions=[];if(!aeKernel.captureDOM()){return resultsAsJSON.stringify(assertions);}// Remove everything that isn't the node we want to test and set to root node
aeKernel.rootTestNode=winnowTestNode(node);// assign unique IDs to all elements prior to building results
nextUniqueId=0;// reset unique ID generator from any previous invocation of this function
const allElements=aeKernel.rootTestNode.querySelectorAll("*");for(const element of allElements){let uniqueId=element.getAttribute("data-ae_uid");if(!uniqueId){// element has not yet been assigned a unique ID, so assign one
uniqueId=nextUniqueId++;element.setAttribute("data-ae_uid",uniqueId);}}for(let i=0;i<testTypeArray.length;i++){const testType=testTypeArray[i];const testTypeName=Object.keys(testTypes).find(key=>testTypes[key]===testType);const tests=aeKernel.getTestsToRun(testType);const testIds=Object.keys(tests);for(let j=0;j<testIds.length;j++){const testId=testIds[j];const test=tests[testId];if(test.mediaType!==mediaTypes.WEB){continue;}let outcome,failingNodes;try{const testFunc=test.testFunc();testFunc.rootTestNode=aeKernel.rootTestNode;const results=testFunc.getNextGenResultsMetadata();outcome=results.outcome;failingNodes=results.failingNodes;}catch(err){aeKernel.tests_failedToRun[testId]=err;continue;}const results=[];for(let k=0;k<failingNodes.length;k++){const failingNode=failingNodes[k];let result;try{result=formatResultForElevin(failingNode,test.fixType,shouldIncludeElement);}catch(err){logger.error(err);continue;}results.push(result);}const assertion={testId:testId,testType:testTypeName,outcome:outcome,results:results};assertions.push(assertion);}}aeKernel.updateSuccess();return resultsAsJSON.stringify(assertions);}/**
	 * Gets the environment-level details from the `window` and `document` objects of the page under test
	 */function getEnvironmentDetails(){const{window,document}=aeKernel;return{width:window?window.innerWidth:null,height:window?window.innerHeight:null,docWidth:document?document.body.scrollWidth:null,docHeight:document?document.body.scrollHeight:null,orientation:window?window.screen.orientation.type:null,userAgent:window?window.navigator.userAgent:null};}// default
// Automatic Tests
test8();test15();test14();test16();test21();test22();test26();test27();test28();test31();test33();test34();test35();test37();test38();test40();test41();test43();test44();test48();test50();test51();test53();test54();test55();test57();test59();test61();test64();test65();test66();test67();test69();test71();test74();test78();test80();test81();test83();test85();test87();test89();test94();test95();test99();test101();test113();test114();test116();test118();test123();test131();test132();test134();test140();test141();test152();test165();test166();test167();test168();test169();test170();test171();test173();test174();test175();test176();test180();test182();test185();test188();test191();test192();test199();test203();test205();test214();test221();test224();test226();test227();test228();test229();test230();test231();test232();test234();test235();test236();test237();test238();test244();test245();test246();test249();test251();test252();test254();test257();test258();test259();test260();test261();test262();test263();test264();test265();test266();test267();test268();test269();test270();test271();test272();test273();test274();test275();test276();test277();test278();test279();test280();test281();test282();test283();test284();test285();test286();test287();test288();test290();test291();test292();test294();test297();test298();test299();test300();test301();test302();test303();test304();test309();test311();test312();test313();test314();test315();test317();test318();test319();test322();test325();test326();test328();test329();test330();test331();test333();test334();test337();test338();test341();test350();test351();test352();test356();test360();test361();test363();test364();test365();test366();test367();test368();test369();test370();test371();test372();test373();test374();test375();test376();test385();test386();test387();test388();test389();test390();test391();test392();test393();test394();test395();test396();test397();test412();test477();test485();test505();test507();test508();test509();test510();test511();test520();test527();test531();test532();test533();test581();test582();test583();test609();test610();test630();test647();test704();test706();test725();test726();test734();test736();test738();test779();test790();test809();test825();test828();test863();test872();test874();test884();test1010();test1017();test1038();test1039();test1040();test1043();test1066();test1077();test1078();test1080();test1097();test1098();test1099();test1100();test1105();test1106();test1107();test1108();test1121();// iOS Automatic Tests
test666();test684();test685();test686();test687();test688();test689();test690();// Android Automatic Tests
test677();test691();test692();test693();test694();test695();test696();test697();test698();test699();test700();test701();test702();test703();test740();test742();test788();// Guided Automatic Tests
test17();test39();test45();test52();test58();test91();test107();test109();test112();test129();test135();test136();test137();test138();test153();test239();test241();test242();test248();test253();test400();test401();test402();test445();test479();test483();test490();test522();test523();test590();test591();test592();test593();test594();test595();test596();test597();test598();test599();test600();test707();test709();test710();test711();test712();test713();test714();test715();test716();test717();test718();test719();test720();test721();test722();test723();test724();test727();test728();test729();test735();test737();test848();test849();test871();test875();test879();test880();test925();test933();test934();test993();test1014();test1092();test1093();test1094();test1109();test1110();test1111();test1112();test1113();test1119();// Android Guided Automatics
test743();// iOS Guided Automatic Tests
test744();test746();test789();// Advisory Automatic Tests
test2();test4();test5();test6();test7();test9();test10();test11();test13();test19();test20();test23();test24();test25();test29();test36();test46();test49();test56();test62();test63();test68();test70();test72();test73();test75();test76();test77();test79();test82();test84();test86();test111();test130();test161();test163();test164();test172();test177();test178();test179();test181();test183();test184();test186();test187();test190();test193();test194();test196();test197();test198();test200();test201();test202();test204();test220();test225();test247();test293();test295();test296();test305();test306();test307();test308();test310();test316();test320();test321();test323();test324();test327();test379();test380();test381();test382();test383();test384();test1042();// iOS Advisory Automatics
test745();// Preview Modes
previewMode_backgroundImages();previewMode_elsWithLangAttribs();previewMode_headings();previewMode_iframes();previewMode_images();previewMode_lists();previewMode_tableCells();previewMode_tableHeadings();previewMode_tables();exports.ACCEPTED_LANGUAGE_FULLTAGS=ACCEPTED_LANGUAGE_FULLTAGS;exports.ACCEPTED_LANGUAGE_SUBTAGS=ACCEPTED_LANGUAGE_SUBTAGS;exports.analytics_runAllTests_returnInstances_JSON=analytics_runAllTests_returnInstances_JSON;exports.analytics_runAllTests_returnInstances_JSON_timer=analytics_runAllTests_returnInstances_JSON_timer;exports.ast_GetLiveDOMElementFromDOMUEL=ast_GetLiveDOMElementFromDOMUEL;exports.ast_getTestById=ast_getTestById;exports.ast_nodeCapture_cleanup=ast_nodeCapture_cleanup;exports.ast_nodeCapture_markdown=ast_nodeCapture_markdown;exports.ast_runAllTests_returnInstances_JSON=ast_runAllTests_returnInstances_JSON;exports.ast_runAllTests_returnInstances_JSON_NodeCapture=ast_runAllTests_returnInstances_JSON_NodeCapture;exports.ast_runMarkDown_GetExpandedShadDOM_ForNode=ast_runMarkDown_GetExpandedShadDOM_ForNode;exports.ast_runPreviewMode_returnInstances_JSON=ast_runPreviewMode_returnInstances_JSON;exports.getDidMarkdownStageExecutionTimeOut=getDidMarkdownStageExecutionTimeOut;exports.getEnvironmentDetails=getEnvironmentDetails;exports.getFingerprint=getFingerprint;exports.getMarkdownFuncs_failedToRun=getMarkdownFuncs_failedToRun;exports.getMarkdownStageExecutionTimeout=getMarkdownStageExecutionTimeout;exports.getRootTestNode=getRootTestNode;exports.getSuccess=getSuccess;exports.getTestInfo=getTestInfo;exports.getTests_failedToRun=getTests_failedToRun;exports.getVersion=getVersion;exports.logger=logger;exports.nextgen_runAllTests_returnInstances_JSON=nextgen_runAllTests_returnInstances_JSON;exports.nextgen_runAllTests_returnInstances_JSON_NodeCapture=nextgen_runAllTests_returnInstances_JSON_NodeCapture;exports.runAllTests_nativeAndroidUI_returnInstances_JSON=runAllTests_nativeAndroidUI_returnInstances_JSON;exports.runAllTests_nativeiOSUI_returnInstances_JSON=runAllTests_nativeiOSUI_returnInstances_JSON;exports.runAllTests_returnFailedNumbers=runAllTests_returnFailedNumbers;exports.runAllTests_returnFailedNumbers_timers=runAllTests_returnFailedNumbers_timers;exports.runAllTests_returnFailedTests=runAllTests_returnFailedTests;exports.runAllTests_returnInstances=runAllTests_returnInstances;exports.runAllTests_returnInstances_JSON=runAllTests_returnInstances_JSON;exports.runAllTests_returnInstances_JSON_NodeCapture=runAllTests_returnInstances_JSON_NodeCapture;exports.runAllTests_returnInstances_timers=runAllTests_returnInstances_timers;exports.runAllTests_returnXPathFilteredInstances_JSON=runAllTests_returnXPathFilteredInstances_JSON;exports.runTest_returnInstances=runTest_returnInstances;exports.runTest_returnNumbers=runTest_returnNumbers;exports.runTest_returnOutcome=runTest_returnOutcome;exports.setMarkdownStageExecutionTimeout=setMarkdownStageExecutionTimeout;exports.setWindowUnderTest=setWindowUnderTest;exports.testTypes=testTypes;exports.test_stages=test_stages;exports.uelAccurate_FromRoot=uelAccurate_FromRoot;Object.defineProperty(exports,'__esModule',{value:true});return exports;}({});
